@@ -59,6 +59,42 @@ typedef
 	}
 	CACHE_ENTRY;
 
+// mySQL datatypes
+
+static CONV_STRING_TYPE _types[] =
+	{
+		{ "tinyint", FIELD_TYPE_TINY },
+		{ "smallint", FIELD_TYPE_SHORT },
+		{ "mediumint", FIELD_TYPE_INT24 },
+		{ "int", FIELD_TYPE_LONG },
+		{ "bigint", FIELD_TYPE_LONGLONG },
+		{ "decimal", FIELD_TYPE_DECIMAL },
+		{ "numeric", FIELD_TYPE_DECIMAL },
+		{ "float", FIELD_TYPE_FLOAT },
+		{ "double", FIELD_TYPE_DOUBLE },
+		{ "real", FIELD_TYPE_DOUBLE },
+		{ "timestamp", FIELD_TYPE_TIMESTAMP },
+		{ "date", FIELD_TYPE_DATE },
+		{ "time", FIELD_TYPE_TIME },
+		{ "datetime", FIELD_TYPE_DATETIME },
+		{ "year", FIELD_TYPE_YEAR },
+		{ "char", FIELD_TYPE_STRING },
+		{ "varchar", FIELD_TYPE_VAR_STRING },
+		{ "blob", FIELD_TYPE_BLOB },
+		{ "tinyblob", FIELD_TYPE_TINY_BLOB },
+		{ "mediumblob", FIELD_TYPE_MEDIUM_BLOB },
+		{ "longblob", FIELD_TYPE_LONG_BLOB },
+		{ "text", FIELD_TYPE_BLOB },
+		{ "tinytext", FIELD_TYPE_TINY_BLOB },
+		{ "mediumtext", FIELD_TYPE_MEDIUM_BLOB },
+		{ "longtext", FIELD_TYPE_LONG_BLOB },
+		{ "set", FIELD_TYPE_SET },
+		{ "enum", FIELD_TYPE_ENUM },
+		{ "bit", FIELD_TYPE_BIT },
+		{ "null", FIELD_TYPE_NULL },
+		{ NULL, 0 },
+	};
+
 /* internal function to quote a value stored as a string */
 
 static void quote_string(const char *data, long len, DB_FORMAT_CALLBACK add)
@@ -127,14 +163,20 @@ static char *get_quote_string(const char *str, int len, char quote)
 	return res;
 }
 
-/* Internal function to convert a database type into a Gambas type */
+// Internal function to convert a database type into a Gambas type
+//
+// Look at https://dev.mysql.com/doc/refman/5.0/en/c-api-data-structures.html
+// for how to make the difference between a text field and a blob field.
 
-static GB_TYPE conv_type(int type, int len)
+#define IS_BINARY_FIELD(_f) ((_f)->charsetnr == 63)
+#define SET_BINARY_FIELD(_f, _v) ((_f)->charsetnr = (_v) ? 63 : 0)
+
+static GB_TYPE conv_type(const MYSQL_FIELD *f)
 {
-	switch(type)
+	switch(f->type)
 	{
 		case FIELD_TYPE_TINY:
-			return (len == 1 ? GB_T_BOOLEAN : GB_T_INTEGER);
+			return (f->max_length == 1 ? GB_T_BOOLEAN : GB_T_INTEGER);
 
 		case FIELD_TYPE_INT24:
 		case FIELD_TYPE_SHORT:
@@ -157,26 +199,22 @@ static GB_TYPE conv_type(int type, int len)
 			return GB_T_DATE;
 
 		case FIELD_TYPE_LONG_BLOB:
-			//fprintf(stderr, "FIELD_TYPE_LONG_BLOB: %d\n", len);
-			return DB_T_BLOB;
-
-		case FIELD_TYPE_BLOB: // TEXT
-			//fprintf(stderr, "FIELD_TYPE_BLOB: %d\n", len);
-			if (len >= 0x1000000 || len <= 0) // LONG BLOB
+		case FIELD_TYPE_TINY_BLOB:
+		case FIELD_TYPE_MEDIUM_BLOB:
+		case FIELD_TYPE_BLOB:
+			if (IS_BINARY_FIELD(f))
 				return DB_T_BLOB;
 			else
 				return GB_T_STRING;
 			
 		case FIELD_TYPE_BIT:
-			if (len == 1)
+			if (f->max_length == 1)
 				return GB_T_BOOLEAN;
-			else if (len <= 32)
+			else if (f->max_length <= 32)
 				return GB_T_INTEGER;
-			else if (len <= 64)
+			else if (f->max_length <= 64)
 				return GB_T_LONG;
 
-		case FIELD_TYPE_TINY_BLOB:
-		case FIELD_TYPE_MEDIUM_BLOB:
 		case FIELD_TYPE_STRING:
 		case FIELD_TYPE_VAR_STRING:
 		case FIELD_TYPE_SET:
@@ -189,51 +227,18 @@ static GB_TYPE conv_type(int type, int len)
 }
 
 
-/* Internal function to convert a string database type into an integer database type */
+// Internal function to convert a string database type
+// into a fake MYSQL_FIELD structure
 
-static int conv_string_type(const char *type, long *len)
+static void conv_string_type(const char *type, MYSQL_FIELD *f)
 {
-	static CONV_STRING_TYPE types[] =
-	{
-		{ "tinyint", FIELD_TYPE_TINY },
-		{ "smallint", FIELD_TYPE_SHORT },
-		{ "mediumint", FIELD_TYPE_INT24 },
-		{ "int", FIELD_TYPE_LONG },
-		{ "bigint", FIELD_TYPE_LONGLONG },
-		{ "decimal", FIELD_TYPE_DECIMAL },
-		{ "numeric", FIELD_TYPE_DECIMAL },
-		{ "float", FIELD_TYPE_FLOAT },
-		{ "double", FIELD_TYPE_DOUBLE },
-		{ "real", FIELD_TYPE_DOUBLE },
-		{ "timestamp", FIELD_TYPE_TIMESTAMP },
-		{ "date", FIELD_TYPE_DATE },
-		{ "time", FIELD_TYPE_TIME },
-		{ "datetime", FIELD_TYPE_DATETIME },
-		{ "year", FIELD_TYPE_YEAR },
-		{ "char", FIELD_TYPE_STRING },
-		{ "varchar", FIELD_TYPE_VAR_STRING },
-		{ "blob", FIELD_TYPE_BLOB },
-		{ "tinyblob", FIELD_TYPE_TINY_BLOB },
-		{ "mediumblob", FIELD_TYPE_MEDIUM_BLOB },
-		{ "longblob", FIELD_TYPE_LONG_BLOB },
-		{ "text", FIELD_TYPE_BLOB },
-		{ "tinytext", FIELD_TYPE_TINY_BLOB },
-		{ "mediumtext", FIELD_TYPE_MEDIUM_BLOB },
-		{ "longtext", FIELD_TYPE_LONG_BLOB },
-		{ "set", FIELD_TYPE_SET },
-		{ "enum", FIELD_TYPE_ENUM },
-		{ "bit", FIELD_TYPE_BIT },
-		{ "null", FIELD_TYPE_NULL },
-		{ NULL, 0 },
-	};
-
 	CONV_STRING_TYPE *cst;
 	long l;
 
 	if (strncmp(type, "national ", 9) == 0)
 		type += 9;
 
-	for (cst = types; cst->pattern; cst++)
+	for (cst = _types; cst->pattern; cst++)
 	{
 		if (strncmp(type, cst->pattern, strlen(cst->pattern)) == 0)
 			break;
@@ -241,37 +246,53 @@ static int conv_string_type(const char *type, long *len)
 
 	if (cst->type)
 	{
-		if (len)
+		SET_BINARY_FIELD(f, FALSE);
+		f->max_length = 0;
+		
+		if (cst->type == FIELD_TYPE_BLOB || cst->type == FIELD_TYPE_TINY_BLOB || cst->type == FIELD_TYPE_MEDIUM_BLOB || cst->type == FIELD_TYPE_LONG_BLOB)
+		{
+			SET_BINARY_FIELD(f, strcmp(&type[strlen(type) - 4], "blob") == 0);
+		}
+		else
 		{
 			type += strlen(cst->pattern);
 			if (sscanf(type, "(%ld)", &l) == 1)
-				*len = l;
-			else if (cst->type == FIELD_TYPE_LONG_BLOB)
-				*len = -1;
-			else if (cst->type == FIELD_TYPE_MEDIUM_BLOB || cst->type == FIELD_TYPE_BLOB)
-				*len = 65535;
-			else
-				*len = 0;
+				f->max_length = l;
 		}
 	}
 
-	return cst->type;
+	f->type = cst->type;
 }
 
+#if 0
+static const char *get_type_name(int type)
+{
+	CONV_STRING_TYPE *cst;
+
+	for (cst = _types; cst->pattern; cst++)
+	{
+		if (cst->type == type)
+			return cst->pattern;
+	}
+
+	return "?";
+}
+#endif
 
 /* Internal function to convert a database value into a Gambas variant value */
 
-static void conv_data(int version, const char *data, long data_length, GB_VARIANT_VALUE *val, int type, int len)
+static void conv_data(int version, const char *data, long data_length, GB_VARIANT_VALUE *val, MYSQL_FIELD *f)
 {
 	GB_VALUE conv;
 	GB_DATE_SERIAL date;
 	double sec;
+	int type = f->type;
 
 	switch (type)
 	{
 		case FIELD_TYPE_TINY:
 
-			if (len == 1)
+			if (f->max_length == 1)
 			{
 				val->type = GB_T_BOOLEAN;
 				/*GB.NumberFromString(GB_NB_READ_INTEGER, data, strlen(data), &conv);*/
@@ -394,23 +415,19 @@ static void conv_data(int version, const char *data, long data_length, GB_VARIAN
 
 			break;
 
-		case FIELD_TYPE_LONG_BLOB:
-			// The BLOB are read by the blob_read() driver function
-			// You must set NULL there.
-			val->type = GB_T_NULL;
-			break;
-
-		// query_fill() only gets this constant, whatever the blob is
 		case FIELD_TYPE_BLOB:
-			if (len == 16777215 || len <= 0) // LONG BLOB
+		case FIELD_TYPE_LONG_BLOB:
+		case FIELD_TYPE_TINY_BLOB:
+		case FIELD_TYPE_MEDIUM_BLOB:
+			if (IS_BINARY_FIELD(f))
 			{
+				// The BLOB are read by the blob_read() driver function
+				// You must set NULL there.
 				val->type = GB_T_NULL;
 				break;
 			}
 			// else continue!
 
-		case FIELD_TYPE_TINY_BLOB:
-		case FIELD_TYPE_MEDIUM_BLOB:
 		case FIELD_TYPE_STRING:
 		case FIELD_TYPE_VAR_STRING:
 		case FIELD_TYPE_SET:
@@ -724,12 +741,8 @@ static int open_database(DB_DESC *desc, DB_DATABASE *db)
 
 	conn = mysql_init(NULL);
 
-	/* (BM) connect by default to the mysql database */
-
-	if (desc->name)
-		name = desc->name;
-	else
-		name = "mysql"; /* Note: Users may not have access to database mysql */
+	// NULL is a possible database name
+	name = desc->name;
 
 	//mysql_options(conn, MYSQL_READ_DEFAULT_GROUP,"Gambas");
 
@@ -991,6 +1004,8 @@ static void query_release(DB_RESULT result, DB_INFO *info)
 	result.
 	<next> is a boolean telling if we want the next row.
 
+	This function must return DB_OK, DB_ERROR or DB_NO_DATA
+	
 	This function must use GB.StoreVariant() to store the value in the
 	buffer.
 
@@ -1010,7 +1025,7 @@ static int query_fill(DB_DATABASE *db, DB_RESULT result, int pos, GB_VARIANT_VAL
 
 	row = mysql_fetch_row(res);
 	mysql_field_seek(res, 0);
-	for ( i=0; i < mysql_num_fields(res); i++)
+	for (i = 0; i < mysql_num_fields(res); i++)
 	{
 		field = mysql_fetch_field(res);
 		data = row[i];
@@ -1019,14 +1034,14 @@ static int query_fill(DB_DATABASE *db, DB_RESULT result, int pos, GB_VARIANT_VAL
 		value.value.type = GB_T_NULL;
 
 		if (data)
-			conv_data(db->version, data, mysql_fetch_lengths(res)[i], &value.value, field->type, field->length);
+			conv_data(db->version, data, mysql_fetch_lengths(res)[i], &value.value, field);
 
 		GB.StoreVariant(&value, &buffer[i]);
 	
 		//fprintf(stderr, "query_fill: %d: (%d, %d) : %s : %d\n", i, field->type, field->length, data, buffer[i].type);
 	}
 
-	return FALSE;
+	return DB_OK;
 }
 
 
@@ -1179,7 +1194,9 @@ static int field_index(DB_RESULT Result, const char *name, DB_DATABASE *db)
 static GB_TYPE field_type(DB_RESULT result, int field)
 {
 	MYSQL_FIELD *f = mysql_fetch_field_direct((MYSQL_RES *)result, field);
-	return conv_type(f->type, f->length);
+	GB_TYPE type = conv_type(f);
+
+	return type;
 }
 
 
@@ -1197,12 +1214,12 @@ static GB_TYPE field_type(DB_RESULT result, int field)
 static int field_length(DB_RESULT result, int field)
 {
 	MYSQL_FIELD *f = mysql_fetch_field_direct((MYSQL_RES *)result, field);
-	GB_TYPE type = conv_type(f->type, f->length);
+	GB_TYPE type = conv_type(f);
 
 	if (type != GB_T_STRING)
 		return 0;
 	else
-		return f->length;
+		return f->max_length;
 }
 
 
@@ -1890,10 +1907,9 @@ static int field_info(DB_DATABASE *db, const char *table, const char *field, DB_
 
 	MYSQL_RES *res;
 	MYSQL_ROW row;
+	MYSQL_FIELD f;
 	GB_VARIANT def;
 	char *val;
-	int type;
-	long len = 0;
 
 	if (do_query_cached(db, "Unable to get field info: &1", &res, "sfc:&1", query, 1, table))
 		return TRUE;
@@ -1905,16 +1921,11 @@ static int field_info(DB_DATABASE *db, const char *table, const char *field, DB_
 	}
 
 	info->name = NULL;
-	type = conv_string_type(row[1], &len);
+	conv_string_type(row[1], &f);
+	info->type = conv_type(&f);
 
-	info->type = conv_type(type, len);
 	if (info->type == GB_T_STRING)
-	{
-		info->length = len;
-		/* (BM) That's new! a TEXT field has a length of 65535 */
-		if (info->length >= 65535)
-			info->length = 0;
-	}
+		info->length = f.max_length;
 	else
 		info->length = 0;
 
@@ -1937,7 +1948,7 @@ static int field_info(DB_DATABASE *db, const char *table, const char *field, DB_
 
 			if (val && *val)
 			{
-				conv_data(db->version, val, 0, &def.value, type, len);
+				conv_data(db->version, val, 0, &def.value, &f);
 				GB.StoreVariant(&def, &info->def);
 			}
 		}
