@@ -39,6 +39,7 @@
 	#undef PACKAGE_STRING
 	#undef PACKAGE_TARNAME
 	#undef PACKAGE_VERSION
+	#undef PACKAGE_URL
 #endif
 
 #include "gb.db.proto.h"
@@ -455,6 +456,11 @@ static void conv_data(const char *data, int len, GB_VARIANT_VALUE *val, Oid type
 			if (bc)
 				date.year = (-date.year);
 
+			// 4713-01-01 BC is used for null dates
+			
+			if (date.year == -4713 && date.month == 1 && date.day == 1)
+				date.year = date.month = date.day = 0;
+
 			GB.MakeDate(&date, (GB_DATE *)&conv);
 
 			val->type = GB_T_DATE;
@@ -721,7 +727,7 @@ static int open_database(DB_DESC *desc, DB_DATABASE *db)
 
 	if (status != PGRES_COMMAND_OK)
 	{
-		GB.Error("Cannot set datestyle to ISO: &1", PQerrorMessage(conn));
+		GB.Error("Cannot set 'datestyle' to 'ISO': &1", PQerrorMessage(conn));
 		PQclear(res);
 		PQfinish(conn);
 		return TRUE;
@@ -732,6 +738,20 @@ static int open_database(DB_DESC *desc, DB_DATABASE *db)
 	db->handle = conn;
 	db->version = db_version(db);
 
+	if (db->version >= 90000)
+	{
+		res = PQexec(conn, "set bytea_output=escape");
+		status = PQresultStatus(res);
+
+		if (status != PGRES_COMMAND_OK)
+		{
+			GB.Error("Cannot set 'bytea_output' to 'escape': &1", PQerrorMessage(conn));
+			PQclear(res);
+			PQfinish(conn);
+			return TRUE;
+		}
+	}
+	
 	/* flags */
 
 	db->flags.no_table_type = TRUE;
@@ -852,24 +872,36 @@ static int format_value(GB_VALUE *arg, DB_FORMAT_CALLBACK add)
 
 			date = GB.SplitDate((GB_DATE *)arg);
 
-			bc = date->year < 0;
+			// Gambas year is between -4801 and +9999
+			// PostgreSQL year is between -4713 and +294276
+			// So let's use -4713 for representing null dates.
 
-			l = sprintf(_buffer, "'%04d-%02d-%02d %02d:%02d:%02d",
-				abs(date->year), date->month, date->day,
-				date->hour, date->min, date->sec);
-
-			add(_buffer, l);
-
-			if (date->msec)
+			if (date->year == 0)
 			{
-				l = sprintf(_buffer, ".%03d", date->msec);
+				l = sprintf(_buffer, "'4713-01-01 %02d:%02d:%02d BC'", date->hour, date->min, date->sec);
 				add(_buffer, l);
 			}
+			else
+			{
+				bc = date->year < 0;
 
-			if (bc)
-				add(" BC", 3);
+				l = sprintf(_buffer, "'%04d-%02d-%02d %02d:%02d:%02d",
+					abs(date->year), date->month, date->day,
+					date->hour, date->min, date->sec);
 
-			add("'", 1);
+				add(_buffer, l);
+
+				if (date->msec)
+				{
+					l = sprintf(_buffer, ".%03d", date->msec);
+					add(_buffer, l);
+				}
+
+				if (bc)
+					add(" BC", 3);
+
+				add("'", 1);
+			}
 
 			return TRUE;
 
@@ -973,7 +1005,10 @@ static void query_release(DB_RESULT result, DB_INFO *info)
 	<pos> is the index of the record in the result.
 	<buffer> points to an array having one element for each field in the
 	result.
+	<next> is a boolean telling if we want the next row.
 
+	This function must return DB_OK, DB_ERROR or DB_NO_DATA
+	
 	This function must use GB.StoreVariant() to store the value in the
 	buffer.
 
@@ -999,7 +1034,7 @@ static int query_fill(DB_DATABASE *db, DB_RESULT result, int pos, GB_VARIANT_VAL
 		GB.StoreVariant(&value, &buffer[i]);
 	}
 
-	return FALSE;
+	return DB_OK;
 }
 
 
@@ -1520,12 +1555,12 @@ static void table_release(DB_DATABASE *db, DB_INFO *info)
 static int table_exist(DB_DATABASE *db, const char *table)
 {
 	const char *query =
-		"select relname from pg_class where (relkind = 'r' or relkind = 'v') "
+		"select relname from pg_class where (relkind = 'r' or relkind = 'v'or relkind = 'm') "
 		"and (relname = '&1') "
 		"and (relnamespace not in (select oid from pg_namespace where nspname = 'information_schema'))";
 
 	const char *query_schema =
-		"select relname from pg_class where (relkind = 'r' or relkind = 'v') "
+		"select relname from pg_class where (relkind = 'r' or relkind = 'v' or relkind = 'm') "
 		"and (relname = '&1') "
 		"and (relnamespace in (select oid from pg_namespace where nspname = '&2'))";
 
@@ -1578,7 +1613,7 @@ static int table_exist(DB_DATABASE *db, const char *table)
 static int table_list_73(DB_DATABASE *db, char ***tables)
 {
 	const char *query =
-		"select pg_class.relname,pg_namespace.nspname from pg_class,pg_namespace where (pg_class.relkind = 'r' or pg_class.relkind = 'v') "
+		"select pg_class.relname,pg_namespace.nspname from pg_class,pg_namespace where (pg_class.relkind = 'r' or pg_class.relkind = 'v' or pg_class.relkind = 'm') "
 		"and (pg_namespace.oid = pg_class.relnamespace) "
 		"and (pg_namespace.oid not in (select oid from pg_namespace where nspname = 'information_schema'))";
 
@@ -1770,7 +1805,7 @@ static int table_primary_key(DB_DATABASE *db, const char *table, char ***primary
 static int table_is_system(DB_DATABASE *db, const char *table)
 {
 	const char *query =
-		"select 1 from pg_class where (relkind = 'r' or relkind = 'v') "
+		"select 1 from pg_class where (relkind = 'r' or relkind = 'v'or relkind = 'm') "
 		"and (relname = '&1') "
 		"and (relnamespace in (select oid from pg_namespace where nspname = 'pg_catalog'))";
 
