@@ -798,6 +798,8 @@ static void query_release(DB_RESULT result, DB_INFO *info)
 	result.
 	<next> is a boolean telling if we want the next row.
 
+	This function must return DB_OK, DB_ERROR or DB_NO_DATA
+	
 	This function must use GB.StoreVariant() to store the value in the
 	buffer.
 
@@ -868,7 +870,7 @@ static int query_fill(DB_DATABASE *db, DB_RESULT result, int pos, GB_VARIANT_VAL
 		GB.StoreVariant(&value, &buffer[i]);
 	}
 
-	return FALSE;
+	return DB_OK;
 }
 
 
@@ -1973,22 +1975,21 @@ static int database_list(DB_DATABASE *db, char ***databases)
 
 	/* Hostname contains home area */
 	dbhome = (char *)conn->getHostName();
-	WalkDirectory( dbhome, databases );
-
-	/* Checks GAMBAS_SQLITE_DBHOME if set, or Current Working Directory */
-	/* Might have to come back and seperate */
-	dbhome = GetDatabaseHome();
-	if (dbhome){
-			//GB.Error("Unable to get databases: &1", "Can't find current directory");
-			WalkDirectory( dbhome, databases );
-			GB.Free(POINTER(&dbhome));
+	if (dbhome && *dbhome)
+	{
+		WalkDirectory( dbhome, databases );
 	}
-
-	/*if (getcwd(cwd, MAX_PATH) != NULL){
-		if (strcmp(cwd, dbhome) != 0){
-				WalkDirectory( cwd, databases );
+	else
+	{
+		/* Checks GAMBAS_SQLITE_DBHOME if set, or Current Working Directory */
+		/* Might have to come back and seperate */
+		dbhome = GetDatabaseHome();
+		if (dbhome){
+				//GB.Error("Unable to get databases: &1", "Can't find current directory");
+				WalkDirectory( dbhome, databases );
+				GB.Free(POINTER(&dbhome));
 		}
-	}*/
+	}
 
 	return GB.Count(databases);
 }
