@@ -28,6 +28,8 @@
 #include "dbus_print_message.h"
 #include "helper.h"
 
+//#define DEBUG_ARG
+
 typedef
 	void (*RETRIEVE_CALLBACK)(GB_TYPE type, void *data, void *param);
 
@@ -123,11 +125,11 @@ CONV_TYPE _conv_type[] =
 };
 
 
-const char *DBUS_to_dbus_type(GB_TYPE type)
+static const char *to_dbus_type(GB_VALUE *arg)
 {
 	CONV_TYPE *p;
 	
-	switch(type)
+	switch(arg->type)
 	{
 		case GB_T_BOOLEAN: return DBUS_TYPE_BOOLEAN_AS_STRING;
 		case GB_T_BYTE: return DBUS_TYPE_BYTE_AS_STRING;
@@ -140,13 +142,27 @@ const char *DBUS_to_dbus_type(GB_TYPE type)
 		case GB_T_STRING: return DBUS_TYPE_STRING_AS_STRING;
 		default: break;
 	}
-	
-	for (p = _conv_type; p->name; p++)
+
+	if (arg->type >= GB_T_OBJECT)
 	{
-		if (GB.FindClass(p->name) == type)
-			return p->dbus;
+		for (p = _conv_type; p->name; p++)
+		{
+			if (GB.FindClass(p->name) == arg->type)
+				return p->dbus;
+		}
+				
+		if (GB.Is(arg->_object.value, GB.FindClass("Array")))
+			return "av";
+		
+		if (GB.Is(arg->_object.value, CLASS_DBusVariant))
+			return "v";
+
+		if (GB.Is(arg->_object.value, CLASS_DBusNull))
+			return "s";
+
+		return NULL;
 	}
-	
+			
 	return "v";
 }
 
@@ -279,6 +295,8 @@ static bool append_arg(DBusMessageIter *iter, const char *signature, GB_VALUE *a
 	int type;
 	GB_TYPE gtype;
 	char *sign;
+	GB_VALUE rarg;
+	GB_VALUE targ;
 	
 	if (arg->type == GB_T_VARIANT)
 		GB.Conv(arg, arg->_variant.value.type);
@@ -292,13 +310,52 @@ static bool append_arg(DBusMessageIter *iter, const char *signature, GB_VALUE *a
 	
 	if (gtype == GB_T_NULL)
 	{
+		GB.Error("Unsupported datatype for signature '&1'", sign);
 		goto __UNSUPPORTED;
 	}
 	else if (gtype != GB_T_VARIANT)
 	{
+		// The contents of arg must not be modified, because it may have to be freed later by the caller
+		
+		if (arg->type >= GB_T_OBJECT)
+		{
+			if (GB.Is(arg->_object.value, CLASS_DBusNull))
+			{
+				targ.type = GB_T_NULL;
+				GB.ReleaseValue(arg);
+				arg = &targ;
+			}
+			else if (GB.Is(arg->_object.value, CLASS_DBusObject) && type == DBUS_TYPE_OBJECT_PATH)
+			{
+				GB_VALUE *val = GB.GetProperty(arg->_object.value, "_Path");
+	
+				if (val && (val->type == GB_T_STRING || val->type != GB_T_CSTRING))
+				{
+					targ = *val;
+					targ.type = GB_T_CSTRING;
+					// The value returned by GB.GetProperty() is not referenced, so it must not be freed.
+					GB.BorrowValue(&targ);
+					GB.ReleaseValue(arg);
+					arg = &targ;
+				}
+			}
+			else if (GB.Is(arg->_object.value, CLASS_DBusVariant))
+			{
+				CDBUSVARIANT *dbusvariant = (CDBUSVARIANT *)arg->_object.value;
+				
+				targ.type = GB_T_VARIANT;
+				targ._variant.value = dbusvariant->value;
+				GB.BorrowValue(&targ);
+				GB.ReleaseValue(arg);
+				arg = &targ;
+			}
+		}
+			
 		if (GB.Conv(arg, gtype))
 		{
+			//BREAKPOINT();
 			GB.ReleaseValue(arg);
+			GB.Error("Type mismatch");
 			return TRUE;
 		}
 	}
@@ -310,6 +367,9 @@ static bool append_arg(DBusMessageIter *iter, const char *signature, GB_VALUE *a
 			unsigned char val;
 			val = arg->_integer.value;
 			dbus_message_iter_append_basic(iter, type, &val);
+#ifdef DEBUG_ARG
+			fprintf(stderr, "%d ", (int)val);
+#endif
 			break;
 		}
 		
@@ -318,6 +378,9 @@ static bool append_arg(DBusMessageIter *iter, const char *signature, GB_VALUE *a
 			dbus_bool_t val;
 			val = arg->_boolean.value ? 1 : 0;
 			dbus_message_iter_append_basic(iter, type, &val);
+#ifdef DEBUG_ARG
+			fprintf(stderr, "%s ", val ? "true" : "false");
+#endif
 			break;
 		}
 			
@@ -327,6 +390,9 @@ static bool append_arg(DBusMessageIter *iter, const char *signature, GB_VALUE *a
 			dbus_int16_t val;
 			val = arg->_integer.value;
 			dbus_message_iter_append_basic(iter, type, &val);
+#ifdef DEBUG_ARG
+			fprintf(stderr, "%d ", (int)val);
+#endif
 			break;
 		}
 			
@@ -336,6 +402,9 @@ static bool append_arg(DBusMessageIter *iter, const char *signature, GB_VALUE *a
 			dbus_int32_t val;
 			val = arg->_integer.value;
 			dbus_message_iter_append_basic(iter, type, &val);
+#ifdef DEBUG_ARG
+			fprintf(stderr, "%d ", val);
+#endif
 			break;
 		}
 			
@@ -345,6 +414,9 @@ static bool append_arg(DBusMessageIter *iter, const char *signature, GB_VALUE *a
 			dbus_int64_t val;
 			val = arg->_long.value;
 			dbus_message_iter_append_basic(iter, type, &val);
+#ifdef DEBUG_ARG
+			fprintf(stderr, "%" PRId64 " ", val);
+#endif
 			break;
 		}
 		
@@ -353,6 +425,9 @@ static bool append_arg(DBusMessageIter *iter, const char *signature, GB_VALUE *a
 			double val;
 			val = arg->_float.value;
 			dbus_message_iter_append_basic(iter, type, &val);
+#ifdef DEBUG_ARG
+			fprintf(stderr, "%g ", val);
+#endif
 			break;
 		}
 
@@ -360,8 +435,22 @@ static bool append_arg(DBusMessageIter *iter, const char *signature, GB_VALUE *a
 		case DBUS_TYPE_OBJECT_PATH:
 		case DBUS_TYPE_SIGNATURE:
 		{
-			const char *str = GB.ToZeroString((GB_STRING *)arg);
+			const char *str;
+			if (arg->type == GB_T_NULL)
+				str = "";
+			else
+				str = GB.ToZeroString((GB_STRING *)arg);
+			
+			if (type == DBUS_TYPE_OBJECT_PATH && DBUS_validate_path(str, -1))
+			{
+				GB.Error("Invalid object path: &1", str);
+				goto __ERROR;
+			}
+			
 			dbus_message_iter_append_basic(iter, type, &str);
+#ifdef DEBUG_ARG
+			fprintf(stderr, "\"%s\" ", str);
+#endif
 			break;
 		}
 		
@@ -381,6 +470,9 @@ static bool append_arg(DBusMessageIter *iter, const char *signature, GB_VALUE *a
 				char *key;
 				int len;
 				
+#ifdef DEBUG_ARG
+			fprintf(stderr, "{ ");
+#endif
 				if (col)
 				{
 					GB_COLLECTION_ITER iter;
@@ -394,6 +486,9 @@ static bool append_arg(DBusMessageIter *iter, const char *signature, GB_VALUE *a
 						key = GB.TempString(key, len);
 						dbus_message_iter_open_container(&citer, DBUS_TYPE_DICT_ENTRY, NULL, &dict_entry_iter);
 						dbus_message_iter_append_basic(&dict_entry_iter, DBUS_TYPE_STRING, &key);
+#ifdef DEBUG_ARG
+			fprintf(stderr, "%s: ", key);
+#endif
 						
 						GB.BorrowValue(&value);
 						if (append_arg(&dict_entry_iter, &contents_signature[2], &value))
@@ -402,16 +497,22 @@ static bool append_arg(DBusMessageIter *iter, const char *signature, GB_VALUE *a
 						dbus_message_iter_close_container(&citer, &dict_entry_iter);
 					}
 				}
+#ifdef DEBUG_ARG
+			fprintf(stderr, "} ");
+#endif
 			}
 			else
 			{
 				GB_ARRAY array = (GB_ARRAY)(arg->_object.value);
 				
+#ifdef DEBUG_ARG
+			fprintf(stderr, "[ ");
+#endif
 				if (array)
 				{
-					value.type = GB.Array.Type(array);
 					for (i = 0; i < GB.Array.Count(array); i++)
 					{
+						value.type = GB.Array.Type(array);
 						GB.ReadValue(&value, GB.Array.Get(array, i), value.type);
 						GB.BorrowValue(&value);
 						if (append_arg(&citer, contents_signature, &value))
@@ -421,6 +522,9 @@ static bool append_arg(DBusMessageIter *iter, const char *signature, GB_VALUE *a
 			}
 			
 			dbus_message_iter_close_container(iter, &citer);
+#ifdef DEBUG_ARG
+			fprintf(stderr, "] ");
+#endif
 			break;
 		}
 		
@@ -435,13 +539,16 @@ static bool append_arg(DBusMessageIter *iter, const char *signature, GB_VALUE *a
 			array = (GB_ARRAY)(arg->_object.value);
 			
 			dbus_message_iter_open_container(iter, DBUS_TYPE_STRUCT, NULL, &citer);
+#ifdef DEBUG_ARG
+			fprintf(stderr, "( ");
+#endif
 			dbus_signature_iter_recurse(&siter, &siter_contents);
 			
 			if (array)
 			{
-				value.type = GB.Array.Type(array);
 				for (i = 0; i < GB.Array.Count(array); i++)
 				{
+					value.type = GB.Array.Type(array);
 					GB.ReadValue(&value, GB.Array.Get(array, i), value.type);
 					GB.BorrowValue(&value);
 					sign = dbus_signature_iter_get_signature(&siter_contents);
@@ -453,6 +560,9 @@ static bool append_arg(DBusMessageIter *iter, const char *signature, GB_VALUE *a
 			}
 			
 			dbus_message_iter_close_container(iter, &citer);
+#ifdef DEBUG_ARG
+			fprintf(stderr, ") ");
+#endif
 			break;
 		}
 
@@ -460,40 +570,50 @@ static bool append_arg(DBusMessageIter *iter, const char *signature, GB_VALUE *a
 		{
 			DBusMessageIter citer;
 			const char *contents_signature;
-			GB_VALUE rarg;
-			GB_VALUE *old_arg = arg;
 			
-			if (arg->type == CLASS_DBusVariant)
+#ifdef DEBUG_ARG
+			fprintf(stderr, "< ");
+#endif
+			if (arg->type >= GB_T_OBJECT && GB.Is(arg->_object.value, CLASS_DBusVariant))
 			{
 				CDBUSVARIANT *dbusvariant = (CDBUSVARIANT *)arg->_object.value;
 				
 				rarg.type = GB_T_VARIANT;
 				rarg._variant.value = dbusvariant->value;
-				arg = &rarg;
 				
-				contents_signature = dbusvariant->signature;
+				contents_signature = CDBUSVARIANT_get_signature(dbusvariant);
+				//fprintf(stderr, "#1: %s\n", contents_signature);
+				GB.BorrowValue(&rarg);
+				dbus_message_iter_open_container(iter, DBUS_TYPE_VARIANT, contents_signature, &citer);
+				if (append_arg(&citer, contents_signature, &rarg))
+					goto __ERROR;
+				dbus_message_iter_close_container(iter, &citer);
 			}
 			else
 			{
-				contents_signature = DBUS_to_dbus_type(arg->type);
+				contents_signature = to_dbus_type(arg);
+				//fprintf(stderr, "#3: %s\n", contents_signature);
+				if (!contents_signature)
+				{
+					GB.Error("Unsupported datatype in variant value");
+					goto __UNSUPPORTED;
+				}
+				
+				dbus_message_iter_open_container(iter, DBUS_TYPE_VARIANT, contents_signature, &citer);
+				GB.BorrowValue(arg);
+				if (append_arg(&citer, contents_signature, arg))
+					goto __ERROR;
+				dbus_message_iter_close_container(iter, &citer);
 			}
+#ifdef DEBUG_ARG
+			fprintf(stderr, "> ");
+#endif
 			
-			if (!contents_signature)
-				goto __UNSUPPORTED;
-			
-			dbus_message_iter_open_container(iter, DBUS_TYPE_VARIANT, contents_signature, &citer);
-			GB.BorrowValue(arg);
-			if (append_arg(&citer, contents_signature, arg))
-			{
-				arg = old_arg;
-				goto __ERROR;
-			}
-			dbus_message_iter_close_container(iter, &citer);
-			arg = old_arg;
 			break;
 		}
 		
 		default:
+			GB.Error("Unsupported signature: &1", sign);
 			goto __UNSUPPORTED;
 	}
 	
@@ -501,7 +621,7 @@ static bool append_arg(DBusMessageIter *iter, const char *signature, GB_VALUE *a
 	return FALSE;
 	
 __UNSUPPORTED:
-	GB.Error("Unsupported datatype");
+	//GB.Error("Unsupported datatype");
 	goto __ERROR;
 
 __ERROR_SIGN:
@@ -679,7 +799,13 @@ static bool define_arguments(DBusMessage *message, const char *signature, GB_ARR
 			GB.BorrowValue(&value);
 
 			sign = dbus_signature_iter_get_signature(&siter);
+#ifdef DEBUG_ARG
+			fprintf(stderr, "[%d] %s: ", n, sign);
+#endif
 			err = append_arg(&iter, sign, &value);
+#ifdef DEBUG_ARG
+			fprintf(stderr, "\n");
+#endif
 			dbus_free(sign);
 			if (err) 
 				return TRUE;
@@ -800,6 +926,40 @@ __RETURN:
 	return ret;
 }
 
+
+bool DBUS_send_signal(DBusConnection *connection, const char *path, const char *interface, const char *signal, const char *signature_in, GB_ARRAY arguments)
+{
+	DBusMessage *message;
+	bool ret;
+	
+	message = dbus_message_new_signal(path, interface, signal);
+	if (!message)
+	{
+		GB.Error("Couldn't allocate D-Bus message");
+		return TRUE;
+	}
+	
+	ret = TRUE;
+	
+	//dbus_message_set_auto_start(message, TRUE);
+
+	if (define_arguments(message, signature_in, arguments))
+		goto __RETURN;
+	
+	//dbus_error_init(&error);
+	dbus_connection_send(connection, message, NULL);
+	//dbus_connection_flush(connection);
+	check_message(connection);
+	ret = FALSE;
+
+__RETURN:
+	
+	dbus_message_unref(message);
+	
+	return ret;
+}
+
+
 bool DBUS_retrieve_message_arguments(DBusMessage *message)
 {
 	DBusMessageIter iter;
@@ -840,8 +1000,11 @@ bool DBUS_reply(DBusConnection *connection, DBusMessage *message, const char *si
 
 	reply = dbus_message_new_method_return(message);
 	
-	if (define_arguments(reply, signature, arguments))
-		goto __RETURN;
+	if (signature && *signature && arguments)
+	{
+		if (define_arguments(reply, signature, arguments))
+			goto __RETURN;
+	}
 	
 	if (!dbus_connection_send(connection, reply, &serial)) 
 	{
@@ -988,6 +1151,8 @@ static DBusHandlerResult filter_func(DBusConnection *connection, DBusMessage *me
 		if (check_filter(obs->interface, dbus_message_get_interface(message)))
 			continue;
 		
+		//print_message(message, FALSE);
+		
 		found = TRUE;
 		obs->message = message;
 		obs->reply = FALSE;
@@ -1256,3 +1421,25 @@ bool DBUS_validate_method(const char *method, int len)
 	return FALSE;
 }
 
+GB_ARRAY DBUS_split_signature(const char *signature)
+{
+	GB_ARRAY result;
+	DBusSignatureIter siter;
+	char *sign;
+	
+	GB.Array.New(POINTER(&result), GB_T_STRING, 0);
+	
+	dbus_signature_iter_init(&siter, signature);
+		
+	for(;;)
+	{
+		sign = dbus_signature_iter_get_signature(&siter);
+		*(char **)GB.Array.Add(result) = GB.NewZeroString(sign);
+		dbus_free(sign);
+			
+		if (!dbus_signature_iter_next(&siter))
+			break;
+	}
+	
+	return result;
+}
