@@ -46,7 +46,11 @@ Atom X11_atom_net_wm_window_type;
 Atom X11_atom_net_wm_window_type_normal;
 Atom X11_atom_net_wm_window_type_utility;
 Atom X11_atom_net_wm_desktop;
+Atom X11_atom_net_wm_user_time;
 Atom X11_atom_net_current_desktop;
+Atom X11_atom_net_workarea = None;
+Atom X11_atom_motif_wm_hints = None;
+Atom X11_atom_net_system_tray = None;
 
 Atom X11_UTF8_STRING;
 
@@ -74,6 +78,7 @@ typedef
 	struct {
 		int count;
 		Atom atoms[MAX_WINDOW_PROP];
+		bool changed;
 		}
 	X11_PROPERTY;
 
@@ -87,26 +92,27 @@ static GB_FUNCTION _x11_configure_notify_func;
 
 static void init_atoms()
 {
-  if (_atom_init)
-    return;
+	if (_atom_init)
+		return;
 
 	//fprintf(stderr, "init_atom: display = %p\n", _display);
 	X11_atom_net_current_desktop = XInternAtom(_display, "_NET_CURRENT_DESKTOP", True);
 	//fprintf(stderr, "init_atom: #1\n");
-  X11_atom_net_wm_state = XInternAtom(_display, "_NET_WM_STATE", True);
-  X11_atom_net_wm_state_above = XInternAtom(_display, "_NET_WM_STATE_ABOVE", True);
-  X11_atom_net_wm_state_below = XInternAtom(_display, "_NET_WM_STATE_BELOW", True);
-  X11_atom_net_wm_state_stays_on_top = XInternAtom(_display, "_NET_WM_STATE_STAYS_ON_TOP", True);
-  X11_atom_net_wm_state_skip_taskbar = XInternAtom(_display, "_NET_WM_STATE_SKIP_TASKBAR", True);
+	X11_atom_net_wm_state = XInternAtom(_display, "_NET_WM_STATE", True);
+	X11_atom_net_wm_state_above = XInternAtom(_display, "_NET_WM_STATE_ABOVE", True);
+	X11_atom_net_wm_state_below = XInternAtom(_display, "_NET_WM_STATE_BELOW", True);
+	X11_atom_net_wm_state_stays_on_top = XInternAtom(_display, "_NET_WM_STATE_STAYS_ON_TOP", True);
+	X11_atom_net_wm_state_skip_taskbar = XInternAtom(_display, "_NET_WM_STATE_SKIP_TASKBAR", True);
 
 	X11_atom_net_wm_desktop = XInternAtom(_display, "_NET_WM_DESKTOP", True);
 	X11_atom_net_wm_window_type = XInternAtom(_display, "_NET_WM_WINDOW_TYPE", True);
 	X11_atom_net_wm_window_type_normal = XInternAtom(_display, "_NET_WM_WINDOW_TYPE_NORMAL", True);
 	X11_atom_net_wm_window_type_utility = XInternAtom(_display, "_NET_WM_WINDOW_TYPE_UTILITY", True);
-	
+	X11_atom_net_wm_user_time = XInternAtom(_display, "_NET_WM_USER_TIME", True);
+
 	X11_UTF8_STRING = XInternAtom(X11_display, "UTF8_STRING", True);
 
-  _atom_init = TRUE;
+	_atom_init = TRUE;
 }
 
 #define PROPERTY_START_READ 1024
@@ -147,24 +153,24 @@ char *X11_get_property(Window wid, Atom prop, Atom *type, int *format, int *pcou
 			False, AnyPropertyType, type, format,
 			&count, &after, &data) != Success)
 		return NULL;
-	
+
 	*pcount += count;
-	
+
 	size = sizeof_format(*format);
 	offset_size = *format == 32 ? sizeof(int32_t) : ( *format == 16 ? sizeof(short) : 1 );
-	
+
 	//fprintf(stderr, "X11_get_property: format = %d size = %d count = %ld after = %ld\n", *format, size, count, after);
-	
+
 	GB.FreeString(&_property_value);
 	_property_value = GB.NewString((char *)data, count * size);
 	XFree(data);
-	
+
 	offset = count * offset_size / sizeof(int32_t);
-	
+
 	while (after)
 	{
 		//fprintf(stderr, "X11_get_property: offset = %ld read = %ld\n", offset, Min(after, PROPERTY_NEXT_READ) / sizeof(int32_t));
-	
+
 		if (XGetWindowProperty(_display, wid, prop, offset, Min(after, PROPERTY_NEXT_READ) / sizeof(int32_t),
 				False, AnyPropertyType, type, format,
 				&count, &after, &data) != Success)
@@ -172,26 +178,26 @@ char *X11_get_property(Window wid, Atom prop, Atom *type, int *format, int *pcou
 
 		*pcount += count;
 		offset += count * offset_size / sizeof(int32_t);
-		
+
 		//fprintf(stderr, "X11_get_property: format = %d size = %d count = %ld after = %ld next offset = %ld\n", *format, size, count, after, offset);
-	
+
 		_property_value = GB.AddString(_property_value, (char *)data, count * size);
 		XFree(data);
 	}
-	
+
 	return _property_value;
 }
 
 #if 0
 static void get_property(Window wid, Atom prop, int maxlength, unsigned char **data, unsigned long *count)
 {
-  Atom type;
-  int format;
-  unsigned long after;
+	Atom type;
+	int format;
+	unsigned long after;
 
-  XGetWindowProperty(_display, wid, prop, 0, maxlength / 4,
-    False, AnyPropertyType, &type, &format,
-    count, &after, data);
+	XGetWindowProperty(_display, wid, prop, 0, maxlength / 4,
+		False, AnyPropertyType, &type, &format,
+		count, &after, data);
 }
 #endif
 
@@ -199,22 +205,22 @@ static char *get_property(Window wid, Atom prop, int *count)
 {
 	Atom type;
 	int format;
-	
+
 	return X11_get_property(wid, prop, &type, &format, count);
 }
 
 Atom X11_get_property_type(Window wid, Atom prop, int *format)
 {
 	uchar *data = NULL;
-  unsigned long count;
-  unsigned long after;
-  Atom type;
+	unsigned long count;
+	unsigned long after;
+	Atom type;
 
 	if (XGetWindowProperty(X11_display, wid, prop, 0, PROPERTY_START_READ / sizeof(int32_t),
 			False, AnyPropertyType, &type, format,
 			&count, &after, &data) != Success)
 		return None;
-	
+
 	XFree(data);
 	return type;
 }
@@ -223,7 +229,7 @@ Atom X11_get_property_type(Window wid, Atom prop, int *format)
 	#if OS_64BITS
 	long padded_data[count];
 	int i;
-	
+
 	if (format == 32)
 	{
 		for (i = 0; i < count; i++)
@@ -241,7 +247,7 @@ void X11_set_property(Window wid, Atom prop, Atom type, int format, void *data,
 Atom X11_intern_atom(const char *name, bool create)
 {
 	int val = atoi(name);
-	
+
 	if (val)
 		return (Atom)val;
 	else
@@ -250,72 +256,76 @@ Atom X11_intern_atom(const char *name, bool create)
 
 static void load_window_state(Window win, Atom prop)
 {
-  int length;
-  char *data;
+	int length;
+	char *data;
 
-  _window_prop.count = 0;
+	_window_prop.count = 0;
+	_window_prop.changed = FALSE;
 
 	data = get_property(win, prop, &length);
 
-  if (length > MAX_WINDOW_PROP)
-    length = MAX_WINDOW_PROP;
+	if (length > MAX_WINDOW_PROP)
+		length = MAX_WINDOW_PROP;
 
-  _window_prop.count = length;
-  memcpy(_window_prop.atoms, data, length * sizeof(Atom));
+	_window_prop.count = length;
+	if (data)
+		memcpy(_window_prop.atoms, data, length * sizeof(Atom));
 }
 
 static void save_window_state(Window win, Atom prop)
 {
-	if (_window_prop.count > 0)
+	if (_window_prop.changed)
 	{
-  	XChangeProperty(_display, win, prop, XA_ATOM, 32, PropModeReplace,
-      	(unsigned char *)_window_prop.atoms, _window_prop.count);
+		//fprintf(stderr, "XChangeProperty: %ld %ld\n", (long)win, (long)prop);
+		XChangeProperty(_display, win, prop, XA_ATOM, 32, PropModeReplace, (unsigned char *)_window_prop.atoms, _window_prop.count);
 	}
 }
 
 static bool has_window_state(Atom prop)
 {
-  int i;
+	int i;
 
-  for (i = 0; i < _window_prop.count; i++)
-  {
-    if (_window_prop.atoms[i] == prop)
-      return TRUE;
-  }
+	for (i = 0; i < _window_prop.count; i++)
+	{
+		if (_window_prop.atoms[i] == prop)
+			return TRUE;
+	}
 
-  return FALSE;
+	return FALSE;
 }
 
 static void set_window_state(Atom prop)
 {
-  if (has_window_state(prop))
-    return;
+	if (has_window_state(prop))
+		return;
 
-  if (_window_prop.count == MAX_WINDOW_PROP)
-  {
-    fprintf(stderr, "X11: set_window_state: Too many properties in window\n");
-    return;
-  }
+	if (_window_prop.count == MAX_WINDOW_PROP)
+	{
+		fprintf(stderr, "X11: set_window_state: Too many properties in window\n");
+		return;
+	}
 
-  _window_prop.atoms[_window_prop.count++] = prop;
+	_window_prop.atoms[_window_prop.count++] = prop;
+	_window_prop.changed = TRUE;
 }
 
 static void clear_window_state(Atom prop)
 {
-  int i;
+	int i;
 
-  for (i = 0; i < _window_prop.count; i++)
-  {
-    if (_window_prop.atoms[i] == prop)
-    {
-      _window_prop.count--;
+	for (i = 0; i < _window_prop.count; i++)
+	{
+		if (_window_prop.atoms[i] == prop)
+		{
+			_window_prop.count--;
 
-      for (; i < _window_prop.count; i++)
-        _window_prop.atoms[i] = _window_prop.atoms[i + 1];
+			for (; i < _window_prop.count; i++)
+				_window_prop.atoms[i] = _window_prop.atoms[i + 1];
 
-      return;
-    }
-  }
+			_window_prop.changed = TRUE;
+			return;
+		}
+	}
 }
 
 
@@ -325,24 +335,24 @@ bool X11_do_init()
 
 	if (X11_ready)
 		return FALSE;
-		
+
 	GB.Component.GetInfo("DISPLAY", POINTER(&_display));
 
 	_root = DefaultRootWindow(_display);
-  
-  X11_ready = _display != NULL;
-  
-  if (!X11_ready)
-  {
-  	fprintf(stderr, "WARNING: X11_init() has failed\n");
-  	return TRUE;
-  }
-  
+
+	X11_ready = _display != NULL;
+
+	if (!X11_ready)
+	{
+		fprintf(stderr, "WARNING: X11_init() has failed\n");
+		return TRUE;
+	}
+
 	init_atoms();
-	
+
 	_has_test_extension = XTestQueryExtension(_display, &event_base, &error_base, &major_version, &minor_version);
-	
-  return FALSE;
+
+	return FALSE;
 }
 
 void X11_exit()
@@ -357,17 +367,17 @@ void X11_exit()
 
 void X11_send_client_message(Window dest, Window window, Atom message, char *data, int format, int count)
 {
-  XEvent e;
-  int mask = (SubstructureRedirectMask | SubstructureNotifyMask);
+	XEvent e;
+	int mask = (SubstructureRedirectMask | SubstructureNotifyMask);
 
 	//fprintf(stderr, "X11_send_client_message: dest = %ld window = %ld message = %ld format = %d count = %d\n", dest, window, message, format, count);
-	
+
 	e.xclient.type = ClientMessage;
 	e.xclient.message_type = message;
 	e.xclient.display = X11_display;
 	e.xclient.window = window;
 	e.xclient.format = format;
-	
+
 	memset(&e.xclient.data.l[0], 0, sizeof(long) * 5);
 	if (data)
 	{
@@ -379,67 +389,69 @@ void X11_send_client_message(Window dest, Window window, Atom message, char *dat
 	}
 
 	XSendEvent(X11_display, dest, False, mask, &e);
+	XFlush(X11_display);
 }
 
 void X11_window_change_property(Window window, bool visible, Atom property, bool set)
 {
-  XEvent e;
-  long mask = (SubstructureRedirectMask | SubstructureNotifyMask);
+	XEvent e;
+	long mask = (SubstructureRedirectMask | SubstructureNotifyMask);
 
-  if (visible)
-  {
-    e.xclient.type = ClientMessage;
-    e.xclient.message_type = X11_atom_net_wm_state;
-    e.xclient.display = _display;
-    e.xclient.window = window;
-    e.xclient.format = 32;
-    e.xclient.data.l[0] = set ? 1 : 0;
-    e.xclient.data.l[1] = property;
-    e.xclient.data.l[2] = 0;
-    e.xclient.data.l[3] = 0;
-    e.xclient.data.l[4] = 0;
+	if (visible)
+	{
+		e.xclient.type = ClientMessage;
+		e.xclient.message_type = X11_atom_net_wm_state;
+		e.xclient.display = _display;
+		e.xclient.window = window;
+		e.xclient.format = 32;
+		e.xclient.data.l[0] = set ? 1 : 0;
+		e.xclient.data.l[1] = property;
+		e.xclient.data.l[2] = 0;
+		e.xclient.data.l[3] = 0;
+		e.xclient.data.l[4] = 0;
 
-    XSendEvent(_display, _root, False, mask, &e);
-  }
-  else
-  {
-    load_window_state(window, X11_atom_net_wm_state);
+		XSendEvent(_display, _root, False, mask, &e);
+		XFlush(X11_display);
+	}
+	else
+	{
+		load_window_state(window, X11_atom_net_wm_state);
 
-    if (set)
-      set_window_state(property);
-    else
-      clear_window_state(property);
+		if (set)
+			set_window_state(property);
+		else
+			clear_window_state(property);
 
-    save_window_state(window, X11_atom_net_wm_state);
-  }
+		save_window_state(window, X11_atom_net_wm_state);
+	}
 }
 
 
 bool X11_window_has_property(Window window, Atom property)
 {
-  load_window_state(window, X11_atom_net_wm_state);
-  return has_window_state(property);
+	load_window_state(window, X11_atom_net_wm_state);
+	return has_window_state(property);
 }
 
 void X11_sync(void)
 {
-  XSync(_display, False);
+	XSync(_display, False);
 }
 
 void X11_window_save_properties(Window window)
 {
-  load_window_state(window, X11_atom_net_wm_state);
-  _window_save[0] = _window_prop;
-  //load_window_state(window, X11_atom_net_wm_window_type);
-  //_window_save[1] = _window_prop;
+	load_window_state(window, X11_atom_net_wm_state);
+	_window_save[0] = _window_prop;
+	//load_window_state(window, X11_atom_net_wm_window_type);
+	//_window_save[1] = _window_prop;
 }
 
 void X11_window_restore_properties(Window window)
 {
-  _window_prop = _window_save[0];
-  save_window_state(window, X11_atom_net_wm_state);
-  //_window_prop = _window_save[1];
-  //save_window_state(window, X11_atom_net_wm_window_type);
+	_window_prop = _window_save[0];
+	save_window_state(window, X11_atom_net_wm_state);
+	//_window_prop = _window_save[1];
+	//save_window_state(window, X11_atom_net_wm_window_type);
 }
 
 #define SYSTEM_TRAY_REQUEST_DOCK    0
@@ -450,60 +462,60 @@ void X11_window_restore_properties(Window window)
 
 void X11_window_dock(Window window)
 {
-  Window xmanager=None;
-  XClientMessageEvent ev;
-  Atom OpCodeAtom;
-  Screen *xscreen;
-  char buf[256];
-  Atom selection_atom;
+	Window xmanager=None;
+	XClientMessageEvent ev;
+	Atom OpCodeAtom;
+	Screen *xscreen;
+	char buf[256];
+	Atom selection_atom;
 
-  buf[0]=0;
+	buf[0]=0;
 
-  xscreen = DefaultScreenOfDisplay(_display);
-  sprintf(buf,"_NET_SYSTEM_TRAY_S%d",XScreenNumberOfScreen(xscreen));
-  selection_atom = XInternAtom(_display, buf, 0);
+	xscreen = DefaultScreenOfDisplay(_display);
+	sprintf(buf,"_NET_SYSTEM_TRAY_S%d",XScreenNumberOfScreen(xscreen));
+	selection_atom = XInternAtom(_display, buf, 0);
 
-  XGrabServer(_display);
+	XGrabServer(_display);
 
-  xmanager = XGetSelectionOwner(_display, selection_atom);
-  if (xmanager != None)
-    XSelectInput(_display, xmanager, StructureNotifyMask);
+	xmanager = XGetSelectionOwner(_display, selection_atom);
+	if (xmanager != None)
+		XSelectInput(_display, xmanager, StructureNotifyMask);
 
-  XUngrabServer(_display);
-  XFlush(_display);
+	XUngrabServer(_display);
+	XFlush(_display);
 
-  /***********************************************
-    Dock Tray Icon
-  ************************************************/
+	/***********************************************
+		Dock Tray Icon
+	************************************************/
 
-  OpCodeAtom = XInternAtom(_display, OPCODE, 0);
+	OpCodeAtom = XInternAtom(_display, OPCODE, 0);
 
-  ev.type = ClientMessage;
-  ev.window = xmanager;
-  ev.message_type = OpCodeAtom;
-  ev.format = 32;
-  ev.data.l[0] = 0;
-  ev.data.l[1] = SYSTEM_TRAY_REQUEST_DOCK;
-  ev.data.l[2] = window;
-  ev.data.l[3] = 0;
-  ev.data.l[4] = 0;
+	ev.type = ClientMessage;
+	ev.window = xmanager;
+	ev.message_type = OpCodeAtom;
+	ev.format = 32;
+	ev.data.l[0] = 0;
+	ev.data.l[1] = SYSTEM_TRAY_REQUEST_DOCK;
+	ev.data.l[2] = window;
+	ev.data.l[3] = 0;
+	ev.data.l[4] = 0;
 
-  XSendEvent(_display, xmanager, 0, NoEventMask, (XEvent *)&ev);
-  XSync(_display, 0);
+	XSendEvent(_display, xmanager, 0, NoEventMask, (XEvent *)&ev);
+	XSync(_display, 0);
 }
 
 void X11_window_startup(Window window, int x, int y, int w, int h)
 {
-  XSizeHints s;
+	XSizeHints s;
 
-  s.flags = USPosition | PPosition | USSize | PSize;
+	s.flags = USPosition | PPosition | USSize | PSize;
 
-  s.x = x;
-  s.y = y;
-  s.width = w;
-  s.height = h;
+	s.x = x;
+	s.y = y;
+	s.width = w;
+	s.height = h;
 
-  XSetWMNormalHints(_display, window, &s);
+	XSetWMNormalHints(_display, window, &s);
 }
 
 void X11_find_windows(Window **window_list, int *count)
@@ -519,10 +531,10 @@ void X11_find_windows(Window **window_list, int *count)
 void X11_get_window_title(Window window, char **result, int *length)
 {
 	static Atom _net_wm_name = 0;
-	
+
 	if (!_net_wm_name)
 		_net_wm_name = XInternAtom(_display, "_NET_WM_NAME", True);
-	
+
 	*result = get_property(window, _net_wm_name, length);
 }
 
@@ -567,7 +579,7 @@ void X11_set_window_tool(Window window, int tool, Window parent)
 int X11_get_window_tool(Window window)
 {
 	load_window_state(window, X11_atom_net_wm_window_type);
-  return has_window_state(X11_atom_net_wm_window_type_utility);
+	return has_window_state(X11_atom_net_wm_window_type_utility);
 }
 
 
@@ -575,72 +587,80 @@ int X11_get_window_tool(Window window)
 
 void X11_window_set_desktop(Window window, bool visible, int desktop)
 {
-  XEvent e;
-  long mask = (SubstructureRedirectMask | SubstructureNotifyMask);
+	XEvent e;
+	long mask = (SubstructureRedirectMask | SubstructureNotifyMask);
 
-  if (visible)
-  {
-    e.xclient.type = ClientMessage;
-    e.xclient.message_type = X11_atom_net_wm_desktop;
-    e.xclient.display = _display;
-    e.xclient.window = window;
-    e.xclient.format = 32;
-    e.xclient.data.l[0] = desktop;
-    e.xclient.data.l[1] = 1;
-    e.xclient.data.l[2] = 0;
-    e.xclient.data.l[3] = 0;
-    e.xclient.data.l[4] = 0;
+	if (visible)
+	{
+		e.xclient.type = ClientMessage;
+		e.xclient.message_type = X11_atom_net_wm_desktop;
+		e.xclient.display = _display;
+		e.xclient.window = window;
+		e.xclient.format = 32;
+		e.xclient.data.l[0] = desktop;
+		e.xclient.data.l[1] = 1;
+		e.xclient.data.l[2] = 0;
+		e.xclient.data.l[3] = 0;
+		e.xclient.data.l[4] = 0;
 
-    XSendEvent(_display, _root, False, mask, &e);
-  }
-  else
-  {
-  	XChangeProperty(_display, window, X11_atom_net_wm_desktop, XA_CARDINAL, 32, PropModeReplace,
-      	(unsigned char *)&desktop, 1);
-  }
+		XSendEvent(_display, _root, False, mask, &e);
+		XFlush(_display);
+	}
+	else
+	{
+		XChangeProperty(_display, window, X11_atom_net_wm_desktop, XA_CARDINAL, 32, PropModeReplace,
+				(unsigned char *)&desktop, 1);
+		XFlush(_display);
+	}
 }
 
 
 int X11_window_get_desktop(Window window)
 {
 	int length;
-	unsigned long *data;
+	char *data = NULL;
+	int desktop = 0;
 
-	data = (unsigned long *)get_property(window, X11_atom_net_wm_desktop, &length);
+	data = get_property(window, X11_atom_net_wm_desktop, &length);
+	if (data)
+		desktop = *((int *)data);
 
-	return data ? *data : 0;
+	return desktop;
 }
 
 int X11_get_current_desktop()
 {
 	int length;
-	unsigned long *data;
+	char *data;
+	int desktop = 0;
 
-	data = (unsigned long *)get_property(_root, X11_atom_net_current_desktop, &length);
+	data = get_property(_root, X11_atom_net_current_desktop, &length);
+	if (data)
+		desktop = *((int *)data);
 
-	return data ? *data : 0;
+	return desktop;
 }
 
 static void init_keycode()
 {
 	int i, j; //, k;
 	KeyCode *pm, *p;
-	
+
 	if (_init_keycode)
 		return;
 
 	XDisplayKeycodes(_display, &_min_keycode, &_max_keycode);
-	
-  _keycode_map = XGetKeyboardMapping(_display, _min_keycode, _max_keycode - _min_keycode + 1, &_keysyms_per_keycode);
-  _modifier_map = XGetModifierMapping(_display);
-  
-  p = _modifier_map->modifiermap;
-  for (i = 0; i < 8; i++)
-  {
-  	pm = p;
-  	for (j = 0; j < _modifier_map->max_keypermod; j++)
-  	{
-  		//for (k = 0; k < 3; k++)
+
+	_keycode_map = XGetKeyboardMapping(_display, _min_keycode, _max_keycode - _min_keycode + 1, &_keysyms_per_keycode);
+	_modifier_map = XGetModifierMapping(_display);
+
+	p = _modifier_map->modifiermap;
+	for (i = 0; i < 8; i++)
+	{
+		pm = p;
+		for (j = 0; j < _modifier_map->max_keypermod; j++)
+		{
+			//for (k = 0; k < 3; k++)
 				switch (XkbKeycodeToKeysym(_display, *p, 0, 0))
 				{
 					case XK_Shift_L: _shift_keycode = pm; break;
@@ -649,20 +669,20 @@ static void init_keycode()
 			p++;
 		}
 	}
-  
-  //fprintf(stderr, "SHIFT: %d  ALTGR: %d\n", _shift_keycode[0], _alt_gr_keycode[0]);
-  
+
+	//fprintf(stderr, "SHIFT: %d  ALTGR: %d\n", _shift_keycode[0], _alt_gr_keycode[0]);
+
 	_init_keycode = TRUE;
 }
 
 static void send_modifiers(KeyCode *codes, bool press)
 {
 	int i;
-	
+
 	for (i = 0; i < _modifier_map->max_keypermod; i++)
 	{
 		if (codes[i])
-			XTestFakeKeyEvent(_display, codes[i], press, CurrentTime);	
+			XTestFakeKeyEvent(_display, codes[i], press, CurrentTime);
 	}
 }
 
@@ -670,9 +690,9 @@ static void handle_modifier(KeyCode code, KeySym keysym, bool press)
 {
 	KeySym *sym;
 	int i;
-	
+
 	sym = &_keycode_map[(code - _min_keycode) * _keysyms_per_keycode];
-	
+
 	for (i = 0; i < _keysyms_per_keycode; i++)
 	{
 		if (keysym == sym[i])
@@ -681,7 +701,7 @@ static void handle_modifier(KeyCode code, KeySym keysym, bool press)
 			break;
 		}
 	}
-	
+
 	switch(i)
 	{
 		case 1:
@@ -703,17 +723,17 @@ char *X11_send_key(char *key, bool press)
 {
 	KeySym keysym;
 	KeyCode code;
-	
+
 	if (!_has_test_extension)
 		return "No XTEST extension";
-	
+
 	if (!_init_keycode)
 		init_keycode();
-	
+
 	//fprintf(stderr, "X11_send_key: '%s' %d\n", key, XStringToKeysym(key));
-	
+
 	if (strlen(key) == 1)
-	{ 
+	{
 		if (*key == '\n')
 			keysym = XK_Return;
 		else if (*key == '\t')
@@ -725,10 +745,10 @@ char *X11_send_key(char *key, bool press)
 	}
 	else
 		keysym = XStringToKeysym(key);
-	
+
 	if (keysym == NoSymbol)
 		return "Unknown key";
-		
+
 	code = XKeysymToKeycode(_display, keysym);
 	if (code)
 	{
@@ -736,13 +756,13 @@ char *X11_send_key(char *key, bool press)
 			handle_modifier(code, keysym, TRUE);
 
 		XTestFakeKeyEvent(_display, code, press, CurrentTime);
-	
+
 		if (press)
 			handle_modifier(code, keysym, FALSE);
 	}
-	
+
 	usleep(1000);
-	
+
 	return NULL;
 }
 
@@ -751,7 +771,7 @@ void X11_enable_event_filter(bool enable)
 	static int count = 0;
 
 	void (*set_event_filter)(void *) = NULL;
-	
+
 	if (enable)
 		count++;
 	else
@@ -786,7 +806,7 @@ void X11_event_filter(XEvent *e)
 			GB_T_INTEGER, e->xconfigure.y,
 			GB_T_INTEGER, e->xconfigure.width,
 			GB_T_INTEGER, e->xconfigure.height);
-			
+
 		GB.Call(&_x11_configure_notify_func, 5, TRUE);
 	}
 
@@ -799,15 +819,15 @@ void X11_get_window_geometry(Window win, int *wx, int *wy, int *ww, int *wh)
 	Window p;
 	int transx, transy;
 	XWindowAttributes wattr;
-	
+
 	*wx = *wy = *ww = *wh = 0;
-	
+
 	if (!XTranslateCoordinates(_display, win, _root, 0, 0, &transx, &transy, &p))
 		return;
-	
+
 	if (!XGetWindowAttributes(_display, win, &wattr))
 		return;
-	
+
 	*wx = transx - wattr.border_width;
 	*wy = transy - wattr.border_width;
 	*ww = wattr.width + wattr.border_width * 2;
