@@ -45,8 +45,11 @@ typedef
 		gMenu *widget;
 		GB_VARIANT_VALUE tag;
 		char *action;
+		char *save_text;
 		unsigned init_shortcut : 1;
 	} 
 	CMENU;
 
+void CMENU_check_popup_click(void);
+
 #endif
