@@ -33,6 +33,7 @@
 #include "gapplication.h"
 #include "gmainwindow.h"
 #include "cpaint_impl.h"
+#include "desktop.h"
 
 extern int CWINDOW_Embedder;
 extern bool CWINDOW_Embedded;
@@ -139,6 +140,12 @@ BEGIN_PROPERTY(Desktop_HasSystemTray)
 
 END_PROPERTY
 
+BEGIN_PROPERTY(Desktop_Type)
+
+	GB.ReturnConstZeroString(DESKTOP_get_type());
+
+END_PROPERTY
+
 
 static void set_font(gFont *font, void *object = 0)
 {
@@ -403,6 +410,8 @@ GB_DESC DesktopDesc[] =
 	
 	GB_STATIC_METHOD("Screenshot", "Picture", Desktop_Screenshot, "[(X)i(Y)i(Width)i(Height)i]"),
 
+	GB_STATIC_PROPERTY_READ("Type", "s", Desktop_Type),
+
 	GB_END_DECLARE
 };
 
