@@ -50,7 +50,7 @@ DECLARE_EVENT(EVENT_Move);
 DECLARE_EVENT(EVENT_Resize);
 DECLARE_EVENT(EVENT_Title);
 DECLARE_EVENT(EVENT_Icon);
-//DECLARE_EVENT(EVENT_State);
+DECLARE_EVENT(EVENT_Font);
 
 void CWINDOW_check_main_window(CWINDOW *win)
 {
@@ -62,62 +62,60 @@ void CWINDOW_check_main_window(CWINDOW *win)
 }
 
 
-static void gb_raise_window_Open(gMainWindow *sender)
+static void cb_open(gMainWindow *sender)
 {
-	CWIDGET *_ob=GetObject(sender);
+	CWIDGET *_object = GetObject(sender);
+	GB.Raise(THIS, EVENT_Open, 0);
+}
 
-	if (!_ob) return;
-	GB.Raise((void*)_ob,EVENT_Open,0);
+static void cb_font_change(gMainWindow *sender)
+{
+	CWIDGET *_object = GetObject(sender);
+	GB.Raise(THIS, EVENT_Font, 0);
 }
 
 /*static void gb_post_window_Open(gMainWindow *sender)
 {
-	GB.Post((GB_POST_FUNC)gb_raise_window_Open, (long)sender);
+	GB.Post((GB_POST_FUNC)cb_open, (long)sender);
 }*/
 
-static void gb_raise_window_Show(gMainWindow *sender)
+static void cb_show(gMainWindow *sender)
 {
-	CWIDGET *ob = GetObject(sender);
+	CWIDGET *_object = GetObject(sender);
 
-	if (!ob) return;
-	GB.Ref(ob);
-	GB.Raise((void*)ob,EVENT_Show,0);
+	GB.Ref(THIS);
+	GB.Raise(THIS, EVENT_Show, 0);
 	if (!sender->spontaneous())
-		CACTION_raise(ob);
-	GB.Unref(POINTER(&ob));
+		CACTION_raise(THIS);
+	GB.Unref(POINTER(&_object));
 }
 
 /*static void gb_post_window_Show(gMainWindow *sender)
 {
-	GB.Post( (void (*)())gb_raise_window_Show,(long)sender);
+	GB.Post( (void (*)())cb_show,(long)sender);
 }*/
 
-static void gb_raise_window_Hide(gMainWindow *sender)
+static void cb_hide(gMainWindow *sender)
 {
-	CWIDGET *ob=GetObject(sender);
+	CWIDGET *_object = GetObject(sender);
 
-	if (!ob) return;
-	GB.Ref(ob);
-	GB.Raise((void*)ob,EVENT_Hide,0);
+	GB.Ref(THIS);
+	GB.Raise(THIS, EVENT_Hide, 0);
 	if (!sender->spontaneous())
-		CACTION_raise(ob);
-	GB.Unref(POINTER(&ob));
+		CACTION_raise(THIS);
+	GB.Unref(POINTER(&_object));
 }
 
-static void gb_raise_window_Move(gMainWindow *sender)
+static void cb_move(gMainWindow *sender)
 {
-	CWIDGET *_ob=GetObject(sender);
-
-	if (!_ob) return;
-	GB.Raise((void*)_ob,EVENT_Move,0);
+	CWIDGET *_object = GetObject(sender);
+	GB.Raise(THIS, EVENT_Move, 0);
 }
 
-static void gb_raise_window_Resize(gMainWindow *sender)
+static void cb_resize(gMainWindow *sender)
 {
-	CWIDGET *_ob=GetObject(sender);
-
-	if (!_ob) return;
-	GB.Raise((void*)_ob,EVENT_Resize,0);
+	CWIDGET *_object = GetObject(sender);
+	GB.Raise(THIS, EVENT_Resize, 0);
 }
 
 static bool close_window(CWINDOW *_object, int ret = 0)
@@ -127,28 +125,6 @@ static bool close_window(CWINDOW *_object, int ret = 0)
 	return WINDOW->close();
 }
 
-static bool closeAll()
-{
-	int i;
-	gMainWindow *win;
-	CWINDOW *window;
-	
-	for(i = 0; i < gMainWindow::count(); i++)
-	{
-		win = gMainWindow::get(i);
-		if (!win)
-			break;
-		//fprintf(stderr, "closeAll: try %p\n", win);
-		window = (CWINDOW *)GetObject(win);
-		if (window == CWINDOW_Main)
-			continue;
-		if (close_window(window))
-			return true;
-	}
-	
-	return false;
-}
-
 void CWINDOW_delete_all()
 {
 	int i;
@@ -196,7 +172,7 @@ static bool gb_raise_window_Close(gMainWindow *sender)
 	
 	if (CWINDOW_Main && sender == CWINDOW_Main->ob.widget)
 	{
-		if (closeAll())
+		if (gMainWindow::closeAll())
 			return true;
 		if (!sender->isPersistent())
 		{
@@ -304,15 +280,15 @@ BEGIN_METHOD(CWINDOW_new, GB_OBJECT parent;)
 
 	InitControl(THIS->ob.widget, (CWIDGET*)THIS);
 	
-	WINDOW->onOpen = gb_raise_window_Open;
-	WINDOW->onShow = gb_raise_window_Show;
-	WINDOW->onHide = gb_raise_window_Hide;
-	WINDOW->onMove = gb_raise_window_Move;
-	WINDOW->onResize = gb_raise_window_Resize;
+	WINDOW->onOpen = cb_open;
+	WINDOW->onShow = cb_show;
+	WINDOW->onHide = cb_hide;
+	WINDOW->onMove = cb_move;
+	WINDOW->onResize = cb_resize;
 	WINDOW->onClose = gb_raise_window_Close;
 	WINDOW->onActivate = cb_activate;
 	WINDOW->onDeactivate = cb_deactivate;
-	//WINDOW->onState = cb_state;
+	WINDOW->onFontChange = cb_font_change;
 
 END_METHOD
 
@@ -395,7 +371,7 @@ END_METHOD
 
 BEGIN_METHOD_VOID(CWINDOW_show)
 
-	WINDOW->show();
+	WINDOW->showActivate();
 
 END_METHOD
 
@@ -710,6 +686,46 @@ BEGIN_PROPERTY(Window_Screen)
 
 END_PROPERTY
 
+BEGIN_PROPERTY(Window_Transparent)
+
+	bool trans = WINDOW->isTransparent();
+
+	if (READ_PROPERTY)
+		GB.ReturnBoolean(trans);
+	else
+	{
+		bool new_trans = VPROP(GB_BOOLEAN);
+		if (trans == new_trans)
+			return;
+
+		if (!new_trans)
+		{
+			GB.Error("Transparent property cannot be reset");
+			return;
+		}
+
+		WINDOW->setTransparent(true);
+	}
+
+END_PROPERTY
+
+BEGIN_PROPERTY(Window_TakeFocus)
+
+	if (READ_PROPERTY)
+		GB.ReturnBoolean(!WINDOW->isNoTakeFocus());
+	else
+		WINDOW->setNoTakeFocus(!VPROP(GB_BOOLEAN));
+
+END_PROPERTY
+
+BEGIN_METHOD_VOID(Window_Activate)
+	
+	WINDOW->activate();
+
+END_METHOD
+
+//-------------------------------------------------------------------------
+
 BEGIN_METHOD_VOID(CFORM_new)
 
 	if (!GB.Parent(_object))
@@ -816,6 +832,8 @@ GB_DESC CWindowDesc[] =
 	GB_METHOD("ShowDialog", "i", CWINDOW_show_modal, NULL),
 	GB_METHOD("ShowPopup", "i", Window_ShowPopup, "[(X)i(Y)i]"),
 	GB_METHOD("Center", NULL, CWINDOW_center, NULL),
+	GB_METHOD("Activate", NULL, Window_Activate, NULL),
+
 	GB_PROPERTY_READ("Modal", "b", CWINDOW_modal),
 	GB_PROPERTY_READ("TopLevel", "b", CWINDOW_top_level),
 	GB_PROPERTY_READ("Closed", "b", CWINDOW_closed),
@@ -835,6 +853,8 @@ GB_DESC CWindowDesc[] =
 	GB_PROPERTY("TopOnly", "b", CWINDOW_top_only),
 	GB_PROPERTY("SkipTaskbar", "b", CWINDOW_skip_taskbar),
 	GB_PROPERTY("Opacity", "i", Window_Opacity),
+	GB_PROPERTY("Transparent", "b", Window_Transparent),
+	GB_PROPERTY("TakeFocus", "b", Window_TakeFocus),
 
 	GB_PROPERTY("Arrangement", "i", Container_Arrangement),
 	GB_PROPERTY("AutoResize", "b", Container_AutoResize),
@@ -869,7 +889,7 @@ GB_DESC CWindowDesc[] =
 	GB_EVENT("Hide", NULL, NULL, &EVENT_Hide),
 	GB_EVENT("Title", NULL, NULL, &EVENT_Title),
 	GB_EVENT("Icon", NULL, NULL, &EVENT_Icon),
-	//GB_EVENT("State", NULL, NULL, &EVENT_State),
+	GB_EVENT("Font", NULL, NULL, &EVENT_Font),
 
 	GB_END_DECLARE
 };
