@@ -22,7 +22,6 @@
 ***************************************************************************/
 
 #include "widgets.h"
-#include "widgets_private.h"
 #include "gapplication.h"
 #include "gmainwindow.h"
 #include "gbutton.h"
@@ -80,7 +79,103 @@ static void cb_click_check(GtkButton *object, gButton *data)
 	data->emit(SIGNAL(data->onClick));
 }
 
+#ifdef GTK3
+static gboolean button_draw(GtkWidget *wid, cairo_t *cr, gButton *data)
+{
+	GdkPixbuf *img;
+	GdkRectangle rpix={0,0,0,0};
+	GdkRectangle rect;
+	GtkCellRendererState state;
+	gint py, px;
+	bool rtl, bcenter=false;
+	gint dx, dy;
+	GtkStateFlags f;
 
+	rtl = gtk_widget_get_default_direction() == GTK_TEXT_DIR_RTL;
+
+	rect.x = rect.y = 0;
+	rect.width = data->width();
+	rect.height = data->height();
+
+	px = rect.width;
+
+	if (gtk_widget_get_state_flags(data->widget) & GTK_STATE_FLAG_ACTIVE)
+	{
+	  gtk_widget_style_get (wid,
+				"child-displacement-x", &dx,
+				"child-displacement-y", &dy,
+				(void *)NULL);
+		rect.x += dx;
+		rect.y += dy;
+	}
+
+	//g_debug("button_expose: %d %d %d %d", e->area.x, e->area.y, e->area.width, e->area.height);
+	//g_debug("rect: %d %d %d %d", rect.x, rect.y, rect.width, rect.height);
+
+	if (data->rendpix)
+	{
+		if (gtk_widget_get_state_flags(data->widget) & GTK_STATE_FLAG_INSENSITIVE)
+		{
+		  if (!data->rendinc)
+		    data->rendinc = gt_pixbuf_create_disabled(data->rendpix);
+		  img = data->rendinc;
+    }
+		else
+		  img = data->rendpix;
+
+		rpix.width = gdk_pixbuf_get_width(img);
+		rpix.height = gdk_pixbuf_get_height(img);
+
+		py = (rect.height - rpix.height)/2;
+
+		bcenter = !(data->text()) || !(*data->text());
+
+		if (bcenter)
+		{
+			//fprintf(stderr, "draw pixbuf: %d %d\n", rect.x + (px-rpix.width)/2, rect.y + py);
+			//gdk_draw_pixbuf(GDK_DRAWABLE(win),gc,img,0,0,rect.x + (px-rpix.width)/2, rect.y + py,
+      //                                  -1,-1,GDK_RGB_DITHER_MAX,0,0);
+
+			gt_cairo_draw_pixbuf(cr, img, rect.x + (px - rpix.width) / 2, rect.y + py, -1, -1, 1.0, NULL);
+			return false;
+		}
+
+		if (rtl)
+			gt_cairo_draw_pixbuf(cr, img, rect.x + rect.width - 6, rect.y + py, -1, -1, 1.0, NULL);
+		else
+			gt_cairo_draw_pixbuf(cr, img, rect.x + 6, rect.y + py, -1, -1, 1.0, NULL);
+
+		rect.width -= rpix.width;
+		rect.x += rpix.width;
+	}
+
+	gt_set_cell_renderer_text_from_font((GtkCellRendererText *)data->rendtxt, data->font());
+	g_object_set(G_OBJECT(data->rendtxt), "sensitive", true, (void *)NULL);
+
+
+	f = gtk_widget_get_state_flags(data->widget);
+
+	if (f & GTK_STATE_INSENSITIVE)
+	{
+		state = GTK_CELL_RENDERER_INSENSITIVE;
+		g_object_set(G_OBJECT(data->rendtxt), "sensitive", false, (void *)NULL);
+	}
+	/*else if (f & GTK_STATE_SELECTED)
+	{
+		state = GTK_CELL_RENDERER_SELECTED;
+	}*/
+	else
+		state = (GtkCellRendererState)0;
+
+	if (rect.width >= 1 && rect.height >= 1 && data->bufText && *data->bufText)
+	{
+		gtk_cell_renderer_set_fixed_size(data->rendtxt, rect.width, rect.height);
+		gtk_cell_renderer_render(data->rendtxt, cr, wid, &rect, &rect, state);
+	}
+
+	return FALSE;
+}
+#else
 static gboolean button_expose(GtkWidget *wid,GdkEventExpose *e,gButton *data)
 {
 	cairo_t *cr;
@@ -198,6 +293,7 @@ static gboolean button_expose(GtkWidget *wid,GdkEventExpose *e,gButton *data)
 	
 	return FALSE;
 }
+#endif
 
 gButton::gButton(gContainer *par, Type typ) : gControl(par)
 {
@@ -205,30 +301,23 @@ gButton::gButton(gContainer *par, Type typ) : gControl(par)
 
 	g_typ = Type_gButton;
 	
+	disable = false;
 	_toggle = false;
 	_radio = false;
 	_animated = false;
 	_stretch = true;
 	_tristate = false;
 	_autoresize = false;
-	scaled = false;
-	disable = false;
 	bufText = NULL;
 	rendtxt = NULL;
-	label = NULL;
 	rendpix = NULL;
 	rendinc = NULL;
+	_label = NULL;
 	pic = NULL;
 	shortcut = 0;
 	
 	switch(typ)
 	{
-		case Toggle:
-			_no_background = true;
-			rendtxt = gtk_cell_renderer_text_new();
-			border = gtk_toggle_button_new();
-			break;
-			
 		case Check:
 			border = gtk_check_button_new();
 			break;
@@ -248,6 +337,12 @@ gButton::gButton(gContainer *par, Type typ) : gControl(par)
 			}
 			break;
 		
+		case Toggle:
+			_no_background = true;
+			rendtxt = gtk_cell_renderer_text_new();
+			border = gtk_toggle_button_new();
+			break;
+
 		case Tool:
 			_no_background = true;
 			rendtxt = gtk_cell_renderer_text_new();
@@ -257,8 +352,8 @@ gButton::gButton(gContainer *par, Type typ) : gControl(par)
 		
 		default:
 			_no_background = true;
-			border=gtk_button_new();
-			rendtxt=gtk_cell_renderer_text_new();
+			border = gtk_button_new();
+			rendtxt = gtk_cell_renderer_text_new();
 			typ = Button;
 			break;
 	}
@@ -272,28 +367,27 @@ gButton::gButton(gContainer *par, Type typ) : gControl(par)
 		g_object_set(G_OBJECT(rendtxt),"xalign",0.5,(void *)NULL);
 		g_object_set(G_OBJECT(rendtxt),"yalign",0.5,(void *)NULL);
 
-		g_signal_connect_after(G_OBJECT(widget),"expose-event",G_CALLBACK(button_expose),(gpointer)this);
-	}
-	else
-	{	
-		label = gtk_label_new_with_mnemonic("");
-		gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_CENTER);
-		gtk_container_add (GTK_CONTAINER(widget), GTK_WIDGET(label));
+		ON_DRAW(widget, this, button_expose, button_draw);
 	}
 	
 	realize();
 	
 	gtk_widget_add_events(widget, GDK_POINTER_MOTION_MASK);
-	onClick=NULL;
+	onClick = NULL;
 	
 	if (type == Radio)
 		g_signal_connect(G_OBJECT(widget),"clicked",G_CALLBACK(cb_click_radio),(gpointer)this);
 	else if (type == Check)
 		g_signal_connect(G_OBJECT(widget), "clicked", G_CALLBACK(cb_click_check), (gpointer)this);	
 	else
+	{
 		g_signal_connect(G_OBJECT(widget),"clicked",G_CALLBACK(cb_click),(gpointer)this);	
+		setColorButton();
+	}
 	
 	
+	setText(NULL);
+
 	if (type == Tool) 
     setBorder(false);
 }
@@ -322,46 +416,6 @@ bool gButton::inconsistent()
 	return vl;	
 }
 
-#if 0
-void gButton::setFont(gFont *ft)
-{
-	PangoFontDescription *desc=pango_context_get_font_description(ft->ct);
-	GtkBox *box;
-	GtkLabel *lbl;
-	GList *chd;
-
-  gControl::setFont(ft);
-
-	return; /* TODO */
-	
-	font_change=true;
-	if (type==4) { gtk_widget_modify_font(widget,desc); return; }
-
-	if (rendtxt)
-	{
-	}
-	else
-	{
-		box=(GtkBox*)gtk_bin_get_child(GTK_BIN(widget));
-		chd=gtk_container_get_children(GTK_CONTAINER(box));
-		chd=chd->next;
-		lbl=GTK_LABEL(chd->data);
-		gtk_widget_modify_font(GTK_WIDGET(lbl),desc);
-		g_list_free(chd);
-	}
-}
-#endif
-
-/*bool gButton::enabled()
-{
-	return GTK_WIDGET_SENSITIVE(widget);
-}
-
-void gButton::setEnabled(bool vl)
-{
-	gtk_widget_set_sensitive(widget, vl);
-}*/
-
 const char* gButton::text()
 {
 	//if (type == Tool) return this->toolTip();
@@ -370,7 +424,6 @@ const char* gButton::text()
 
 void gButton::setText(const char *st)
 {
-	GtkLabel *lbl=NULL;
 	GtkAccelGroup *accel;
 	char *buf;
 	
@@ -383,38 +436,46 @@ void gButton::setText(const char *st)
 		g_free(bufText);
 	}
 
- 	bufText = g_strdup(st);
-	updateSize();
-	resize();
-	
-	if (!rendtxt)
+ 	bufText = st ? g_strdup(st) : NULL;
+
+	if (rendtxt)
 	{
-		lbl=(GtkLabel*)gtk_bin_get_child(GTK_BIN(widget));
-		if (!st) st="";
-		if (!strlen(st))
-		{ 
-			gtk_label_set_text(lbl,"");
-			g_object_set(G_OBJECT(lbl),"visible",false,(void *)NULL);
-			return;
+		if (bufText && *bufText)
+		{
+			shortcut = (int)gMnemonic_correctMarkup(bufText, &buf);
+
+			if (shortcut)
+				gtk_widget_add_accelerator(widget, "clicked", accel, (guint)shortcut, GDK_MOD1_MASK, (GtkAccelFlags)0);
+
+			if (rendtxt)
+				g_object_set(G_OBJECT(rendtxt), "markup", buf, (void *)NULL);
+
+			g_free(buf);
 		}
-		gMnemonic_correctText((char*)st,&buf);
-		gtk_label_set_text_with_mnemonic(lbl,buf);
-		g_object_set(G_OBJECT(lbl),"visible",true,(void *)NULL);
-		g_free(buf);
-		return;
-		
+		else
+		{
+			g_object_set(G_OBJECT(rendtxt), "markup", "", (void *)NULL);
+		}
+
+		refresh();
 	}
-	
-	shortcut = (int)gMnemonic_correctMarkup((char*)st,&buf);
-	
-	if (shortcut) 
-		gtk_widget_add_accelerator(widget, "clicked", accel, (guint)shortcut, GDK_MOD1_MASK, (GtkAccelFlags)0);
-	
-	g_object_set(G_OBJECT(rendtxt),"markup",buf,(void *)NULL);
-	
-	g_free(buf);
-	
-	refresh();
+	else
+	{
+		if (bufText && *bufText)
+		{
+			gMnemonic_correctText((char*)st, &buf);
+			gtk_button_set_use_underline(GTK_BUTTON(widget), TRUE);
+			gtk_button_set_label(GTK_BUTTON(widget), buf);
+			g_free(buf);
+		}
+		else
+			gtk_button_set_label(GTK_BUTTON(widget), "");
+
+		_label = gtk_bin_get_child(GTK_BIN(widget));
+		set_gdk_fg_color(_label, foreground());
+	}
+
+	updateFont();
 }
 
 
@@ -443,7 +504,7 @@ void gButton::setPicture(gPicture *npic)
   
 	if (rendpix) { g_object_unref(G_OBJECT(rendpix)); rendpix = NULL; }
 	if (rendinc) { g_object_unref(G_OBJECT(rendinc)); rendinc = NULL; }
-	
+
 	rendpix = new_rendpix;
 	
 	updateSize();
@@ -483,12 +544,20 @@ void gButton::setDefault(bool vl)
 	if (vl)
 	{
 		win->_default = this;
+#if GTK_CHECK_VERSION(2, 18, 0)
+		gtk_widget_set_can_default(widget, true);
+#else
 		GTK_WIDGET_SET_FLAGS (widget, GTK_CAN_DEFAULT);
+#endif
 		//gtk_widget_grab_default (widget);
 	}
 	else
 	{
+#if GTK_CHECK_VERSION(2, 18, 0)
+		gtk_widget_set_can_default(widget, false);
+#else
 		GTK_WIDGET_UNSET_FLAGS(widget, GTK_CAN_DEFAULT);
+#endif
 		if (win->_default == this)
 			win->_default = NULL;
 	}
@@ -548,13 +617,22 @@ void gButton::animateClick(bool on)
 	
 	if (!on && !_animated)
 	{
+#ifdef GTK3
+		gtk_widget_set_state_flags(widget, GTK_STATE_FLAG_ACTIVE, FALSE);
+#else
 		gtk_widget_set_state(widget, GTK_STATE_ACTIVE);
+#endif
+		refresh();
 		_animated = true;
 	}
 	else if (on && _animated)
 	{
 		_animated = false;
+#ifdef GTK3
+		gtk_widget_set_state_flags(widget, GTK_STATE_FLAG_NORMAL, FALSE);
+#else
 		gtk_widget_set_state(widget, GTK_STATE_NORMAL);
+#endif
 		gtk_button_clicked(GTK_BUTTON(widget));
 	}
 }
@@ -638,6 +716,11 @@ void gButton::setRealForeground(gColor color)
 {
 	gControl::setRealForeground(color);
 
+#ifndef GTK3
+	if (_label)
+		set_gdk_fg_color(_label, color);
+#endif
+
 	if (rendtxt)
 	{
 		if (color == COLOR_DEFAULT)
@@ -656,9 +739,6 @@ void gButton::setRealForeground(gColor color)
 				(void *)NULL);
 		}
 	}
-	
-	if (label)
-		set_gdk_fg_color(label, color);
 }
 
 void gButton::setTristate(bool vl)
@@ -676,7 +756,6 @@ void gButton::setAutoResize(bool vl)
 
 void gButton::updateSize()
 {
-	GtkRequisition req;
 	int mw, mh;
 	
 	if (!_autoresize)
@@ -687,15 +766,27 @@ void gButton::updateSize()
 	
 	if (bufText && *bufText)
 	{
-		int m = font()->width(bufText, strlen(bufText));
+		gint m;
 	
 		if (type == Check || type == Radio)
 		{
+#ifdef GTK3
+			int indicator_size, indicator_spacing, focus_width, focus_pad;
+			gtk_widget_style_get(widget,
+				"indicator-size", &indicator_size,
+				"indicator-spacing", &indicator_spacing,
+				"focus-line-width", &focus_width,
+        "focus-padding", &focus_pad,
+				(char *)NULL);
+			m = (indicator_size + indicator_spacing * 2 + 2 * (focus_width + focus_pad)) + indicator_spacing + font()->width(bufText, strlen(bufText));
+#else
+			GtkRequisition req;
 			g_signal_emit_by_name(border, "size-request",	&req);
-			m += req.width;
-			if (req.height > m)
-				m = req.height;
+			m = req.width;
+#endif
 		}
+		else
+			m = font()->width(bufText, strlen(bufText)) + 16;
 		
 		mw += m;
 	}
@@ -706,8 +797,6 @@ void gButton::updateSize()
 		mw += pic->width();
 	}
 	
-	mw += 16;
-	
 	if (mh < height())
 		mh = height();
 
