@@ -60,9 +60,12 @@ public:
 	void remove(int pos);
 	
 	virtual void resize(int w, int h);
+#ifdef GTK3
+	virtual void updateColor();
+#else
 	virtual void setRealBackground(gColor vl);
+#endif
 	virtual void setRealForeground(gColor vl);
-	virtual void setFont(gFont *f);
 	virtual void setFocus();
 	
 //"Signals"
@@ -78,6 +81,7 @@ public:
 	GtkWidget *_button;
 	int _model_dirty_timeout;
 	
+	virtual void updateFont();
 	void updateModel();
 	void updateSort();
 	char *indexToKey(int index);
