@@ -85,14 +85,18 @@ public:
 	void setFullArrangement(gContainerArrangement *arr);
 	
 	virtual void performArrange();
+	void getMaxSize(int xc, int yc, int wc, int hc, int *w, int *h);
 
+#ifndef GTK3
 	virtual void setBackground(gColor color = COLOR_DEFAULT);
+#else
+	virtual void updateColor();
+#endif
 	virtual void setForeground(gColor color = COLOR_DEFAULT);
-	virtual void setFont(gFont *ft);
+	virtual void updateFont();
 	
 	bool hasBackground() const;
 	bool hasForeground() const;
-	bool hasFont() const;
 
 	virtual void resize(int w, int h);
 
