@@ -48,6 +48,8 @@ public:
 	static gColor lightfgColor();
 	static gColor tooltipForeground();
 	static gColor tooltipBackground();
+	static gColor linkForeground();
+	static gColor visitedForeground();
 
 	static gFont* font();
 	static void setFont(gFont *vl);
@@ -66,7 +68,7 @@ public:
 	
 	static void geometry(GdkRectangle *rect) { geometry(0, rect); }
 	static void availableGeometry(GdkRectangle *rect) { availableGeometry(0, rect); }
-	
+
 private:
 
 	static int _desktop_scale;
