@@ -21,9 +21,6 @@
 
 ***************************************************************************/
 
-#include "widgets.h"
-#include "widgets_private.h"
-
 #ifndef GAMBAS_DIRECTFB
 #ifdef GDK_WINDOWING_X11
 #include <gdk/gdkx.h>
@@ -32,12 +29,39 @@
 #endif
 #endif
 
+#include "widgets.h"
 #include "gapplication.h"
 #include "gmainwindow.h"
+#include "gmouse.h"
 #include "gclipboard.h"
 #include "gdrag.h"
 
+//#define DEBUG_ME 1
+
+#ifdef GTK3
+struct _GtkTargetList
+{
+  GList *list;
+  guint ref_count;
+};
+
+#if GTK_CHECK_VERSION(3, 12, 0)
+#else
+struct _GtkTargetPair
+{
+  GdkAtom   target;
+  guint     flags;
+  guint     info;
+};
+#endif
+
+typedef
+	struct _GtkTargetPair GtkTargetPair;
+#endif
+
+static int _current_clipboard = gClipboard::Clipboard;
 static GtkClipboard *_clipboard = NULL;
+static GtkClipboard *_selection = NULL;
 
 static char *convert_format(char *fmt)
 {
@@ -48,21 +72,62 @@ static char *convert_format(char *fmt)
 	return fmt;
 }
 
+#if GTK_CHECK_VERSION(2, 14, 0)
+#else
+static gint gtk_selection_data_get_length(const GtkSelectionData *sel)
+{
+	return sel->length;
+}
+
+static const guchar *gtk_selection_data_get_data(const GtkSelectionData *sel)
+{
+	return sel->data;
+}
+#endif
+
+#if GTK_CHECK_VERSION(2, 22, 0)
+#else
+static GList *gdk_drag_context_list_targets(GdkDragContext *context)
+{
+	return context->targets;
+}
+#endif
+
 /***********************************************************************
 
 	Clipboard
 
 ************************************************************************/
 
-void gClipboard::init()
+static GtkClipboard *get_clipboard()
 {
-	if (!_clipboard) 
-		_clipboard = gtk_clipboard_get_for_display(gdk_display_get_default(), GDK_SELECTION_CLIPBOARD);
+	if (_current_clipboard == gClipboard::Selection)
+	{
+		if (!_selection)
+			_selection = gtk_clipboard_get(GDK_SELECTION_PRIMARY);
+		return _selection;
+	}
+	else
+	{
+		if (!_clipboard)
+			_clipboard = gtk_clipboard_get(GDK_SELECTION_CLIPBOARD);
+		return _clipboard;
+	}
+}
+
+void gClipboard::setCurrent(int clipboard)
+{
+	_current_clipboard = clipboard;
+}
+
+int gClipboard::getCurrent()
+{
+	return _current_clipboard;
 }
 
 void gClipboard::clear()
 {
-	gtk_clipboard_clear(_clipboard);
+	gtk_clipboard_clear(get_clipboard());
 }
 
 char *gClipboard::getFormat(int n)
@@ -72,7 +137,7 @@ char *gClipboard::getFormat(int n)
 	GdkAtom *targets;
 	char *fmt, *cfmt;
 
-	if (!gtk_clipboard_wait_for_targets(_clipboard, &targets, &n_tg))
+	if (!gtk_clipboard_wait_for_targets(get_clipboard(), &targets, &n_tg))
 		return NULL;
 	
 	for (i = 0; i < n_tg; i++)
@@ -102,25 +167,24 @@ static void cb_clear_text(GtkClipboard *clipboard, gpointer text)
 
 static void cb_get_text(GtkClipboard *clipboard, GtkSelectionData *selection, guint info, gpointer text)
 {
-	gtk_selection_data_set_text(selection, (const char *)text, -1);
+	gtk_selection_data_set(selection, gtk_selection_data_get_target(selection), 8, (guchar *)text, strlen((char *)text));
 }
 
 int gClipboard::getType()
 {
-	if (gtk_clipboard_wait_is_text_available(_clipboard)) return Text;
-	if (gtk_clipboard_wait_is_image_available(_clipboard)) return Image;
-	
+	if (gtk_clipboard_wait_is_image_available(get_clipboard())) return Image;
+	if (gtk_clipboard_wait_is_text_available(get_clipboard())) return Text;
 	return Nothing;
 }
 
 void gClipboard::setImage(gPicture *image)
 {
-	gtk_clipboard_set_image(_clipboard, image->getPixbuf());
+	gtk_clipboard_set_image(get_clipboard(), image->getPixbuf());
 }
 
 gPicture * gClipboard::getImage()
 {
-  return new gPicture(gtk_clipboard_wait_for_image(_clipboard));
+  return new gPicture(gtk_clipboard_wait_for_image(get_clipboard()));
 }
 
 static void 
@@ -164,7 +228,7 @@ void gClipboard::setText(char *text, int len, char *format)
 	if (!text)
 		return;
 		
-	gt_clipboard_set_text(_clipboard, format, text, len);
+	gt_clipboard_set_text(get_clipboard(), format, text, len);
 }
 
 char *gClipboard::getText(int *len, const char *format)
@@ -179,7 +243,7 @@ char *gClipboard::getText(int *len, const char *format)
 
 	*len = 0;
 	
-	if (!gtk_clipboard_wait_for_targets(_clipboard, &targets, &n_tg) || n_tg <= 0)
+	if (!gtk_clipboard_wait_for_targets(get_clipboard(), &targets, &n_tg) || n_tg <= 0)
 		return NULL;
 	
 	for (i = 0; i < n_tg; i++)
@@ -197,14 +261,13 @@ char *gClipboard::getText(int *len, const char *format)
 	if (i >= n_tg)
 		return NULL;
 	
-	if (!gtk_clipboard_wait_is_target_available(_clipboard, target))
+	if (!gtk_clipboard_wait_is_target_available(get_clipboard(), target))
 		return NULL;
 	
-	data = gtk_clipboard_wait_for_contents(_clipboard, target);
-	
-	*len = data->length;
+	data = gtk_clipboard_wait_for_contents(get_clipboard(), target);
+	*len = gtk_selection_data_get_length(data);
 	text = (char *)g_malloc(*len);
-	memcpy(text, data->data, *len);
+	memcpy(text, gtk_selection_data_get_data(data), *len);
 	
 	gtk_selection_data_free(data);
 	
@@ -235,20 +298,21 @@ int gDrag::_x = -1;
 int gDrag::_y = -1;
 GdkDragContext *gDrag::_context = NULL;
 guint32 gDrag::_time = 0;
-volatile bool gDrag::_got_data = false;
 bool gDrag::_local = false;
-bool gDrag::_end = false;
+volatile bool gDrag::_got_data = false;
+volatile bool gDrag::_end = false;
 
 void gDrag::setIcon(gPicture *vl)
 {  
-	//g_debug("gDrag::setIcon: %p", vl);
 	gPicture::assign(&_icon, vl);
 }
 
 void gDrag::cancel()
 {
-	//fprintf(stderr, "gDrag::cancel\n");
-	
+	#if DEBUG_ME
+		fprintf(stderr, "gDrag::cancel\n");
+	#endif
+
 	hide();
 	setIcon(NULL);
 	setDropText(NULL);
@@ -264,6 +328,10 @@ void gDrag::cancel()
 	_got_data = false;
 	_local = false;
 	_active = false;
+
+#if DEBUG_ME
+	fprintf(stderr, "_active -> false\n");
+#endif
 }
 
 void gDrag::exit()
@@ -275,18 +343,30 @@ gControl *gDrag::drag(gControl *source, GtkTargetList *list)
 {
 	GdkDragContext *ct;
 	gControl *dest;
+	int button;
 
-	_local = true;
-	
-	//fprintf(stderr, "gtk_drag_begin: start\n");
+	#if DEBUG_ME
+		fprintf(stderr, "gDrag::drag: source = %p list = %p event = %p\n", source, list, gApplication::lastEvent());
+	#endif
 
-	ct = gtk_drag_begin(source->border, list, GDK_ACTION_COPY, 1, NULL);
+	button = gMouse::left() ? 1 : gMouse::middle() ? 2 : gMouse::right() ? 3 : 0;
+
+#if GTK_CHECK_VERSION(3, 10, 0)
+	ct = gtk_drag_begin_with_coordinates(source->border, list, GDK_ACTION_COPY, button, gApplication::lastEvent(), -1, -1);
+#else
+	ct = gtk_drag_begin(source->border, list, GDK_ACTION_COPY, button, gApplication::lastEvent());
+#endif
 	if (!ct)
 		return NULL;
 
-	//fprintf(stderr, "gtk_drag_begin: finish\n");
-	
-	if (_icon) 
+	_local = true;
+	_active = true;
+
+#if DEBUG_ME
+	fprintf(stderr, "gDrag::drag: begin\n");
+#endif
+
+	if (_icon)
 	{
 		GdkPixbuf *icon = _icon->getIconPixbuf();
 		gtk_drag_set_icon_pixbuf(ct, icon, _icon_x, _icon_y);
@@ -294,16 +374,19 @@ gControl *gDrag::drag(gControl *source, GtkTargetList *list)
 			g_object_unref(G_OBJECT(icon));
 	}
 	
+	_end = false;
 	while (!_end)
 		MAIN_do_iteration(true);
 	
 	gtk_target_list_unref(list);	
 	
-	_end = false;
-	
 	dest = _destination;
 	cancel();
 	
+#if DEBUG_ME
+	fprintf(stderr, "gDrag::drag: end\n");
+#endif
+
 	return dest;
 }
 
@@ -313,8 +396,10 @@ gControl *gDrag::dragText(gControl *source, char *text, char *format)
 	
 	//cancel();
 	
-	//fprintf(stderr, "dragText: %s\n", text);
-	
+	#if DEBUG_ME
+		fprintf(stderr, "gDrag::dragText: %s\n", text);
+	#endif
+
 	setDropText(text);
 	
 	list = gtk_target_list_new (NULL, 0);
@@ -362,8 +447,10 @@ void gDrag::setDropInfo(int type, char *format)
 
 void gDrag::setDropData(int action, int x, int y, gControl *source, gControl *dest)
 {
-	//g_debug("gDrag::setDropData: action = %d x = %d y = %d source = %p\n", action, x, y, source);
-	
+	#if DEBUG_ME
+		fprintf(stderr, "gDrag::setDropData: action = %d x = %d y = %d source = %p\n", action, x, y, source);
+	#endif
+
 	_x = x;
 	_y = y;
 	_action = action;
@@ -374,8 +461,10 @@ void gDrag::setDropData(int action, int x, int y, gControl *source, gControl *de
 
 void gDrag::setDropText(char *text, int len)
 {
-	//fprintf(stderr, "gDrag::setDropText: text = '%s' %d\n", text, len);
-	
+	#if DEBUG_ME
+		fprintf(stderr, "gDrag::setDropText: text = '%s' %d\n", text, len);
+	#endif
+
 	g_free(_text);
 	if (text)
 	{
@@ -423,12 +512,20 @@ void gDrag::setDropImage(char *buf, int len)
 
 bool gDrag::checkThreshold(gControl *control, int x, int y, int sx, int sy)
 {
-  return gtk_drag_check_threshold(control->border, sx, sy, x, y);
+	if (_active)
+		return false;
+	else
+		return gtk_drag_check_threshold(control->border, sx, sy, x, y);
 } 
 
 GdkDragContext *gDrag::enable(GdkDragContext *context, gControl *control, guint32 time)
 {
 	GdkDragContext *old = _context;
+
+	#if DEBUG_ME
+		fprintf(stderr, "gDrag::enable\n");
+	#endif
+
 	_enabled++;
 	_context = context;
 	_time = time;
@@ -439,6 +536,11 @@ GdkDragContext *gDrag::enable(GdkDragContext *context, gControl *control, guint3
 GdkDragContext *gDrag::disable(GdkDragContext *context)
 {
 	GdkDragContext *old = _context;
+
+	#if DEBUG_ME
+		fprintf(stderr, "gDrag::disable\n");
+	#endif
+
 	_context = context;
 	_enabled--;
 	return old;
@@ -475,9 +577,8 @@ static void show_frame(gControl *control, int x, int y, int w, int h)
 	int i;
 	GdkWindowAttr attr = { 0 };
 	GdkWindow *window;
-	GdkColor color;
 	GdkWindow *parent;
-	GtkAllocation *a;
+	GtkAllocation a;
 	
 	if (w < 0) w = control->width() - control->getFrameWidth() * 2;
 	if (h < 0) h = control->height() - control->getFrameWidth() * 2;
@@ -493,20 +594,26 @@ static void show_frame(gControl *control, int x, int y, int w, int h)
 	// Don't know why I should do that...
 	if (control->_scroll)
 	{
-		parent = control->widget->window;
+		parent = gtk_widget_get_window(control->widget);
 	}
 	else
 	{
-		parent = control->border->window;
-		a = &control->border->allocation;
-		x += a->x;
-		y += a->y;
+		parent = gtk_widget_get_window(control->border);
+		gtk_widget_get_allocation(control->border, &a);
+		x += a.x;
+		y += a.y;
 	}
 	
 	
 	if (!_frame_visible)
 	{
+#ifdef GTK3
+		GdkRGBA rgba;
+		gt_from_color(0, &rgba);
+#else
+		GdkColor color;
 		fill_gdk_color(&color, 0);
+#endif
 		
 		attr.wclass = GDK_INPUT_OUTPUT;
 		attr.window_type = GDK_WINDOW_CHILD;
@@ -514,7 +621,11 @@ static void show_frame(gControl *control, int x, int y, int w, int h)
 		for (i = 0; i < 4; i++)
 		{
 			window = gdk_window_new(parent, &attr, 0);
+#ifdef GTK3
+			gdk_window_set_background_rgba(window, &rgba);
+#else
 			gdk_window_set_background(window, &color);
+#endif
 			_frame[i] = window;
 		}
 	}
@@ -599,7 +710,7 @@ char *gDrag::getFormat(int n)
 	if (!_context)
 		return NULL;
 	
-	tg = g_list_first(_context->targets);
+	tg = g_list_first(gdk_drag_context_list_targets(_context));
 	
 	while (tg)
 	{
@@ -645,11 +756,14 @@ int gDrag::getType()
 
 static void cb_drag_data_received(GtkWidget *widget, GdkDragContext *context, gint x, gint y, GtkSelectionData *sel, guint info, guint time, gControl *data)
 {
+#if DEBUG_ME
+	fprintf(stderr, "cb_drag_data_received\n");
+#endif
+
 	if (gDrag::getType() == gDrag::Text)
 	{
-		//fprintf(stderr, "cb_drag_data_received: %d '%.*s'\n", sel->length, sel->length, (char *)sel->data);
-		if (sel->length != -1)
-			gDrag::setDropText((char*)sel->data, sel->length);
+		if (gtk_selection_data_get_length(sel) != -1)
+			gDrag::setDropText((char*)gtk_selection_data_get_data(sel), gtk_selection_data_get_length(sel));
 		else
 			gDrag::setDropText(NULL);
 	}
@@ -657,8 +771,8 @@ static void cb_drag_data_received(GtkWidget *widget, GdkDragContext *context, gi
 	if (gDrag::getType() == gDrag::Image)
 	{
 		//fprintf(stderr, "Image\n");
-		if (sel->length != -1)
-			gDrag::setDropImage((char*)sel->data, sel->length);
+		if (gtk_selection_data_get_length(sel) != -1)
+			gDrag::setDropImage((char*)gtk_selection_data_get_data(sel), gtk_selection_data_get_length(sel));
 		else
 			gDrag::setDropImage(NULL);
 	}
@@ -674,13 +788,16 @@ bool gDrag::getData(const char *prefix)
 	char *cfmt;
 	gulong id;
 	static bool norec = false;
+	gControl *dest;
 	
-	//fprintf(stderr, "getData: norec = %d _local = %d\n", norec, _local);
+#if DEBUG_ME
+	fprintf(stderr, "getData: norec = %d _local = %d\n", norec, _local);
+#endif
 	
 	if (norec || _local) // local DnD
 		return false;
 	
-	tg = g_list_first(_context->targets);
+	tg = g_list_first(gdk_drag_context_list_targets(_context));
 	
 	while (tg)
 	{
@@ -692,8 +809,12 @@ bool gDrag::getData(const char *prefix)
 		if (strlen(cfmt) >= strlen(prefix) && !strncasecmp(cfmt, prefix, strlen(prefix)))
 		{ 
 			g_free(format);
-			id = g_signal_connect(_dest->border, "drag-data-received", G_CALLBACK(cb_drag_data_received), (gpointer)_dest);
-			
+
+			dest = _dest;
+
+			id = g_signal_connect(dest->border, "drag-data-received", G_CALLBACK(cb_drag_data_received), (gpointer)dest);
+			//fprintf(stderr, "gDrag::getData: g_signal_connect -> %p %ld\n", dest->border, id);
+
 			_got_data = false;
 			
 			norec = true;
@@ -705,8 +826,9 @@ bool gDrag::getData(const char *prefix)
 	
 			norec = false;
 			
-			g_signal_handler_disconnect(_dest->border, id);
-	
+			//fprintf(stderr, "gDrag::getData: g_signal_disconnect -> %p %ld\n", dest->border, id);
+			g_signal_handler_disconnect(dest->border, id);
+
 			return false;
 		}
 			
@@ -747,3 +869,8 @@ gPicture *gDrag::getImage(bool fromOutside)
 	return _picture;
 }
 
+void gDrag::end()
+{
+	_end = true;
+	_active = false;
+}
\ No newline at end of file
