@@ -37,7 +37,7 @@ public:
 	virtual void setBackground(gColor color = COLOR_DEFAULT);
 
 private:
-	
+
 	void create();
 };
 
@@ -46,14 +46,25 @@ class gFrame : public gContainer
 public:
 	gFrame(gContainer *parent);
 
-	char* text();
-	void setText(char* vl);
+	const char *text();
+	void setText(const char *vl);
 
+	virtual int containerX();
+	virtual int containerY();
+	virtual int clientX();
+	virtual int clientY();
+	virtual int clientWidth();
+	virtual int clientHeight();
+
+	virtual void updateFont();
+#ifdef GTK3
+	virtual void updateColor();
+#else
 	virtual void setRealForeground(gColor color);
+#endif
 
 //"Private"
-  GtkWidget *fr;
-	GtkWidget *label;
+	GtkWidget *fr;
 };
 
 #endif
