@@ -43,17 +43,28 @@ public:
 //"Private"
 	static void disable();
 	static bool enable(gControl *control, GdkEventKey *e);
+	static bool canceled() { return _canceled; }
 	static void init();
 	static void exit();
 	
 	static void setActiveControl(gControl *control);
 
-private:
+	static bool raiseEvent(int type, gControl *control, const char *text);
+
+	static bool mustIgnoreEvent(GdkEventKey *e);
+
+	static bool gotCommit();
+
+	static bool _canceled;
 	static GdkEventKey _event;
+	static int _last_key_press;
+	static int _last_key_release;
+
+private:
 	static bool _valid;
-	static GtkIMContext *_im_context;
-	static gControl *_im_control;
-	static bool _no_input_method;
 };
 
+void gcb_im_commit(GtkIMContext *context, const char *str, gpointer pointer);
+gboolean gcb_key_event(GtkWidget *widget, GdkEvent *event, gControl *data);
+
 #endif
