@@ -25,17 +25,8 @@
 #define __GMOUSE_CPP
 
 #include "widgets.h"
-#include "widgets_private.h"
 #include "gmouse.h"
 
-#ifndef GAMBAS_DIRECTFB
-#ifdef GDK_WINDOWING_X11
-#include <gdk/gdkx.h>
-#include <X11/X.h>
-#include <X11/Xlib.h>
-#endif
-#endif
-
 int gMouse::_isValid = 0;
 int gMouse::_x;
 int gMouse::_y;
@@ -47,22 +38,25 @@ int gMouse::_delta;
 int gMouse::_orientation;
 int gMouse::_start_x;
 int gMouse::_start_y;
+int gMouse::_dx = 0;
+int gMouse::_dy = 0;
 GdkEvent *gMouse::_event = 0;
 
+#ifdef GTK3
+static GdkDevice *get_pointer()
+{
+	return gdk_device_manager_get_client_pointer(gdk_display_get_device_manager(gdk_display_get_default()));
+}
+#endif
+
 void gMouse::move(int x, int y)
 {
-	GdkDisplay* dpy;
-	GdkWindow*  win = gdk_get_default_root_window();
-	#ifdef GAMBAS_DIRECTFB
-	stub("DIRECTFB/gMouse::move");
-	#else	
-	#ifdef GDK_WINDOWING_X11
-	dpy = gdk_display_get_default();
-	XWarpPointer(GDK_DISPLAY_XDISPLAY(dpy), GDK_WINDOW_XID(win), GDK_WINDOW_XID(win), 0, 0, 0, 0, x, y);
-	#else
-	stub("no-X11/gMouse::move");
-	#endif
-	#endif
+	GdkDisplay* dpy = gdk_display_get_default();
+#ifdef GTK3
+	gdk_device_warp(get_pointer(), gdk_display_get_default_screen(dpy), x, y);
+#else
+	gdk_display_warp_pointer(dpy, gdk_display_get_default_screen(dpy), x, y);
+#endif
 }
 
 int gMouse::button()
@@ -117,12 +111,12 @@ bool gMouse::normal()
 
 int gMouse::x()
 {
-	return _isValid ? _x : -1;
+	return _isValid ? _x + _dx : -1;
 }
 
 int gMouse::y()
 {
-	return _isValid ? _y : -1;
+	return _isValid ? _y + _dy : -1;
 }
 
 void gMouse::getScreenPos(int *x, int *y)
@@ -134,7 +128,11 @@ void gMouse::getScreenPos(int *x, int *y)
 	}
 	else
 	{
+#ifdef GTK3
+		gdk_device_get_position(get_pointer(), NULL, x, y);
+#else
 		gdk_display_get_pointer(gdk_display_get_default(), NULL, x, y, NULL);
+#endif
 	}
 }
 
@@ -145,7 +143,11 @@ int gMouse::screenX()
 	if (_isValid)
 		x = _screen_x;
 	else
+#ifdef GTK3
+		gdk_device_get_position(get_pointer(), NULL, &x, NULL);
+#else
 		gdk_display_get_pointer(gdk_display_get_default(), NULL, &x, NULL, NULL);
+#endif
 	
 	return x;
 }
@@ -157,7 +159,11 @@ int gMouse::screenY()
 	if (_isValid)
 		y = _screen_y;
 	else
+#ifdef GTK3
+		gdk_device_get_position(get_pointer(), NULL, NULL, &y);
+#else
 		gdk_display_get_pointer(gdk_display_get_default(), NULL, NULL, &y, NULL);
+#endif
 	
 	return y;
 }
@@ -267,6 +273,7 @@ int gMouse::getType()
 
 void gMouse::initDevices()
 {
+#ifndef GTK3
 	static bool done = false;
 	
 	GList *devices;
@@ -293,16 +300,17 @@ void gMouse::initDevices()
 	}
 	
 	done = true;
+#endif
 }
 
 double gMouse::getPointerX()
 {
-	return ((GdkEventMotion *)_event)->x;
+	return ((GdkEventMotion *)_event)->x + _dx;
 }
 
 double gMouse::getPointerY()
 {
-	return ((GdkEventMotion *)_event)->y;
+	return ((GdkEventMotion *)_event)->y + _dy;
 }
 
 double gMouse::getPointerScreenX()
@@ -327,4 +335,10 @@ void gMouse::invalidate()
 			_event = 0;
 		}
 	}
-}
\ No newline at end of file
+}
+
+void gMouse::translate(int dx, int dy)
+{
+	_dx = dx;
+	_dy = dy;
+}
