@@ -1,23 +1,23 @@
 /***************************************************************************
 
-  gpicture.h
+	gpicture.h
 
-  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
+	(c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2, or (at your option)
-  any later version.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2, or (at your option)
+	any later version.
 
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY; without even the implied warranty of
+	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+	GNU General Public License for more details.
 
-  You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-  MA 02110-1301, USA.
+	You should have received a copy of the GNU General Public License
+	along with this program; if not, write to the Free Software
+	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+	MA 02110-1301, USA.
 
 ***************************************************************************/
 
@@ -35,24 +35,26 @@ class gPicture : public gShare
 {
 public:
 
-  enum gPictureType
-  {
-    VOID,
-    PIXBUF,
-    PIXMAP,
+	enum gPictureType
+	{
+		VOID,
+		PIXBUF,
+#ifndef GTK3
+		PIXMAP,
+#endif
 		SURFACE
-  };
-  
-  gPicture();
+	};
+
+	gPicture();
 	gPicture(gPictureType type, int w, int h, bool trans);
 	~gPicture();
 	gPicture *copy();
-	
-	bool isVoid() { return _type == VOID; }
-  
-  static void assign(gPicture **dst, gPicture *src = 0) { gShare::assign((gShare **)dst, src); }
 
-  gPictureType type() const { return _type; }
+	bool isVoid() { return _type == VOID; }
+
+	static void assign(gPicture **dst, gPicture *src = 0) { gShare::assign((gShare **)dst, src); }
+
+	gPictureType type() const { return _type; }
 	int width() const { return _width; }
 	int height() const { return _height; }
 	int depth();
@@ -83,25 +85,33 @@ public:
 	static gPicture *fromData(const char *data, int width, int height);
 
 //"Private"
+#ifndef GTK3
 	GdkPixmap *pixmap;
 	GdkBitmap *mask;
+#endif
 	GdkPixbuf *pixbuf;
 	cairo_surface_t *surface;
-  
-  gPictureType _type;
-  bool _transparent;
-  int _width;
-  int _height;
-  
-  gPicture(GdkPixbuf *image, bool trans = true);
-  gPicture(GdkPixmap *pixmap);
-  void initialize();
+
+	gPictureType _type;
+	bool _transparent;
+	int _width;
+	int _height;
+
+	gPicture(GdkPixbuf *image, bool trans = true);
+#ifdef GTK3
+	gPicture(cairo_surface_t *surf);
+#else
+	gPicture(GdkPixmap *pix);
+#endif
+	void initialize();
 	GdkPixbuf *getPixbuf();
+#ifndef GTK3
 	GdkPixmap *getPixmap();
 	GdkBitmap *getMask();
+#endif
 	GdkPixbuf *getIconPixbuf();
 	cairo_surface_t *getSurface();
-	
+
 	static gPicture* fromPixbuf(GdkPixbuf *buf) { return new gPicture(buf); }
 
 // "Private"
