@@ -36,6 +36,9 @@ public:
 //"Methods"
 	virtual void resize(int w,int h);
 	void ensureVisible(int x, int y, int w, int h);
+	void setBorder(bool b);
+	bool hasBorder() const;
+
 
 //"Signals"
 	void (*onScroll)(gScrollView *sender);
@@ -43,13 +46,15 @@ public:
 //"Private"
 	virtual void updateCursor(GdkCursor *cursor);
 	virtual void performArrange();
+	virtual void updateScrollBar();
+	virtual int getFrameWidth();
   void updateSize();
   guint _timer;
 	
 private:
 	GtkWidget *viewport;
 	int _maxw, _maxh;
-  int _mw, _mh;
+	int _sw, _sh;
 };
 
 #endif
