@@ -49,8 +49,8 @@ public:
 	void setText(const char *txt, int len = -1);
 	void setWrap(bool vl);
 	
-	int textWidth();
-	int textHeight();
+	//int textWidth();
+	//int textHeight();
 	
 	int alignment() const;
 	void setAlignment(int vl);
@@ -90,8 +90,14 @@ public:
 	void (*onCursor)(gTextArea *sender);
 
 //"Private"
-  void updateCursor(GdkCursor *cursor);
-  void waitForLayout(int *tw, int *th);
+  virtual void updateCursor(GdkCursor *cursor);
+#ifdef GTK3
+	virtual void updateColor();
+	virtual int minimumWidth() const;
+	virtual int minimumHeight() const;
+#endif
+	virtual GtkIMContext *getInputMethod();
+  //void waitForLayout(int *tw, int *th);
 	void clearUndoStack();
 	void clearRedoStack();
 	
