@@ -1,32 +1,95 @@
 /***************************************************************************
 
-  gtextbox.cpp
+	gtextbox.cpp
 
-  (c) 2004-2006 - Daniel Campos Fernández <dcamposf@gmail.com>
+	(c) 2004-2006 - Daniel Campos Fernández <dcamposf@gmail.com>
 
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2, or (at your option)
-  any later version.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2, or (at your option)
+	any later version.
 
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY; without even the implied warranty of
+	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+	GNU General Public License for more details.
 
-  You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-  MA 02110-1301, USA.
+	You should have received a copy of the GNU General Public License
+	along with this program; if not, write to the Free Software
+	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+	MA 02110-1301, USA.
 
 ***************************************************************************/
 
 #include "widgets.h"
-#include "widgets_private.h"
+#include "gapplication.h"
+#include "gkey.h"
 #include "gtextbox.h"
 
+#ifdef GTK3
+
+#define MAX_ICONS 2
+
+#if GTK_CHECK_VERSION(3,14,0)
+	
+struct _GtkEntryPrivate
+{
+  void         *icons[MAX_ICONS];
+
+  GtkEntryBuffer        *buffer;
+  GtkIMContext          *im_context;
+  GtkWidget             *popup_menu;
+
+  GdkWindow             *text_area;
+};
+
+#else
+
+struct _GtkEntryPrivate
+{
+	void         *icons[MAX_ICONS];
+
+	GtkEntryBuffer        *buffer;
+	GtkIMContext          *im_context;
+	GtkWidget             *popup_menu;
+
+	GdkDevice             *device;
+
+	GdkWindow             *text_area;
+};
+
+#endif
+
+#endif
+
+#ifdef GTK3
+GtkCssProvider *gTextBox::_style_provider = NULL;
+#endif
+
+static gboolean raise_change(gTextBox *data)
+{
+	if (data->_changed)
+	{
+		data->emit(SIGNAL(data->onChange));
+		data->_changed = false;
+	}
+	return FALSE;
+}
+
+static void cb_before_insert(GtkEditable *editable, gchar *new_text, gint new_text_length, gint *position, gTextBox *data)
+{
+	if (gKey::gotCommit())
+	{
+		gcb_im_commit(NULL, new_text, NULL);
+		if (gKey::canceled())
+			g_signal_stop_emission_by_name (G_OBJECT(editable), "insert-text");
+		*position = gtk_editable_get_position(editable);
+	}
+}
+
 static void cb_change_insert(GtkEditable *editable, gchar *new_text, gint new_text_length, gint *position, gTextBox *data)
 {
+	data->_changed = false;
 	gtk_editable_set_position(editable, *position);
 	data->emit(SIGNAL(data->onChange));
 	*position = gtk_editable_get_position(editable);
@@ -34,7 +97,11 @@ static void cb_change_insert(GtkEditable *editable, gchar *new_text, gint new_te
 
 static void cb_change_delete(GtkEditable *editable, gint start_pos, gint end_pos, gTextBox *data)
 {
-	data->emit(SIGNAL(data->onChange));	
+	if (!data->_changed)
+	{
+		data->_changed = true;
+		g_timeout_add(0, (GSourceFunc)raise_change, data);
+	}
 }
 
 static void cb_activate(GtkEntry *editable,gTextBox *data)
@@ -45,31 +112,67 @@ static void cb_activate(GtkEntry *editable,gTextBox *data)
 
 gTextBox::gTextBox(gContainer *parent, bool combo) : gControl(parent)
 {
+#ifdef GTK3
+	if (!_style_provider)
+	{
+		const char *css;
+
+		_style_provider = gtk_css_provider_new();
+
+		/*if (strcmp(gApplication::getStyleName(), "Clearlooks-Phenix") == 0)
+			css = ".entry { border-width: 0; padding: 0; border-radius: 0; margin: 0; border-style: none; box-shadow: none; background-image: none; }";
+		else*/
+		css = ".entry { border-width: 0; padding: 1px 2px 0px 2px; border-radius: 0; margin: 0; border-style: none; box-shadow: none; background-image: none; }";
+
+		gtk_css_provider_load_from_data(_style_provider, css, -1, NULL);
+	}
+
+	g_object_ref(_style_provider);
+#endif
+
 	if (!combo)
 	{
 		g_typ=Type_gTextBox;
 		
 		have_cursor = true;
-		use_base = true;
+		_no_background = TRUE;
 		
-		entry = widget = gtk_entry_new();	
+		entry = widget = gtk_entry_new();
 		realize();
+		setColorBase();
 		initEntry();
 	}
 	
+	_changed = false;
 	onChange = NULL;
 	onActivate = NULL;
 }
 
+gTextBox::~gTextBox()
+{
+#ifdef GTK3
+	g_object_unref(_style_provider);
+#endif
+}
+
+/*static void cb_im_commit(GtkIMContext *context, const char *str, gpointer pointer)
+{
+	fprintf(stderr, "cb_im_commit: %s\n", str);
+}*/
+
 void gTextBox::initEntry()
 {
+	_has_input_method = entry != NULL;
+
 	if (!entry)
 		return;
 	
 	// This imitates the QT behaviour, where change signal is raised after the cursor has been moved.
+	g_signal_connect(G_OBJECT(entry), "insert-text", G_CALLBACK(cb_before_insert), (gpointer)this);
 	g_signal_connect_after(G_OBJECT(entry), "insert-text", G_CALLBACK(cb_change_insert), (gpointer)this);
 	g_signal_connect_after(G_OBJECT(entry), "delete-text", G_CALLBACK(cb_change_delete), (gpointer)this);
 	g_signal_connect(G_OBJECT(entry), "activate", G_CALLBACK(cb_activate), (gpointer)this);
+	//g_signal_connect(getInputMethod(), "commit", G_CALLBACK(cb_im_commit), (gpointer)this);
 }
 
 char* gTextBox::text()
@@ -138,9 +241,9 @@ void gTextBox::setPosition(int pos)
 		
 	if (pos < 0) 
 		pos = 0;
-	else if (pos >= len)
+	else if (pos > len)
 		pos = -1;
-		
+
 	gtk_editable_set_position(GTK_EDITABLE(entry), pos);
 }
 
@@ -162,15 +265,26 @@ void gTextBox::setBorder(bool vl)
 	
 	gtk_entry_set_has_frame(GTK_ENTRY(entry), vl);
 	
-	/*if (vl)
+#ifdef GTK3
+	GtkStyleContext *style = gtk_widget_get_style_context(entry);
+	if (vl)
+		gtk_style_context_remove_provider(style, GTK_STYLE_PROVIDER(_style_provider));
+	else
+		gtk_style_context_add_provider(style, GTK_STYLE_PROVIDER(_style_provider), GTK_STYLE_PROVIDER_PRIORITY_USER);
+
+	//gtk_style_context_invalidate(style);
+#else
+	if (vl)
 		gtk_entry_set_inner_border(GTK_ENTRY(entry), NULL);
 	else
-	{*/
+	{
 		GtkBorder *border = gtk_border_new();
+		border->left = border->right = 2;
+		border->top = 1;
 		gtk_entry_set_inner_border(GTK_ENTRY(entry), border);
 		gtk_border_free(border);
-	//}
-	
+	}
+#endif
 }
 
 void gTextBox::insert(char *txt, int len)
@@ -312,24 +426,33 @@ void gTextBox::setAlignment(int al)
 
 void gTextBox::updateCursor(GdkCursor *cursor)
 {
-  GdkWindow *win;
-  
-  gControl::updateCursor(cursor);
-  if (!entry)
-  	return;
-  	
-  win = GTK_ENTRY(entry)->text_area;
-  if (!win)
-  	return;
-  	
-  if (cursor)
-    gdk_window_set_cursor(win, cursor);
-  else
-  {
-    cursor = gdk_cursor_new_for_display(gtk_widget_get_display(widget), GDK_XTERM);
-    gdk_window_set_cursor(win, cursor);
-    gdk_cursor_unref(cursor);
-  }
+	GdkWindow *win;
+	
+	gControl::updateCursor(cursor);
+	if (!entry)
+		return;
+
+#ifdef GTK3
+	win = GTK_ENTRY(entry)->priv->text_area;
+#else
+	win = GTK_ENTRY(entry)->text_area;
+#endif
+
+	if (!win)
+		return;
+		
+	if (cursor)
+		gdk_window_set_cursor(win, cursor);
+	else
+	{
+		cursor = gdk_cursor_new_for_display(gtk_widget_get_display(widget), GDK_XTERM);
+		gdk_window_set_cursor(win, cursor);
+#ifdef GTK3
+		g_object_unref(cursor);
+#else
+		gdk_cursor_unref(cursor);
+#endif
+	}
 }
 
 void gTextBox::clear()
@@ -346,3 +469,12 @@ int gTextBox::minimumHeight()
 	return req.height - 4;*/
 	return font()->height() + hasBorder() ? 4 : 2;
 }
+
+GtkIMContext *gTextBox::getInputMethod()
+{
+#ifdef GTK3
+	return entry ? GTK_ENTRY(entry)->priv->im_context : NULL;
+#else
+	return entry ? GTK_ENTRY(entry)->im_context : NULL;
+#endif
+}
