@@ -28,6 +28,7 @@ class gTextBox : public gControl
 {
 public:
 	gTextBox(gContainer *parent, bool combo = false);
+	~gTextBox();
 
 //"Properties"
 	int alignment();
@@ -65,10 +66,17 @@ public:
 	void (*onActivate)(gTextBox *sender);
 
 //"Private"
-  void updateCursor(GdkCursor *cursor);
+  virtual void updateCursor(GdkCursor *cursor);
   void initEntry();
   GtkWidget *entry;
 	virtual int minimumHeight();
+	virtual GtkIMContext *getInputMethod();
+
+	unsigned _changed : 1;
+
+#ifdef GTK3
+	static GtkCssProvider *_style_provider;
+#endif
 };
 
 #endif
