@@ -34,6 +34,7 @@
 #include "gkey.h"
 
 #include "x11.h"
+#include "desktop.h"
 #include "CScreen.h"
 #include "CStyle.h"
 #include "CDraw.h"
@@ -72,6 +73,7 @@
 #include <gtk/gtk.h>
 #include <string.h>
 
+GB_CLASS CLASS_Control;
 GB_CLASS CLASS_Picture;
 GB_CLASS CLASS_Image;
 GB_CLASS CLASS_DrawingArea;
@@ -102,252 +104,251 @@ static void *_old_hook_main;
 
 int MAIN_scale = 0;
 bool MAIN_debug_busy = false;
+bool MAIN_rtl = false;
 
 extern "C"
 {
-	GB_INTERFACE GB EXPORT;
-	IMAGE_INTERFACE IMAGE EXPORT;
 
-	GB_DESC *GB_CLASSES[] EXPORT =
-	{
-		ScreenDesc,
-		ScreensDesc,
-		DesktopDesc,
-		ApplicationDesc,
-		StyleDesc,
-		CSelectDesc,
-		CAlignDesc,
-		CArrangeDesc,
-		CBorderDesc,
-		CScrollDesc,
-		CColorDesc,
-		CFontsDesc,
-		CFontDesc,
-		CKeyDesc,
-		CImageDesc,
-		CPictureDesc,
-		CClipboardDesc,
-		CDragDesc,
-		CCursorDesc,
-		CMouseDesc,
-		CPointerDesc,
-		CMessageDesc,
-		CDialogDesc,
-		CWatcherDesc,
-		CWidgetDesc,
-		CChildrenDesc,
-		CContainerDesc,
-		CDrawingAreaDesc,
-		CFrameDesc,
-		CUserControlDesc,
-		CUserContainerDesc,
-		CPanelDesc,
-		CHBoxDesc,
-		CVBoxDesc,
-		CHPanelDesc,
-		CVPanelDesc,
-		CMenuDesc,
-		CMenuChildrenDesc,
-		CTrayIconDesc,
-		CTrayIconsDesc,
-		CWindowMenusDesc,
-		CWindowControlsDesc,
-		CWindowDesc,
-		CWindowsDesc,
-		CFormDesc,
-		CLabelDesc,
-		CTextLabelDesc,
-		CSliderDesc,
-		CScrollBarDesc,
-		CButtonDesc,
-		CToggleButtonDesc,
-		CCheckBoxDesc,
-		CRadioButtonDesc,
-		CToolButtonDesc,
-		CPictureBoxDesc,
-		CMovieBoxDesc,
-		CTextBoxSelectionDesc,
-		CTextBoxDesc,
-		CTextAreaDesc,
-		CTextAreaSelectionDesc,
-		CComboBoxDesc,
-		CComboBoxItemDesc,
-		CTabStripDesc,
-		CTabStripContainerDesc,
-		CTabStripContainerChildrenDesc,
-		CPluginDesc,
-		CScrollViewDesc,
-		CSpinBoxDesc,
-		CSeparatorDesc,
-		CStockDesc,
-		PrinterDesc,
-		SvgImageDesc,
-		NULL
-	};
+GB_INTERFACE GB EXPORT;
+IMAGE_INTERFACE IMAGE EXPORT;
 
-	void *GB_GTK_1[] EXPORT =
-	{
-		(void *)GTK_INTERFACE_VERSION,
-		(void *)GTK_CreateGLArea,
-		NULL
-	};
+static void declare_tray_icon()
+{
+GB.Component.Declare(TrayIconsDesc);
+GB.Component.Declare(TrayIconDesc);
+}
 
-	const char *GB_INCLUDE EXPORT = "gb.draw,gb.gui.base";
+GB_DESC *GB_CLASSES[] EXPORT =
+{
+	ScreenDesc,
+	ScreensDesc,
+	DesktopDesc,
+	ApplicationDesc,
+	StyleDesc,
+	CSelectDesc,
+	CAlignDesc,
+	CArrangeDesc,
+	CBorderDesc,
+	CScrollDesc,
+	CColorDesc,
+	CFontsDesc,
+	CFontDesc,
+	CKeyDesc,
+	CImageDesc,
+	CPictureDesc,
+	CClipboardDesc,
+	CDragDesc,
+	CCursorDesc,
+	CMouseDesc,
+	CPointerDesc,
+	CMessageDesc,
+	CDialogDesc,
+	CWatcherDesc,
+	CWidgetDesc,
+	CChildrenDesc,
+	CContainerDesc,
+	CDrawingAreaDesc,
+	CFrameDesc,
+	CUserControlDesc,
+	CUserContainerDesc,
+	CPanelDesc,
+	CHBoxDesc,
+	CVBoxDesc,
+	CHPanelDesc,
+	CVPanelDesc,
+	CMenuDesc,
+	CMenuChildrenDesc,
+	CWindowMenusDesc,
+	CWindowControlsDesc,
+	CWindowDesc,
+	CWindowsDesc,
+	CFormDesc,
+	CLabelDesc,
+	CTextLabelDesc,
+	CSliderDesc,
+	CScrollBarDesc,
+	CButtonDesc,
+	CToggleButtonDesc,
+	CCheckBoxDesc,
+	CRadioButtonDesc,
+	CToolButtonDesc,
+	CPictureBoxDesc,
+	CMovieBoxDesc,
+	CTextBoxSelectionDesc,
+	CTextBoxDesc,
+	CTextAreaDesc,
+	CTextAreaSelectionDesc,
+	CComboBoxDesc,
+	CComboBoxItemDesc,
+	CTabStripDesc,
+	CTabStripContainerDesc,
+	CTabStripContainerChildrenDesc,
+	CPluginDesc,
+	CScrollViewDesc,
+	CSpinBoxDesc,
+	CSeparatorDesc,
+	CStockDesc,
+	PrinterDesc,
+	SvgImageDesc,
+	NULL
+};
 
-	int EXPORT GB_INIT(void)
-	{
-		char *env;
-		
-		env = getenv("GB_GUI_BUSY");
-		if (env && atoi(env))
-			MAIN_debug_busy = true;
-		
-		GB.Hook(GB_HOOK_QUIT, (void *)my_quit);
-		_old_hook_main = GB.Hook(GB_HOOK_MAIN, (void *)my_main);
-		GB.Hook(GB_HOOK_WAIT, (void *)my_wait);
-		GB.Hook(GB_HOOK_LOOP, (void *)my_loop);
-		GB.Hook(GB_HOOK_TIMER,(void *)my_timer);
-		GB.Hook(GB_HOOK_WATCH,(void *)my_watch);
-		GB.Hook(GB_HOOK_POST,(void*)my_post);
-		GB.Hook(GB_HOOK_ERROR,(void*)my_error);
-		GB.Hook(GB_HOOK_LANG,(void*)my_lang);
+void *GB_GTK_1[] EXPORT =
+{
+	(void *)GTK_INTERFACE_VERSION,
+	(void *)GTK_CreateGLArea,
+	NULL
+};
 
-		GB.Component.Load("gb.draw");
-		GB.Component.Load("gb.image");
-		GB.Component.Load("gb.gui.base");
-		GB.GetInterface("gb.image", IMAGE_INTERFACE_VERSION, &IMAGE);
-		IMAGE.SetDefaultFormat(GB_IMAGE_RGBA);
-		DRAW_init();
-		
-		CWatcher::init();
+const char *GB_INCLUDE EXPORT = "gb.draw,gb.gui.base";
+
+int EXPORT GB_INIT(void)
+{
+	char *env;
+
+	env = getenv("GB_GUI_BUSY");
+	if (env && atoi(env))
+		MAIN_debug_busy = true;
+
+	GB.Hook(GB_HOOK_QUIT, (void *)my_quit);
+	_old_hook_main = GB.Hook(GB_HOOK_MAIN, (void *)my_main);
+	GB.Hook(GB_HOOK_WAIT, (void *)my_wait);
+	GB.Hook(GB_HOOK_LOOP, (void *)my_loop);
+	GB.Hook(GB_HOOK_TIMER,(void *)my_timer);
+	GB.Hook(GB_HOOK_WATCH,(void *)my_watch);
+	GB.Hook(GB_HOOK_POST,(void*)my_post);
+	GB.Hook(GB_HOOK_ERROR,(void*)my_error);
+	GB.Hook(GB_HOOK_LANG,(void*)my_lang);
+
+	GB.Component.Load("gb.draw");
+	GB.Component.Load("gb.image");
+	GB.Component.Load("gb.gui.base");
+	GB.GetInterface("gb.image", IMAGE_INTERFACE_VERSION, &IMAGE);
+	IMAGE.SetDefaultFormat(GB_IMAGE_RGBA);
+	DRAW_init();
+
+	CWatcher::init();
+
+	CLASS_Control = GB.FindClass("Control");
+	//CLASS_Container = GB.FindClass("Container");
+	//CLASS_UserControl = GB.FindClass("UserControl");
+	//CLASS_UserContainer = GB.FindClass("UserContainer");
+	CLASS_Window = GB.FindClass("Window");
+	CLASS_Menu = GB.FindClass("Menu");
+	CLASS_Picture = GB.FindClass("Picture");
+	//CLASS_Drawing = GB.FindClass("Drawing");
+	CLASS_DrawingArea = GB.FindClass("DrawingArea");
+	CLASS_Printer = GB.FindClass("Printer");
+	CLASS_Image = GB.FindClass("Image");
+	CLASS_SvgImage = GB.FindClass("SvgImage");
 
-		//CLASS_Control = GB.FindClass("Control");
-		//CLASS_Container = GB.FindClass("Container");
-		//CLASS_UserControl = GB.FindClass("UserControl");
-		//CLASS_UserContainer = GB.FindClass("UserContainer");
-		CLASS_Window = GB.FindClass("Window");
-		CLASS_Menu = GB.FindClass("Menu");
-		CLASS_Picture = GB.FindClass("Picture");
-		//CLASS_Drawing = GB.FindClass("Drawing");
-		CLASS_DrawingArea = GB.FindClass("DrawingArea");
-		CLASS_Printer = GB.FindClass("Printer");
-		CLASS_Image = GB.FindClass("Image");
-		CLASS_SvgImage = GB.FindClass("SvgImage");
-		
 #if !defined(GLIB_VERSION_2_36)
-		g_type_init();
+	g_type_init();
 #endif /* !defined(GLIB_VERSION_2_36) */
-		
-		my_lang(GB.System.Language(), GB.System.IsRightToLeft());
 
-		return -1;
-	}
+	my_lang(GB.System.Language(), GB.System.IsRightToLeft());
 
-	void EXPORT GB_EXIT()
-	{
-		CWatcher::exit();
-	}
+	return -1;
+}
 
-	int EXPORT GB_INFO(const char *key, void **value)
-	{
-		if (!strcasecmp(key, "DISPLAY"))
-		{
-			#ifdef GDK_WINDOWING_X11
-			#ifndef GAMBAS_DIRECTFB			
-			*value = (void *)gdk_x11_display_get_xdisplay(gdk_display_get_default());
-			#else
-			// For DirectFB
-			*value=0;
-			stub("DIRECTFB/EXPORT GB_INFO");
-			#endif
-			#else
-			// For Win32
-			*value=0;
-			stub("no-X11/EXPORT GB_INFO");
-			#endif
-			return TRUE;
-		}
-		else if (!strcasecmp(key, "ROOT_WINDOW"))
-		{
-			#ifdef GDK_WINDOWING_X11
-			#ifndef GAMBAS_DIRECTFB
-			*value = (void *)gdk_x11_get_default_root_xwindow();
-			#else
-			// For DirectFB
-			*value=0;
-			stub("DIRECTFB/EXPORT GB_INFO");
-			#endif
-			#else
-			// For Win32
-			*value=0;
-			stub("no-X11/EXPORT GB_INFO");
-			#endif
-			return TRUE;
-		}
-		else if (!strcasecmp(key, "GET_HANDLE"))
-		{
-			*value = (void *)CWIDGET_get_handle;
-			return TRUE;
-		}
-		else if (!strcasecmp(key, "SET_EVENT_FILTER"))
-		{
-			*value = (void *)gApplication::setEventFilter;
-			return TRUE;
-		}
-		else if (!strcasecmp(key, "TIME"))
-		{
-			*value = (void *)(intptr_t)gtk_get_current_event_time(); //gdk_x11_display_get_user_time(gdk_display_get_default());
-			return TRUE;
-		}
-		else
-			return FALSE;
-	}
+void EXPORT GB_EXIT()
+{
+	CWatcher::exit();
+}
 
-	static void activate_main_window(intptr_t)
+int EXPORT GB_INFO(const char *key, void **value)
+{
+	if (!strcasecmp(key, "DISPLAY"))
 	{
-		if (gMainWindow::_active)
-			gtk_window_present(GTK_WINDOW(gMainWindow::_active->topLevel()->border));
+		*value = (void *)gdk_x11_display_get_xdisplay(gdk_display_get_default());
+		return TRUE;
 	}
-	
-	void EXPORT GB_SIGNAL(int signal, void *param)
+	else if (!strcasecmp(key, "ROOT_WINDOW"))
 	{
-		static GtkWidget *save_popup_grab = NULL;
-		
-		switch(signal)
-		{
-			case GB_SIGNAL_DEBUG_BREAK:
-				if (gApplication::_popup_grab)
-				{
-					save_popup_grab = gApplication::_popup_grab;
-					gApplication::ungrabPopup();
-				}
-				break;
-				
-			case GB_SIGNAL_DEBUG_FORWARD:
-				//while (qApp->activePopupWidget())
-				//	delete qApp->activePopupWidget();
-				if (gdk_display_get_default())
-				gdk_display_sync(gdk_display_get_default());
-				break;
-				
-			case GB_SIGNAL_DEBUG_CONTINUE:
-				GB.Post((GB_CALLBACK)activate_main_window, 0);
-				if (save_popup_grab)
-				{
-					gApplication::_popup_grab = save_popup_grab;
-					save_popup_grab = NULL;
-					gApplication::grabPopup();
-				}
-				break;
-		}
+		*value = (void *)gdk_x11_get_default_root_xwindow();
+		return TRUE;
+	}
+	else if (!strcasecmp(key, "GET_HANDLE"))
+	{
+		*value = (void *)CWIDGET_get_handle;
+		return TRUE;
+	}
+	else if (!strcasecmp(key, "SET_EVENT_FILTER"))
+	{
+		*value = (void *)gApplication::setEventFilter;
+		return TRUE;
+	}
+	else if (!strcasecmp(key, "TIME"))
+	{
+		*value = (void *)(intptr_t)gtk_get_current_event_time(); //gdk_x11_display_get_user_time(gdk_display_get_default());
+		return TRUE;
+	}
+	else if (!strcasecmp(key, "DECLARE_TRAYICON"))
+	{
+		*value = (void *)declare_tray_icon;
+		return TRUE;
+	}
+	else
+		return FALSE;
+}
+
+static void activate_main_window(intptr_t)
+{
+	if (gMainWindow::_active)
+		gtk_window_present(GTK_WINDOW(gMainWindow::_active->topLevel()->border));
+}
+
+void EXPORT GB_SIGNAL(int signal, void *param)
+{
+	static GtkWidget *save_popup_grab = NULL;
+
+	switch(signal)
+	{
+		case GB_SIGNAL_DEBUG_BREAK:
+			if (gApplication::_popup_grab)
+			{
+				save_popup_grab = gApplication::_popup_grab;
+				gApplication::ungrabPopup();
+			}
+			break;
+
+		case GB_SIGNAL_DEBUG_FORWARD:
+			//while (qApp->activePopupWidget())
+			//	delete qApp->activePopupWidget();
+			if (gdk_display_get_default())
+			gdk_display_sync(gdk_display_get_default());
+			break;
+
+		case GB_SIGNAL_DEBUG_CONTINUE:
+			GB.Post((GB_CALLBACK)activate_main_window, 0);
+			if (save_popup_grab)
+			{
+				gApplication::_popup_grab = save_popup_grab;
+				save_popup_grab = NULL;
+				gApplication::grabPopup();
+			}
+			break;
 	}
 }
 
+} // extern "C"
+
 void my_quit (void)
 {
+	GB_FUNCTION func;
+
 	while (gtk_events_pending())
 		gtk_main_iteration();
-  
+
+	if (GB.ExistClass("TrayIcons"))
+	{
+		if (!GB.GetFunction(&func, (void *)GB.FindClass("TrayIcons"), "DeleteAll", NULL, NULL))
+			GB.Call(&func, 0, FALSE);
+	}
+
+	if (!GB.GetFunction(&func, (void *)GB.FindClass("_Gui"), "_Quit", NULL, NULL))
+		GB.Call(&func, 0, FALSE);
+
 	CWINDOW_delete_all();
 	gControl::cleanRemovedControls();
 
@@ -369,10 +370,10 @@ static void my_main(int *argc, char ***argv)
 {
 	static bool init = false;
 	char *env;
-	
+
 	if (init)
 		return;
-	
+
 	env = getenv("GB_X11_INIT_THREADS");
 		if (env && atoi(env))
 	XInitThreads();
@@ -380,10 +381,10 @@ static void my_main(int *argc, char ***argv)
 	gApplication::init(argc, argv);
 	gApplication::setDefaultTitle(GB.Application.Title());
 	gDesktop::init();
-	
+
 	gApplication::onEnterEventLoop = GB.Debug.EnterEventLoop;
 	gApplication::onLeaveEventLoop = GB.Debug.LeaveEventLoop;
-		
+
 	MAIN_scale = gDesktop::scale();
 	#ifdef GDK_WINDOWING_X11
   	X11_init(gdk_x11_display_get_xdisplay(gdk_display_get_default()), gdk_x11_get_default_root_xwindow());
@@ -416,10 +417,10 @@ typedef
 
 gboolean my_timer_function(GB_TIMER *timer)
 {
-	if (timer->id)  
+	if (timer->id)
 	{
 		GB.RaiseTimer(timer);
-		
+
 		if (timer->id)
 		{
 			MyTimerId *id = (MyTimerId *)timer->id;
@@ -435,7 +436,7 @@ gboolean my_timer_function(GB_TIMER *timer)
 			//fprintf(stderr, "elapsed = %d  delay = %d  next = %d\n", elapsed, timer->delay, next);
 		}
 	}
-	
+
 	return false;
 }
 
@@ -475,7 +476,7 @@ static int my_loop()
 {
 	gControl::cleanRemovedControls();
 	_must_check_quit = true;
-	
+
 	for(;;)
 	{
 		if (_must_check_quit)
@@ -490,12 +491,18 @@ static int my_loop()
 	}
 
 	my_quit();
-	
+
   return 0;
 }
 
 static void my_wait(int duration)
 {
+	if (gDrawingArea::inAnyDrawEvent())
+	{
+		GB.Error("Wait is forbidden during a repaint event");
+		return;
+	}
+
 	if (duration > 0 && gKey::valid())
 	{
 #ifdef GTK3
@@ -531,18 +538,20 @@ static void my_error(int code,char *error,char *where)
 
 static void my_lang(char *lang, int rtl)
 {
-	int bucle, ct;
+	int i, n;
 	gControl *iter;
 
+	MAIN_rtl = rtl;
+
 	if (rtl)
 		gtk_widget_set_default_direction(GTK_TEXT_DIR_RTL);
 	else
 		gtk_widget_set_default_direction(GTK_TEXT_DIR_LTR);
 
-	ct=gApplication::controlCount();
-	for (bucle=0;bucle<ct;bucle++)
+	n = gApplication::controlCount();
+	for (i = 0; i < n; i++)
 	{
-		iter=gApplication::controlItem(bucle);
+		iter = gApplication::controlItem(i);
 		if (iter->isVisible() && iter->isContainer())
 			((gContainer*)iter)->performArrange();
 	}
@@ -563,9 +572,9 @@ void MAIN_do_iteration(bool do_not_block, bool do_not_sleep)
 		if (gtk_events_pending ())
 			gtk_main_iteration_do (false);
 	}
-	else	
+	else
 		gtk_main_iteration_do(true);
-	
+
 	gApplication::_loopLevel--;
 
 	if (_post_check)
@@ -573,7 +582,7 @@ void MAIN_do_iteration(bool do_not_block, bool do_not_sleep)
 		_post_check = false;
 		GB.CheckPost();
 	}
-	
+
 	gControl::cleanRemovedControls();
 }
 
