@@ -96,6 +96,8 @@ typedef
 		GtkPrintContext *print_context;
 		CFONT *font;
 		CFONT **font_stack;
+		PangoLayout *layout;
+		float ascent;
 		cairo_matrix_t init;
 		double dx;
 		double dy;
@@ -273,8 +275,8 @@ static int Begin(GB_PAINT *d)
 
 		}
 
-		d->resolutionX = gDesktop::resolution(); //device->physicalDpiX();
-		d->resolutionY = gDesktop::resolution(); //device->physicalDpiY();
+		rx = gDesktop::resolution(); //device->physicalDpiX();
+		ry = gDesktop::resolution(); //device->physicalDpiY();
 
 #else
 		GdkDrawable *dr;
@@ -298,8 +300,8 @@ static int Begin(GB_PAINT *d)
 			dr = gtk_widget_get_window(wid->widget);
 		}
 
-		d->resolutionX = gDesktop::resolution(); //device->physicalDpiX();
-		d->resolutionY = gDesktop::resolution(); //device->physicalDpiY();
+		rx = gDesktop::resolution(); //device->physicalDpiX();
+		ry = gDesktop::resolution(); //device->physicalDpiY();
 		
 		EXTRA(d)->context = gdk_cairo_create(dr);
 #endif
@@ -368,8 +370,12 @@ static void End(GB_PAINT *d)
 	void *device = d->device;
 	GB_PAINT_EXTRA *dx = EXTRA(d);
 
+	if (dx->layout)
+		g_object_unref(dx->layout);	
+
 	if (dx->font_stack)
 		GB.FreeArray(POINTER(&dx->font_stack));
+	
 	GB.Unref(POINTER(&dx->font));
 	
 	if (GB.Is(device, CLASS_Picture))
@@ -431,6 +437,41 @@ static void Antialias(GB_PAINT *d, int set, int *antialias)
 		*antialias = (cairo_get_antialias(CONTEXT(d)) == CAIRO_ANTIALIAS_NONE) ? 0 : 1;
 }
 
+static void _Font(GB_PAINT *d, int set, GB_FONT *font);
+
+static void update_layout(GB_PAINT *d)
+{
+	CFONT *font;
+	GB_PAINT_EXTRA *dx = EXTRA(d);
+	
+	if (dx->layout)
+	{
+		_Font(d, FALSE, (GB_FONT *)&font);
+		gt_add_layout_from_font(dx->layout, font->font, d->resolutionY);
+		dx->ascent = font->font->ascentF();
+		
+		pango_cairo_context_set_font_options(pango_layout_get_context(dx->layout), gdk_screen_get_font_options (gdk_screen_get_default()));
+		
+		/*cairo_font_options_t *options = cairo_font_options_create(); //cairo_font_options_copy(pango_cairo_context_get_font_options(pango_layout_get_context(dx->layout)));
+		cairo_font_options_set_antialias(options, CAIRO_ANTIALIAS_GRAY);
+		cairo_font_options_set_hint_style(options, CAIRO_HINT_STYLE_MEDIUM);
+		cairo_font_options_set_hint_metrics(options, CAIRO_HINT_METRICS_ON);
+		cairo_font_options_set_subpixel_order(options, CAIRO_SUBPIXEL_ORDER_RGB);
+		pango_cairo_context_set_font_options(pango_layout_get_context(dx->layout), options);
+		cairo_font_options_destroy(options);*/
+		
+		pango_layout_context_changed(dx->layout);
+	
+	}
+}
+
+static void apply_font(gFont *font, void *object = 0)
+{
+	GB_PAINT *d = (GB_PAINT *)DRAW.Paint.GetCurrent();
+	update_layout(d);
+}
+
+
 // Font is used by X11!
 static void _Font(GB_PAINT *d, int set, GB_FONT *font)
 {
@@ -439,11 +480,13 @@ static void _Font(GB_PAINT *d, int set, GB_FONT *font)
 		GB.Unref(POINTER(&EXTRA(d)->font));
 		if (*font)
 		{
-			EXTRA(d)->font = CFONT_create(((CFONT *)(*font))->font->copy());
+			EXTRA(d)->font = CFONT_create(((CFONT *)(*font))->font->copy(), apply_font);
 			GB.Ref(EXTRA(d)->font);
 		}
 		else
 			EXTRA(d)->font = NULL;
+		
+		update_layout(d);
 	}
 	else
 	{
@@ -452,11 +495,11 @@ static void _Font(GB_PAINT *d, int set, GB_FONT *font)
 			if (GB.Is(d->device, CLASS_DrawingArea))
 			{
 				gDrawingArea *wid = (gDrawingArea *)((CWIDGET *)d->device)->widget;
-				EXTRA(d)->font = CFONT_create(wid->font()->copy());
+				EXTRA(d)->font = CFONT_create(wid->font()->copy(), apply_font);
 			}
 			else
 			{
-				EXTRA(d)->font = CFONT_create(new gFont());
+				EXTRA(d)->font = CFONT_create(new gFont(), apply_font);
 			}
 			
 			GB.Ref(EXTRA(d)->font);
@@ -926,14 +969,21 @@ static PangoLayout *create_pango_layout(GB_PAINT *d)
 	/*if (dx->print_context)
 		return gtk_print_context_create_pango_layout(dx->print_context);
 	else*/
-		return pango_cairo_create_layout(dx->context);
+
+	if (!dx->layout)
+	{
+		dx->layout = pango_cairo_create_layout(dx->context);
+		
+		update_layout(d);
+	}
+		
+	return dx->layout;
 }
 
 static void draw_text(GB_PAINT *d, bool rich, const char *text, int len, float w, float h, int align, bool draw)
 {
 	char *html = NULL;
 	PangoLayout *layout;
-	CFONT *font;
 	float tw, th, offx, offy;
 
 	layout = create_pango_layout(d);
@@ -949,9 +999,6 @@ static void draw_text(GB_PAINT *d, bool rich, const char *text, int len, float w
 	else
 		pango_layout_set_text(layout, text, len);
 	
-	_Font(d, FALSE, (GB_FONT *)&font);
-	gt_add_layout_from_font(layout, font->font, d->resolutionY);
-	
 	if (align == GB_DRAW_ALIGN_DEFAULT)
 		align = ALIGN_TOP_NORMAL;
 	
@@ -964,7 +1011,7 @@ static void draw_text(GB_PAINT *d, bool rich, const char *text, int len, float w
 	else
 	{
 		offx = 0;
-		offy = -(font->font->ascentF());
+		offy = -(EXTRA(d)->ascent);
 	}
 	
 	cairo_rel_move_to(CONTEXT(d), offx + DX(d), offy + DY(d));
@@ -973,8 +1020,6 @@ static void draw_text(GB_PAINT *d, bool rich, const char *text, int len, float w
 	else
 		pango_cairo_layout_path(CONTEXT(d), layout);
 
-	g_object_unref(layout);	
-	
 	if (html) g_free(html);
 }
 
@@ -1019,11 +1064,10 @@ static void get_text_extents(GB_PAINT *d, bool rich, const char *text, int len,
 	GetCurrentPoint(d, &x, &y);
 	
 	ext->x1 = (float)rect.x / PANGO_SCALE + x;
-	ext->y1 = (float)rect.y / PANGO_SCALE + y - font->font->ascentF();
+	ext->y1 = (float)rect.y / PANGO_SCALE + y - EXTRA(d)->ascent;
 	ext->x2 = ext->x1 + (float)rect.width / PANGO_SCALE;
 	ext->y2 = ext->y1 + (float)rect.height / PANGO_SCALE;
 	
-	g_object_unref(layout);
 	if (html) g_free(html);
 }
 
@@ -1044,9 +1088,11 @@ static void TextSize(GB_PAINT *d, const char *text, int len, float *w, float *h)
 	_Font(d, FALSE, (GB_FONT *)&font);
 	
 	scale = (float)d->resolutionY / gDesktop::resolution();
+	
+	font->font->textSize(text, len, w, h);
 
-	*w = font->font->width(text, len) * scale;
-	*h = font->font->height(text, len) * scale;
+	*w *= scale;
+	*h *= scale;
 }
 
 static void RichTextSize(GB_PAINT *d, const char *text, int len, float sw, float *w, float *h)
