@@ -234,6 +234,7 @@ void gMainWindow::initialize()
 	_transparent = false;
 	_utility = false;
 	_no_take_focus = false;
+	_moved = false;
 
 	onOpen = NULL;
 	onShow = NULL;
@@ -243,6 +244,7 @@ void gMainWindow::initialize()
 	onActivate = NULL;
 	onDeactivate = NULL;
 	onState = NULL;
+	onFontChange = NULL;
 	
 	accel = gtk_accel_group_new();
 }
@@ -451,6 +453,9 @@ void gMainWindow::move(int x, int y)
 	
 	if (isTopLevel())
 	{
+		if (!_moved && (x || y))
+			_moved = true;
+		
 		if (x == bufX && y == bufY) 
 			return;
 	
@@ -557,11 +562,19 @@ void gMainWindow::emitOpen()
 	//_no_resize_event = false;
 }
 
+void gMainWindow::present()
+{
+	if (_no_take_focus)
+		gtk_widget_show(GTK_WIDGET(border));
+	else
+		gtk_window_present(GTK_WINDOW(border));
+}
+
 void gMainWindow::afterShow()
 {
 	if (_activate)
 	{
-		gtk_window_present(GTK_WINDOW(border));
+		present();
 		_activate = false;
 	}
 }
@@ -586,13 +599,10 @@ void gMainWindow::setVisible(bool vl)
 		visible = true;
 		_hidden = false;
 		
-		setTransparent(_transparent);
+		setTransparent(_transparent); // must not call gtk_window_present!
 
 		if (isTopLevel())
 		{
-			if (!_title || !*_title)
-				gtk_window_set_title(GTK_WINDOW(border), gApplication::defaultTitle());
-			
 			/*if (!_xembed)
 			{
 				fprintf(stderr, "gtk_window_group_add_window: %p -> %p\n", border, gApplication::currentGroup());
@@ -609,16 +619,20 @@ void gMainWindow::setVisible(bool vl)
 			#endif
 
 			gtk_window_move(GTK_WINDOW(border), bufX, bufY);
+			
 			if (isPopup())
 			{
 				gtk_widget_show_now(border);
 				gtk_widget_grab_focus(border);
 			}
-			else if (_no_take_focus)
-				gtk_widget_show(border);
 			else
-				gtk_window_present(GTK_WINDOW(border));
+			{
+				present();
+			}
 
+			if (!_title || !*_title)
+				gtk_window_set_title(GTK_WINDOW(border), gApplication::defaultTitle());
+			
 			if (isUtility())
 			{
 				gMainWindow *parent = _current;
@@ -712,7 +726,7 @@ void gMainWindow::setFullscreen(bool vl)
 	{
 		gtk_window_fullscreen(GTK_WINDOW(border));
 		if (isVisible())
-			gtk_window_present(GTK_WINDOW(border));
+			present();
 	}
 	else
 		gtk_window_unfullscreen(GTK_WINDOW(border));
@@ -832,9 +846,17 @@ void gMainWindow::showActivate()
 {
 	bool v = isTopLevel() && isVisible() && !_no_take_focus;
 
+	if (!_moved)
+		center();
 	show();
 	if (v)
-		gtk_window_present(GTK_WINDOW(border));
+		present();
+}
+
+void gMainWindow::activate()
+{
+	if (isTopLevel() && isVisible())
+		present();
 }
 
 void gMainWindow::showPopup()
@@ -847,7 +869,7 @@ void gMainWindow::showPopup()
 void gMainWindow::raise()
 {
 	if (!isTopLevel()) { gControl::raise(); return; }
-	gtk_window_present(GTK_WINDOW(border));
+	present();
 }
 
 const char* gMainWindow::text()
@@ -1446,6 +1468,7 @@ void gMainWindow::updateFont()
 {
 	gContainer::updateFont();
 	gMenu::updateFont(this);
+	emit(SIGNAL(onFontChange));
 }
 
 void gMainWindow::checkMenuBar()
@@ -1625,7 +1648,7 @@ void gMainWindow::setTransparent(bool vl)
 	bufW = w - 1;
 	resize(w, h);
 
-	gtk_window_present(GTK_WINDOW(border));
+	//gtk_window_present(GTK_WINDOW(border));
 }
 
 bool gMainWindow::closeAll()
