@@ -34,44 +34,27 @@
 
 #include "main.h"
 
-
-#ifdef __cplusplus
-extern "C" {
-#endif
-
-
-
 GB_INTERFACE GB EXPORT;
 
-
-
 GB_DESC *GB_CLASSES[] EXPORT =
 {
   CNetDesc,
   CProxyDesc,
-  CCurlDesc,
+  CurlSSLDesc,
+  CurlDesc,
   CHttpClientDesc,
   CFtpClientDesc,
   NULL
 };
 
-
-
 int EXPORT GB_INIT(void)
 {
-  curl_global_init(0);
+  curl_global_init(CURL_GLOBAL_ALL);
   return 0;
 }
 
-
-
 void EXPORT GB_EXIT()
 {
   curl_global_cleanup();
 }
 
-
-#ifdef _cpluscplus
-}
-#endif
-
