@@ -45,6 +45,9 @@
 #include "CSocket.h"
 #include "CServerSocket.h"
 #include "CDnsClient.h"
+
+//#define DEBUG_ME 1
+
 #define MAX_CLIENT_BUFFER_SIZE 65536
 #define UNIXPATHMAX 108
 
@@ -69,6 +72,25 @@ GB_STREAM_DESC SocketStream =
 	handle: CSocket_stream_handle
 };
 
+#if DEBUG_ME
+static void set_status(CSOCKET *_object, int status)
+{
+	static const char *status_name[] = { "Inactive", "Active", "Pending", "Accepting", "Receiving data", "Searching", "Connecting", "Connected" };
+	static const char *error_name[] = { NULL, NULL, "Cannot create socket", "Connection refused", "Cannot read", "Cannot write", "Host not found", NULL, NULL, NULL,
+		"Cannot bind socket", NULL, NULL, NULL, "Cannot listen", "Cannot bind interface", "Cannot authenticate" };
+	SOCKET->status = status;
+	
+	fprintf(stderr, "gb.net: socket %p: ", THIS);
+	if (status >= 0 && status < 7)
+		fprintf(stderr, "%s", status_name[status]);
+	else if (status >= -16 && status < 0)
+		fprintf(stderr, "%s", error_name[-status]);
+	
+	fprintf(stderr, " (%d)\n", status);
+}
+#else
+#define set_status(_object, _status) SOCKET->status = (_status)
+#endif
 
 bool SOCKET_update_timeout(CSOCKET_COMMON *socket)
 {
@@ -95,6 +117,13 @@ bool SOCKET_update_timeout(CSOCKET_COMMON *socket)
 	return FALSE;
 }
 
+void SOCKET_set_blocking(CSOCKET_COMMON *socket, bool block)
+{
+	int do_not_block = block ? 0 : 1;
+	
+	ioctl(socket->socket, FIONBIO, &do_not_block);
+	SOCKET_update_timeout(socket);
+}
 
 /**********************************
 Routines to call events
@@ -139,66 +168,77 @@ static void CSocket_close(CSOCKET *_object)
 		SOCKET->stream.desc = NULL;
 		close(SOCKET->socket);
 		SOCKET->socket = -1;
-		SOCKET->status = NET_INACTIVE;
+		set_status(THIS, NET_INACTIVE);
 	}
 	
 	if (THIS->OnClose)
 		THIS->OnClose(_object);
 }
 
-/**************************************************
-This function is called by DnsClient to inform
+
+/*
+	This function is called by DnsClient to inform
 	that it has finished its work
-	*************************************************/
+*/
+
 void CSocket_CallBackFromDns(void *_object)
 {
-	//int NoBlock=1;
 	int myval=0;
 
-	if ( SOCKET->status != NET_SEARCHING) return;
-	if ( !THIS->DnsTool->sHostIP)
+	if (SOCKET->status != NET_SEARCHING) 
+		return;
+	
+	if (!THIS->DnsTool->sHostIP)
 	{
-		/* error host not found */
+		// Host not found
 		CSocket_stream_internal_error(THIS, NET_HOST_NOT_FOUND, TRUE);
 		return;
 	}
 
 	GB.FreeString (&THIS->sRemoteHostIP);
 	THIS->sRemoteHostIP = GB.NewZeroString (THIS->DnsTool->sHostIP);
-	/* Let's turn socket to async mode */
-	//ioctl(SOCKET->socket,FIONBIO,&NoBlock);
-	/* Third, we connect the socket */
-	THIS->Server.sin_family=AF_INET;
-		THIS->Server.sin_port=htons(THIS->iPort);
-		THIS->Server.sin_addr.s_addr =inet_addr(THIS->DnsTool->sHostIP);
-		bzero(&(THIS->Server.sin_zero),8);
-	myval=connect(SOCKET->socket,(struct sockaddr*)&(THIS->Server), sizeof(struct sockaddr));
-	if (!myval || errno==EINPROGRESS) /* this is the good answer : connect in progress */
+
+	// We connect to the socket
+	
+	THIS->Server.sin_family = AF_INET;
+	THIS->Server.sin_port = htons(THIS->iPort);
+	THIS->Server.sin_addr.s_addr = inet_addr(THIS->DnsTool->sHostIP);
+	bzero(&(THIS->Server.sin_zero), 8);
+	
+	// Don't block, so that connect() returns immediately
+	
+	SOCKET_set_blocking(SOCKET, FALSE);
+	myval = connect(SOCKET->socket,(struct sockaddr*)&(THIS->Server), sizeof(struct sockaddr));
+	SOCKET_set_blocking(SOCKET, TRUE);
+	
+	if (!myval || errno == EINPROGRESS) // Rhis is the good answer : connect in progress
 	{
-		SOCKET->status = NET_CONNECTING;
-		GB.Watch (SOCKET->socket,GB_WATCH_WRITE,(void *)CSocket_CallBackConnecting,(intptr_t)THIS);
+		set_status(THIS, NET_CONNECTING);
+		GB.Watch(SOCKET->socket, GB_WATCH_WRITE, (void *)CSocket_CallBackConnecting, (intptr_t)THIS);
 	}
 	else
 	{
-		GB.Watch (SOCKET->socket , GB_WATCH_NONE , (void *)CSocket_CallBack,0);
-		SOCKET->stream.desc=NULL;
+		GB.Watch(SOCKET->socket , GB_WATCH_NONE, NULL, 0);
+		SOCKET->stream.desc = NULL;
 		close(SOCKET->socket);
-		SOCKET->status = NET_INACTIVE;
+		set_status(THIS, NET_INACTIVE);
 	}
+	
 	if (THIS->DnsTool)
-		{
+	{
 		dns_close_all(THIS->DnsTool);
 		GB.Unref(POINTER(&THIS->DnsTool));
-		THIS->DnsTool=NULL;
-		}
-	if ( SOCKET->status <= NET_INACTIVE )
+		THIS->DnsTool = NULL;
+	}
+	
+	if (SOCKET->status <= NET_INACTIVE)
 	{
 		CSocket_stream_internal_error(THIS, NET_CONNECTION_REFUSED, TRUE);
 		return;
 	}
 
 	GB.Ref(THIS);
-	GB.Post(CSocket_post_hostfound,(intptr_t)THIS);
+	GB.Post(CSocket_post_hostfound, (intptr_t)THIS);
 }
 
 
@@ -218,7 +258,7 @@ void CSocket_CallBackConnecting(int t_sock,int type,intptr_t param)
 	int mylen;
 	void *_object = (void *)param;
 
-	GB.Watch(SOCKET->socket, GB_WATCH_NONE, (void *)CSocket_CallBackConnecting, 0);
+	GB.Watch(SOCKET->socket, GB_WATCH_NONE, NULL, 0);
 	
 	if (SOCKET->status != NET_CONNECTING) return;
 	
@@ -227,7 +267,7 @@ void CSocket_CallBackConnecting(int t_sock,int type,intptr_t param)
 	an error trying to connect
 	****************************************************/
 	
-	SOCKET->status = CheckConnection(SOCKET->socket);
+	set_status(THIS, CheckConnection(SOCKET->socket));
 	if (SOCKET->status == NET_INACTIVE)
 	{
 		CSocket_stream_internal_error(THIS, NET_CONNECTION_REFUSED, TRUE);
@@ -308,7 +348,7 @@ void CSocket_stream_internal_error(void *_object, int ncode, bool post)
 	CSocket_close(THIS);
 
 	/* fatal socket error handling */
-	SOCKET->status = ncode;
+	set_status(THIS, ncode);
 	
 	if (post)
 	{
@@ -388,7 +428,6 @@ int CSocket_stream_read(GB_STREAM *stream, char *buffer, int len)
 {
 	void *_object = stream->tag;
 	int npos=-1;
-	//int NoBlock=0;
 	int bytes;
 
 	if (!THIS) return -1;
@@ -402,10 +441,7 @@ int CSocket_stream_read(GB_STREAM *stream, char *buffer, int len)
 	if (bytes < len)
 		len = bytes;
 
-	//ioctl(SOCKET->socket,FIONBIO,&NoBlock);
 	USE_MSG_NOSIGNAL(npos=recv(SOCKET->socket,(void*)buffer,len*sizeof(char),MSG_NOSIGNAL));
-	//NoBlock++;
-	//ioctl(SOCKET->socket,FIONBIO,&NoBlock);
 	
 	GB.Stream.SetBytesRead(stream, npos);
 	
@@ -421,11 +457,9 @@ int CSocket_stream_write(GB_STREAM *stream, char *buffer, int len)
 {
 	void *_object = stream->tag;
 	int npos=-1;
-	//int NoBlock=0;
 
 	if (!THIS) return -1;
 
-	//ioctl(SOCKET->socket,FIONBIO,&NoBlock);
 	while (len > 0)
 	{
 		USE_MSG_NOSIGNAL(npos=send(SOCKET->socket,(void*)buffer,len*sizeof(char),MSG_NOSIGNAL));
@@ -434,8 +468,6 @@ int CSocket_stream_write(GB_STREAM *stream, char *buffer, int len)
 		len -= npos;
 		buffer += npos;
 	}
-	//NoBlock++;
-	//ioctl(SOCKET->socket,FIONBIO,&NoBlock);
 
 	if (npos >= 0 || errno == EAGAIN) 
 	{
@@ -463,8 +495,8 @@ To start a UNIX connection
 **************************************************************************/
 int CSocket_connect_unix(void *_object,char *sPath, int lenpath)
 {
-	int doNotBlock = 0;
-
+	int ret;
+	
 	if ( SOCKET->status > NET_INACTIVE ) return 1;
 	if (!sPath) return 7;
 	if ( (lenpath<1) || (lenpath>UNIXPATHMAX) ) return 7;
@@ -476,7 +508,7 @@ int CSocket_connect_unix(void *_object,char *sPath, int lenpath)
 	strcpy(THIS->UServer.sun_path,sPath);
 	if ( (SOCKET->socket=socket(AF_UNIX,SOCK_STREAM,0))==-1 )
 	{
-		SOCKET->status = NET_CANNOT_CREATE_SOCKET;
+		set_status(THIS, NET_CANNOT_CREATE_SOCKET);
 		GB.Ref (THIS);
 		CSocket_post_error(_object); /* Unable to create socket */
 		return 2;
@@ -487,14 +519,16 @@ int CSocket_connect_unix(void *_object,char *sPath, int lenpath)
 	
 	THIS->conn_type = NET_TYPE_INTERNET;
 	
-	if (connect(SOCKET->socket,(struct sockaddr*)&THIS->UServer,sizeof(struct sockaddr_un))==0)
+	ret = connect(SOCKET->socket,(struct sockaddr*)&THIS->UServer,sizeof(struct sockaddr_un));
+	
+	// Set socket to blocking mode, after the connect() call!
+	SOCKET_set_blocking(SOCKET, TRUE);
+
+	if (ret == 0)
 	{
-		SOCKET->status = NET_CONNECTED;
-		//ioctl(SOCKET->socket,FIONBIO,&NoBlock);
+		set_status(THIS, NET_CONNECTED);
 		CSOCKET_init_connected(THIS);
-		/*GB.Watch (SOCKET->socket,GB_WATCH_READ,(void *)CSocket_CallBack,(intptr_t)THIS);
-		if (GB.CanRaise(THIS, EVENT_Write))
-			GB.Watch(SOCKET->socket, GB_WATCH_WRITE, (void *)CSocket_CallBackWrite, (intptr_t)THIS);*/
+
 		// $BM
 		if (THIS->Host) GB.FreeString(&THIS->Host);
 		if (THIS->Path) GB.FreeString(&THIS->Path);
@@ -505,15 +539,12 @@ int CSocket_connect_unix(void *_object,char *sPath, int lenpath)
 
 		return 0;
 	}
-	
-	// Set socket to blocking mode, after the connect() call!
-	ioctl(SOCKET->socket, FIONBIO, &doNotBlock);
 		
-	/* Error */
-	SOCKET->stream.desc=NULL;
+	// Error
+	SOCKET->stream.desc = NULL;
 	close(SOCKET->socket);
 	GB.FreeString(&THIS->sPath);
-	SOCKET->status = NET_CONNECTION_REFUSED;
+	set_status(THIS, NET_CONNECTION_REFUSED);
 
 	GB.Ref (THIS);
 	CSocket_post_error(_object); /* Unable to connect to remote host */
@@ -526,8 +557,6 @@ To start a TCP connection
 **************************************************************************/
 int CSocket_connect_socket(void *_object,char *sHost,int lenhost,int myport)
 {
-	int doNotBlock = 0;
-	
 	if ( SOCKET->status > NET_INACTIVE ) return 1;
 	if (!lenhost) return 9;
 	if (!sHost)   return 9;
@@ -538,14 +567,14 @@ int CSocket_connect_socket(void *_object,char *sHost,int lenhost,int myport)
 
 	if ( (SOCKET->socket=socket(AF_INET,SOCK_STREAM,0))==-1 )
 	{
-		SOCKET->status = NET_CANNOT_CREATE_SOCKET;
+		set_status(THIS, NET_CANNOT_CREATE_SOCKET);
 		GB.Ref (THIS);
 		CSocket_post_error(_object);
 		return 2;
 	}
 
-	// Set socket to non-blocking mode
-	ioctl(SOCKET->socket, FIONBIO, &doNotBlock);
+	// Set socket to blocking mode
+	SOCKET_set_blocking(SOCKET, TRUE);
 
 	THIS->iPort=myport;
 	THIS->conn_type = NET_TYPE_INTERNET;
@@ -571,7 +600,7 @@ int CSocket_connect_socket(void *_object,char *sHost,int lenhost,int myport)
 	it will call to CSocket_CallBack_fromDns,
 	and we'll continue there connection proccess
 	********************************************/
-	SOCKET->status = NET_SEARCHING; /* looking for IP */
+	set_status(THIS, NET_SEARCHING); /* looking for IP */
 	dns_thread_getip(THIS->DnsTool);
 	SOCKET->stream.desc=&SocketStream;
 	THIS->iUsePort=THIS->iPort;
@@ -599,7 +628,6 @@ MaxLen -> 0 no limit, >0 max. data t read
 int CSocket_peek_data(void *_object,char **buf,int MaxLen)
 {
 	int retval=0;
-	//int NoBlock=0;
 	int nread=0;
 	int bytes=0;
 
@@ -615,10 +643,7 @@ int CSocket_peek_data(void *_object,char **buf,int MaxLen)
 		if (MaxLen >0 ) bytes=MaxLen;
 		GB.Alloc((void**)buf,bytes);
 		(*buf)[0]='\0';
-		//ioctl(SOCKET->socket,FIONBIO,&NoBlock);
 		USE_MSG_NOSIGNAL(retval=recv(SOCKET->socket,(void*)(*buf),bytes*sizeof(char),MSG_PEEK|MSG_NOSIGNAL));
-		//NoBlock++;
-		//ioctl(SOCKET->socket,FIONBIO,&NoBlock);
 	}
 
 	if (retval==-1)
@@ -632,7 +657,7 @@ int CSocket_peek_data(void *_object,char **buf,int MaxLen)
 		GB.Watch (SOCKET->socket , GB_WATCH_NONE , (void *)CSocket_CallBack,0);
 		SOCKET->stream.desc=NULL;
 		close(SOCKET->socket);
-		SOCKET->status = NET_CANNOT_READ;
+		set_status(THIS, NET_CANNOT_READ);
 		GB.Ref (THIS);
 		CSocket_post_error(_object);
 		return -1;
