@@ -3,7 +3,7 @@
   main.h
 
   (c) 2004 Rob Kudla <pcre-component@kudla.org>
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -26,9 +26,12 @@
 #define __MAIN_H
 
 #include "gambas.h"
+#include "gb.pcre.h"
 
 #ifndef __MAIN_C
-extern GB_INTERFACE GB;
+extern GB_INTERFACE *GB_PTR;
 #endif
 
+#define GB (*GB_PTR)
+
 #endif /* __MAIN_H */
