@@ -3,7 +3,7 @@
   regexp.h
 
   (c) 2004 Rob Kudla <pcre-component@kudla.org>
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -44,11 +44,13 @@ typedef
 		char *subject;
 		char *pattern;
 		int *ovector;
+		int ovecsize;
 		int count;
 		int eopts;
 		int copts;
 		pcre *code;
 		int _submatch;
+		int error;
 	}
 	CREGEXP;
 
@@ -56,4 +58,6 @@ typedef
 
 #endif
 
+bool REGEXP_match(const char *subject, int lsubject, const char *pattern, int lpattern, int coptions, int eoptions);
+
 #endif
