@@ -56,7 +56,7 @@ void MyCheckBox::adjust(bool force)
 	a = _autoResize;
 	_autoResize = false;
 	hint = sizeHint();
-	CWIDGET_resize(THIS, hint.width(), QMAX(hint.height(), height()));
+	CWIDGET_resize(THIS, hint.width(), qMax(hint.height(), height()));
 	_autoResize = a;
 }
 
@@ -89,7 +89,7 @@ END_METHOD
 BEGIN_PROPERTY(CheckBox_Text)
 
   if (READ_PROPERTY)
-    GB.ReturnNewZeroString(TO_UTF8(WIDGET->text()));
+    RETURN_NEW_STRING(WIDGET->text());
   else
 	{
     WIDGET->setText(QSTRING_PROP());
