@@ -103,10 +103,11 @@ DECLARE_PROPERTY(Container_Spacing);
 DECLARE_PROPERTY(Container_Margin);
 DECLARE_PROPERTY(Container_Indent);
 DECLARE_PROPERTY(Container_Border);
+DECLARE_PROPERTY(Container_SimpleBorder);
 DECLARE_PROPERTY(Container_Invert);
 
 void CCONTAINER_arrange(void *_object);
-void CCONTAINER_get_max_size(void *_object, int *w, int *h);
+void CCONTAINER_get_max_size(void *_object, int xc, int yc, int wc, int hc, int *w, int *h);
 void CCONTAINER_insert_child(void *_object);
 void CCONTAINER_remove_child(void *_object);
 void CCONTAINER_decide(CWIDGET *control, bool *width, bool *height);
@@ -128,6 +129,7 @@ public:
 	void drawFrame(QPainter *);
 	int frameWidth();
 	void setPixmap(QPixmap *pixmap);
+	void setPaintBackgroundColor(bool bg) { _bg = bg; }
 
 protected:
 
@@ -138,6 +140,7 @@ private:
 
 	int _frame;
 	QPixmap *_pixmap;
+	bool _bg;
 };
 
 class MyContainer : public MyFrame
