@@ -23,8 +23,6 @@
 
 #define __CMENU_CPP
 
-#undef QT3_SUPPORT
-
 #include <QMenuBar>
 #include <QMenu>
 #include <QKeyEvent>
@@ -329,7 +327,7 @@ BEGIN_METHOD(Menu_new, GB_OBJECT parent; GB_BOOLEAN hidden)
 
 		if (!menu->menu)
 		{
-			menu->menu = new QMenu();
+			menu->menu = new MyMenu();
 			menu->menu->setSeparatorsCollapsible(true);
 			((QAction *)(menu->widget.widget))->setMenu(menu->menu);
 
@@ -440,7 +438,7 @@ BEGIN_PROPERTY(Menu_Text)
 		if (THIS->save_text)
 			GB.ReturnString(THIS->save_text);
 		else
-			GB.ReturnNewZeroString(TO_UTF8(ACTION->text()));
+			RETURN_NEW_STRING(ACTION->text());
 	}
 	else
 	{
@@ -692,6 +690,8 @@ void CMENU_popup(CMENU *_object, const QPoint &pos)
 			THIS->disabled = true;
 		}
 
+		//qDebug("CMENU_popup: %p: open", THIS);
+
 		// The Click event is posted, it does not occur immediately.
 		save = CWIDGET_enter_popup();
 		THIS->exec = true;
@@ -706,6 +706,8 @@ void CMENU_popup(CMENU *_object, const QPoint &pos)
 
 		//CWIDGET_check_hovered();
 
+		//qDebug("CMENU_popup: %p: close", THIS);
+		
 		if (_popup_menu_clicked)
 		{
 			CMENU *menu = _popup_menu_clicked;
@@ -713,6 +715,8 @@ void CMENU_popup(CMENU *_object, const QPoint &pos)
 			send_click_event(menu);
 		}
 
+		//qDebug("CMENU_popup: %p: end", THIS);
+		
 		MENU_popup_count++;
 
 		//MyMainWindow *toplevel = (MyMainWindow *)(THIS->toplevel);
@@ -1047,3 +1051,12 @@ void CMenu::slotDestroyed(void)
 	#endif
 }
 
+//---------------------------------------------------------------------------
+
+void MyMenu::setVisible(bool visible)
+{
+	if (!visible)
+		setAttribute(Qt::WA_NoMouseReplay);
+	QMenu::setVisible(visible);
+}
+
