@@ -23,8 +23,6 @@
 
 #define __CWINDOW_CPP
 
-#undef QT3_SUPPORT
-
 #include <qnamespace.h>
 #include <qapplication.h>
 #include <qmenubar.h>
@@ -51,9 +49,11 @@
 #include "main.h"
 
 #ifndef NO_X_WINDOW
+#ifndef QT5
 #include <QX11EmbedWidget>
 #include <QX11EmbedContainer>
 #endif
+#endif
 
 #include "gambas.h"
 
@@ -98,6 +98,7 @@ DECLARE_EVENT(EVENT_Show);
 DECLARE_EVENT(EVENT_Hide);
 DECLARE_EVENT(EVENT_Title);
 DECLARE_EVENT(EVENT_Icon);
+DECLARE_EVENT(EVENT_Font);
 
 DECLARE_METHOD(Window_Show);
 
@@ -107,27 +108,10 @@ CWINDOW *CWINDOW_Current = 0;
 CWINDOW *CWINDOW_LastActive = 0;
 CWINDOW *CWINDOW_Active = 0;
 
+#ifndef QT5
 int CWINDOW_Embedder = 0;
 bool CWINDOW_Embedded = false;
 static int CWINDOW_EmbedState = 0;
-
-#ifndef NO_X_WINDOW
-void CWINDOW_change_property(QWidget *w, Atom property, bool set)
-{
-	if (!w->isWindow())
-		return;
-
-	X11_window_change_property(w->winId(), w->isVisible(), property, set);
-}
-
-bool CWINDOW_has_property(QWidget *w, Atom property)
-{
-	if (!w->isWindow())
-		return false;
-
-	return X11_window_has_property(w->winId(), property);
-}
-
 #endif
 
 // Fix a QT little boring visual bug on menubars
@@ -140,7 +124,7 @@ void CWINDOW_fix_menubar(CWINDOW *window)
 		window->menuBar->setFocus();
 		QKeyEvent e(QEvent::KeyPress, Qt::Key_Escape, Qt::NoModifier);
 		qApp->sendEvent(window->menuBar, &e);
-		if (save) 
+		if (save)
 			save->setFocus();
 	}
 }
@@ -160,8 +144,8 @@ static void clear_mask(CWINDOW *_object)
 		//WINDOW->setResizable(WINDOW->isResizable(), true);
 		if (v && THIS->reallyMasked)
 		{
-			X11_window_remap(WINDOW->winId());
-			WINDOW->initProperties();
+			X11_window_remap(WINDOW->effectiveWinId());
+			WINDOW->initProperties(PROP_ALL);
 		}
 		#endif
 	}
@@ -180,7 +164,7 @@ void CWINDOW_define_mask(CWINDOW *_object)
 
 	if (THIS->picture)
 		background = *(THIS->picture->pixmap);
-	
+
 	if (background.isNull())
 	{
 		clear_mask(THIS);
@@ -193,7 +177,7 @@ void CWINDOW_define_mask(CWINDOW *_object)
 		if (THIS->masked && background.hasAlpha())
 		{
 			THIS->reallyMasked = true;
-			WINDOW->setMask(background.mask());
+			WINDOW->setBetterMask(background);
 		}
 		else
 		{
@@ -203,7 +187,7 @@ void CWINDOW_define_mask(CWINDOW *_object)
 
 		THIS->container->setPixmap(THIS->picture->pixmap);
 	}
-	
+
 	THIS->container->update();
 }
 
@@ -211,13 +195,13 @@ static bool emit_open_event(void *_object)
 {
 	//if (THIS->opening)
 	//	return true;
-	
+
 	if (THIS->opened)
 		return false;
-	
+
 	CWIDGET_clear_flag(THIS, WF_CLOSED);
 	THIS->opened = true;
-	
+
 	if (!THIS->minw && !THIS->minh)
 	{
 		THIS->minw = THIS->w;
@@ -238,7 +222,7 @@ static bool emit_open_event(void *_object)
 		THIS->opened = false;
 		return true;
 	}
-	
+
 	THIS->opened = true;
 	THIS->hidden = false;
 	return false;
@@ -276,16 +260,18 @@ static void reparent_window(CWINDOW *_object, void *parent, bool move, int x = 0
 {
 	QPoint p;
 	QWidget *newParentWidget;
-	
+	bool moved = THIS->moved;
+
 	if (move)
 	{
 		p.setX(x);
 		p.setY(y);
+		moved = true;
 	}
 	else if (THIS->toplevel)
   {
-	   p.setX(THIS->x);
-	   p.setY(THIS->y);
+		p.setX(THIS->x);
+		p.setY(THIS->y);
   }
   else
     p = WIDGET->pos();
@@ -306,12 +292,14 @@ static void reparent_window(CWINDOW *_object, void *parent, bool move, int x = 0
 	}
 	else
 		CWIDGET_move(THIS, p.x(), p.y());
+
+	THIS->moved = moved;
 }
 
 void CWINDOW_ensure_active_window()
 {
 	void *_object = CWINDOW_Active;
-	
+
 	if (THIS)
 		WINDOW->activateWindow();
 }
@@ -338,20 +326,23 @@ BEGIN_METHOD(Window_new, GB_OBJECT parent)
 
 	MyMainWindow *win = 0;
 	MyContainer *container;
-	#ifndef NO_X_WINDOW
+#ifndef NO_X_WINDOW
+#ifndef QT5
 	QX11EmbedWidget *client = 0;
-	#endif
+#endif
+#endif
 	const char *name = GB.GetClassName(THIS);
 
 	//THIS->widget.flag.fillBackground = true;
 
 	if (MISSING(parent) || !VARG(parent))
 	{
-		#ifndef NO_X_WINDOW
+#ifndef NO_X_WINDOW
+#ifndef QT5
 		if (CWINDOW_Embedder && !CWINDOW_Embedded)
 		{
 			client = new QX11EmbedWidget;
-			
+
 			win = new MyMainWindow(client, name, true);
 			container = new MyContainer(win);
 			container->raise();
@@ -360,7 +351,8 @@ BEGIN_METHOD(Window_new, GB_OBJECT parent)
 			THIS->xembed = true;
 		}
 		else
-		#endif
+#endif
+#endif
 		{
 			//win = new MyMainWindow(CWINDOW_Main ? (MyMainWindow *)QWIDGET(CWINDOW_Main) : 0, name);
 			win = new MyMainWindow(0, name);
@@ -410,7 +402,7 @@ BEGIN_METHOD(Window_new, GB_OBJECT parent)
 	if (THIS->toplevel || THIS->xembed)
 	{
 		CWindow::insertTopLevel(THIS);
-		
+
 		/*if (CWINDOW_Main == 0)
 		{
 			#if DEBUG_WINDOW
@@ -420,7 +412,8 @@ BEGIN_METHOD(Window_new, GB_OBJECT parent)
 		}*/
 	}
 
-	#ifndef NO_X_WINDOW
+#ifndef NO_X_WINDOW
+#ifndef QT5
 	if (THIS->xembed)
 	{
 		CWINDOW_Embedded = true;
@@ -452,7 +445,8 @@ BEGIN_METHOD(Window_new, GB_OBJECT parent)
 			GB.Error("Embedding has failed");
 		}
 	}
-	#endif
+#endif
+#endif
 
 	#if 1
 	if (THIS->embedded && !THIS->xembed)
@@ -469,7 +463,7 @@ BEGIN_METHOD(Window_new, GB_OBJECT parent)
 	//else
 	//	THIS->hidden = TRUE;
 	#endif
-	
+
 	THIS->showMenuBar = true;
 
 END_METHOD
@@ -479,7 +473,7 @@ BEGIN_METHOD_VOID(CFORM_new)
 
 	if (!GB.Parent(_object))
 		GB.Attach(_object, _object, "Form");
-	
+
 	CWIDGET_set_name((CWIDGET *)THIS, GB.GetClassName((void *)THIS));
 
 END_METHOD
@@ -488,10 +482,10 @@ END_METHOD
 BEGIN_METHOD_VOID(CFORM_main)
 
 	CWINDOW *form = (CWINDOW *)GB.AutoCreate(GB.GetClass(NULL), 0);
-	
+
 	if (!form->hidden)
 		Window_Show(form, NULL);
-		
+
 END_METHOD
 
 
@@ -560,7 +554,7 @@ END_METHOD
 static bool do_close(CWINDOW *_object, int ret, bool destroyed = false)
 {
 	bool closed;
-	
+
 	#if DEBUG_WINDOW
 	qDebug("do_close: (%s %p) %d %d", GB.GetClassName(THIS), THIS, THIS->closing, CWIDGET_test_flag(THIS, WF_CLOSED));
 	#endif
@@ -617,7 +611,7 @@ static bool do_close(CWINDOW *_object, int ret, bool destroyed = false)
 	}
 
 	#if 0
-	if (closed || destroyed) 
+	if (closed || destroyed)
 	{
 		if (CWINDOW_Active == THIS)
 			CWINDOW_activate(CWidget::get(WIDGET->parentWidget()));
@@ -669,14 +663,17 @@ BEGIN_METHOD_VOID(Window_Show)
 
 	if (emit_open_event(THIS))
 		return;
-	
+
 	if (!THIS->toplevel)
 	{
 		CWIDGET_set_visible((CWIDGET *)THIS, true);
-    #ifndef NO_X_WINDOW
-    if (THIS->xembed)
-    	XEMBED->show();
-    #endif
+		CWIDGET_check_visibility((CWIDGET *)THIS);
+#ifndef NO_X_WINDOW
+#ifndef QT5
+		if (THIS->xembed)
+			XEMBED->show();
+#endif
+#endif
 		post_show_event(THIS);
 	}
 	else
@@ -690,7 +687,7 @@ END_METHOD
 BEGIN_METHOD_VOID(Window_Hide)
 
 	THIS->hidden = true;
-	
+
 	if (THIS->toplevel && WINDOW->isModal())
 	{
 		do_close(THIS, 0);
@@ -705,7 +702,6 @@ END_METHOD
 BEGIN_METHOD_VOID(Window_ShowModal)
 
 	THIS->ret = 0;
-	THIS->mustCenter = true;
 
 	if (!emit_open_event(THIS))
 	{
@@ -725,12 +721,12 @@ END_METHOD
 BEGIN_METHOD(Window_ShowPopup, GB_INTEGER x; GB_INTEGER y)
 
 	QPoint pos;
-	
+
 	if (MISSING(x) || MISSING(y))
 		pos = QCursor::pos();
 	else
 		pos = QPoint(VARG(x), VARG(y));
-	
+
 	THIS->ret = 0;
 
 	if (THIS->toplevel)
@@ -806,7 +802,7 @@ END_PROPERTY
 BEGIN_PROPERTY(Window_Text)
 
 	if (READ_PROPERTY)
-		GB.ReturnNewZeroString(TO_UTF8(WIDGET->windowTitle()));
+		RETURN_NEW_STRING(WIDGET->windowTitle());
 	else
 	{
 		QString s = QSTRING_PROP();
@@ -859,7 +855,7 @@ BEGIN_PROPERTY(Window_Picture)
 	else
 	{
 		CPICTURE *new_pict = (CPICTURE *)VPROP(GB_OBJECT);
-		
+
 		if (new_pict != THIS->picture)
 		{
 			CPICTURE *old = THIS->picture;
@@ -888,7 +884,7 @@ BEGIN_PROPERTY(Window_Mask)
 		else
 		{
 			bool new_masked = VPROP(GB_BOOLEAN);
-			
+
 			if (new_masked != THIS->masked)
 			{
 				THIS->masked = new_masked;
@@ -943,77 +939,17 @@ BEGIN_PROPERTY(Window_FullScreen)
 END_PROPERTY
 
 
-#ifdef NO_X_WINDOW
-
-BEGIN_PROPERTY(Window_Stacking)
-
-	if (READ_PROPERTY)
-		GB.ReturnInteger(0);
-
-END_PROPERTY
-
-
-BEGIN_PROPERTY(Window_TopOnly)
-
-	if (READ_PROPERTY)
-		GB.ReturnBoolean(FALSE);
-		
-END_PROPERTY
-
-
-BEGIN_PROPERTY(Window_SkipTaskbar)
-
-	if (READ_PROPERTY)
-		GB.ReturnBoolean(FALSE);
-
-END_PROPERTY
-
-
-BEGIN_PROPERTY(Window_Sticky)
-
-	if (READ_PROPERTY)
-		GB.ReturnBoolean(FALSE);
-
-END_PROPERTY
-
-BEGIN_PROPERTY(Window_Utility)
-
-	if (READ_PROPERTY)
-		GB.ReturnBoolean(FALSE);
-
-END_PROPERTY
-
-#endif
-
-#ifndef NO_X_WINDOW
-
-static void manage_window_property(void *_object, void *_param, Atom property)
-{
-	if (!THIS->toplevel)
-	{
-		if (READ_PROPERTY)
-			GB.ReturnBoolean(FALSE);
-	}
-	else
-	{
-		if (READ_PROPERTY)
-			GB.ReturnBoolean(CWINDOW_has_property(WINDOW, property));
-		else
-			CWINDOW_change_property(WINDOW, property, VPROP(GB_BOOLEAN));
-	}
-}
-
 BEGIN_PROPERTY(Window_Stacking)
 
 	int p;
-	
+
 	if (!THIS->toplevel)
 	{
 		if (READ_PROPERTY)
 			GB.ReturnInteger(0);
 		return;
 	}
-	
+
 	if (READ_PROPERTY)
 	{
 		GB.ReturnInteger(THIS->stacking);
@@ -1024,7 +960,7 @@ BEGIN_PROPERTY(Window_Stacking)
 		if (p >= 0 && p <= 2)
 		{
 			THIS->stacking = p;
-			WINDOW->initProperties();
+			WINDOW->initProperties(PROP_STACKING);
 		}
 	}
 
@@ -1039,7 +975,7 @@ BEGIN_PROPERTY(Window_TopOnly)
 			GB.ReturnBoolean(FALSE);
 		return;
 	}
-	
+
 	if (READ_PROPERTY)
 	{
 		GB.ReturnBoolean(THIS->stacking == 1);
@@ -1047,18 +983,30 @@ BEGIN_PROPERTY(Window_TopOnly)
 	else
 	{
 		THIS->stacking = VPROP(GB_BOOLEAN) ? 1 : 0;
-		WINDOW->initProperties();
+		WINDOW->initProperties(PROP_STACKING);
 	}
-	
+
 END_PROPERTY
 
 
 BEGIN_PROPERTY(Window_SkipTaskbar)
 
-	manage_window_property(_object, _param, X11_atom_net_wm_state_skip_taskbar);
+	if (!THIS->toplevel)
+	{
+		if (READ_PROPERTY)
+			GB.ReturnBoolean(FALSE);
+		return;
+	}
 
-	if (!READ_PROPERTY)
+	if (READ_PROPERTY)
+	{
+		GB.ReturnBoolean(THIS->skipTaskbar);
+	}
+	else
+	{
 		THIS->skipTaskbar = VPROP(GB_BOOLEAN);
+		WINDOW->initProperties(PROP_SKIP_TASKBAR);
+	}
 
 END_PROPERTY
 
@@ -1071,31 +1019,17 @@ BEGIN_PROPERTY(Window_Sticky)
 			GB.ReturnBoolean(FALSE);
 		return;
 	}
-	
+
 	if (READ_PROPERTY)
 		GB.ReturnBoolean(THIS->sticky);
 	else
 	{
 		THIS->sticky = VPROP(GB_BOOLEAN);
-		X11_window_set_desktop(WINDOW->winId(), WINDOW->isVisible(), THIS->sticky ? 0xFFFFFFFF : X11_get_current_desktop());
+		WINDOW->initProperties(PROP_STICKY);
 	}
 
 END_PROPERTY
 
-#if 0
-BEGIN_PROPERTY(CWINDOW_type)
-
-	if (READ_PROPERTY)
-		GB.ReturnInteger(WINDOW->getType());
-	else
-	{
-		qDebug("gb.qt4: warning: Window.Type is deprecated");
-		WINDOW->setType(VPROP(GB_INTEGER));
-	}
-
-END_PROPERTY
-#endif
-
 BEGIN_PROPERTY(Window_Utility)
 
 	if (READ_PROPERTY)
@@ -1108,15 +1042,13 @@ BEGIN_PROPERTY(Window_Utility)
 
 END_PROPERTY
 
-#endif //------------------------------------------------------------------------------------------
-
 
 BEGIN_METHOD_VOID(Window_Center)
 
 	if (!THIS->toplevel)
 		return;
 
-	WINDOW->center(true);
+	WINDOW->center();
 
 END_METHOD
 
@@ -1141,7 +1073,9 @@ BEGIN_PROPERTY(Window_Visible)
 		GB.ReturnBoolean(!WINDOW->isHidden());
 	else
 	{
-		if (VPROP(GB_BOOLEAN))
+		bool show = !!VPROP(GB_BOOLEAN);
+
+		if (show)
 			Window_Show(_object, _param);
 		else
 			Window_Hide(_object, _param);
@@ -1209,7 +1143,7 @@ END_METHOD
 BEGIN_METHOD(Window_Controls_get, GB_STRING name)
 
 	CWIDGET *control = WINDOW->names[GB.ToZeroString(ARG(name))];
-	
+
 	if (!control || CWIDGET_check(control))
 		GB.ReturnNull();
 	else
@@ -1368,6 +1302,13 @@ BEGIN_PROPERTY(Window_TakeFocus)
 
 END_PROPERTY
 
+BEGIN_METHOD_VOID(Window_Activate)
+
+	if (THIS->toplevel && WINDOW->isVisible() && !WINDOW->isHidden())
+		WINDOW->activateWindow();
+
+END_METHOD
+
 
 /***************************************************************************/
 
@@ -1381,7 +1322,7 @@ GB_DESC CWindowMenusDesc[] =
   GB_METHOD("Show", NULL, CWINDOW_menu_show, NULL),
   GB_METHOD("Hide", NULL, CWINDOW_menu_hide, NULL),
   GB_PROPERTY("Visible", "b", CWINDOW_menu_visible),
-  
+
 	GB_END_DECLARE
 };
 
@@ -1440,6 +1381,8 @@ GB_DESC CWindowDesc[] =
 	GB_METHOD("ShowDialog", "i", Window_ShowModal, NULL),
 	GB_METHOD("ShowPopup", "i", Window_ShowPopup, "[(X)i(Y)i]"),
 	GB_METHOD("Center", NULL, Window_Center, NULL),
+	GB_METHOD("Activate", NULL, Window_Activate, NULL),
+
 	GB_PROPERTY_READ("Modal", "b", Window_Modal),
 	GB_PROPERTY_READ("TopLevel", "b", Window_TopLevel),
 	GB_PROPERTY_READ("Closed", "b", Window_Closed),
@@ -1474,7 +1417,7 @@ GB_DESC CWindowDesc[] =
 	GB_PROPERTY("AutoResize", "b", Container_AutoResize),
   GB_PROPERTY("Invert", "b", Container_Invert),
   GB_PROPERTY("Indent", "b", Container_Indent),
-	
+
 	//GB_PROPERTY("Type", "i", CWINDOW_type),
 	GB_PROPERTY("Utility", "b", Window_Utility),
 	GB_PROPERTY("Border", "b", Window_Border),
@@ -1497,7 +1440,8 @@ GB_DESC CWindowDesc[] =
 	GB_EVENT("Hide", NULL, NULL, &EVENT_Hide),
 	GB_EVENT("Title", NULL, NULL, &EVENT_Title),
 	GB_EVENT("Icon", NULL, NULL, &EVENT_Icon),
-	
+	GB_EVENT("Font", NULL, NULL, &EVENT_Font),
+
 	//GB_INTERFACE("Draw", &DRAW_Interface),
 
 	GB_END_DECLARE
@@ -1524,7 +1468,7 @@ GB_DESC CFormDesc[] =
 	GB_STATIC_METHOD("Main", NULL, CFORM_main, NULL),
 	GB_STATIC_METHOD("Load", NULL, CFORM_load, "[(Parent)Control;]"),
 	GB_METHOD("_new", NULL, CFORM_new, NULL),
-	
+
 	FORM_DESCRIPTION,
 
 	GB_END_DECLARE
@@ -1548,7 +1492,8 @@ MyMainWindow::MyMainWindow(QWidget *parent, const char *name, bool embedded) :
 	_enterLoop = false;
 	_utility = false;
 	_state = windowState();
-	
+	_screen = -1;
+
 	//setAttribute(Qt::WA_KeyCompression, true);
 	//setAttribute(Qt::WA_InputMethodEnabled, true);
 	setAttribute(Qt::WA_QuitOnClose, false);
@@ -1567,20 +1512,18 @@ MyMainWindow::~MyMainWindow()
 	#if DEBUG_WINDOW
 	qDebug("~MyMainWindow: %s %s %p", GB.GetClassName(THIS), THIS->widget.name, THIS);
 	#endif
-	
+
 	do_close(THIS, 0, true);
 
 	if (CWINDOW_Active == THIS)
 		CWINDOW_Active = 0;
+
 	if (CWINDOW_LastActive == THIS)
-	{
 		CWINDOW_LastActive = 0;
-		//qDebug("CWINDOW_LastActive = 0");
-	}
 
 	if (sg)
 		delete sg;
-		
+
 	GB.Detach(THIS);
 
 	if (THIS->menuBar)
@@ -1593,7 +1536,7 @@ MyMainWindow::~MyMainWindow()
 	}
 
 	CWindow::removeTopLevel(THIS);
-	
+
 	_deleted = true;
 
 	//qDebug("~MyMainWindow %p (end)", this);
@@ -1610,7 +1553,7 @@ bool MyMainWindow::event(QEvent *e)
 			//CWINDOW_activate((CWIDGET *)THIS);
 			GB.Ref(THIS);
 			GB.Post((void (*)())activate_later, (intptr_t)THIS);
-			
+
 		}
 		else*/ if (e->type() == QEvent::WindowDeactivate)
 		{
@@ -1619,7 +1562,7 @@ bool MyMainWindow::event(QEvent *e)
 				CWINDOW_activate(NULL);
 		}
 	}
-	
+
 	return QWidget::event(e);
 }
 #endif
@@ -1627,11 +1570,11 @@ bool MyMainWindow::event(QEvent *e)
 void MyMainWindow::showEvent(QShowEvent *e)
 {
 	CWINDOW *_object = (CWINDOW *)CWidget::get(this);
-	
+
 	//qDebug("showEvent: %s\n", GB.GetClassName(THIS));
-	
+
 	emit_open_event(THIS);
-	
+
 	//CWINDOW_fix_menubar(THIS);
 
 	if (_activate)
@@ -1640,7 +1583,7 @@ void MyMainWindow::showEvent(QShowEvent *e)
 		raise();
 		//setFocus();
 		activateWindow();
-		//X11_window_activate(winId());
+		//X11_window_activate(effectiveWinId());
 		_activate = false;
 	}
 
@@ -1648,29 +1591,53 @@ void MyMainWindow::showEvent(QShowEvent *e)
 }
 
 
-void MyMainWindow::initProperties()
+void MyMainWindow::initProperties(int which)
 {
 	#ifndef NO_X_WINDOW
 	CWIDGET *_object = CWidget::get(this);
 
-	if (!THIS->toplevel)
+	if (!THIS->toplevel || effectiveWinId() == 0)
 		return;
 
-	CWINDOW_change_property(this, X11_atom_net_wm_state_above, THIS->stacking == 1);
-	CWINDOW_change_property(this, X11_atom_net_wm_state_stays_on_top, THIS->stacking == 1);
-	CWINDOW_change_property(this, X11_atom_net_wm_state_below, THIS->stacking == 2);
-	CWINDOW_change_property(this, X11_atom_net_wm_state_skip_taskbar, THIS->skipTaskbar);
-	X11_set_window_type(winId(), _type);
+	if (!THIS->title && _border)
+		setWindowTitle(TO_QSTRING(GB.Application.Title()));
+
+	//qDebug("initProperties: %d", which);
+	X11_flush();
+
+	if (which & (PROP_STACKING | PROP_SKIP_TASKBAR))
+	{
+		X11_window_change_begin(effectiveWinId(), isVisible());
+
+		if (which & PROP_STACKING)
+		{
+			X11_window_change_property(X11_atom_net_wm_state_above, THIS->stacking == 1);
+			X11_window_change_property(X11_atom_net_wm_state_stays_on_top, THIS->stacking == 1);
+			X11_window_change_property(X11_atom_net_wm_state_below, THIS->stacking == 2);
+		}
+		if (which & PROP_SKIP_TASKBAR)
+			X11_window_change_property(X11_atom_net_wm_state_skip_taskbar, THIS->skipTaskbar);
+
+		X11_window_change_end();
+	}
+
+	//if (which == PROP_ALL)
+	//	X11_set_window_type(effectiveWinId(), _type);
+
+	if (which & PROP_BORDER)
+		X11_set_window_decorated(effectiveWinId(), _border);
+
+	if (which & PROP_STICKY)
+		X11_window_set_desktop(effectiveWinId(), isVisible(), THIS->sticky ? 0xFFFFFFFF : X11_get_current_desktop());
+
+	X11_flush();
 	#endif
 }
 
-void MyMainWindow::afterShow()
+void MyMainWindow::setEventLoop()
 {
 	if (!CWIDGET_test_flag(THIS, WF_CLOSED))
-	{
-		//define_mask(THIS, THIS->picture, THIS->masked);	
 		THIS->loopLevel = CWINDOW_Current ? CWINDOW_Current->loopLevel : 0;
-	}
 }
 
 void MyMainWindow::activateLater()
@@ -1678,17 +1645,34 @@ void MyMainWindow::activateLater()
 	activateWindow();
 }
 
-void MyMainWindow::present()
+void MyMainWindow::present(QWidget *parent)
 {
+	//CWIDGET *_object = CWidget::get(this);
+	//CWIDGET *_parent = parent ? CWidget::get(parent) : 0;
+	//qDebug("present: %p %s: parent = %p %s", THIS, _object->name, _parent, _parent ? _parent->name : "");
+
+	if (parent)
+		_screen = QApplication::desktop()->screenNumber(parent);
+	else
+		_screen = -1;
+
 	if (!isVisible())
 	{
-		//X11_window_startup(WINDOW->winId(), THIS->x, THIS->y, THIS->w, THIS->h);
+		//X11_window_startup(WINDOW->effectiveWinId(), THIS->x, THIS->y, THIS->w, THIS->h);
 
 		if (isUtility() && _resizable)
 			setMinimumSize(THIS->minw, THIS->minh);
-		
+
 		setAttribute(Qt::WA_ShowWithoutActivating, THIS->noTakeFocus);
 
+#ifndef QT5
+		if (effectiveWinId() == 0)
+		{
+			createWinId();
+		}
+		initProperties(PROP_ALL);
+#endif
+
 		if (getState() & Qt::WindowMinimized)
 			showMinimized();
 		else if (getState() & Qt::WindowFullScreen)
@@ -1697,37 +1681,41 @@ void MyMainWindow::present()
 			showMaximized();
 		else
 			show();
-		
-		if (isUtility() && _resizable)
-    	setSizeGrip(true);
+
+#ifdef QT5
+		//qDebug("createWinId: %p", (void *)effectiveWinId());
+		if (THIS->noTakeFocus)
+			X11_window_set_user_time(effectiveWinId(), 0);
+		initProperties(PROP_ALL);
+		if (THIS->noTakeFocus)
+			X11_window_set_user_time(effectiveWinId(), 0);
+#else
+		initProperties(PROP_SKIP_TASKBAR);
+#endif
+
+		/*if (isUtility() && _resizable)
+			setSizeGrip(true);
 		else
-			setSizeGrip(false);
-		
-		if (hasBorder() && !THIS->noTakeFocus)
-		{
-			//MAIN_process_events();
-			//usleep(50000);
-			//_activate = TRUE;
-			//if (isToolbar())
-			//QTimer::singleShot(50, this, SLOT(activateLater()));
-			//else
-			activateWindow();
-		}
+			setSizeGrip(false);*/
 	}
 	else
 	{
 		//_activate = true;
-		
+
 		if (getState() & Qt::WindowMinimized)
 		{
 			setState(windowState() & ~Qt::WindowMinimized);
 			//qDebug("_activate set #2");
 		}
-		
-		raise();
-		if (hasBorder())
-			activateWindow();
 	}
+
+	if (!THIS->noTakeFocus) // && (parent || hasBorder()))
+		activateWindow();
+
+	if (parent)
+		X11_set_transient_for(effectiveWinId(), parent->effectiveWinId());
+
+	raise();
 }
 
 void MyMainWindow::showActivate(QWidget *transient)
@@ -1737,11 +1725,11 @@ void MyMainWindow::showActivate(QWidget *transient)
 	//qDebug("showActivate: %s %d", THIS->widget.name, isToolbar());
 
 	// Reparent the window if, for example, there is an already modal window displayed
-	
+
 	if (CWINDOW_Current && THIS != CWINDOW_Current)
 	{
 		newParentWidget = CWINDOW_Current->widget.widget;
-		
+
 		if (!isVisible())
 		{
 			if (newParentWidget && parentWidget() != newParentWidget)
@@ -1755,26 +1743,14 @@ void MyMainWindow::showActivate(QWidget *transient)
 
 	//CWIDGET_clear_flag(THIS, WF_CLOSED);
 
-	if (!THIS->title && _border)
-		setWindowTitle(TO_QSTRING(GB.Application.Title()));
-
-	initProperties();
-	//::sleep(1);
-
-	present();
-
-	afterShow();
-
-	#ifndef NO_X_WINDOW
 	if (isUtility())
 	{
 		if (!newParentWidget && CWINDOW_Main && THIS != CWINDOW_Main)
 			newParentWidget = CWidget::getTopLevel((CWIDGET *)CWINDOW_Main)->widget.widget;
-		
-		if (newParentWidget)
-			X11_set_transient_for(winId(), newParentWidget->winId());
 	}
-	#endif
+
+	present(newParentWidget);
+	setEventLoop();
 }
 
 void on_error_show_modal(MODAL_INFO *info)
@@ -1782,7 +1758,7 @@ void on_error_show_modal(MODAL_INFO *info)
 	#ifdef DEBUG_WINDOW
 	qDebug("on_error_show_modal");
 	#endif
-	
+
 	// info->that can be NULL if the dialog is destroyed during the event loop
 
 	if (info->that)
@@ -1794,7 +1770,7 @@ void on_error_show_modal(MODAL_INFO *info)
 
 	MyApplication::eventLoop = info->old;
 	CWINDOW_Current = info->save;
-	
+
 	if (info->that && info->that->isPersistent())
 	{
 		info->that->setSizeGrip(false);
@@ -1806,6 +1782,7 @@ void MyMainWindow::showModal(void)
 {
 	//Qt::WindowFlags flags = windowFlags() & ~Qt::WindowType_Mask;
 	CWIDGET *_object = CWidget::get(this);
+	CWINDOW *parent;
 	bool persistent = CWIDGET_test_flag(THIS, WF_PERSISTENT);
 	//QPoint p = pos();
 	QEventLoop eventLoop;
@@ -1815,10 +1792,12 @@ void MyMainWindow::showModal(void)
 	if (isModal())
 		return;
 
+	CWIDGET_finish_focus();
+
 	info.that = this;
 	info.old = MyApplication::eventLoop;
 	info.save = CWINDOW_Current;
-	
+
 	MyApplication::eventLoop = &eventLoop;
 
 	setWindowModality(Qt::ApplicationModal);
@@ -1830,46 +1809,42 @@ void MyMainWindow::showModal(void)
 	}
 
 	_enterLoop = false; // Do not call exitLoop() if we do not entered the loop yet!
-	
-	present();
-	afterShow();
-	
-	#ifndef NO_X_WINDOW
-	if (CWINDOW_Active)
-	{
-		//qDebug("Active = %p X11_set_transient_for(0x%08x, 0x%08x)", CWINDOW_Active, winId(), CWINDOW_Active->widget.widget->winId());
-		X11_set_transient_for(winId(), CWidget::getTopLevel((CWIDGET *)CWINDOW_Active)->widget.widget->winId());
-	}
-	#endif
+
+	parent = CWINDOW_Current;
+	if (!parent)
+		parent = CWINDOW_Main;
+
+	present(parent ? CWidget::getTopLevel((CWIDGET *)parent)->widget.widget : 0);
+	setEventLoop();
 
 	THIS->loopLevel++;
 	CWINDOW_Current = THIS;
-	
+
 	_enterLoop = true;
-	
+
 	GB.Debug.EnterEventLoop();
 
 	handler.handler = (GB_CALLBACK)on_error_show_modal;
 	handler.arg1 = (intptr_t)&info;
-	
+
 	GB.OnErrorBegin(&handler);
-	
+
 	eventLoop.exec();
-	
+
 	GB.OnErrorEnd(&handler);
-	
+
 	GB.Debug.LeaveEventLoop();
 	//eventLoop.processEvents(QEventLoop::ExcludeUserInputEvents | QEventLoop::DeferredDeletion, 0);
-	
+
 	MyApplication::eventLoop = info.old;
 	CWINDOW_Current = info.save;
-		
+
 	if (persistent)
 	{
 		setSizeGrip(false);
 		setWindowModality(Qt::NonModal);
 	}
-	
+
 	CWINDOW_ensure_active_window();
 }
 
@@ -1886,6 +1861,7 @@ void MyMainWindow::showPopup(QPoint &pos)
 
 	setWindowFlags(Qt::Popup | flags);
 	setWindowModality(Qt::ApplicationModal);
+	THIS->popup = true;
 
 	/*if (_resizable && _border)
 	{
@@ -1900,22 +1876,22 @@ void MyMainWindow::showPopup(QPoint &pos)
 	setFocus();
 	show();
 	raise();
-	afterShow();
+	setEventLoop();
 	//QTimer::singleShot(50, this, SLOT(activateLater()));
-	
+
 	THIS->loopLevel++;
 	CWINDOW_Current = THIS;
 
 	//handle_focus(THIS);
 	//activateWindow();
-	
+
 	save_popup = CWIDGET_enter_popup();
-	
+
 	_enterLoop = true;
-	
+
 	QEventLoop eventLoop;
 	QEventLoop *old;
-	
+
 	old = MyApplication::eventLoop;
 	MyApplication::eventLoop = &eventLoop;
 	GB.Debug.EnterEventLoop();
@@ -1924,17 +1900,18 @@ void MyMainWindow::showPopup(QPoint &pos)
 	MyApplication::eventLoop = old;
 	//eventLoop.exec();
 	//eventLoop.processEvents(QEventLoop::ExcludeUserInputEvents | QEventLoop::DeferredDeletion, 0);
-	
+
 	CWINDOW_Current = save;
-		
+
 	if (persistent)
 	{
 		setWindowModality(Qt::NonModal);
 		setWindowFlags(Qt::Window | flags);
+		THIS->popup = false;
 	}
-	
+
 	CWIDGET_leave_popup(save_popup);
-	
+
 	//CWIDGET_check_hovered();
 }
 
@@ -1995,18 +1972,27 @@ void MyMainWindow::setBorder(bool b)
 {
 	if (_border == b)
 		return;
-		
+
 	_border = b;
 	if (!isWindow())
 		return;
+
+	if (effectiveWinId())
+	{
+		//qDebug("effectiveWinId");
+		initProperties(PROP_BORDER);
+		X11_window_remap(effectiveWinId());
+	}
+#ifndef QT5
 	doReparent(parentWidget(), pos());
+#endif
 }
 
 void MyMainWindow::setResizable(bool b)
 {
 	if (_resizable == b)
 		return;
-		
+
 	_resizable = b;
 	if (!isWindow())
 		return;
@@ -2016,12 +2002,12 @@ void MyMainWindow::setResizable(bool b)
 void MyMainWindow::setUtility(bool b)
 {
 	Qt::WindowFlags flags;
-	
+
 	if (_utility == b)
 		return;
-		
+
 	_utility = b;
-	
+
 	doReparent(parentWidget(), pos());
 }
 
@@ -2033,14 +2019,14 @@ int MyMainWindow::getType()
 		return 0;
 	else
 		return _type;
-	//return X11_get_window_type(winId());
+	//return X11_get_window_type(effectiveWinId());
 }
 
 void MyMainWindow::setType(int type)
 {
 	if (!isWindow())
 		return;
-	X11_set_window_type(winId(), type);
+	X11_set_window_type(effectiveWinId(), type);
 	_type = type;
 }
 #endif
@@ -2116,11 +2102,13 @@ void MyMainWindow::resizeEvent(QResizeEvent *e)
 			CCONTAINER_arrange(THIS);
 	}
 
-  #ifndef NO_X_WINDOW
-  if (THIS->xembed)
-  	XEMBED->resize(THIS->w, THIS->h);
-  #endif
-  	
+#ifndef NO_X_WINDOW
+#ifndef QT5
+	if (THIS->xembed)
+		XEMBED->resize(THIS->w, THIS->h);
+#endif
+#endif
+
   	//qDebug("resizeEvent %ld %ld isHidden:%s shown:%s ", THIS->w, THIS->h, isHidden() ? "1" : "0", shown ? "1" : "0");
 	//qDebug("THIS->h = %ld  THIS->container->height() = %ld  height() = %ld", THIS->h, THIS->container->height(), height());
 
@@ -2134,28 +2122,11 @@ void MyMainWindow::keyPressEvent(QKeyEvent *e)
 	CWINDOW *_object = (CWINDOW *)CWidget::getReal(this);
 	QPushButton *test = 0;
 	CWIDGET *ob;
-	
+
 	e->ignore();
 
 	//qDebug("MyMainWindow::keyPressEvent: (%p '%s' %s)", this, this ? this->caption().latin1() : 0, GB.GetClassName(CWidget::get(this)));
-	
-	/*if (GB.CanRaise(THIS, EVENT_KeyPress))
-	{
-		CKEY_clear(true);
 
-		GB.FreeString(&CKEY_info.text);
-		CKEY_info.text = GB.NewZeroString(TO_UTF8(e->text()));
-		CKEY_info.state = e->modifiers();
-		CKEY_info.code = e->key();
-
-		cancel = GB.Raise(THIS, EVENT_KeyPress, 0);
-
-		CKEY_clear(false);
-
-		if (cancel)
-			return;
-	}*/
-	
 	if ((e->modifiers() == Qt::NoModifier || (e->modifiers() & Qt::KeypadModifier && e->key() == Qt::Key_Enter )))
 	{
 		switch (e->key())
@@ -2260,14 +2231,14 @@ bool CWINDOW_must_quit()
 {
 	CWINDOW *win;
 	int i;
-	
+
 	for (i = 0; i < CWindow::list.count(); i++)
 	{
 		win = CWindow::list.at(i);
 		if (win->opened)
 			return false;
 	}
-	
+
 	return true;
 }
 
@@ -2283,7 +2254,7 @@ void MyMainWindow::closeEvent(QCloseEvent *e)
 	#if DEBUG_WINDOW
 		qDebug("closeEvent: CWINDOW_Current = %p / %d <-> %p / %d", CWINDOW_Current, CWINDOW_Current ? CWINDOW_Current->loopLevel : -1, THIS, THIS->loopLevel);
 	#endif
-	
+
 	if (THIS->opened)
 	{
 		// If a window is not opened, then it can be closed whatever the loop level is
@@ -2355,7 +2326,7 @@ void MyMainWindow::closeEvent(QCloseEvent *e)
 	#endif
 	THIS->opened = false;
 	MAIN_check_quit();
-	
+
 	return;
 
 IGNORE:
@@ -2395,23 +2366,18 @@ void MyMainWindow::doReparent(QWidget *parent, const QPoint &pos)
 	THIS->embedded = !THIS->toplevel;
 
 	f &= ~Qt::WindowType_Mask;
-	
+
 	if (THIS->toplevel)
 	{
 		if (_utility)
 			f |= Qt::Tool;
 		else
 			f |= Qt::Window;
-		
-		if (_border)
-			f &= ~(Qt::FramelessWindowHint);
-		else
-			f |= Qt::FramelessWindowHint;
-		
+
 		if (!old_toplevel)
 			CWindow::insertTopLevel(THIS);
 	}
-	else	
+	else
 	{
 		if (old_toplevel)
 		{
@@ -2421,32 +2387,34 @@ void MyMainWindow::doReparent(QWidget *parent, const QPoint &pos)
 		}
 	}
 
-	//qDebug("doReparent: %s %p: visible = %d opened = %d hidden = %d isVisible = %d isHidden = %d shown = %d", 
+	//qDebug("doReparent: %s %p: visible = %d opened = %d hidden = %d isVisible = %d isHidden = %d shown = %d",
 	//				THIS->widget.name, THIS, THIS->widget.flag.visible, THIS->opened, THIS->hidden, isVisible(), isHidden(), THIS->widget.flag.shown);
+
 	//if (!THIS->hidden) showIt = true;
 	//hide();
+
 	hidden = THIS->hidden || !WIDGET->isVisible();
-	//qDebug("doReparent: %s %p: hidden = %d", THIS->widget.name, THIS, hidden);
 	if (parent != parentWidget() || f != windowFlags())
 	{
 		reparented = true;
 		setParent(parent, f);
+		//qDebug("setParent %d", f != windowFlags());
 	}
-	
+
 	move(pos);
 	//qDebug("doReparent: (%s %p) (%d %d) -> (%d %d)", GB.GetClassName(THIS), THIS, pos.x(), pos.y(), WIDGET->x(), WIDGET->y());
-	
+
 	if (!THIS->embedded)
 	{
 		#ifndef NO_X_WINDOW
-			initProperties();
+			initProperties(PROP_ALL);
 			if (active && hasBorder())
 				activateWindow();
 		#endif
 
 		setWindowIcon(icon);
 	}
-	
+
 	if (!_resizable && _border && isWindow())
 	{
 		setMinimumSize(width(), height());
@@ -2457,26 +2425,12 @@ void MyMainWindow::doReparent(QWidget *parent, const QPoint &pos)
 		setMinimumSize(0, 0);
 		setMaximumSize(QWIDGETSIZE_MAX, QWIDGETSIZE_MAX);
 	}
-		
-	//qDebug("--> isVisible = %d isHidden = %d", isVisible(), isHidden());
-	
-	/*if (THIS->embedded && !THIS->hidden)
-	{
-		qDebug("doReparent: %s %p: show_later", THIS->widget.name, THIS);
-		#if DEBUG_WINDOW
-		qDebug("post show_later %s %p", GB.GetClassName(THIS), THIS);
-		#endif
-		GB.Ref(THIS);
-		//GB.Post((void (*)())show_later, (intptr_t)THIS);
-		show_later(THIS);
-		//WIDGET->show();
-	}*/
-	
+
 	/*if (parentWidget())
 		qDebug("doReparent (%s %p): new parent = (%s %p)", THIS->widget.name, THIS, CWidget::get(parentWidget())->name, CWidget::get(parentWidget()));
 	else
 		qDebug("doReparent (%s %p): new parent = 0", THIS->widget.name, THIS);*/
-	
+
 	if (reparented)
 	{
 		if (!hidden)
@@ -2484,19 +2438,26 @@ void MyMainWindow::doReparent(QWidget *parent, const QPoint &pos)
 	}
 }
 
+int MyMainWindow::currentScreen() const
+{
+	if (_screen >= 0)
+		return _screen;
 
-void MyMainWindow::center(bool force = false)
+	if (CWINDOW_Active)
+		return QApplication::desktop()->screenNumber(CWINDOW_Active->widget.widget);
+	else if (CWINDOW_Main)
+		return QApplication::desktop()->screenNumber(CWINDOW_Main->widget.widget);
+	else
+		return QApplication::desktop()->primaryScreen();
+}
+
+void MyMainWindow::center()
 {
 	CWINDOW *_object = (CWINDOW *)CWidget::get(this);
 	QPoint p;
 	QRect r;
 
-	if (!force && !THIS->mustCenter)
-		return;
-
-	THIS->mustCenter = false;
-
-	r = QApplication::desktop()->availableGeometry(QApplication::desktop()->screenNumber(this));
+	r = QApplication::desktop()->availableGeometry(currentScreen());
 
 	CWIDGET_move(THIS, r.x() + (r.width() - width()) / 2, r.y() + (r.height() - height()) / 2);
 }
@@ -2507,19 +2468,19 @@ void MyMainWindow::configure()
 	QMenuBar *menuBar = THIS->menuBar;
 	bool arrange = false;
 	QRect geom;
-	
+
 	//qDebug("THIS->menuBar = %p  menuBar() = %p", THIS->menuBar, menuBar());
 
 	if (menuBar && THIS->showMenuBar && !THIS->hideMenuBar)
 	{
 		int h = menuBar->sizeHint().height();
-		
+
 		if (h == 0)
 			h = menuBar->height();
-		
+
 		menuBar->show();
 		geom = QRect(0, h, this->width(), this->height() - h);
-		
+
 		if (THIS->container->geometry() != geom)
 		{
 			arrange = true;
@@ -2535,18 +2496,18 @@ void MyMainWindow::configure()
 			menuBar->lower();
 		}
 		//qDebug("configure: %s (%d %d)", GB.GetClassName(THIS), this->width(), this->height());
-		
+
 		geom = QRect(0, 0, this->width(), this->height());
-		
+
 		if (THIS->container->geometry() != geom)
 		{
 			arrange = true;
 			THIS->container->setGeometry(geom);
 		}
-		
+
 		THIS->container->raise();
 	}
-	
+
 	if (arrange)
 		CCONTAINER_arrange(THIS);
 
@@ -2560,7 +2521,7 @@ void MyMainWindow::setName(const char *name, CWIDGET *control)
 {
 	if (_deleted)
 		return;
-		
+
 	names.remove(name);
 	if (control)
 		names.insert(name, control);
@@ -2569,12 +2530,12 @@ void MyMainWindow::setName(const char *name, CWIDGET *control)
 void MyMainWindow::resize(int w, int h)
 {
 	bool save = _resizable;
-	
+
 	if (!_resizable && _border)
 		setResizable(true);
-		
+
 	QWidget::resize(w, h);
-	
+
 	if (_resizable != save)
 		setResizable(save);
 }
@@ -2582,12 +2543,12 @@ void MyMainWindow::resize(int w, int h)
 void MyMainWindow::setGeometry(int x, int y, int w, int h)
 {
 	bool save = _resizable;
-	
+
 	if (!_resizable && _border)
 		setResizable(true);
-		
+
 	QWidget::setGeometry(x, y, w, h);
-	
+
 	if (_resizable != save)
 		setResizable(save);
 }
@@ -2596,8 +2557,13 @@ void MyMainWindow::setGeometry(int x, int y, int w, int h)
 void MyMainWindow::changeEvent(QEvent *e)
 {
 	QWidget::changeEvent(e);
+
 	if (e->type() == QEvent::StyleChange || e->type() == QEvent::FontChange)
+	{
 		configure();
+		void *_object = CWidget::get(this);
+		GB.Raise(THIS, EVENT_Font, 0);
+	}
 	/*else if (e->type() == QEvent::WindowStateChange)
 	{
 		qDebug("WindowStateChange");
@@ -2619,6 +2585,48 @@ void MyMainWindow::setState(Qt::WindowStates state)
 		_state = state;
 }
 
+void MyMainWindow::setVisible(bool visible)
+{
+	if (!visible)
+		setAttribute(Qt::WA_NoMouseReplay);
+	QWidget::setVisible(visible);
+}
+
+void MyMainWindow::setBetterMask(QPixmap &bg)
+{
+	if (!bg.hasAlphaChannel())
+			return;
+
+	static const uchar qt_pixmap_bit_mask[] = { 0x01, 0x02, 0x04, 0x08, 0x10, 0x20, 0x40, 0x80 };
+	const QImage img = bg.toImage();
+	const QImage image = (img.depth() < 32 ? img.convertToFormat(QImage::Format_ARGB32_Premultiplied) : img);
+	const int w = image.width();
+	const int h = image.height();
+
+	QImage mask(w, h, QImage::Format_MonoLSB);
+	if (mask.isNull()) // allocation failed
+		return;
+
+	mask.setColorCount(2);
+	mask.setColor(0, QColor(Qt::color0).rgba());
+	mask.setColor(1, QColor(Qt::color1).rgba());
+
+	const int bpl = mask.bytesPerLine();
+
+	for (int y = 0; y < h; ++y) {
+		const QRgb *src = reinterpret_cast<const QRgb*>(image.scanLine(y));
+		uchar *dest = mask.scanLine(y);
+		memset(dest, 0, bpl);
+		for (int x = 0; x < w; ++x) {
+				if (qAlpha(*src) >= 128)
+						dest[x >> 3] |= qt_pixmap_bit_mask[x & 7];
+				++src;
+		}
+	}
+
+	setMask(QBitmap::fromImage(mask));
+}
+
 /***************************************************************************
 
 	CWindow
@@ -2645,8 +2653,8 @@ void CWINDOW_activate(CWIDGET *ob)
 {
 	CWINDOW *active;
 
-	//qDebug("CWINDOW_activate: %s", ob ? ob->name : "NULL");
-	
+	//qDebug("CWINDOW_activate: %s", ob ? ob->name : NULL);
+
 	if (ob)
 	{
 		active = CWidget::getWindow(ob);
@@ -2665,14 +2673,14 @@ void CWINDOW_activate(CWIDGET *ob)
 	if (active == CWINDOW_Active)
 		return;
 
-	//qDebug("activate: (%s %p): (%s %p) -> (%s %p)", ob ? GB.GetClassName(ob) : "", ob, CWINDOW_Active ? GB.GetClassName(CWINDOW_Active) : "", CWINDOW_Active, active ? GB.GetClassName(active) : "", active);
+	//qDebug("CWINDOW_activate: (%s %p): (%s %p) -> (%s %p)", ob ? GB.GetClassName(ob) : "", ob, CWINDOW_Active ? GB.GetClassName(CWINDOW_Active) : "", CWINDOW_Active, active ? GB.GetClassName(active) : "", active);
 
 	if (CWINDOW_Active)
 	{
 		GB.Raise(CWINDOW_Active, EVENT_Deactivate, 0);
 		CWINDOW_Active = 0;
 	}
-	
+
 	if (active)
 		GB.Raise(active, EVENT_Activate, 0);
 
@@ -2701,7 +2709,7 @@ void CWINDOW_set_default_button(CWINDOW *win, QPushButton *button, bool on)
 
 void CWINDOW_set_cancel_button(CWINDOW *win, QPushButton *button, bool on)
 {
-	//qDebug("CWINDOW_set_cancel_button: (%s %p) (%s %p) %d", GB.GetClassName(win), win, GB.GetClassName(CWidget::get(button)), CWidget::get(button), on); 
+	//qDebug("CWINDOW_set_cancel_button: (%s %p) (%s %p) %d", GB.GetClassName(win), win, GB.GetClassName(CWidget::get(button)), CWidget::get(button), on);
 	if (on)
 	{
 		win->cancelButton = button;
@@ -2724,20 +2732,20 @@ bool CWindow::eventFilter(QObject *o, QEvent *e)
 		{
 			MyMainWindow *w = (MyMainWindow *)o;
 
-			if (THIS->toplevel)
+			if (THIS->toplevel && !THIS->popup && (!THIS->moved || w->isModal()))
 				w->center();
-			
+
 			//handle_focus(THIS);
 			emit_open_event(THIS);
-			
+
 			//qDebug("eventFilter: Show: %s %d (%d) focus = %p", GB.GetClassName(THIS), !WINDOW->isHidden(), e->spontaneous(), THIS->focus);
-			
+
 			post_show_event(THIS);
 			//CWINDOW_define_mask(THIS);
-			
+
 			GB.Raise(THIS, EVENT_Show, 0);
 			if (!e->spontaneous())
-				CACTION_raise(THIS);			
+				CACTION_raise(THIS);
 		}
 		else if (e->type() == QEvent::Hide) // && !e->spontaneous())
 		{
@@ -2754,7 +2762,19 @@ bool CWindow::eventFilter(QObject *o, QEvent *e)
 	return QObject::eventFilter(o, e);    // standard event processing
 }
 
+#ifdef QT5
+void CWindow::error(void)
+{
+}
 
+void CWindow::embedded(void)
+{
+}
+
+void CWindow::closed(void)
+{
+}
+#else
 void CWindow::error(void)
 {
 	//CWINDOW *_object = (CWINDOW *)CWidget::getReal((QObject *)sender());
@@ -2776,6 +2796,7 @@ void CWindow::closed(void)
 	//CWIDGET_destroy(CWidget::getReal((QObject *)sender()));
 	delete sender();
 }
+#endif
 
 void CWindow::destroy(void)
 {
@@ -2788,16 +2809,18 @@ void CWindow::destroy(void)
 		CWindow::removeTopLevel(THIS);
 	}
 
+#ifndef QT5
 	CWINDOW_EmbedState = EMBED_WAIT;
 	CWINDOW_Embedded = false;
 	CWINDOW_Embedder = 0;
+#endif
 }
 
 void CWindow::insertTopLevel(CWINDOW *_object)
 {
 	if (!THIS->toplevel)
 		return;
-	
+
 	list.append(THIS);
 	count = list.count();
 
@@ -2813,7 +2836,7 @@ void CWindow::removeTopLevel(CWINDOW *_object)
 
 	list.removeAll(THIS);
   count = list.count();
-	
+
 	#if DEBUG_WINDOW
 	qDebug("removeTopLevel: count = %d (%p %s %s)", count, THIS, THIS->widget.name, THIS->embedded ? "E" : "W");
 	#endif
@@ -2825,7 +2848,7 @@ CMENU *CWindow::findMenu(CWINDOW *_object, const char *name)
 {
 	int i;
 	CMENU *menu;
-	
+
 	if (THIS->menuBar)
 	{
 		for (i = 0; i < THIS->menuBar->actions().count(); i++)
@@ -2835,7 +2858,7 @@ CMENU *CWindow::findMenu(CWINDOW *_object, const char *name)
 				return menu;
 		}
 	}
-	
+
 	return NULL;
 }
 
