@@ -25,8 +25,10 @@
 
 #include "main.h"
 
-#include <qobject.h>
-#include <qpalette.h>
+#include <QObject>
+#include <QPalette>
+#include <QApplication>
+
 #include "gdocument.h"
 
 #include "CEditor.h"
@@ -175,7 +177,7 @@ END_PROPERTY
 BEGIN_PROPERTY(Highlight_Text)
 
 	if (READ_PROPERTY)
-		GB.ReturnNewZeroString(TO_UTF8(_highlight_text));
+		RETURN_NEW_STRING(_highlight_text);
 	else
 		_highlight_text = QSTRING_PROP();
 
@@ -225,7 +227,7 @@ END_PROPERTY
 BEGIN_PROPERTY(Editor_Text)
 
 	if (READ_PROPERTY)
-		GB.ReturnNewZeroString(DOC->getText().utf8());
+		RETURN_NEW_STRING(DOC->getText().getString());
 	else
 		DOC->setText(QSTRING_PROP());
 
@@ -276,6 +278,15 @@ BEGIN_PROPERTY(CEDITOR_read_only)
 
 END_PROPERTY
 
+BEGIN_PROPERTY(CEDITOR_overwrite)
+
+	if (READ_PROPERTY)
+		GB.ReturnBoolean(WIDGET->getInsertMode());
+	else
+		WIDGET->setInsertMode(VPROP(GB_BOOLEAN));
+
+END_PROPERTY
+
 BEGIN_METHOD_VOID(CEDITOR_clear)
 
 	DOC->clear();
@@ -291,19 +302,41 @@ BEGIN_METHOD(CEDITOR_insert, GB_STRING str; GB_INTEGER y; GB_INTEGER x)
 
 END_METHOD
 
+static void print_newline(void *_object)
+{
+	int line, col;
+
+	WIDGET->getCursor(&line, &col);
+
+	if (line < (DOC->numLines() - 1))
+		WIDGET->cursorGoto(line + 1, WIDGET->getColumn(), false);
+	else
+	{
+		WIDGET->cursorGoto(line, DOC->lineLength(line), false);
+		WIDGET->insert("\n");
+	}
+
+	if (THIS->terminal)
+		WIDGET->cursorGoto(WIDGET->getLine(), col, false);
+
+	//qApp->processEvents(QEventLoop::ExcludeUserInputEvents, 0);
+}
+
 static void print_text(void *_object, const char *str, int lstr, bool esc = false)
 {
 	QString s = QString::fromUtf8(str, lstr);
 	int line, col;
 	uint i, len;
 	
+	//fprintf(stderr, "-> %.*s\n", lstr, str);
+
 	WIDGET->getCursor(&line, &col);
-	if (col == 0)
+	/*if (col == 0)
 	{
-		//qDebug("remove: %p: %d, %d, %d, %d", THIS, line, 0, line, DOC->lineLength(line));
 		DOC->remove(line, 0, line, DOC->lineLength(line));
 		WIDGET->cursorGoto(line, 0, false);
-	}
+	}*/
+
 // 	if (col < DOC->lineLength(line))
 // 	{
 // 		end = col + s.length();
@@ -319,12 +352,13 @@ static void print_text(void *_object, const char *str, int lstr, bool esc = fals
 		{
 			if (col == MAX_CONSOLE_WIDTH)
 			{
-				WIDGET->insert("\n");
+				print_newline(THIS);
 				col = 0;
 			}
 			len = s.length() - i;
 			if ((col + len) >= MAX_CONSOLE_WIDTH)
 				len = MAX_CONSOLE_WIDTH - col;
+			DOC->remove(WIDGET->getLine(), col, WIDGET->getLine(), col + len);
 			WIDGET->insert(s.mid(i, len));
 			i += len;	
 			if (i >= (uint)s.length())
@@ -335,9 +369,186 @@ static void print_text(void *_object, const char *str, int lstr, bool esc = fals
 	else
 	{
 		if (col >= MAX_CONSOLE_WIDTH)
-			WIDGET->insert("\n");
+			print_newline(THIS);
+		DOC->remove(WIDGET->getLine(), col, WIDGET->getLine(), col + s.length());
 		WIDGET->insert(s);
 	}
+
+	//qApp->processEvents(QEventLoop::ExcludeUserInputEvents, 0);
+	//BREAKPOINT();
+}
+
+static int ansi_read_integer(const char *str, int len, int def, int *pos)
+{
+	int value = 0;
+	int n = 0;
+	uchar c;
+
+	str += *pos;
+	len -= *pos;
+
+	while (len > 0)
+	{
+		c = str[n];
+		if (c < '0' || c > '9')
+			break;
+		value = value * 10 + c - '0';
+		len--;
+		n++;
+		if (n > 6)
+		{
+			value = -1;
+			break;
+		}
+	}
+
+	if (n == 0)
+		value = def;
+
+	while (len > 0)
+	{
+		c = str[n];
+		len--;
+		n++;
+		if (c == ';' || c < '0' || c > '9')
+			break;
+	}
+
+	*pos += n;
+	return value;
+}
+
+static int ansi_process(void *_object, const char *str, int len)
+{
+	uchar c;
+	int n, m, l, pos;
+	bool print;
+
+	if (len == 0)
+		return 0;
+
+	c = *str;
+	if (c == '[' || c == ']' || c == '(' || c == ')')
+	{
+		for (l = 0; l < len; l++)
+		{
+			c = str[l];
+			if ((c >= 'A' && c <= 'Z') || (c >= 'a' && c <= 'z'))
+				break;
+		}
+	}
+	else
+		l = 0;
+
+	if (l >= len)
+		return 0;
+
+	print = false;
+
+	if (*str == '[')
+	{
+		pos = 1;
+
+		switch(str[l])
+		{
+			case 'A':
+				n = ansi_read_integer(str, l, 1, &pos);
+				if (n > 0) WIDGET->cursorRelGoto(-n, 0, false);
+				break;
+
+			case 'B':
+				n = ansi_read_integer(str, l, 1, &pos);
+				if (n > 0) WIDGET->cursorRelGoto(n, 0, false);
+				break;
+
+			case 'C':
+				n = ansi_read_integer(str, l, 1, &pos);
+				if (n > 0) WIDGET->cursorRelGoto(0, n, false);
+				break;
+
+			case 'D':
+				n = ansi_read_integer(str, l, 1, &pos);
+				if (n > 0) WIDGET->cursorRelGoto(0, -n, false);
+				break;
+
+			case 'G':
+				n = ansi_read_integer(str, l, 1, &pos);
+				if (n > 0) WIDGET->cursorGoto(WIDGET->getLine(), n - 1, false);
+				break;
+
+			case 'd':
+				n = ansi_read_integer(str, l, 1, &pos);
+				if (n > 0) WIDGET->cursorGoto(n - 1, WIDGET->getColumn(), false);
+				break;
+
+			case 'H': case 'f':
+				n = ansi_read_integer(str, l, 1, &pos);
+				m = ansi_read_integer(str, l, 1, &pos);
+
+				while (DOC->numLines() < n)
+					DOC->insertLine(DOC->numLines());
+
+				WIDGET->cursorGoto(n - 1, m - 1, false);
+				THIS->terminal = TRUE;
+				//qApp->processEvents(QEventLoop::ExcludeUserInputEvents, 0);
+				break;
+
+			case 'J':
+				n = ansi_read_integer(str, l, 1, &pos);
+				switch(n)
+				{
+					case 0: WIDGET->clearDocument(false, true); break;
+					case 1: WIDGET->clearDocument(true, false); break;
+					case 2: WIDGET->clearDocument(true, true); break;
+				}
+
+			case 'K':
+				n = ansi_read_integer(str, l, 0, &pos);
+				switch (n)
+				{
+					case 0: WIDGET->clearLine(false, true); break;
+					case 1: WIDGET->clearLine(true, false); break;
+					case 2: WIDGET->clearLine(true, true); break;
+				}
+				break;
+
+			case 's':
+				WIDGET->saveCursor();
+				break;
+
+			case 'u':
+				WIDGET->restoreCursor();
+				break;
+
+			case 'X':
+				n = ansi_read_integer(str, l, 1, &pos);
+				WIDGET->clearAfter(n);
+				break;
+
+			default:
+				print = FALSE;
+		}
+	}
+
+	if (print)
+	{
+		fprintf(stderr, "ESC ");
+		for (n = 0; n <= l; n++)
+		{
+			uchar c = str[n];
+
+			if (c < 32)
+				fprintf(stderr, "\\x%02X", c);
+			else
+				fputc(c, stderr);
+		}
+		fputc('\n', stderr);
+
+		//qApp->processEvents(QEventLoop::ExcludeUserInputEvents, 0);
+		//BREAKPOINT();
+	}
+
+	return l;
 }
 
 BEGIN_METHOD(CEDITOR_print, GB_STRING str; GB_INTEGER y; GB_INTEGER x)
@@ -345,9 +556,10 @@ BEGIN_METHOD(CEDITOR_print, GB_STRING str; GB_INTEGER y; GB_INTEGER x)
 	char *str = STRING(str);
 	int len = LENGTH(str);
 	int i, j;
-	int line, col;
 	unsigned char c;
 
+	DOC->begin();
+
 	if (!MISSING(y) && !MISSING(x))
 		WIDGET->cursorGoto(VARG(y), VARG(x), false);
 	
@@ -364,23 +576,21 @@ BEGIN_METHOD(CEDITOR_print, GB_STRING str; GB_INTEGER y; GB_INTEGER x)
 				print_text(THIS, &str[j], i - j);
 				
 			j = i + 1;
-			
-			WIDGET->getCursor(&line, &col);		
+
+			/*if (c != 27)
+				fprintf(stderr, "\\x%02X\n", c);*/
 			
 			if (c == '\t')
 			{
-				//int l = 8 - col % 8;
-				//print_text(THIS, "        ", l);
 				WIDGET->insert("\t");
 			}
 			else if (c == '\r')
 			{
-				WIDGET->cursorGoto(line, 0, false);
+				WIDGET->cursorGoto(WIDGET->getLine(), 0, false);
 			}
 			else if (c == '\n')
 			{
-				WIDGET->cursorGoto(line, DOC->lineLength(line), false);
-				WIDGET->insert("\n");
+				print_newline(THIS);
 			}
 			else if (c == '\f') // CTRL+L
 			{
@@ -390,6 +600,17 @@ BEGIN_METHOD(CEDITOR_print, GB_STRING str; GB_INTEGER y; GB_INTEGER x)
 			{
 				WIDGET->flash();
 			}
+			else if (c == '\b')
+			{
+				WIDGET->cursorRelGoto(0, -1, false); //WIDGET->getLine(), WIDGET->getColumn() - 1, false);
+			}
+			else if (c == 27)
+			{
+				i++;
+				i += ansi_process(THIS, &str[i], len - i);
+				j = i + 1;
+				i--;
+			}
 			else
 			{
 				QString tmp;
@@ -401,6 +622,8 @@ BEGIN_METHOD(CEDITOR_print, GB_STRING str; GB_INTEGER y; GB_INTEGER x)
 
 	if (i > j)
 		print_text(THIS, &str[j], i - j);
+
+	DOC->end();
 		
 END_METHOD
 
@@ -632,7 +855,7 @@ END_PROPERTY
 
 BEGIN_PROPERTY(CEDITOR_sel_text)
 
-	GB.ReturnNewZeroString(WIDGET->getSelectedText().utf8());
+	RETURN_NEW_STRING(WIDGET->getSelectedText().getString());
 
 END_PROPERTY
 
@@ -715,7 +938,7 @@ END_METHOD
 BEGIN_PROPERTY(CEDITOR_line_text)
 
 	if (READ_PROPERTY)
-		GB.ReturnNewZeroString(DOC->getLine(THIS->line).utf8());
+		RETURN_NEW_STRING(DOC->getLine(THIS->line).getString());
 	else
 	{
 		GString s = GString(QSTRING_PROP());
@@ -811,12 +1034,6 @@ BEGIN_METHOD(CEDITOR_line_purge, GB_BOOLEAN comment; GB_BOOLEAN string; GB_STRIN
 	
 	line = DOC->getLine(THIS->line);
 	
-	/*if (comment && string)	
-	{
-		GB.ReturnNewZeroString(line.utf8());
-		return;
-	}*/
-	
 	for (i = 0; i < line.length(); i++)
 	{
 		state = DOC->getCharState(THIS->line, i);
@@ -826,7 +1043,7 @@ BEGIN_METHOD(CEDITOR_line_purge, GB_BOOLEAN comment; GB_BOOLEAN string; GB_STRIN
 			result += line.at(i);
 	}
 	
-	GB.ReturnNewZeroString(result.utf8());
+	RETURN_NEW_STRING(result.getString());
 
 END_METHOD
 
@@ -1265,7 +1482,7 @@ BEGIN_PROPERTY(Editor_Mouse)
 	QT.MouseProperty(_object, _param);
 
 	if (!READ_PROPERTY)
-		WIDGET->saveCursor();
+		WIDGET->saveMouseCursor();
 
 END_PROPERTY
 
@@ -1501,6 +1718,7 @@ GB_DESC CEditorDesc[] =
 	GB_CONSTANT("ShowDots", "i", GEditor::ShowDots),
 	GB_CONSTANT("HideMargin", "i", GEditor::HideMargin),
 	GB_CONSTANT("NoFolding", "i", GEditor::NoFolding),
+	GB_CONSTANT("AlwaysShowCursor", "i", GEditor::AlwaysShowCursor),
 
 	GB_METHOD("_new", NULL, CEDITOR_new, "(Parent)Container;"),
 	GB_METHOD("_free", NULL, CEDITOR_free, NULL),
@@ -1577,6 +1795,7 @@ GB_DESC CEditorDesc[] =
 	GB_METHOD("Reset", NULL, CEDITOR_reset, NULL),
 
 	GB_PROPERTY("ReadOnly", "b", CEDITOR_read_only),
+	GB_PROPERTY("Overwrite", "b", CEDITOR_overwrite),
 
 	GB_METHOD("Clear", NULL, CEDITOR_clear, NULL),
 
