@@ -121,6 +121,7 @@ void GEditor::reset()
 	_posOutside = false;
 	_checkCache = true;
 	_ensureCursorVisibleLater = false;
+	_save_x = _save_y = 0;
 	
 	foldClear();
 }
@@ -1188,16 +1189,20 @@ void GEditor::paintCell(QPainter &p, int row, int)
 	}
 
 	// Text cursor
-	if (_cursor && realRow == y)
+	if (realRow == y && (_cursor || getFlag(AlwaysShowCursor)))
 	{
 		QColor color = styles[GLine::Normal].color;
 		int xc = lineWidth(realRow, x);
 		int wc = 2;
 		//p.fillRect(QMIN((int)l->s.length(), x) * charWidth + margin, 0, 1, _cellh, styles[GLine::Normal].color);
 		if (_insertMode)
+		{
 			wc = lineWidth(realRow, x + 1) - xc;
-		
-		color.setAlpha(160);
+			color.setAlpha(80);
+		}
+		else
+			color.setAlpha(160);
+
 		p.fillRect(xc, 0, wc, _cellh, color);
 	}
 
@@ -1931,6 +1936,79 @@ void GEditor::deleteCurrentLine()
 	_insertMode = im;
 }
 
+void GEditor::clearLine(bool before, bool after)
+{
+	int ox = x;
+
+	doc->begin();
+
+	if (before)
+	{
+		QString ins;
+
+		doc->remove(y, 0, y, x + 1);
+		ins.fill(' ', ox + 1);
+		doc->insert(y, 0, ins);
+		x = ox;
+	}
+
+	if (after)
+	{
+		doc->remove(y, x, y, doc->lineLength(y) -1);
+	}
+
+	doc->end();
+}
+
+void GEditor::clearAfter(int n)
+{
+	int ox = x;
+	QString ins;
+
+	doc->begin();
+	doc->remove(y, x, y, x + n);
+	ins.fill(' ', n);
+	doc->insert(y, ox, ins);
+	x = ox;
+	doc->end();
+}
+
+void GEditor::clearDocument(bool before, bool after)
+{
+	int i;
+	int ox = x;
+
+	doc->begin();
+
+	if (before && after)
+	{
+		for (i = 0; i < doc->numLines(); i++)
+			doc->remove(i, 0, i, doc->lineLength(i));
+	}
+	else if (before)
+	{
+		QString ins;
+
+		for (i = 0; i < y; i++)
+			doc->remove(i, 0, i, doc->lineLength(i));
+
+		doc->remove(y, 0, y, x);
+		ins.fill(' ', ox);
+		doc->insert(y, 0, ins);
+		x = ox;
+	}
+	else if (after)
+	{
+		doc->remove(y, x, y, doc->lineLength(y) -1);
+
+		for (i = y + 1; i < doc->numLines(); i++)
+			doc->remove(i, 0, i, doc->lineLength(i));
+	}
+
+	x = ox;
+	doc->end();
+}
+
 void GEditor::tab(bool back)
 {
 	QString ins;
@@ -3347,7 +3425,23 @@ void GEditor::expand(bool shift)
 	}
 }
 
-void GEditor::saveCursor()
+void GEditor::saveMouseCursor()
 {
 	_saveCursor = viewport()->cursor();
 }
+
+void GEditor::saveCursor()
+{
+	_save_x = x;
+	_save_y = y;
+}
+
+void GEditor::restoreCursor()
+{
+	cursorGoto(_save_x, _save_y, false);
+}
+
+bool GEditor::cursorRelGoto(int dy, int dx, bool mark)
+{
+	return cursorGoto(QMAX(0, y + dy), QMAX(0, x + dx), mark);
+}
