@@ -94,14 +94,23 @@
 #include "cprinter.h"
 #include "csvgimage.h"
 #include "cpaint_impl.h"
+#include "ctrayicon.h"
 
-#ifndef NO_X_WINDOW
 #include <QX11Info>
+#ifndef QT5
 #include "CEmbedder.h"
-#include "CTrayIcon.h"
-#include "x11.h"
 #endif
 
+#include "desktop.h"
+#include "x11.h"
+
+#ifdef QT5
+#include <xcb/xcb.h>
+#include <xcb/xproto.h>
+#include <QAbstractNativeEventFilter>
+#endif
+
+#include "fix_breeze.h"
 #include "main.h"
 
 /*#define DEBUG*/
@@ -145,6 +154,7 @@ static QTranslator *_translator = NULL;
 static bool _application_keypress = false;
 static GB_FUNCTION _application_keypress_func;
 static bool _check_quit_posted = false;
+static int _prevent_quit = 0;
 
 #ifndef NO_X_WINDOW
 static void (*_x11_event_filter)(XEvent *) = 0;
@@ -155,6 +165,10 @@ static QHash<void *, void *> _link_map;
 static QPointer<QWidget> MAIN_mouseGrabber = 0;
 static QPointer<QWidget> MAIN_keyboardGrabber = 0;
 
+static QByteArray _utf8_buffer[UTF8_NBUF];
+static int _utf8_count = 0;
+static int _utf8_length = 0;
+
 
 //static MyApplication *myApp;
 
@@ -282,13 +296,13 @@ bool MyAbstractEventDispatcher::processEvents(QEventLoop::ProcessEventsFlags fla
 	for(;;)
 	{
 		ptr = &CWIDGET_destroy_list;
-	
+
 		for(;;)
 		{
 			ob = *ptr;
 			if (!ob)
 				return ret;
-	
+
 			//if (MAIN_loop_level <= ob->level && !ob->flag.notified)
 			if (!ob->flag.notified)
 			{
@@ -355,12 +369,12 @@ MyApplication::MyApplication(int &argc, char **argv)
 
 void MyApplication::initClipboard()
 {
-  QObject::connect(clipboard(), SIGNAL(dataChanged()), qApp, SLOT(clipboardHasChanged()));
+  QObject::connect(clipboard(), SIGNAL(changed(QClipboard::Mode)), qApp, SLOT(clipboardHasChanged(QClipboard::Mode)));
 }
 
-void MyApplication::clipboardHasChanged()
+void MyApplication::clipboardHasChanged(QClipboard::Mode m)
 {
-	CLIPBOARD_has_changed();
+	CLIPBOARD_has_changed(m);
 }
 
 static bool QT_EventFilter(QEvent *e)
@@ -369,15 +383,15 @@ static bool QT_EventFilter(QEvent *e)
 
 	if (!_application_keypress)
 		return false;
-		
+
 	if (e->type() == QEvent::KeyPress)
 	{
 		QKeyEvent *kevent = (QKeyEvent *)e;
-		
+
 		CKEY_clear(true);
 
 		GB.FreeString(&CKEY_info.text);
-		CKEY_info.text = GB.NewZeroString(QT_ToUTF8(kevent->text()));
+		CKEY_info.text = GB.NewZeroString(QT_ToUtf8(kevent->text()));
 		CKEY_info.state = kevent->modifiers();
 		CKEY_info.code = kevent->key();
 
@@ -392,7 +406,7 @@ static bool QT_EventFilter(QEvent *e)
 
 			GB.FreeString(&CKEY_info.text);
 			//qDebug("IMEnd: %s", imevent->text().latin1());
-			CKEY_info.text = GB.NewZeroString(QT_ToUTF8(imevent->commitString()));
+			CKEY_info.text = GB.NewZeroString(QT_ToUtf8(imevent->commitString()));
 			CKEY_info.state = 0;
 			CKEY_info.code = 0;
 		}
@@ -400,9 +414,9 @@ static bool QT_EventFilter(QEvent *e)
 
 	GB.Call(&_application_keypress_func, 0, FALSE);
 	cancel = GB.Stopped();
-	
+
 	CKEY_clear(false);
-		
+
 	return cancel;
 }
 
@@ -432,7 +446,7 @@ bool MyApplication::eventFilter(QObject *o, QEvent *e)
 		{
 			QWidget *widget = (QWidget *)o;
 			CWIDGET *control;
-			
+
 			if (widget->isWindow())
 			{
 				if (e->type() == QEvent::WindowActivate)
@@ -462,9 +476,9 @@ bool MyApplication::eventFilter(QObject *o, QEvent *e)
 {
 	if (o->isWidgetType())
 	{
-		CWIDGET *ob = CWidget::get(o);		
+		CWIDGET *ob = CWidget::get(o);
 		bool old, res;
-	
+
 		if (ob)
 		{
 			old = QT_Notify(ob, true);
@@ -473,7 +487,7 @@ bool MyApplication::eventFilter(QObject *o, QEvent *e)
 			return res;
 		}
 	}
-	
+
 	return QApplication::notify(o, e);
 }*/
 
@@ -498,7 +512,7 @@ void MyApplication::setTooltipEnabled(bool b)
 	b = !b;
 	if (b == _tooltip_disable)
 		return;
-		
+
 	_tooltip_disable = b;
 	setEventFilter(b);
 }
@@ -542,13 +556,186 @@ void MyApplication::commitDataRequested(QSessionManager &session)
 	session.setRestartCommand(cmd);
 }
 
-#ifndef NO_X_WINDOW
+//---------------------------------------------------------------------------
 
 static void x11_set_event_filter(void (*filter)(XEvent *))
 {
 	_x11_event_filter = filter;
 }
 
+#ifdef QT5
+
+class MyNativeEventFilter: public QAbstractNativeEventFilter
+{
+public:
+
+	static MyNativeEventFilter manager;
+
+	virtual bool nativeEventFilter(const QByteArray &eventType, void *message, long *)
+	{
+		xcb_generic_event_t *ev = static_cast<xcb_generic_event_t *>(message);
+		int type = ev->response_type & ~0x80;
+
+		switch(type)
+		{
+			case XCB_KEY_PRESS:
+			case XCB_KEY_RELEASE:
+				MAIN_x11_last_key_code = ((xcb_key_press_event_t *)ev)->detail;
+				break;
+		}
+
+		if (_x11_event_filter)
+		{
+			XEvent xev;
+
+			CLEAR(&xev);
+			xev.xany.type = type;
+			xev.xany.display = QX11Info::display();
+			xev.xany.send_event = ev->response_type & 0x80 ? 1 : 0;
+
+			switch (type)
+			{
+				//case XCB_KEY_PRESS:
+				//case XCB_KEY_RELEASE:
+				case XCB_EXPOSE:
+				{
+					xcb_expose_event_t *e = (xcb_expose_event_t *)ev;
+					xev.xexpose.window = e->window;
+					xev.xexpose.x = e->x;
+					xev.xexpose.y = e->y;
+					xev.xexpose.width = e->width;
+					xev.xexpose.height = e->height;
+					xev.xexpose.count = e->count;
+					break;
+				}
+
+				case XCB_VISIBILITY_NOTIFY:
+				{
+					xcb_visibility_notify_event_t *e = (xcb_visibility_notify_event_t *)ev;
+					xev.xvisibility.window = e->window;
+					xev.xvisibility.state = e->state;
+					break;
+				}
+
+				case XCB_DESTROY_NOTIFY:
+				{
+					xcb_destroy_notify_event_t *e = (xcb_destroy_notify_event_t *)ev;
+					xev.xdestroywindow.event = e->event;
+					xev.xdestroywindow.window = e->window;
+					break;
+				}
+
+				case XCB_MAP_NOTIFY:
+				{
+					xcb_map_notify_event_t *e = (xcb_map_notify_event_t *)ev;
+					xev.xmap.event = e->event;
+					xev.xmap.window = e->window;
+					xev.xmap.override_redirect = e->override_redirect;
+					break;
+				}
+
+				case XCB_UNMAP_NOTIFY:
+				{
+					xcb_unmap_notify_event_t *e = (xcb_unmap_notify_event_t *)ev;
+					xev.xunmap.event = e->event;
+					xev.xunmap.window = e->window;
+					xev.xunmap.from_configure = e->from_configure;
+					break;
+				}
+
+				case XCB_REPARENT_NOTIFY:
+				{
+					xcb_reparent_notify_event_t *e = (xcb_reparent_notify_event_t *)ev;
+					xev.xreparent.event = e->event;
+					xev.xreparent.window = e->window;
+					xev.xreparent.parent = e->parent;
+					xev.xreparent.x = e->x;
+					xev.xreparent.y = e->y;
+					xev.xreparent.override_redirect = e->override_redirect;
+					break;
+				}
+
+				case XCB_CONFIGURE_NOTIFY:
+				{
+					xcb_configure_notify_event_t *e = (xcb_configure_notify_event_t *)ev;
+					xev.xconfigure.event = e->event;
+					xev.xconfigure.window = e->window;
+					xev.xconfigure.x = e->x;
+					xev.xconfigure.y = e->y;
+					xev.xconfigure.width = e->width;
+					xev.xconfigure.height = e->height;
+					xev.xconfigure.border_width = e->border_width;
+					xev.xconfigure.override_redirect = e->override_redirect;
+					break;
+				}
+				case XCB_PROPERTY_NOTIFY:
+				{
+					xcb_property_notify_event_t *e = (xcb_property_notify_event_t *)ev;
+					xev.xproperty.window = e->window;
+					xev.xproperty.atom = e->atom;
+					xev.xproperty.time = e->time;
+					xev.xproperty.state = e->state;
+					break;
+				}
+
+				case XCB_SELECTION_CLEAR:
+				{
+					xcb_selection_clear_event_t *e = (xcb_selection_clear_event_t *)ev;
+					xev.xselectionclear.window = e->owner;
+					xev.xselectionclear.selection = e->selection;
+					xev.xselectionclear.time = e->time;
+					break;
+				}
+
+				case XCB_SELECTION_REQUEST:
+				{
+					xcb_selection_request_event_t *e = (xcb_selection_request_event_t *)ev;
+					xev.xselectionrequest.owner = e->owner;
+					xev.xselectionrequest.requestor = e->requestor;
+					xev.xselectionrequest.selection = e->selection;
+					xev.xselectionrequest.target = e->target;
+					xev.xselectionrequest.property = e->property;
+					xev.xselectionrequest.time = e->time;
+					break;
+				}
+
+				case XCB_SELECTION_NOTIFY:
+				{
+					xcb_selection_notify_event_t *e = (xcb_selection_notify_event_t *)ev;
+					xev.xselection.requestor = e->requestor;
+					xev.xselection.selection = e->selection;
+					xev.xselection.target = e->target;
+					xev.xselection.property = e->property;
+					xev.xselection.time = e->time;
+					break;
+				}
+
+				case XCB_CLIENT_MESSAGE:
+				{
+					xcb_client_message_event_t *e = (xcb_client_message_event_t *)ev;
+					xev.xclient.window = e->window;
+					xev.xclient.message_type = e->type;
+					xev.xclient.format = e->format;
+					memcpy(&xev.xclient.data, &e->data, 20);
+					break;
+				}
+
+				default:
+					//qDebug("gb.qt5: warning: unhandled xcb event: %d", type);
+					return false;
+			}
+
+			(*_x11_event_filter)(&xev);
+		}
+
+		return false;
+	}
+};
+
+MyNativeEventFilter MyNativeEventFilter::manager;
+
+#else
+
 bool MyApplication::x11EventFilter(XEvent *e)
 {
 	// Workaround for input methods that void the key code of KeyRelease eventFilter
@@ -556,15 +743,16 @@ bool MyApplication::x11EventFilter(XEvent *e)
 		MAIN_x11_last_key_code = e->xkey.keycode;
 	else if (e->type == XKeyRelease)
 		MAIN_x11_last_key_code = e->xkey.keycode;
-	
+
 	if (_x11_event_filter)
 		(*_x11_event_filter)(e);
-	
+
 	return false;
 }
+
 #endif
 
-/** MyTimer ****************************************************************/
+//---------------------------------------------------------------------------
 
 MyTimer::MyTimer(GB_TIMER *t) : QObject(0)
 {
@@ -583,14 +771,14 @@ void MyTimer::timerEvent(QTimerEvent *e)
 		GB.RaiseTimer(timer);
 }
 
-/***************************************************************************/
+//---------------------------------------------------------------------------
 
 static void release_grab()
 {
 	MAIN_mouseGrabber = QWidget::mouseGrabber();
 	MAIN_keyboardGrabber = QWidget::keyboardGrabber();
-	
-	if (MAIN_mouseGrabber) 
+
+	if (MAIN_mouseGrabber)
 	{
 		//qDebug("releaseMouse");
 		MAIN_mouseGrabber->releaseMouse();
@@ -600,7 +788,7 @@ static void release_grab()
 		//qDebug("releaseKeyboard");
 		MAIN_keyboardGrabber->releaseKeyboard();
 	}
-	
+
 	#ifndef NO_X_WINDOW
 	if (qApp->activePopupWidget())
 	{
@@ -619,7 +807,7 @@ static void unrelease_grab()
 		MAIN_mouseGrabber->grabMouse();
 		MAIN_mouseGrabber = 0;
 	}
-	
+
 	if (MAIN_keyboardGrabber)
 	{
 		//qDebug("grabKeyboard");
@@ -631,23 +819,30 @@ static void unrelease_grab()
 static bool must_quit(void)
 {
 	#if DEBUG_WINDOW
-	qDebug("must_quit: Window = %d Watch = %d in_event_loop = %d", CWindow::count, CWatch::count, in_event_loop);
+	qDebug("must_quit: Window = %d Watch = %d in_event_loop = %d MAIN_in_message_box = %d _prevent_quit = %d", CWindow::count, CWatch::count, in_event_loop, MAIN_in_message_box, _prevent_quit);
 	#endif
-	return CWINDOW_must_quit() && CWatch::count == 0 && in_event_loop && MAIN_in_message_box == 0 && TRAYICON_count == 0;
+	return CWINDOW_must_quit() && CWatch::count == 0 && in_event_loop && MAIN_in_message_box == 0 && _prevent_quit == 0;
 }
 
 static void check_quit_now(intptr_t param)
 {
 	static bool exit_called = false;
-	
+
 	if (must_quit() && !exit_called)
 	{
 		if (QApplication::instance())
 		{
-			#ifndef NO_X_WINDOW
-				CTRAYICON_close_all();
-				qApp->syncX();
-			#endif
+			GB_FUNCTION func;
+
+			if (GB.ExistClass("TrayIcons"))
+			{
+				if (!GB.GetFunction(&func, (void *)GB.FindClass("TrayIcons"), "DeleteAll", NULL, NULL))
+					GB.Call(&func, 0, FALSE);
+			}
+
+#ifndef QT5
+			qApp->syncX();
+#endif
 			qApp->exit();
 			exit_called = true;
 		}
@@ -660,20 +855,14 @@ void MAIN_check_quit(void)
 {
 	if (_check_quit_posted)
 		return;
-	
+
 	GB.Post((GB_CALLBACK)check_quit_now, 0);
 	_check_quit_posted = true;
 }
 
-void MAIN_update_scale(void)
+void MAIN_update_scale(const QFont &font)
 {
-	//QFontMetrics fm(qApp->desktop()->font());
-	//MAIN_scale = GET_DESKTOP_SCALE(fm.height());
-	#ifdef NO_X_WINDOW
-	MAIN_scale = GET_DESKTOP_SCALE(qApp->desktop()->font().pointSize(), 96);
-	#else
-	MAIN_scale = GET_DESKTOP_SCALE(qApp->desktop()->font().pointSize(), QX11Info::appDpiY());
-	#endif
+	MAIN_scale = GET_DESKTOP_SCALE(font.pointSize(), QX11Info::appDpiY());
 }
 
 static void QT_InitEventLoop(void)
@@ -692,23 +881,33 @@ static void QT_Init(void)
 
 	//qApp->setAttribute(Qt::AA_ImmediateWidgetCreation);
 
-	#ifndef NO_X_WINDOW
-		X11_init(QX11Info::display(), QX11Info::appRootWindow());
-	#endif
+	X11_init(QX11Info::display(), QX11Info::appRootWindow());
 
 	/*QX11Info::setAppDpiX(0, 92);
 	QX11Info::setAppDpiY(0, 92);*/
-		
-	/*fcntl(ConnectionNumber(qt_xdisplay()), F_SETFD, FD_CLOEXEC);*/
-	
-	//Q3MimeSourceFactory::addFactory(&myMimeSourceFactory);
 
-	MAIN_update_scale();
+	/*fcntl(ConnectionNumber(qt_xdisplay()), F_SETFD, FD_CLOEXEC);*/
+
+	if (::strcmp(qApp->style()->metaObject()->className(), "Breeze::Style") == 0)
+	{
+		char *env = getenv("GB_QT_NO_BREEZE_FIX");
+		if (!env || atoi(env) == 0)
+		{
+			CSTYLE_fix_breeze = TRUE;
+			qApp->setStyle(new FixBreezeStyle);
+		}
+	}
+
+
+	MAIN_update_scale(qApp->desktop()->font());
 
 	qApp->installEventFilter(&CWidget::manager);
+#ifdef QT5
+	qApp->installNativeEventFilter(&MyNativeEventFilter::manager);
+#endif
 
 	MyApplication::setEventFilter(true);
-	
+
 	if (GB.GetFunction(&_application_keypress_func, (void *)GB.Application.StartupClass(), "Application_KeyPress", "", "") == 0)
 	{
 		_application_keypress = true;
@@ -717,40 +916,43 @@ static void QT_Init(void)
 
 	//qt_x11_set_global_double_buffer(false);
 
-	//Q3StyleSheet::defaultSheet()->item("tt")->setFontFamily("Monospace");
-	//Q3StyleSheet::defaultSheet()->item("pre")->setFontFamily("Monospace");
-
 	qApp->setQuitOnLastWindowClosed(false);
-	
+
 	MyApplication::initClipboard();
-	
+
 	init = true;
 }
 
 static bool try_to_load_translation(QString &locale)
 {
+#ifdef QT5
+	return (!_translator->load(QString("qt_") + locale, QString(getenv("QTDIR")) + "/translations")
+		  && !_translator->load(QString("qt_") + locale, QString("/usr/lib/qt5/translations"))
+		  && !_translator->load(QString("qt_") + locale, QString("/usr/share/qt5/translations")));
+#else
 	return (!_translator->load(QString("qt_") + locale, QString(getenv("QTDIR")) + "/translations")
 		  && !_translator->load(QString("qt_") + locale, QString("/usr/lib/qt4/translations"))
 		  && !_translator->load(QString("qt_") + locale, QString("/usr/share/qt4/translations")));
+#endif
 }
 
 static void init_lang(char *lang, bool rtl)
 {
 	int pos;
 	QString locale(lang);
-	
+
 	pos = locale.lastIndexOf(".");
 	if (pos >= 0) locale = locale.left(pos);
-	
+
 	if (_translator)
 	{
 		qApp->removeTranslator(_translator);
 		delete _translator;
 		_translator = NULL;
 	}
-	
+
 	_translator = new QTranslator();
-	
+
 	if (!try_to_load_translation(locale))
 		goto __INSTALL_TRANSLATOR;
 
@@ -764,10 +966,10 @@ static void init_lang(char *lang, bool rtl)
 
 	delete _translator;
 	_translator = NULL;
-	
+
 	//if (strcmp(lang, "C"))
 	//	qDebug("gb.qt4: warning: unable to load Qt translation: %s", lang);
-	
+
 	goto __SET_DIRECTION;
 
 __INSTALL_TRANSLATOR:
@@ -828,18 +1030,23 @@ static void hook_main(int *argc, char ***argv)
 
 static void hook_quit()
 {
+	GB_FUNCTION func;
+
 	CWINDOW_close_all(true);
 	CWINDOW_delete_all(true);
 
 	qApp->sendPostedEvents(); //processEvents(QEventLoop::ExcludeUserInputEvents | QEventLoop::DeferredDeletion, 0);
 	qApp->sendPostedEvents(0, QEvent::DeferredDelete);
+
+	if (!GB.GetFunction(&func, (void *)GB.FindClass("_Gui"), "_Quit", NULL, NULL))
+		GB.Call(&func, 0, FALSE);
 }
 
 
 static void hook_loop()
 {
 	//qDebug("**** ENTERING EVENT LOOP");
-	
+
 	qApp->sendPostedEvents();
 	//qApp->processEvents(QEventLoop::ExcludeUserInputEvents | QEventLoop::DeferredDeletion, 0);
 
@@ -849,18 +1056,24 @@ static void hook_loop()
 		qApp->exec();
 	else
 		MAIN_check_quit();
-	
+
 	hook_quit();
 }
 
 
 static void hook_wait(int duration)
 {
+	if (MyDrawingArea::inAnyDrawEvent())
+	{
+		GB.Error("Wait is forbidden during a repaint event");
+		return;
+	}
+
 	MAIN_in_wait++;
 	if (duration > 0)
 	{
 		if (CKEY_is_valid())
-			fprintf(stderr, "gb.qt4: warning: calling the event loop during a keyboard event handler is ignored\n");
+			fprintf(stderr, QT_NAME ": warning: calling the event loop during a keyboard event handler is ignored\n");
 		else
 			qApp->processEvents(QEventLoop::AllEvents, duration);
 	}
@@ -879,7 +1092,7 @@ static void hook_timer(GB_TIMER *timer, bool on)
 		t->deleteLater();
 		timer->id = 0;
 	}
-	
+
 	if (on)
 		timer->id = (intptr_t)(new MyTimer(timer));
 }
@@ -917,7 +1130,7 @@ static void hook_error(int code, char *error, char *where)
 	CWatch::stop();
 
 	msg = "<b>This application has raised an unexpected<br>error and must abort.</b><br><br>";
-	
+
 	if (code > 0)
 	{
 		msg = msg + "[%1] %2.<br>%3";
@@ -941,7 +1154,7 @@ static void hook_error(int code, char *error, char *where)
 
 static void QT_InitWidget(QWidget *widget, void *object, int fill_bg)
 {
-	((CWIDGET *)object)->flag.fillBackground = fill_bg;	
+	((CWIDGET *)object)->flag.fillBackground = fill_bg;
 	CWIDGET_new(widget, object);
 }
 
@@ -965,22 +1178,37 @@ static QPixmap *QT_GetPixmap(CPICTURE *pict)
 	return pict->pixmap;
 }
 
-const char *QT_ToUTF8(const QString &str)
+const char *QT_ToUtf8(const QString &str)
 {
-	static QByteArray buf[UTF8_NBUF];
-	static int cpt = 0;
-
 	const char *res;
 
-	buf[cpt] = str.toUtf8();
-	res = buf[cpt].data();
-	cpt++;
-	if (cpt >= UTF8_NBUF)
-		cpt = 0;
+	_utf8_buffer[_utf8_count] = str.toUtf8();
+	res = _utf8_buffer[_utf8_count].data();
+	_utf8_length = _utf8_buffer[_utf8_count].length();
+	_utf8_count++;
+	if (_utf8_count >= UTF8_NBUF)
+		_utf8_count = 0;
 
 	return res;
 }
 
+int QT_GetLastUtf8Length()
+{
+	return _utf8_length;
+}
+
+char *QT_NewString(const QString &str)
+{
+	const char *res = QT_ToUtf8(str);
+	return GB.NewString(res, _utf8_length);
+}
+
+void QT_ReturnNewString(const QString &str)
+{
+	const char *res = QT_ToUtf8(str);
+	GB.ReturnNewString(res, _utf8_length);
+}
+
 static void *QT_CreatePicture(const QPixmap &p)
 {
 	return CPICTURE_create(&p);
@@ -1006,6 +1234,36 @@ void *QT_GetLink(QObject *qobject)
 	return _link_map.value(qobject, 0);
 }
 
+void QT_PreventQuit(bool inc)
+{
+	if (inc)
+		_prevent_quit++;
+	else
+	{
+		_prevent_quit--;
+		MAIN_check_quit();
+	}
+}
+
+QMenu *QT_FindMenu(void *parent, const char *name)
+{
+	CMENU *menu = NULL;
+
+	if (parent && GB.Is(parent, CLASS_Control))
+	{
+		CWINDOW *window = CWidget::getWindow((CWIDGET *)parent);
+		menu = CWindow::findMenu(window, name);
+	}
+
+	return menu ? menu->menu : NULL;
+}
+
+static void declare_tray_icon()
+{
+	GB.Component.Declare(TrayIconDesc);
+	GB.Component.Declare(TrayIconsDesc);
+}
+
 extern "C" {
 
 GB_DESC *GB_CLASSES[] EXPORT =
@@ -1034,17 +1292,21 @@ GB_DESC *GB_CLASSES[] EXPORT =
 	CSliderDesc, CSpinBoxDesc, CMovieBoxDesc, CScrollBarDesc,
 	CWindowMenusDesc, CWindowControlsDesc, CWindowDesc, CWindowsDesc, CFormDesc,
 	CDialogDesc,
-	#ifndef NO_X_WINDOW
-	CEmbedderDesc, CTrayIconDesc, CTrayIconsDesc,
-	#endif
+#ifndef QT5
+	CEmbedderDesc,
+#endif
 	CWatcherDesc,
 	PrinterDesc,
 	SvgImageDesc,
 	NULL
 };
 
-void *GB_QT4_1[] EXPORT = {
-
+#ifdef QT5
+void *GB_QT5_1[] EXPORT =
+#else
+void *GB_QT4_1[] EXPORT =
+#endif
+{
 	(void *)1,
 
 	(void *)QT_InitEventLoop,
@@ -1061,7 +1323,10 @@ void *GB_QT4_1[] EXPORT = {
 	(void *)QT_CreatePicture,
 	//(void *)QT_MimeSourceFactory,
 	(void *)QT_GetPixmap,
-	(void *)QT_ToUTF8,
+	(void *)QT_ToUtf8,
+	(void *)QT_GetLastUtf8Length,
+	(void *)QT_NewString,
+	(void *)QT_ReturnNewString,
 	(void *)QT_EventFilter,
 	(void *)QT_Notify,
 	(void *)CCONST_alignment,
@@ -1095,18 +1360,18 @@ const char *GB_INCLUDE EXPORT = "gb.draw,gb.gui.base";
 
 int EXPORT GB_INIT(void)
 {
-	// Do not disable GLib support
-	
 	char *env;
-	
+
+	// Do not disable GLib support
+
 	env = getenv("KDE_FULL_SESSION");
 	if (env && !strcasecmp(env, "true"))
 		putenv((char *)"QT_NO_GLIB=1");
-	
+
 	env = getenv("GB_GUI_BUSY");
 	if (env && atoi(env))
 		MAIN_debug_busy = true;
-	
+
 	//putenv((char *)"QT_SLOW_TOPLEVEL_RESIZE=1");
 
 	_old_hook_main = GB.Hook(GB_HOOK_MAIN, (void *)hook_main);
@@ -1125,7 +1390,7 @@ int EXPORT GB_INIT(void)
 	GB.GetInterface("gb.image", IMAGE_INTERFACE_VERSION, &IMAGE);
   IMAGE.SetDefaultFormat(GB_IMAGE_BGRP);
 	DRAW_init();
-	
+
 	CLASS_Control = GB.FindClass("Control");
 	CLASS_Container = GB.FindClass("Container");
 	CLASS_UserControl = GB.FindClass("UserControl");
@@ -1188,6 +1453,11 @@ int EXPORT GB_INFO(const char *key, void **value)
 		*value = (void *)QX11Info::appTime();
 		return TRUE;
 	}
+	else if (!strcasecmp(key, "DECLARE_TRAYICON"))
+	{
+		*value = (void *)declare_tray_icon;
+		return TRUE;
+	}
 	else
 		return FALSE;
 }
@@ -1200,13 +1470,13 @@ extern Time	qt_x_time;
 static void activate_main_window(intptr_t value)
 {
 	CWINDOW *active;
-	
+
 	active = CWINDOW_Active;
 	if (!active) active = CWINDOW_LastActive;
-	
+
 	if (!active)
 		return;
-	
+
 	MyMainWindow *win = (MyMainWindow *)active->widget.widget;
 	if (win && !win->isWindow())
 		win = (MyMainWindow *)win->window();
@@ -1224,19 +1494,21 @@ void EXPORT GB_SIGNAL(int signal, void *param)
 {
 	if (!qApp)
 		return;
-	
+
 	switch(signal)
 	{
 		case GB_SIGNAL_DEBUG_BREAK:
 			release_grab();
 			break;
-			
+
 		case GB_SIGNAL_DEBUG_FORWARD:
 			//while (qApp->activePopupWidget())
 			//	delete qApp->activePopupWidget();
+#ifndef QT5
 			qApp->syncX();
+#endif
 			break;
-			
+
 		case GB_SIGNAL_DEBUG_CONTINUE:
 			GB.Post((GB_CALLBACK)activate_main_window, 0);
 			unrelease_grab();
