@@ -85,7 +85,10 @@ void X11_exit();
 void X11_sync(void);
 
 /* Functions to deal with the _NET_WM_STATE and _NET_WM_TYPE property */
-void X11_window_change_property(Window window, bool visible, Atom property, bool set);
+void X11_window_change_property(Atom property, bool set);
+void X11_window_change_begin(Window window, bool visible);
+void X11_window_change_end();
+
 bool X11_window_has_property(Window window, Atom property);
 void X11_window_save_properties(Window window);
 void X11_window_restore_properties(Window window);
@@ -104,6 +107,8 @@ void X11_set_window_tool(Window window, int tool, Window parent);
 int X11_get_window_tool(Window window);
 void X11_window_set_desktop(Window window, bool visible, int desktop);
 int X11_window_get_desktop(Window window);
+void X11_window_set_user_time(Window win, int timestamp);
+
 int X11_get_current_desktop();
 
 int X11_get_window_type(Window window);
