@@ -2,7 +2,7 @@
 
   gambas.h
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -24,6 +24,10 @@
 #ifndef __GAMBAS_H
 #define __GAMBAS_H
 
+#ifdef __CYGWIN__
+#include <strings.h>
+#endif
+
 #include "config.h"
 #include <stdint.h>
 #include <stddef.h>
@@ -61,7 +65,7 @@
 #define EXPORT
 #endif
 
-#if defined(__cplusplus)
+#if defined(__cplusplus) && !defined(__clang__)
 	#define __null ((intptr_t)0)
 #else
 	#ifdef bool
@@ -285,7 +289,6 @@ typedef
 		int64_t _pad;
 		#endif
     }
-	PACKED
   GB_VARIANT;
 
 
@@ -297,6 +300,7 @@ typedef
 		GB_BOOLEAN _boolean;
 		GB_INTEGER _integer;
 		GB_LONG _long;
+		GB_SINGLE _single;
 		GB_FLOAT _float;
 		GB_DATE _date;
 		GB_STRING _string;
@@ -310,10 +314,13 @@ typedef
 /* Predefined errors constants */
 
 #define GB_ERR_TYPE       ((char *)6)
+#define GB_ERR_OVERFLOW   ((char *)7)
 #define GB_ERR_NSYMBOL    ((char *)11)
+#define GB_ERR_NWRITE     ((char *)16)
 #define GB_ERR_NPROPERTY  ((char *)17)
 #define GB_ERR_ARG        ((char *)20)
 #define GB_ERR_BOUND      ((char *)21)
+#define GB_ERR_ZERO       ((char *)26)
 
 
 /* Gambas description start macro */
@@ -347,11 +354,17 @@ typedef
 
 #define GB_VIRTUAL_CLASS() { GB_VIRTUAL_CLASS_ID }, { GB_NOT_CREATABLE_ID }
 
+#define GB_DECLARE_VIRTUAL(name) \
+	{ name, (intptr_t)GB_VERSION, (intptr_t)0 }, GB_VIRTUAL_CLASS()
+
 
 /* Not creatable class macro */
 
 #define GB_NOT_CREATABLE() { GB_NOT_CREATABLE_ID }
 
+#define GB_DECLARE_STATIC(name) \
+	{ name, (intptr_t)GB_VERSION, (intptr_t)0 }, GB_NOT_CREATABLE()
+
 
 /* Auto creatable class macro */
 
@@ -374,6 +387,9 @@ typedef
 #define GB_CONSTANT(symbol, type, value) \
 	{ "C" symbol, (intptr_t)type, (intptr_t)value }
 
+#define GB_FLOAT_CONSTANT(symbol, value) \
+	{ "C" symbol, (intptr_t)"f", (intptr_t)0, (intptr_t)0, (double)value }
+
 #define GB_PROPERTY(symbol, type, proc) \
 	{ "p" symbol, (intptr_t)type, (intptr_t)proc }
 
@@ -524,8 +540,12 @@ typedef
 		intptr_t val1;
 		intptr_t val2;
 		intptr_t val3;
-		intptr_t val4;
+		#if __WORDSIZE == 64
+		double val4;
 		intptr_t val5;
+		#else
+		double val4;
+		#endif
 		}
 	GB_DESC;
 
@@ -567,6 +587,10 @@ typedef
 #define GB_HOOK_ERROR     9
 #define GB_HOOK_TIMEOUT   10
 
+/* Macro for calling the previous hook */
+
+#define CALL_HOOK_MAIN(_hook, _pargc, _pargv) do { if (_hook) { ((void (*)(int *, char ***))(_hook))((_pargc), (_pargv)); } } while (0);
+
 /* Constants that represent interpreter signals catched by GB_SIGNAL function */
 
 #define GB_SIGNAL_DEBUG_BREAK         1
@@ -608,7 +632,13 @@ typedef
 /* Type of the GB.BrowseProject() callback */
 
 typedef
-	void (*GB_BROWSE_CALLBACK)(const char *, int64_t);
+	void (*GB_BROWSE_PROJECT_CALLBACK)(const char *, int64_t);
+
+
+/* Type of the GB.BrowseDirectory() callback */
+
+typedef
+	void (*GB_BROWSE_CALLBACK)(const char *);
 
 
 /* Type of a timer callback */
@@ -617,24 +647,24 @@ typedef
 	int (*GB_TIMER_CALLBACK)(intptr_t);
 
 
-/* Type of a posted function */
+/* Type of a signal callback */
 
 typedef
-	void (*GB_POST_FUNC)();
+	void (*GB_SIGNAL_CALLBACK)(int, intptr_t);
 
 
 /* A structure for the components of a date */
 
 typedef
 	struct {
-		short year;
-		short month;
-		short day;
-		short hour;
-		short min;
-		short sec;
-		short weekday;
-		short msec;
+		int year;
+		int month;
+		int day;
+		int hour;
+		int min;
+		int sec;
+		int weekday;
+		int msec;
 		}
 	GB_DATE_SERIAL;
 
@@ -656,6 +686,17 @@ typedef
 typedef
 	void *GB_ARRAY;
 
+typedef
+	struct {
+		int size;
+		int count;
+		GB_TYPE type;
+		void *data;
+		int *dim;
+		void *ref;
+		}
+	GB_ARRAY_BASE;
+
 
 /* Opaque type of a Gambas Collection */
 
@@ -685,6 +726,13 @@ typedef
 	void (*GB_HASHTABLE_ENUM_FUNC)(void *);
 
 
+/* Constants for end-of-line format */
+
+#define GB_EOL_UNIX      0
+#define GB_EOL_WINDOWS   1
+#define GB_EOL_MAC       2
+
+
 /* opaque type for a Stream object */
 
 struct GB_STREAM;
@@ -711,6 +759,7 @@ typedef
 		int64_t _reserved;
 		intptr_t _reserved2;
 		intptr_t _reserved3;
+		void *tag;
 		}
 	GB_STREAM_BASE;
 
@@ -753,25 +802,6 @@ typedef
 #define GB_SC_UNICODE ((char *)-1)
 
 
-/* Opaque type for a SubCollection object */
-
-typedef
-	void *GB_SUBCOLLECTION;
-
-
-/* SubCollection description */
-
-typedef
-	struct {
-		char *klass;
-		void *(*get)(void *, const char *);
-		int (*exist)(void *, const char *);
-		void (*list)(void *, char ***);
-		void (*release)(void *, void *);
-	}
-	GB_SUBCOLLECTION_DESC;
-
-
 /* Timer object */
 
 typedef
@@ -779,18 +809,36 @@ typedef
 		GB_BASE object;
 		intptr_t id;
 		intptr_t tag;
-		int delay;
+		unsigned delay : 24;
+		unsigned triggered : 1;
+		unsigned _reserved : 7;
+		#if __WORDSIZE == 64
+		int _pad;
+		#endif
 		GB_TIMER_CALLBACK callback;
 		}
 	GB_TIMER;
 
-/* Structure for GB.RaiseCatch handler */
+/* Structure for GB.OnErrorBegin() handler */
+
+typedef
+	struct {
+		void *prev;
+		void *context;
+		GB_CALLBACK handler;
+		intptr_t arg1;
+		intptr_t arg2;
+		}
+	GB_ERROR_HANDLER;
+	
+/* Structure for GB.RaiseBegin handler */
 
 typedef
 	struct {
 		void (*callback)(intptr_t);
 		intptr_t data;
 		void *old;
+		int level;
 	}
 	GB_RAISE_HANDLER;
 
@@ -799,6 +847,73 @@ typedef
 	
 #define POINTER(_pointer) (void **)(void *)_pointer
 
+/* For classes that implements arithmetic operators (e.g. complex numbers...) */
+
+typedef
+	struct {
+		int (*equal)(void *, void *, bool);
+		int (*equalf)(void *, double);
+		int (*equalo)(void *, void *, bool);
+		int (*comp)(void *, void *, bool);
+		int (*compf)(void *, double);
+		int (*compo)(void *, void *, bool);
+		void *(*add)(void *, void *, bool);
+		void *(*addf)(void *, double, bool);
+		void *(*addo)(void *, void *, bool);
+		void *(*sub)(void *, void *, bool);
+		void *(*subf)(void *, double, bool);
+		void *(*subo)(void *, void *, bool);
+		void *(*mul)(void *, void *, bool);
+		void *(*mulf)(void *, double, bool);
+		void *(*mulo)(void *, void *, bool);
+		void *(*div)(void *, void *, bool);
+		void *(*divf)(void *, double, bool);
+		void *(*divo)(void *, void *, bool);
+		void *(*pow)(void *, void *, bool);
+		void *(*powf)(void *, double, bool);
+		void *(*powo)(void *, void *, bool);
+		void *(*neg)(void *);
+		void *(*abs)(void *);
+		double (*fabs)(void *);
+		int (*sgn)(void *);
+		intptr_t _reserved;
+	}
+	GB_OPERATOR_DESC;
+
+/* Double-linked list API */
+
+typedef
+	struct {
+		void *next;
+		void *prev;
+	}
+	GB_LIST;
+
+/* Information about a file */
+	
+typedef
+	struct {
+		short type;
+		short mode;
+		int atime;
+		int mtime;
+		int ctime;
+		int64_t size;
+		uid_t uid;
+		gid_t gid;
+		char hidden;
+		}
+	GB_FILE_STAT;
+
+/* Constants for the GB_FILE_STAT structure */
+	
+#define GB_STAT_FILE        1
+#define GB_STAT_DIRECTORY   2
+#define GB_STAT_DEVICE      3
+#define GB_STAT_PIPE        4
+#define GB_STAT_SOCKET      5
+#define GB_STAT_LINK        6
+	
 /* Gambas Application Programming Interface */
 
 typedef
@@ -809,19 +924,28 @@ typedef
 
 		void *(*Hook)(int, void *);
 
-		bool (*LoadComponent)(const char *);
-		bool (*ExistComponent)(const char *);
-		char *(*CurrentComponent)(void);
-		bool (*GetComponentInfo)(const char *, void **);
+		struct {
+			bool (*Load)(const char *);
+			bool (*Exist)(const char *);
+			bool (*IsLoaded)(const char *);
+			char *(*Current)(void);
+			bool (*GetInfo)(const char *, void **);
+			void (*Signal)(int, void *);
+			void (*Declare)(GB_DESC *);
+		}
+		Component;
 
 		void (*Push)(int, ...);
 		bool (*GetFunction)(GB_FUNCTION *, void *, const char *, const char *, const char *);
 		GB_VALUE *(*Call)(GB_FUNCTION *, int, int);
 		void *(*GetClassInterface)(GB_CLASS, const char *);
-		void (*GetProperty)(void *, const char *);
-		void (*SetProperty)(void *, const char *, ...);
+		void *(*GetProperty)(void *, const char *);
+		bool (*SetProperty)(void *, const char *, GB_VALUE *value);
+		bool (*Serialize)(const char *path, GB_VALUE *value);
+		bool (*UnSerialize)(const char *path, GB_VALUE *value);
 
 		bool (*Loop)(int);
+		void (*Wait)(int);
 		void (*Post)(GB_CALLBACK, intptr_t);
 		void (*Post2)(GB_CALLBACK, intptr_t, intptr_t);
 		GB_TIMER *(*Every)(int, GB_TIMER_CALLBACK, intptr_t);
@@ -832,10 +956,9 @@ typedef
 		void (*CheckPost)(void);
 		bool (*CanRaise)(void *, int);
 		int (*GetEvent)(GB_CLASS, const char *);
-		char *(*GetLastEventName)();
+		char *(*GetLastEventName)(void);
 		void (*RaiseTimer)(void *);
 		bool (*Stopped)(void);
-		void (*Signal)(int, void *);
 
 		int (*NParam)(void);
 		bool (*Conv)(GB_VALUE *, GB_TYPE);
@@ -843,6 +966,9 @@ typedef
 		
 		void (*Error)(const char *, ...);
 		void (*Propagate)(void);
+		void (*Deprecated)(const char *, const char *, const char *);
+		void (*OnErrorBegin)(GB_ERROR_HANDLER *);
+		void (*OnErrorEnd)(GB_ERROR_HANDLER *);
 
 		GB_CLASS (*GetClass)(void *);
 		char *(*GetClassName)(void *);
@@ -853,20 +979,23 @@ typedef
 		bool (*Is)(void *, GB_CLASS);
 		void (*Ref)(void *);
 		void (*Unref)(void **);
-		void (*UnrefKeep)(void **, int);
+		//void (*UnrefKeep)(void **, int);
 		void (*Detach)(void *);
 		void (*Attach)(void *, void *, const char *);
 		void *(*Parent)(void *);
+		void *(*Create)(GB_CLASS, char *, void *);
 		void *(*New)(GB_CLASS, char *, void *);
 		void *(*AutoCreate)(GB_CLASS, int);
 		bool (*CheckObject)(void *);
 
-		void *(*GetEnum)();
-		void (*StopEnum)();
-		void (*ListEnum)(void *);
-		bool (*NextEnum)();
+		void *(*GetEnum)(void);
+		void (*StopEnum)(void);
+		void *(*BeginEnum)(void *);
+		void (*EndEnum)(void *);
+		bool (*NextEnum)(void);
 		void (*StopAllEnum)(void *);
 
+		GB_VALUE *(*GetReturnValue)(void);
 		void (*Return)(GB_TYPE, ...);
 		void (*ReturnInteger)(int);
 		void (*ReturnLong)(int64_t);
@@ -875,9 +1004,12 @@ typedef
 		void (*ReturnDate)(GB_DATE *);
 		void (*ReturnObject)(void *);
 		void (*ReturnNull)(void);
+		void (*ReturnSingle)(float);
 		void (*ReturnFloat)(double);
 		void (*ReturnVariant)(GB_VARIANT_VALUE *);
-		void (*ReturnConvVariant)();
+		void (*ReturnConvVariant)(void);
+		void (*ReturnBorrow)(void);
+		void (*ReturnRelease)(void);
 		void (*ReturnPtr)(GB_TYPE, void *);
 		void (*ReturnSelf)(void *);
 
@@ -892,6 +1024,7 @@ typedef
 		char *(*NewZeroString)(const char *);
 		char *(*TempString)(const char *, int);
 		void (*FreeString)(char **);
+		char *(*FreeStringLater)(char *);
 		char *(*ExtendString)(char *, int);
 		char *(*AddString)(char *, const char *, int);
 		char *(*AddChar)(char *, char);
@@ -914,7 +1047,9 @@ typedef
 		char *(*TempDir)(void);
 		char *(*TempFile)(const char *);
 		bool (*CopyFile)(const char *, const char *);
-		void (*BrowseProject)(GB_BROWSE_CALLBACK);
+		void (*BrowseProject)(GB_BROWSE_PROJECT_CALLBACK);
+		void (*BrowseDirectory)(const char *, GB_BROWSE_CALLBACK, GB_BROWSE_CALLBACK);
+		bool (*StatFile)(const char *, GB_FILE_STAT *, bool);
 
 		void (*Store)(GB_TYPE, GB_VALUE *, void *);
 		void (*StoreString)(GB_STRING *, char **);
@@ -923,6 +1058,7 @@ typedef
 		void (*ReadValue)(GB_VALUE *, void *, GB_TYPE);
 		void (*BorrowValue)(GB_VALUE *);
 		void (*ReleaseValue)(GB_VALUE *);
+		int (*CompVariant)(GB_VARIANT_VALUE *, GB_VARIANT_VALUE *);
 
 		GB_DATE_SERIAL *(*SplitDate)(GB_DATE *);
 		bool (*MakeDate)(GB_DATE_SERIAL *, GB_DATE *);
@@ -934,6 +1070,7 @@ typedef
 		GB_VALUE *(*Eval)(void *, void *);
 
 		void (*Alloc)(void **, int);
+		void (*AllocZero)(void **, int);
 		void (*Free)(void **);
 		void (*Realloc)(void **, int);
 
@@ -944,15 +1081,6 @@ typedef
 		void *(*Insert)(void *, int, int);
 		void (*Remove)(void *, int, int);
 
-		struct {
-			void (*New)(GB_SUBCOLLECTION *, GB_SUBCOLLECTION_DESC *, void *);
-			void (*Add)(void *, const char *, int, void *);
-			void (*Remove)(void *, const char *, int);
-			void *(*Get)(void *, const char *, int);
-			void *(*Container)(void *);
-			}
-		SubCollection;
-
 		int (*ToLower)(int);
 		int (*ToUpper)(int);
 		int (*StrCaseCmp)(const char *, const char *);
@@ -973,6 +1101,9 @@ typedef
 			char *(*DomainName)(void);
 			bool (*IsRightToLeft)(void);
 			char *(*Path)(void);
+			void (*HasForked)(void);
+			bool (*Debug)(void);
+			char *(*Home)(void);
 			}
 		System;
 
@@ -1009,31 +1140,49 @@ typedef
 			GB_STREAM *(*Get)(void *object);
 			void (*SetBytesRead)(GB_STREAM *stream, int length);
 			void (*SetSwapping)(GB_STREAM *stream, int swap);
+			void (*SetAvailableNow)(GB_STREAM *stream, int available_now);
 			bool (*Block)(GB_STREAM *stream, int block);
 			int (*Read)(GB_STREAM *stream, void *addr, int len);
 			int (*Write)(GB_STREAM *stream, void *addr, int len);
+			int (*GetReadable)(GB_STREAM *stream, int *len);
+			bool (*Eof)(GB_STREAM *stream);
 			}
 		Stream;
 
 		struct {
 			void (*Start)(int length);
-			char *(*End)();
+			char *(*End)(void);
 			void (*Add)(const char *src, int len);
 			}
 		String;
 		
 		struct {
-			char *(*GetCurrentPosition)();
+			char *(*GetCurrentPosition)(void);
+			void (*EnterEventLoop)(void);
+			void (*LeaveEventLoop)(void);
 			}
 		Debug;
 		
+		struct {
+			GB_SIGNAL_CALLBACK *(*Register)(int signum, void (*func)(int, intptr_t), intptr_t data);
+			void (*Unregister)(int signum, GB_SIGNAL_CALLBACK *cb);
 		}
+		Signal;
+		
+		struct {
+			void (*Add)(void *p_first, void *node, GB_LIST *list);
+			void (*Remove)(void *p_first, void *node, GB_LIST *list);
+		}
+		List;
+	
+	}
 	GB_INTERFACE;
 
 
 /*
 
   Special methods that can be declared in a class
+  -----------------------------------------------
 
   _get        array reading operator
   _put        array writing operator
@@ -1043,23 +1192,24 @@ typedef
   _call       called when the object or the class is used as a function
   _unknown    called when the name of the property or method is unknown
 
-*/
-
-/*
 
   Syntax of a method or event signature
+  -------------------------------------
 
   Gambas datatype      String representation
 
-  BOOLEAN              b
-  INTEGER              i
-  LONG                 l
-  FLOAT                f
-  DATE                 d
-  STRING               s
-  VARIANT              v
-  OBJECT               o
-  POINTER              p
+  Boolean              b
+  Byte                 c
+  Short                h
+  Integer              i
+  Long                 l
+  Single               g
+  Float                f
+  Date                 d
+  String               s
+  Variant              v
+  Object               o
+  Pointer              p
   Any class            ClassName;
 
 */
