@@ -95,6 +95,9 @@ void XMLNode_NewGBObject(Node *node)
     case Node::DocumentNode:
         node->GBObject = (CNode*)GB.New(GB.FindClass("XmlDocument"), 0, 0);
         break;
+    case Node::AttributeNode:
+        node->GBObject = (CNode*)GB.New(GB.FindClass("XmlNode"), 0, 0);
+        break;
     default:
         fprintf(stderr, "FATAL : tried to create a Gambas object with invalid type.");
         exit(EXIT_FAILURE);
@@ -388,10 +391,10 @@ void XMLNode_addGBChildrenByAttributeValue(Node *node, const char *attrName, con
 {
     if(node->type == Node::ElementNode)
     {
-        Attribute *attr = XMLElement_GetAttribute((Element*)node, attrName, lenAttrName, mode);
+        Attribute *attr = XMLElement_GetAttribute((Element*)node, attrName, lenAttrName);
         if(attr)
         {
-            if(GB_MatchString(attr->attrValue, attr->lenAttrValue, attrValue, lenAttrValue))
+            if(GB_MatchString(attr->attrValue, attr->lenAttrValue, attrValue, lenAttrValue, mode))
             {
                 *(reinterpret_cast<void **>((GB.Array.Add(*array)))) = XMLNode_GetGBObject(node);
                 GB.Ref(node->GBObject);
