@@ -2,7 +2,7 @@
 
   gb_error.c
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -44,6 +44,7 @@ static const char *_message[] =
 	/*  6 E_EXPECTED */ "&1 expected",
 	/*  7 E_MISSING */ "Missing &1",
 	/*  8 E_SYNTAX_MISSING */ "Syntax error. Missing &1",
+	/*  9 E_TOOLONG */ "File name is too long",
   NULL
 };
 
@@ -267,7 +268,7 @@ void ERROR_warning(const char *warning, ...)
 
   fflush(NULL);
 
-  fprintf(stderr, "gbc" GAMBAS_VERSION_STRING ": warning: ");
+  fprintf(stderr, "gbc: warning: ");
   vfprintf(stderr, warning, args);
   putc('\n', stderr);
 }
