@@ -36,6 +36,7 @@
 #include "gbc_compile.h"
 #include "gbc_trans.h"
 #include "gbc_header.h"
+#include "gb_code.h"
 
 /*#define DEBUG*/
 
@@ -460,7 +461,7 @@ static bool header_declaration(TRANS_DECL *decl)
 	bool is_static = FALSE;
 	bool is_public = FALSE;
 	bool is_const = FALSE;
-	//bool has_static = FALSE;
+	bool no_warning = FALSE;
 
 	/* static ! */
 
@@ -490,6 +491,12 @@ static bool header_declaration(TRANS_DECL *decl)
 		look++;
 	}
 
+	if (PATTERN_is(*look, RS_LBRA))
+	{
+		no_warning = TRUE;
+		look++;
+	}
+
 	if (!PATTERN_is_identifier(*look))
 		return FALSE;
 
@@ -498,6 +505,14 @@ static bool header_declaration(TRANS_DECL *decl)
 	decl->index = PATTERN_index(*look);
 	look++;
 
+	if (no_warning)
+	{
+		if (!PATTERN_is(*look, RS_RBRA))
+			return FALSE;
+		look++;
+		decl->no_warning = TRUE;
+	}
+
 	save = JOB->current;
 	JOB->current = look;
 
@@ -886,7 +901,7 @@ static bool header_library(void)
 		return FALSE;
 
 	if (!PATTERN_is_string(*JOB->current))
-		THROW("Library name must be a string");
+		THROW("Extern library name must be a string");
 
 	JOB->default_library = PATTERN_index(*JOB->current);
 	JOB->current++;
@@ -976,6 +991,35 @@ static bool header_structure(void)
 	return TRUE;
 }
 
+static bool header_use(void)
+{
+	if (!TRANS_is(RS_USE))
+		return FALSE;
+
+	CLASS_begin_init_function(JOB->class, FUNC_INIT_STATIC);
+
+	for(;;)
+	{
+		if (!PATTERN_is_string(*JOB->current))
+			THROW("Component name must be a string");
+
+		TRANS_string(*JOB->current);
+		TRANS_subr(TS_SUBR_USE, 1);
+		CODE_drop();
+
+		JOB->current++;
+
+		if (!TRANS_is(RS_COMMA))
+			break;
+	}
+
+	if (!PATTERN_is_newline(*JOB->current))
+		THROW("Syntax error");
+
+	return TRUE;
+}
+
+
 static void check_class_header()
 {
 	while (TRUE) //(JOB->current < JOB->end)
@@ -1074,6 +1118,9 @@ void HEADER_do(void)
 		if (header_library())
 			continue;
 
+		if (header_use())
+			continue;
+
 		/*if (PATTERN_is_command(*JOB->current))
 		{
 			JOB->current++;
