@@ -130,6 +130,8 @@ static void READ_exit(void)
 	int len;
 	int index;
 	bool local;
+	bool has_static;
+	char c;
 
 	local = FALSE;
 	
@@ -145,12 +147,30 @@ static void READ_exit(void)
 		}
 		else if (*name != '.')
 		{
+			has_static = FALSE;
+
+			for(;;)
+			{
+				c = name[len - 1];
+				if (c == '!')
+				{
+					has_static = TRUE;
+					len--;
+				}
+				else if (c == '?')
+					len--;
+				else
+					break;
+			}
+
 			if (TABLE_find_symbol(JOB->class->table, name, len, &index))
 			{
 				if (local)
-					CLASS_add_class_unused(JOB->class, index);
+					index = CLASS_add_class_unused(JOB->class, index);
 				else
-					CLASS_add_class_exported_unused(JOB->class, index);
+					index = CLASS_add_class_exported_unused(JOB->class, index);
+
+				JOB->class->class[index].has_static = has_static;
 			}
 		}
 	}
@@ -755,7 +775,7 @@ static void add_identifier()
 			break;
 			
 		__OTHERS:
-			if (last_type || last_identifier)
+			if (last_type || last_identifier || (PATTERN_is(last_pattern, RS_LBRA) && car == ')' && PATTERN_is_reserved(get_last_last_pattern())))
 				can_be_reserved = FALSE;
 			else
 				can_be_reserved = canres_car[car];
@@ -775,8 +795,11 @@ static void add_identifier()
 		index = RESERVED_find_subr(start, len);
 		if (index >= 0)
 		{
-			type = RT_SUBR;
-			goto __ADD_PATTERN;
+			if (COMP_subr_info[index].min_param == 0 || car == '(')
+			{
+				type = RT_SUBR;
+				goto __ADD_PATTERN;
+			}
 		}
 	}
 
@@ -977,7 +1000,7 @@ static void add_string()
 							i = xdigit_val(get_char_offset(2));
 							if (i >= 0)
 							{
-								car = (car << 4) | i;
+								car = (car << 4) | (uchar)i;
 								*p = car;
 								source_ptr += 2;
 								continue;
