@@ -2,7 +2,7 @@
 
   gbc_read.h
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -29,7 +29,7 @@
 #include <ctype.h>
 
 #undef isdigit
-#define isdigit(_c) (READ_digit_car[_c])
+#define isdigit(_c) (READ_digit_car[(uchar)(_c)])
 #undef isspace
 #define isspace(_c) (((uchar)_c) <= ' ')
 
