@@ -87,6 +87,7 @@ enum {
 	TS_SUBR_ERROR,
 	TS_SUBR_LOCK,
 	TS_SUBR_UNLOCK,
+	TS_SUBR_LOCK_WAIT,
 	TS_SUBR_INPUT_FROM,
 	TS_SUBR_OUTPUT_TO,
 	TS_SUBR_DEBUG,
@@ -98,7 +99,9 @@ enum {
 	TS_SUBR_OPEN_MEMORY,
 	TS_SUBR_CHMOD,
 	TS_SUBR_CHOWN,
-	TS_SUBR_CHGRP
+	TS_SUBR_CHGRP,
+	TS_SUBR_USE,
+	TS_SUBR_CHECK_EXEC,
 	};
 
 enum {
@@ -156,6 +159,7 @@ void TRANS_operation(short op, short nparam, bool output, PATTERN previous);
 void TRANS_new(void);
 TYPE TRANS_variable_get_type(void);
 void TRANS_class(int index);
+bool TRANS_string(PATTERN pattern);
 
 /* trans_tree.c */
 
@@ -241,6 +245,7 @@ void TRANS_error(void);
 void TRANS_scan(void);
 void TRANS_randomize(void);
 void TRANS_mid(void);
+void TRANS_use(void);
 
 #endif
 
