@@ -54,9 +54,9 @@ static void trans_subr(int subr, int nparam)
 		{ ".Flush" }, { ".Exec" }, { ".Shell" }, { ".Wait" }, { ".Kill" }, 
 		{ ".Move" }, { ".Mkdir" }, { ".Rmdir" }, { ".Array" }, {".Collection" }, 
 		{ ".Copy" }, { ".Link" },  { ".Error" }, { ".Lock" }, { ".Unlock" }, 
-		{ ".InputFrom" }, { ".OutputTo" }, { ".Debug" }, { ".Sleep" }, { ".Randomize" }, 
-		{ ".ErrorTo" }, { "Left" }, { "Mid" }, { ".OpenMemory" }, { ".Chmod" },
-		{ ".Chown" }, { ".Chgrp" }
+		{ ".LockWait" }, { ".InputFrom" }, { ".OutputTo" }, { ".Debug" }, { ".Sleep" },
+		{ ".Randomize" }, { ".ErrorTo" }, { "Left" }, { "Mid" }, { ".OpenMemory" },
+		{ ".Chmod" }, { ".Chown" }, { ".Chgrp" }, { ".Use" }, { ".CheckExec" }
 	};
 
 	TRANS_SUBR_INFO *tsi = &subr_info[subr];
@@ -68,13 +68,18 @@ static void trans_subr(int subr, int nparam)
 			ERROR_panic("Unknown intern subroutine: %s", tsi->name);
 	}
 
-	CODE_subr(tsi->info->opcode, nparam, tsi->info->optype, FALSE, tsi->info->min_param == tsi->info->max_param);
+	if (subr == TS_SUBR_ARRAY && nparam > MAX_PARAM_OP)
+		CODE_subr(tsi->info->opcode, MAX_PARAM_OP + 1, CODE_CALL_VARIANT + MAX_PARAM_OP, FALSE);
+	else if (subr == TS_SUBR_COLLECTION && nparam > MAX_PARAM_OP)
+		CODE_subr(tsi->info->opcode, MAX_PARAM_OP, CODE_CALL_VARIANT + MAX_PARAM_OP - 1, FALSE);
+	else
+		CODE_subr(tsi->info->opcode, nparam, tsi->info->optype, tsi->info->min_param == tsi->info->max_param);
 }
 
 
 static bool trans_stream_check(int default_stream, bool check)
 {
-	if (TRANS_is(RS_SHARP))
+	if (TRANS_is(RS_SHARP) || default_stream == TS_NONE)
 	{
 		TRANS_expression(FALSE);
 		
@@ -97,8 +102,8 @@ static bool trans_stream_check(int default_stream, bool check)
 	}
 	else
 	{
-		if (default_stream == TS_NONE)
-			THROW("Syntax error. &1 expected", "'#'");
+		//if (default_stream == TS_NONE)
+		//	THROW("Syntax error. &1 expected", "'#'");
 
 		CODE_push_number(default_stream);
 		return TRUE;
@@ -577,8 +582,14 @@ void TRANS_lock(void)
 		THROW("Useless LOCK");
 		
 	TRANS_expression(FALSE);
-	
-	trans_subr(TS_SUBR_LOCK, 1);
+
+	if (TRANS_is(RS_WAIT))
+	{
+		TRANS_expression(FALSE);
+		trans_subr(TS_SUBR_LOCK_WAIT, 2);
+	}
+	else
+		trans_subr(TS_SUBR_LOCK, 1);
 }
 
 
@@ -941,6 +952,8 @@ void TRANS_mid()
 	JOB->current = save;
 }
 
+
+
 #if 0
 void TRANS_scan(void)
 {
@@ -979,4 +992,3 @@ void TRANS_subr(int subr, int nparam)
 	trans_subr(subr, nparam);
 }
 
-
