@@ -45,11 +45,16 @@ static void analyze_expr(short priority, short op_main);
 static void analyze_array();
 
 
+static void THROW_EXPR_TOO_COMPLEX()
+{
+	THROW("Expression too complex");
+}
+
 static void inc_level()
 {
 	level++;
 	if (level > MAX_EXPR_LEVEL)
-	THROW("Expression too complex");
+		THROW_EXPR_TOO_COMPLEX();
 }
 
 
@@ -58,11 +63,6 @@ static void dec_level()
 	level--;
 }
 
-static void THROW_EXPR_TOO_COMPLEX()
-{
-	THROW("Expression too complex");
-}
-
 #define add_pattern(_pattern) \
 do { \
 	if (tree_length >= MAX_EXPR_PATTERN) \
@@ -244,8 +244,8 @@ static void analyze_make_array()
 		for(;;)
 		{
 			n++;
-			if (n > MAX_PARAM_OP)
-				THROW("Too many arguments");
+			/*if (n > MAX_PARAM_OP)
+				THROW("Too many arguments");*/
 			analyze_expr(0, RS_NONE);
 			
 			if (!checked)
@@ -260,14 +260,32 @@ static void analyze_make_array()
 					THROW(E_MISSING, "':'");
 				current++;
 				n++;
-				if (n > MAX_PARAM_OP)
-					THROW("Too many arguments");
+				/*if (n > MAX_PARAM_OP)
+					THROW("Too many arguments");*/
 				analyze_expr(0, RS_NONE);
 			}
 			
 			if (!PATTERN_is(*current, RS_COMMA))
 				break;
+
 			current++;
+
+			if (collection)
+			{
+				if (n == (MAX_PARAM_OP - 1))
+				{
+					add_operator(RS_COLON, MAX_PARAM_OP + 1);
+					n = 0;
+				}
+			}
+			else
+			{
+				if (n == MAX_PARAM_OP)
+				{
+					add_operator(RS_RSQR, MAX_PARAM_OP + 1);
+					n = 0;
+				}
+			}
 		}
 	}
 
@@ -493,6 +511,12 @@ static void analyze_call()
 			{
 				add_reserved_pattern(RS_OPTIONAL);
 			}
+			else if (optional && PATTERN_is(*current, RS_3PTS) && PATTERN_is(current[1], RS_RBRA))
+			{
+				current++;
+				add_reserved_pattern(RS_3PTS);
+				nparam_post--;
+			}
 			else
 			{
 				if (PATTERN_is(*current, RS_AT) || PATTERN_is(*current, RS_BYREF))
@@ -506,7 +530,7 @@ static void analyze_call()
 			}
 	
 			nparam_post++;
-	
+
 			if (nparam_post >= MAX_PARAM_FUNC)
 				THROW("Too many arguments");
 		}
@@ -755,8 +779,8 @@ void TRANS_tree(bool check_statement, TRANS_TREE **result, int *count)
 	#endif
 
 	tree_length = 0;
-
 	current = JOB->current;
+	level = 0;
 
 	TRY
 	{
