@@ -2,7 +2,7 @@
 
 	gb_common.c
 
-	(c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+	(c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
 	This program is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -64,8 +64,8 @@ void CHECK_enter(void)
 
 void CHECK_leave(void)
 {
-	SIGNAL_uninstall(&_SIGSEGV_handler);
-	SIGNAL_uninstall(&_SIGBUS_handler);
+	SIGNAL_uninstall(&_SIGSEGV_handler, SIGSEGV);
+	SIGNAL_uninstall(&_SIGBUS_handler, SIGBUS);
 }
 
 bool CHECK_got_error(void)
@@ -73,7 +73,7 @@ bool CHECK_got_error(void)
 	return _got_error;
 }
 
-bool CHECK_address(void *ptr, size_t len)
+bool CHECK_address(void *ptr, ssize_t len)
 {
 	offset_t i;
 
@@ -92,7 +92,7 @@ bool CHECK_address(void *ptr, size_t len)
 	return _got_error;
 }
 
-bool CHECK_strlen(char *ptr, size_t *len)
+bool CHECK_strlen(char *ptr, ssize_t *len)
 {
 	size_t l = 0;
 
