@@ -2,7 +2,7 @@
 
   gb_common_check.h
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -34,7 +34,7 @@ EXTERN sigjmp_buf CHECK_jump;
 void CHECK_enter(void);
 void CHECK_leave(void);
 bool CHECK_got_error(void);
-bool CHECK_address(void *ptr, size_t len);
-bool CHECK_strlen(char *ptr, size_t *len);
+bool CHECK_address(void *ptr, ssize_t len);
+bool CHECK_strlen(char *ptr, ssize_t *len);
 
 #endif
