@@ -67,6 +67,7 @@ FILE *log_file;
 
 static bool _welcome = FALSE;
 static bool _quit_after_main = FALSE;
+static bool _eval = FALSE;
 
 static void NORETURN my_exit(int ret)
 {
@@ -125,10 +126,12 @@ static void main_exit(bool silent)
 		EXTERN_release();
 		STREAM_exit();
 		OBJECT_exit();
+		CFILE_exit();
+
 		CLASS_clean_up(silent);
+
 		SUBR_exit();
 		DEBUG_exit();
-		CFILE_exit();
 		WATCH_exit();
 		#if USE_PROFILE
 		PROFILE_exit();
@@ -142,9 +145,16 @@ static void main_exit(bool silent)
 		FILE_exit();
 		STACK_exit();
 		ERROR_exit();
-		STRING_exit();
+	}
+	CATCH
+	{
+		if (!silent)
+			ERROR_print_at(stderr, _eval, TRUE);
+		_exit(1);
 	}
 	END_TRY
+
+	STRING_exit();
 }
 
 static bool is_option(const char *arg, char option)
@@ -186,6 +196,7 @@ int main(int argc, char *argv[])
 	int i, n;
 	char *file = NULL;
 	int ret = 0;
+	const char *redirect_stderr = NULL;
 
 	//char log_path[256];
 	//sprintf(log_path, "/tmp/gambas-%d.log", getuid());
@@ -224,14 +235,15 @@ int main(int argc, char *argv[])
 			printf(
 				"Options:\n"
 				"  -g               enter debugging mode\n"
+				"  -s <class>       override startup class\n"
 				"  -p <path>        activate profiling and debugging mode\n"
 				"  -k               do not unload shared libraries\n"
 				"  -H --httpd       run through an embedded http server\n"
 				);
+
 			if (!EXEC_arch)
-			{
 				printf("  -e               evaluate an expression\n");
-			}
+
 			printf(
 				"  -V --version     display version\n"
 				"  -L --license     display license\n"
@@ -265,6 +277,8 @@ int main(int argc, char *argv[])
 		if (argc < 3)
 			ERROR_fatal("-e option needs an expression.");
 		
+		_eval = TRUE;
+
 		TRY
 		{
 			init(NULL, argc, argv);
@@ -298,6 +312,17 @@ int main(int argc, char *argv[])
 		{
 			EXEC_fifo = TRUE;
 		}
+		else if (is_option_arg(argv, argc, &i, 's', &PROJECT_startup))
+		{
+			continue;
+		}
+		else if (is_option_arg(argv, argc, &i, 't', &redirect_stderr))
+		{
+			int fd = open(redirect_stderr, O_WRONLY | O_CLOEXEC);
+			if (fd < 0)
+				ERROR_fatal("cannot redirect stderr.");
+			dup2(fd, STDERR_FILENO);
+		}
 		else if (is_option(argv[i], 'k'))
 		{
 			EXEC_keep_library = TRUE;
@@ -368,7 +393,7 @@ int main(int argc, char *argv[])
 	{
 		ERROR_hook();
 		
-		if (EXEC_debug)
+		if (EXEC_debug && DEBUG_is_init())
 		{
 			if (!_welcome)
 				DEBUG.Main(TRUE);
@@ -417,7 +442,7 @@ int main(int argc, char *argv[])
 		{
 			ERROR_hook();
 			
-			if (EXEC_debug && DEBUG_is_init())
+			if (EXEC_debug)
 			{
 				DEBUG.Main(TRUE);
 				main_exit(TRUE);
