@@ -57,6 +57,8 @@ static bool _daemon = FALSE;
 extern char **environ;
 
 
+//-------------------------------------------------------------------------
+
 BEGIN_PROPERTY(Application_Path)
 
   GB_ReturnString(PROJECT_path);
@@ -99,12 +101,20 @@ BEGIN_PROPERTY(Application_Version)
 END_PROPERTY
 
 
+//-------------------------------------------------------------------------
+
 BEGIN_PROPERTY(Application_Args_Count)
 
   GB_ReturnInt(PROJECT_argc);
 
 END_PROPERTY
 
+BEGIN_PROPERTY(Application_Args_Max)
+
+  GB_ReturnInt(PROJECT_argc - 1);
+
+END_PROPERTY
+
 
 BEGIN_METHOD(Application_Args_get, GB_INTEGER index)
 
@@ -139,6 +149,26 @@ BEGIN_METHOD_VOID(Application_Args_next)
 END_METHOD
 
 
+BEGIN_PROPERTY(Application_Args_All)
+
+	GB_ARRAY array;
+	int i;
+	char *arg;
+
+	GB_ArrayNew(&array, GB_T_STRING, PROJECT_argc);
+	for (i = 0; i < PROJECT_argc; i++)
+	{
+		arg = PROJECT_argv[i];
+		if (arg && *arg)
+			*(char **)GB_ArrayGet(array, i) = STRING_new_zero(arg);
+	}
+
+  GB_ReturnObject(array);
+
+END_PROPERTY
+
+//-------------------------------------------------------------------------
+
 BEGIN_PROPERTY(Application_Env_Count)
 
 	int n = 0;
@@ -186,6 +216,8 @@ BEGIN_METHOD_VOID(Application_Env_next)
 
 END_METHOD
 
+//-------------------------------------------------------------------------
+
 static void init_again(int old_pid)
 {
 	char old[PATH_MAX];
@@ -252,11 +284,15 @@ END_PROPERTY
 
 #endif
 
+//-------------------------------------------------------------------------
+
 GB_DESC NATIVE_AppArgs[] =
 {
-  GB_DECLARE_VIRTUAL("Args"),
+  GB_DECLARE_STATIC("Args"),
 
   GB_STATIC_PROPERTY_READ("Count", "i", Application_Args_Count),
+  GB_STATIC_PROPERTY_READ("Max", "i", Application_Args_Max),
+  GB_STATIC_PROPERTY_READ("All", "String[]", Application_Args_All),
   GB_STATIC_METHOD("_get", "s", Application_Args_get, "(Index)i"),
   GB_STATIC_METHOD("_next", "s", Application_Args_next, NULL),
 
@@ -266,7 +302,7 @@ GB_DESC NATIVE_AppArgs[] =
 
 GB_DESC NATIVE_AppEnv[] =
 {
-  GB_DECLARE_VIRTUAL("Env"),
+  GB_DECLARE_STATIC("Env"),
 
   GB_STATIC_PROPERTY_READ("Count", "i", Application_Env_Count),
   GB_STATIC_METHOD("_get", "s", Application_Env_get, "(Key)s"),
@@ -279,7 +315,7 @@ GB_DESC NATIVE_AppEnv[] =
 
 GB_DESC NATIVE_App[] =
 {
-  GB_DECLARE_VIRTUAL("Application"),
+  GB_DECLARE_STATIC("Application"),
 
   GB_STATIC_PROPERTY_SELF("Args", "Args"),
   GB_STATIC_PROPERTY_SELF("Env", "Env"),
