@@ -78,6 +78,7 @@ void *CARRAY_get_data_multi(CARRAY *_object, GB_INTEGER *arg, int nparam);
 void *CARRAY_out_of_bound();
 CLASS *CARRAY_get_array_class(CLASS *class, CTYPE ctype);
 int *CARRAY_get_array_bounds(CARRAY *_object);
+void CARRAY_resize(CARRAY *_object, int size);
 
 CARRAY *CARRAY_create_static(CLASS *class, void *ref, CLASS_ARRAY *desc, void *data);
 int CARRAY_get_static_count(CLASS_ARRAY *desc);
@@ -96,6 +97,15 @@ void CARRAY_release_static(CLASS *class, CLASS_ARRAY *desc, void *data);
  	__data; \
 })
 
+#define CARRAY_get_data_throw(_array, _index) \
+({ \
+	int __index = (_index); \
+	CARRAY *__array = (CARRAY *)(_array); \
+  if ((__index < 0) || (__index >= __array->count)) \
+  	THROW(E_BOUND); \
+	(void *)((char *)(__array->data) + __index * __array->size); \
+})
+
 
 #endif  // #ifndef __GBX_CLASS_INFO_C 
 
