@@ -108,14 +108,21 @@ END_METHOD
 
 BEGIN_METHOD(Component_IsLoaded, GB_STRING name)
 
-	const char *name = GB_ToZeroString(ARG(name));
-	COMPONENT *comp;
-
-	comp = COMPONENT_find(name);
-	GB_ReturnBoolean(comp && comp->loaded);
+	GB_ReturnBoolean(COMPONENT_is_loaded(GB_ToZeroString(ARG(name))));
 
 END_METHOD
 
+BEGIN_METHOD(Component_FindFromPath, GB_STRING path)
+
+	ARCHIVE *arch = NULL;
+	const char *path = GB_ToZeroString(ARG(path));
+	ARCHIVE_find_from_path(&arch, &path);
+	if (arch)
+		GB_ReturnNewZeroString(arch->name);
+	else
+		GB_ReturnVoidString();
+
+END_METHOD
 
 //---- Classes ------------------------------------------------------------
 
@@ -636,6 +643,59 @@ BEGIN_METHOD(Object_Call, GB_OBJECT object; GB_STRING method; GB_OBJECT params)
 
 END_METHOD
 
+BEGIN_METHOD(Object_Raise, GB_OBJECT object; GB_STRING event; GB_OBJECT params)
+
+	CLASS *class;
+	CLASS_DESC *desc;
+	int index;
+	int i, np;
+	void *object = VARG(object);
+	GB_ARRAY params = VARGOPT(params, NULL);
+	int len = LENGTH(event);
+	char event[len + 2];
+
+	if (GB_CheckObject(object))
+		return;
+	
+	class = OBJECT_class(object);
+	
+	event[0] = ':';
+	memcpy(&event[1], STRING(event), len);
+	event[len + 1] = 0;
+	
+	index = CLASS_find_symbol(class, event);
+	if (index == NO_SYMBOL)
+		goto __UNKNOWN_EVENT;
+
+	desc = class->table[index].desc;
+	if (CLASS_DESC_get_type(desc) != CD_EVENT)
+		goto __UNKNOWN_EVENT;
+	
+	if (params)
+		np = GB_ArrayCount(params);
+	else
+		np = 0;
+
+	if (np)
+	{
+		STACK_check(np);
+
+		for (i = 0; i < np; i++)
+		{
+			CARRAY_get_value(params, i, SP);
+			PUSH();
+		}
+	}
+
+	GB_ReturnBoolean(GB_Raise(object, desc->event.index, -np));
+	return;
+	
+__UNKNOWN_EVENT:
+
+	GB_Error("Unknown event");
+	
+END_METHOD
+
 
 BEGIN_METHOD(Object_IsValid, GB_OBJECT object)
 
@@ -810,6 +870,7 @@ GB_DESC NATIVE_Component[] =
 	//GB_STATIC_METHOD("_get", "Component", library_get, "(Name)s"),
 	GB_STATIC_METHOD("Load", "Component", Component_Load, "(Name)s"),
 	GB_STATIC_METHOD("IsLoaded", "b", Component_IsLoaded, "(Name)s"),
+	GB_STATIC_METHOD("FindFromPath", "s", Component_FindFromPath, "(Path)s"),
 	GB_STATIC_PROPERTY_READ("Path", "s", Component_Path),
 
 	GB_PROPERTY_READ("Name", "s", Component_Name),
@@ -880,6 +941,7 @@ GB_DESC NATIVE_Object[] =
 	GB_STATIC_METHOD("SizeOf", "i", Object_SizeOf, "(Object)o"),
 	GB_STATIC_METHOD("Address", "p", Object_Address, "(Object)o"),
 	GB_STATIC_METHOD("CanRaise", "b", Object_CanRaise, "(Object)o(Event)s"),
+	GB_STATIC_METHOD("Raise", "b", Object_Raise, "(Object)o(Event)s[(Arguments)Array;]"),
 
 	GB_END_DECLARE
 };
