@@ -2,7 +2,7 @@
 
   gbx_c_enum.c
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -47,10 +47,10 @@ CENUM *CENUM_create(void *enum_object)
   CENUM *_object;
   
   _object = OBJECT_new(CLASS_Enum, NULL, NULL);
-  OBJECT_UNREF_KEEP(_object, "CENUM_create");
+  OBJECT_UNREF_KEEP(_object);
   
   THIS->enum_object = enum_object;
-  OBJECT_REF(enum_object, "CENUM_create");
+  OBJECT_REF(enum_object);
 
   LIST_insert(&_enum_list, THIS, &THIS->list);
   
@@ -77,13 +77,23 @@ BEGIN_METHOD_VOID(CENUM_free)
   #endif
 
   LIST_remove(&_enum_list, THIS, &THIS->list);
-  OBJECT_UNREF(THIS->enum_object, "CENUM_free");
+  OBJECT_UNREF(THIS->enum_object);
   
   if (THIS->variant)
     GB_StoreVariant(NULL, THIS->data);
 
 END_METHOD
 
+static bool check_enum()
+{
+	if (!EXEC_enum)
+	{
+		GB_Error("No current enumeration");
+		return TRUE;
+	}
+	else
+		return FALSE;
+}
 
 BEGIN_METHOD_VOID(CENUM_stop)
 
@@ -91,6 +101,9 @@ BEGIN_METHOD_VOID(CENUM_stop)
   fprintf(stderr, "CENUM_stop: %p <%p>\n", EXEC_enum, EXEC_enum->enum_object);
   #endif
   
+	if (check_enum())
+		return;
+
   EXEC_enum->stop = TRUE;
 
 END_METHOD
@@ -98,6 +111,9 @@ END_METHOD
 
 BEGIN_PROPERTY(CENUM_index)
 
+	if (check_enum())
+		return;
+
   _object = EXEC_enum;
 
   if (READ_PROPERTY)
@@ -112,7 +128,7 @@ BEGIN_PROPERTY(CENUM_index)
     else
     {
       #ifdef DEBUG_ME
-      fprintf(stderr, "CENUM_index: %p <%p>: -> value (%d)\n", THIS, THIS->enum_object, ((VARIANT *)THIS->data)->type);
+      fprintf(stderr, "CENUM_index: %p <%p>: -> value (%ld)\n", THIS, THIS->enum_object, ((VARIANT *)THIS->data)->type);
       #endif  
       GB_ReturnVariant((GB_VARIANT_VALUE *)THIS->data);
      }
@@ -162,6 +178,14 @@ BEGIN_METHOD_VOID(CENUM_next)
 
 END_METHOD
 
+BEGIN_PROPERTY(Enum_Stopped)
+
+	if (check_enum())
+		return;
+
+	GB_ReturnBoolean(EXEC_enum->stop);
+
+END_PROPERTY
 
 #endif
 
@@ -172,6 +196,7 @@ GB_DESC NATIVE_Enum[] =
   GB_METHOD("_free", NULL, CENUM_free, NULL),
   
   GB_STATIC_PROPERTY("Index", "v", CENUM_index),
+  GB_STATIC_PROPERTY_READ("Stopped", "b", Enum_Stopped),
   GB_STATIC_METHOD("Stop", NULL, CENUM_stop, NULL),
   
   GB_STATIC_METHOD("_next", NULL, CENUM_next, NULL),
