@@ -658,6 +658,61 @@ BEGIN_METHOD(File_Save, GB_STRING path; GB_STRING data)
 
 END_METHOD
 
+BEGIN_METHOD(File_IsRelative, GB_STRING path)
+
+	char *path = STRING(path);
+	int len = LENGTH(path);
+	
+	if (len <= 0)
+	{
+		GB_ReturnBoolean(FALSE);
+		return;
+	}
+	
+	GB_ReturnBoolean(FILE_is_relative(path));
+
+END_METHOD
+
+BEGIN_METHOD(File_IsHidden, GB_STRING path)
+
+	char *path = STRING(path);
+	int len = LENGTH(path);
+	int i;
+	char c;
+	
+	for (i = 0; i < len; i++)
+	{
+		if (path[i] != '.')
+			continue;
+		
+		if (i > 0 && path[i - 1] != '/')
+			continue;
+		
+		if (i == (len - 1))
+			continue;
+		
+		c = path[i + 1];
+		if (c == '/')
+			continue;
+		
+		if (c == '.')
+		{
+			if (i == (len - 2))
+				continue;
+		
+			if (path[i + 2] == '/')
+				continue;
+		}
+		
+		GB_ReturnBoolean(TRUE);
+		return;
+	}
+	
+	GB_ReturnBoolean(FALSE);
+
+END_METHOD
+
+//---------------------------------------------------------------------------
 
 BEGIN_PROPERTY(Stream_Handle)
 
@@ -893,6 +948,9 @@ GB_DESC NATIVE_File[] =
 	GB_STATIC_METHOD("SetExt", "s", File_SetExt, "(Path)s(NewExt)s"),
 	GB_STATIC_METHOD("SetBaseName", "s", File_SetBaseName, "(Path)s(NewBaseName)s"),
 
+	GB_STATIC_METHOD("IsRelative", "b", File_IsRelative, "(Path)s"),
+	GB_STATIC_METHOD("IsHidden", "b", File_IsHidden, "(Path)s"),
+
 	GB_STATIC_METHOD("Load", "s", File_Load, "(FileName)s"),
 	GB_STATIC_METHOD("Save", NULL, File_Save, "(FileName)s(Data)s"),
 
