@@ -56,6 +56,7 @@ typedef
     unsigned to_string : 1;
 		unsigned process_group : 1;
 		unsigned ignore : 1;
+		unsigned error : 1;
     char *result;
     GB_VARIANT_VALUE tag;
   }
@@ -74,5 +75,7 @@ enum
 
 CPROCESS *CPROCESS_create(int mode, void *cmd, char *name, CARRAY *env);
 void CPROCESS_wait_for(CPROCESS *process, int timeout);
+void CPROCESS_check(void *_object);
+const char *CPROCESS_search_program_in_path(char *name);
 
 #endif
