@@ -2,7 +2,7 @@
 
   gbx_class_desc.h
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -57,7 +57,6 @@ typedef
 		#endif
 		struct _CLASS *class;
 		}
-	PACKED
 	CLASS_DESC_PROPERTY;
 
 typedef
@@ -72,7 +71,6 @@ typedef
 		#endif
 		struct _CLASS *class;
 		}
-	PACKED
 	CLASS_DESC_VARIABLE;
 
 typedef
@@ -92,14 +90,13 @@ typedef
 		#endif
 		struct _CLASS *class;
 		}
-	PACKED
 	CLASS_DESC_METHOD;
 
 typedef
 	struct {
 		char *name;
 		TYPE type;                  // return value datatype - N/A
-		int *index;                 // event index
+		intptr_t index;             // event index
 		TYPE *signature;            // event signature
 		char npmin;                 // minimum number of arguments
 		char npmax;                 // maximum number of arguments
@@ -110,7 +107,6 @@ typedef
 		#endif
 		struct _CLASS *class;
 		}
-	PACKED
 	CLASS_DESC_EVENT;
 
 typedef
@@ -128,7 +124,6 @@ typedef
 		#endif
 		struct _CLASS *class;
 		}
-	PACKED
 	CLASS_DESC_EXTERN;
 
 typedef
@@ -151,7 +146,6 @@ typedef
 		#endif
 		struct _CLASS *class;
 		}
-	PACKED
 	CLASS_DESC_CONSTANT;
 
 typedef
@@ -159,7 +153,6 @@ typedef
 		char *name;
 		void (*func)();
 		}
-	PACKED
 	CLASS_DESC_HOOK;
 
 typedef
@@ -168,8 +161,10 @@ typedef
 		intptr_t type;
 		intptr_t val1;
 		intptr_t val2;
-		intptr_t val3;
-		intptr_t val4;
+		union {
+			double _double;
+			intptr_t _int[2];
+			}	val3;
 		}
 	CLASS_DESC_GAMBAS;
 
@@ -185,7 +180,6 @@ typedef
 		CLASS_DESC_GAMBAS gambas;
 		CLASS_DESC_EXTERN ext;
 		}
-	PACKED
 	CLASS_DESC;
 
 typedef
@@ -199,9 +193,11 @@ typedef
 
 
 #define CLASS_DESC_get_type(d) (*(d)->gambas.name)
+#define CLASS_DESC_is_static_method(d) (CLASS_DESC_get_type((CLASS_DESC *)d) == 'M')
 #define CLASS_DESC_SELF (-1)
 
 char *CLASS_DESC_get_signature(CLASS_DESC *cd);
 const char *CLASS_DESC_get_type_name(const CLASS_DESC *desc);
 
+
 #endif
