@@ -345,7 +345,7 @@ static void load_structure(CLASS *class, int *structure, int nfield)
 	else
 		sclass = CLASS_find(name);
 	
-	if (sclass->state)
+	if (CLASS_is_loaded(sclass))
 	{
 		if (!sclass->is_struct)
 			THROW_CLASS(class, "Class already exists: ", name);
@@ -466,7 +466,7 @@ static void load_structure(CLASS *class, int *structure, int nfield)
 	
 	sclass->is_struct = TRUE;
 	
-	sclass->state = CS_READY;
+	sclass->loaded = TRUE;
 	sclass->ready = TRUE;
 	return;
 	
@@ -496,6 +496,7 @@ static void load_and_relocate(CLASS *class, int len_data, CLASS_DESC **pstart, i
 	char *name;
 	int len;
 	bool have_jit_functions = FALSE;
+	uchar flag;
 	
 	ALLOC_ZERO(&class->load, sizeof(CLASS_LOAD));
 
@@ -576,12 +577,27 @@ static void load_and_relocate(CLASS *class, int len_data, CLASS_DESC **pstart, i
 	{
 		func = &class->load->func[i];
 		func->code = (ushort *)get_section("code", &section, NULL, _s);
+
+		flag = ((FUNCTION_FLAG *)func)->flag;
+
+		func->fast = (flag & 1) != 0;
+		func->optional = (func->npmin < func->n_param);
+		func->use_is_missing = (flag & 2) != 0;
+
 		if (func->fast)
 		{
 			func->fast = JIT_load();
 			if (func->fast)
 				have_jit_functions = TRUE;
 		}
+
+		if (func->use_is_missing)
+		{
+			func->stack_usage++;
+			func->n_ctrl++;
+		}
+
+		func->_reserved = 0;
 	}
 
 	/* Creation flags */
@@ -931,12 +947,15 @@ static void load_without_inits(CLASS *class, bool in_jit_compilation)
 	
 	//size_t alloc = MEMORY_size;
 	
-	if (class->state >= CS_LOADED)
+	if (CLASS_is_loaded(class))
 		return;
 
 	if (class->error)
 		THROW_CLASS(class, "Loading has already failed", "");
-		
+
+	if (CLASS_exiting)
+		THROW_CLASS(class, "Program is exiting", "");
+
 	class->error = TRUE;
 
 	#if DEBUG_LOAD
@@ -1224,7 +1243,7 @@ static void load_without_inits(CLASS *class, bool in_jit_compilation)
 
 	/* ...and usable ! */
 
-	class->state = CS_LOADED;
+	class->loaded = TRUE;
 	class->error = FALSE;
 
 	/* Init breakpoints */
@@ -1280,7 +1299,7 @@ void CLASS_load_real(CLASS *class)
 
 	_load_class_from_jit = FALSE;
 	
-	if (class->state == CS_NULL)
+	if (!CLASS_is_loaded(class))
 	{
 		if (len >= 3 && name[len - 2] == '[' && name[len - 1] == ']' && !class->array_type)
 		{
@@ -1293,12 +1312,12 @@ void CLASS_load_real(CLASS *class)
 	
 	if (load_from_jit)
 	{
-		class->state = CS_LOADED;
+		class->loaded = TRUE;
 		class->ready = FALSE;
 	}
 	else
 	{
-		class->state = CS_READY;
+		class->loaded = TRUE;
 		class->ready = TRUE;
 		
 		CLASS_run_inits(class);
