@@ -55,8 +55,8 @@
 #include "gbx_component.h"
 
 
-//#define DEBUG_COMP
-/*#define DEBUG_PRELOAD*/
+//#define DEBUG_COMP 1
+//#define DEBUG_PRELOAD
 
 COMPONENT *COMPONENT_current = NULL;
 COMPONENT *COMPONENT_main;
@@ -137,6 +137,7 @@ void COMPONENT_load_all(void)
 	_load_all = FALSE;
 }
 
+
 void COMPONENT_load_all_finish(void)
 {
 	COMPONENT *comp;
@@ -149,6 +150,12 @@ void COMPONENT_load_all_finish(void)
 	{
 		ARCHIVE_load_exported_class(comp->archive, AR_LOAD_ONLY);
 	}
+
+	LIST_for_each(comp, _component_list)
+	{
+		if (comp->library)
+			LIBRARY_after_init(comp->library);
+	}
 }
 
 
@@ -221,7 +228,7 @@ COMPONENT *COMPONENT_create(const char *name)
 		//fprintf(stderr, "COMPONENT_create: %s\n", path);
 
 		if (FILE_exist(path))
-			comp->library = LIBRARY_create(name);
+			comp->library = LIBRARY_create(comp->name);
 
 		if (can_archive)
 		{
@@ -229,7 +236,7 @@ COMPONENT *COMPONENT_create(const char *name)
 			sprintf(path, ARCH_PATTERN, COMPONENT_path, name);
 
 			if (FILE_exist(path))
-				comp->archive = ARCHIVE_create(name, NULL);
+				comp->archive = ARCHIVE_create(comp->name, NULL);
 		}
 	}
 
@@ -240,6 +247,24 @@ COMPONENT *COMPONENT_create(const char *name)
 	if (!comp->library && !comp->archive && !same_name_as_project)
 	{
 		COMPONENT_delete(comp);
+		
+#if 0
+		// If gb.qt5 components are not present, automatically switch to gb.qt4 components
+		
+		if (strncmp(name, "gb.qt5", 6) == 0 && (name[6] == 0 || name[6] == '.'))
+		{
+			char new_name[strlen(name) + 1];
+			
+			if (name[6] == 0)
+				ERROR_warning("gb.qt5 not found, using gb.qt4 instead.");
+			
+			strcpy(new_name, "gb.qt4");
+			strcat(new_name, &name[6]);
+			
+			return COMPONENT_create(new_name);
+		}
+#endif
+		
 		THROW(E_LIBRARY, name, "cannot find component");
 	}
 
@@ -385,3 +410,10 @@ void COMPONENT_exec(const char *name, int argc, char **argv)
 		LIBRARY_exec(comp->library, argc, argv);
 }
 
+bool COMPONENT_is_loaded(const char *name)
+{
+	COMPONENT *comp;
+
+	comp = COMPONENT_find(name);
+	return comp && comp->loaded;
+}
