@@ -61,24 +61,26 @@ bool EXEC_big_endian;
 /* Current iterator */
 CENUM *EXEC_enum;
 
+const char *EXEC_profile_path = NULL; // profile file path
+const char *EXEC_fifo_name = NULL; // fifo name
+EXEC_HOOK EXEC_Hook = { NULL };
+EXEC_GLOBAL EXEC;
+uint64_t EXEC_byref = 0;
+uchar EXEC_quit_value = 0;
+
 bool EXEC_debug = FALSE; // debugging mode
 bool EXEC_task = FALSE; // I am a background task
 bool EXEC_profile = FALSE; // profiling mode
 bool EXEC_profile_instr = FALSE; // profiling mode at instruction level
-const char *EXEC_profile_path = NULL; // profile file path
 bool EXEC_arch = FALSE; // executing an archive
 bool EXEC_fifo = FALSE; // debugging through a fifo
-const char *EXEC_fifo_name = NULL; // fifo name
 bool EXEC_keep_library = FALSE; // do not unload libraries
 bool EXEC_string_add = FALSE; // next '&' operator is done for a '&='
-uchar EXEC_quit_value = 0;
-EXEC_HOOK EXEC_Hook = { NULL };
-EXEC_GLOBAL EXEC;
 bool EXEC_main_hook_done = FALSE;
 bool EXEC_got_error = FALSE;
-uint64_t EXEC_byref = 0;
+bool EXEC_break_on_error = FALSE; // if we must break into the debugger as soon as there is an error.
 
-const char EXEC_should_borrow[] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 2, 2, 0, 0 };
+const char EXEC_should_borrow[] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 2, 2, 0, 0, 1 };
 
 const char *EXEC_unknown_name;
 bool EXEC_unknown_property;
@@ -419,9 +421,11 @@ void EXEC_enter(void)
 {
 	int i;
 	FUNCTION *func; // = EXEC.func;
+	bool optional;
 	int nparam = EXEC.nparam;
 	void *object = EXEC.object;
 	CLASS *class = EXEC.class;
+	int64_t optargs;
 
 	#if DEBUG_STACK
 	fprintf(stderr, "\n| >> EXEC_enter(%s, %d, %d)\n", EXEC.class->name, EXEC.index, nparam);
@@ -439,6 +443,8 @@ void EXEC_enter(void)
 		fprintf(stderr, "%s.%s\n", EXEC.class->name, func->debug->name);
 	#endif
 	
+	optional = func->optional;
+
 	// Check number of arguments
 
 	if (UNLIKELY(nparam < func->npmin))
@@ -451,14 +457,19 @@ void EXEC_enter(void)
 	for (i = 0; i < func->npmin; i++)
 		VALUE_conv(SP - nparam + i, func->param[i].type);
 
-	if (func->npmin < func->n_param)
+	if (optional)
 	{
+		optargs = 0;
+
 		// Optional arguments
 
 		for (i = func->npmin; i < Min(func->n_param, nparam); i++)
 		{
 			if (SP[- nparam + i].type == T_VOID)
+			{
+				optargs |= (1 << i);
 				SP[- nparam + i]._void.ptype = func->param[i].type;
+			}
 			else
 				VALUE_conv(SP - nparam + i, func->param[i].type);
 		}
@@ -471,6 +482,7 @@ void EXEC_enter(void)
 
 			for (i = nparam; i < func->n_param; i++)
 			{
+				optargs |= (1 << i);
 				SP->type = T_VOID;
 				SP->_void.ptype = func->param[i].type;
 				SP++;
@@ -528,6 +540,15 @@ void EXEC_enter(void)
 			SP->type = T_VOID;
 			SP++;
 		}
+
+		// Optional argument map
+		if (optional && func->use_is_missing)
+		{
+			SP--;
+			SP->type = T_LONG;
+			SP->_long.value = optargs;
+			SP++;
+		}
 	}
 
 	RP->type = T_VOID;
@@ -639,7 +660,7 @@ static int exec_leave_byref(ushort *pc, int nparam)
 	nbyref = 1 + (*pc & 0xF);
 	pc_func = FP->code;
 
-	if (LIKELY(!PCODE_is(*pc_func, C_BYREF)))
+	if (!PCODE_is(*pc_func, C_BYREF))
 		return 0;
 
 	nbyref_func = 1 + (*pc_func & 0xF);
@@ -674,9 +695,11 @@ static int exec_leave_byref(ushort *pc, int nparam)
 	}
 
 	pc--;
+
 	n = SP - PP;
 	RELEASE_MANY(SP, n);
 	SP -= nparam;
+
 	SP += nb;
 	OBJECT_UNREF(OP);
 	SP -= nb;
@@ -688,6 +711,7 @@ static int exec_leave_byref(ushort *pc, int nparam)
 	return nb;
 }
 
+#if 0
 #define EXEC_LEAVE_BYREF() \
 	pc++; \
 	nbyref = 1 + (*pc & 0xF); \
@@ -738,7 +762,7 @@ static int exec_leave_byref(ushort *pc, int nparam)
 	STACK_pop_frame(&EXEC_current); \
 	PC += nbyref + 1; \
 	goto __RETURN_VALUE;
-
+#endif
 
 void EXEC_leave_drop()
 {
@@ -923,21 +947,25 @@ void EXEC_function_loop()
 					//STACK_pop_frame(&EXEC_current);
 					PROPAGATE();
 				}
-				// We are in a TRY
-				else if (EP != NULL)
+
+				if (EXEC_break_on_error)
+					DEBUG.Main(TRUE);
+
+				// Are we in a TRY?
+				if (EP != NULL)
 				{
 					#if DEBUG_ERROR
 					fprintf(stderr, "#1 EP = %d  SP = %d\n", EP - (VALUE *)STACK_base, SP - (VALUE *)STACK_base);
 					fprintf(stderr, "TRY\n");
 					#endif
 					ERROR_set_last(FALSE);
-					
+
 					// No need to unwind the Gosub stack until the TRY stack position, because TRY GOSUB is forbidden
 					/*while (GP > EP)
 					{
 						...
 					}*/
-					
+
 					// The stack is popped until reaching the stack position before the TRY
 					while (SP > EP)
 						POP();
@@ -946,9 +974,11 @@ void EXEC_function_loop()
 					PC = EC;
 					EP = NULL;
 					retry = TRUE;
+					goto __CONTINUE;
 				}
-				// There is a CATCH in the function
-				else if (EC != NULL)
+
+				// Is there a CATCH in the function?
+				if (EC != NULL)
 				{
 					#if DEBUG_ERROR
 					fprintf(stderr, "#2 EC = %p\n", EC);
@@ -956,126 +986,128 @@ void EXEC_function_loop()
 					#endif
 
 					ERROR_set_last(TRUE);
-					
+
 					// The stack is popped until reaching the stack position at the function start
 					while (SP > (BP + FP->n_local + FP->n_ctrl))
 						POP();
-					
+
 					// Reset the Gosub stack pointer as all Gosub control variables saved have been released
 					GP = NULL;
-					
+
 					PC = EC;
 					EC = NULL;
 					retry = TRUE;
+					goto __CONTINUE;
 				}
+
 				// There is no error handler in the function
+
+				#if DEBUG_ERROR
+				fprintf(stderr, "#3\n");
+				fprintf(stderr, "NOTHING\n");
+				#endif
+				//ERROR_INFO save = { 0 };
+				//ERROR_save(&save);
+
+				ERROR_set_last(TRUE);
+
+				if (EXEC_debug && !STACK_has_error_handler())
+				{
+					ERROR_hook();
+					DEBUG.Main(TRUE);
+
+					if (TP && TC)
+					{
+						ERROR_lock();
+						while (BP > TP)
+						{
+							EXEC_leave_drop();
+							if (!PC)
+								STACK_pop_frame(&EXEC_current);
+						}
+						while (SP > TP)
+							POP();
+						PC = TC;
+						ERROR_unlock();
+					}
+
+					retry = TRUE;
+				}
 				else
 				{
-					#if DEBUG_ERROR
-					fprintf(stderr, "#3\n");
-					fprintf(stderr, "NOTHING\n");
-					#endif
-					//ERROR_INFO save = { 0 };
-					//ERROR_save(&save);
+					// We leave stack frames until we find either:
+					// - A stack frame that has an error handler.
+					// - A void stack frame created by EXEC_enter_real()
 
-					ERROR_set_last(TRUE);
+					// First we leave stack frames for JIT functions
+					// on top of the stack. We have just propagated
+					// past these some lines below.
+					ERROR_lock();
+					while (PC != NULL && EC == NULL && FP->fast)
+						EXEC_leave_drop();
+					ERROR_unlock();
 
-					if (EXEC_debug && !STACK_has_error_handler())
+					// We can only leave stack frames for non-JIT functions.
+					ERROR_lock();
+					while (PC != NULL && EC == NULL && !FP->fast)
+						EXEC_leave_drop();
+					ERROR_unlock();
+
+					// If the JIT function has set up an exception handler, call that now.
+					// If not, we must still propagate past that JIT function.
+					if (PC != NULL && FP->fast)
+						PROPAGATE();
+
+					// If we got the void stack frame, then we remove it and raise the error again
+					if (PC == NULL)
 					{
-						ERROR_hook();
-						DEBUG.Main(TRUE);
-						
-						if (TP && TC)
-						{
-							ERROR_lock();
-							while (BP > TP)
-							{
-								EXEC_leave_drop();
-								if (!PC)
-									STACK_pop_frame(&EXEC_current);
-							}
-							while (SP > TP)
-								POP();
-							PC = TC;
-							ERROR_unlock();
-						}
+						/*printf("try to propagate\n");*/
+						STACK_pop_frame(&EXEC_current);
 
-						retry = TRUE;
+						//ERROR_restore(&save);
+						//ERROR_set_last();
+						PROPAGATE();
 					}
-					else
+
+					// We have a TRY too, handle it.
+					if (EP != NULL)
 					{
-						// We leave stack frames until we find either:
-						// - A stack frame that has an error handler.
-						// - A void stack frame created by EXEC_enter_real()
-						
-						// First we leave stack frames for JIT functions
-						// on top of the stack. We have just propagated
-						// past these some lines below.
-						ERROR_lock();
-						while (PC != NULL && EC == NULL && FP->fast)
-							EXEC_leave_drop();
-						ERROR_unlock();
-						
-						// We can only leave stack frames for non-JIT functions.
-						ERROR_lock();
-						while (PC != NULL && EC == NULL && !FP->fast)
-							EXEC_leave_drop();
-						ERROR_unlock();
-
-						// If the JIT function has set up an exception handler, call that now.
-						// If not, we must still propagate past that JIT function.
-						if (PC != NULL && FP->fast)
-							PROPAGATE();
-
-						// If we got the void stack frame, then we remove it and raise the error again
-						if (PC == NULL)
-						{
-							/*printf("try to propagate\n");*/
-							STACK_pop_frame(&EXEC_current);
-							
-							//ERROR_restore(&save);
-							//ERROR_set_last();
-							PROPAGATE();
-						}
-
-						// We have a TRY too, handle it.
-						if (EP != NULL)
-						{
-							#if DEBUG_ERROR
-							fprintf(stderr, "#4 EP = %d  SP = %d\n", EP - (VALUE *)STACK_base, SP - (VALUE *)STACK_base);
-							#endif
-
-							ERROR_lock();
-							while (SP > EP)
-								POP();
-							ERROR_unlock();
-
-							EP = NULL;
-							/* On va directement sur le END TRY */
-						}
-
-						// Now we can handle the CATCH
-						
-						// The stack is popped until reaching the stack position at the function start
-						ERROR_lock();
 						#if DEBUG_ERROR
-						DEBUG_where();
-						fprintf(stderr, "#5 BP + local + ctrl = %d  SP = %d\n", BP + FP->n_local + FP->n_ctrl - (VALUE *)STACK_base, SP - (VALUE *)STACK_base);
+						fprintf(stderr, "#4 EP = %d  SP = %d\n", EP - (VALUE *)STACK_base, SP - (VALUE *)STACK_base);
 						#endif
-						while (SP > (BP + FP->n_local + FP->n_ctrl))
+
+						ERROR_lock();
+						while (SP > EP)
 							POP();
 						ERROR_unlock();
-					
-						PC = EC;
-						EC = NULL;
 
-						retry = TRUE;
+						EP = NULL;
+						/* On va directement sur le END TRY */
 					}
-					
-					//ERROR_restore(&save);
-					//ERROR_set_last();
+
+					// Now we can handle the CATCH
+
+					// The stack is popped until reaching the stack position at the function start
+					ERROR_lock();
+					#if DEBUG_ERROR
+					DEBUG_where();
+					fprintf(stderr, "#5 BP + local + ctrl = %d  SP = %d\n", BP + FP->n_local + FP->n_ctrl - (VALUE *)STACK_base, SP - (VALUE *)STACK_base);
+					#endif
+					while (SP > (BP + FP->n_local + FP->n_ctrl))
+						POP();
+					ERROR_unlock();
+
+					PC = EC;
+					EC = NULL;
+
+					retry = TRUE;
 				}
 
+				//ERROR_restore(&save);
+				//ERROR_set_last();
+
+			__CONTINUE:
+
 				while (SP < EXEC_super)
 					EXEC_super = ((VALUE *)EXEC_super)->_object.super;
 			}
@@ -2004,3 +2036,30 @@ void EXEC_push_complex(void)
 	OBJECT_REF(ob);
 	SP++;
 }
+
+void EXEC_push_vargs(void)
+{
+	int i;
+	int nargs = (FP && FP->vararg) ? BP - PP : 0;
+
+	if (nargs == 0)
+		return;
+
+	STACK_check(nargs);
+
+	for (i = 0; i < nargs; i++)
+	{
+		*SP = PP[i];
+		PUSH();
+	}
+}
+
+void EXEC_drop_vargs(void)
+{
+	int nargs = (FP && FP->vararg) ? BP - PP : 0;
+
+	if (nargs == 0)
+		return;
+
+	RELEASE_MANY(SP, nargs);
+}
