@@ -101,6 +101,7 @@ extern bool EXEC_fifo;
 extern const char *EXEC_fifo_name;
 extern bool EXEC_keep_library;
 extern bool EXEC_string_add;
+extern bool EXEC_break_on_error;
 
 extern EXEC_HOOK EXEC_Hook;
 
@@ -176,6 +177,14 @@ void EXEC_loop(void);
 		EXEC_object_other(_val, _pclass, _pobject); \
 })
 
+#define EXEC_object_array(_val, _vclass, _vobject) \
+	_vobject = (_val)->_object.object; \
+	if (!(_vobject)) \
+		THROW(E_NULL); \
+	_vclass = (_vobject)->class; \
+	if ((_val) == EXEC_super) \
+		EXEC_super = (_val)->_object.super;
+
 CLASS *EXEC_object_real(VALUE *val, OBJECT **pobject);
 CLASS *EXEC_object_variant(VALUE *val, OBJECT **pobject);
 bool EXEC_object_other(VALUE *val, CLASS **pclass, OBJECT **pobject);
@@ -202,7 +211,7 @@ void EXEC_special_inheritance(int special, CLASS *class, OBJECT *object, int npa
 
 void EXEC_nop(void);
 
-void EXEC_push_unknown(ushort code);
+void EXEC_push_unknown(void);
 void EXEC_push_array(ushort code);
 int EXEC_push_unknown_event(bool unknown);
 //void EXEC_push_special(void);
@@ -228,6 +237,9 @@ void RELEASE_many(VALUE *val, int n);
 
 void EXEC_push_complex(void);
 
+void EXEC_push_vargs(void);
+void EXEC_drop_vargs(void);
+
 #define BORROW(_value) \
 do { \
 	VALUE *_v = (_value); \
