@@ -74,7 +74,7 @@ void STREAM_exit(void)
 	STREAM_close(&_temp_stream);
 }
 
-static void wait_for_fd_ready_to_read(fd)
+static void wait_for_fd_ready_to_read(int fd)
 {
 	if (fd >= 0)
 		WATCH_process(fd, -1, 0);
@@ -189,6 +189,9 @@ void STREAM_open(STREAM *stream, const char *path, int mode)
 		if (FILE_is_relative(path) && !((mode & ST_DIRECT) && path[0] == '.' && isdigit(path[1])))
 		{
 			ARCHIVE *arch = NULL;
+			const char *tpath = path;
+			
+			/*ARCHIVE *arch = NULL;
 			
 			if (strncmp(path, "../", 3))
 				ARCHIVE_get_current(&arch);
@@ -199,10 +202,18 @@ void STREAM_open(STREAM *stream, const char *path, int mode)
 			{
 				sclass = &STREAM_arch;
 				goto _OPEN;
-			}
+			}*/
 
 			if ((mode & ST_ACCESS) != ST_READ || mode & ST_PIPE)
 				THROW(E_ACCESS);
+			
+			if (!ARCHIVE_find_from_path(&arch, &tpath))
+			{
+				sclass = &STREAM_arch;
+				goto __OPEN;
+			}
+			
+			path = tpath;
 		}
 
 		if (mode & ST_DIRECT)
@@ -211,7 +222,7 @@ void STREAM_open(STREAM *stream, const char *path, int mode)
 			sclass = &STREAM_buffer;
 	}
 
-_OPEN:
+__OPEN:
 
 	stream->common.mode = mode;
 	stream->common.swap = FALSE;
@@ -245,8 +256,7 @@ _OPEN:
 	#endif
 }
 
-
-void STREAM_release(STREAM *stream)
+static void release_buffer(STREAM *stream)
 {
 	if (stream->common.buffer)
 	{
@@ -257,7 +267,11 @@ void STREAM_release(STREAM *stream)
 		stream->common.buffer_pos = 0;
 		stream->common.buffer_len = 0;
 	}
-	
+}
+
+void STREAM_release(STREAM *stream)
+{
+	release_buffer(stream);
 	STREAM_cancel(stream);
 }
 
@@ -539,6 +553,8 @@ void STREAM_seek(STREAM *stream, int64_t pos, int whence)
 				THROW_SYSTEM(errno, NULL);
 		}
 	}
+	
+	release_buffer(stream);
 }
 
 static void fill_buffer(STREAM *stream, char *addr, bool do_not_wait_ready)
