@@ -67,17 +67,10 @@ static int stream_close(STREAM *stream)
 
 static int stream_read(STREAM *stream, char *buffer, int len)
 {
-	bool ret = STREAM_read_direct(FDR, buffer, len);
-
-  return ret;
+	return STREAM_read_direct(FDR, buffer, len);
 }
 
-static int stream_getchar(STREAM *stream, char *buffer)
-{
-	bool ret = read(FDR, buffer, 1) <= 0;
-
-  return ret;
-}
+#define stream_getchar NULL
 
 static int stream_write(STREAM *stream, char *buffer, int len)
 {
