@@ -211,7 +211,7 @@ void SUBR_exist(ushort code);
 void SUBR_dir(ushort code);
 void SUBR_kill(ushort code);
 void SUBR_mkdir(ushort code);
-void SUBR_rmdir(void);
+void SUBR_rmdir(ushort code);
 void SUBR_move(ushort code);
 void SUBR_link(ushort code);
 void SUBR_temp(ushort code);
@@ -229,7 +229,7 @@ void SUBR_alloc(ushort code);
 void SUBR_free(void);
 void SUBR_realloc(ushort code);
 void SUBR_strptr(ushort code);
-void SUBR_varptr(void);
+void SUBR_varptr(ushort code);
 void SUBR_ptr(ushort code);
 void SUBR_make(ushort code);
 
@@ -238,7 +238,7 @@ void SUBR_make(ushort code);
 void SUBR_error(void);
 void SUBR_shell(void);
 void SUBR_wait(ushort code);
-void SUBR_sleep(void);
+void SUBR_sleep(ushort code);
 void SUBR_exec(ushort code);
 void SUBR_eval(ushort code);
 void SUBR_array(ushort code);
