@@ -67,29 +67,59 @@ void SUBR_wait(ushort code)
 {
 	SUBR_ENTER();
 
+	EXEC_set_native_error(FALSE);
+
 	if (NPARAM == 0)
 		GB_Wait(0);
 	else
 		GB_Wait((int)(SUBR_get_float(PARAM) * 1000 + 0.5));
+
+	if (EXEC_has_native_error())
+	{
+		EXEC_set_native_error(FALSE);
+		PROPAGATE();
+	}
 	
 	SUBR_LEAVE_VOID();
 }
 
 
-void SUBR_sleep(void)
+void SUBR_sleep(ushort code)
 {
-	double wait;
-	struct timespec rem;
-
 	SUBR_ENTER_PARAM(1);
 
-	wait = SUBR_get_float(PARAM);
+	switch(code & 0x3F)
+	{
+		case 0: // Sleep
+		{
+			double wait;
+			struct timespec rem;
 
-	rem.tv_sec = (time_t)(int)wait;
-	rem.tv_nsec = (int)(frac(wait) * 1E9);
+			wait = SUBR_get_float(PARAM);
 
-	while (nanosleep(&rem, &rem) < 0);
+			rem.tv_sec = (time_t)(int)wait;
+			rem.tv_nsec = (int)(frac(wait) * 1E9);
 
+			while (nanosleep(&rem, &rem) < 0);
+			break;
+		}
+		case 1: // Use
+		{
+			char *name = SUBR_get_string(PARAM);
+			COMPONENT *comp = COMPONENT_find(name);
+			if (!comp)
+				comp = COMPONENT_create(name);
+
+			COMPONENT_load(comp);
+			break;
+		}
+		case 2: // CheckExec
+		{
+			CPROCESS_check(PARAM->_object.object);
+			break;
+		}
+	}
+	
 	SUBR_LEAVE();
 }
 
@@ -290,45 +320,87 @@ _FREE:
 
 void SUBR_array(ushort code)
 {
+	static bool reuse = FALSE;
+
 	TYPE type;
-	int i;
-	GB_ARRAY array;
+	int i, j;
+	CARRAY *array;
+	bool next_reuse;
 
 	SUBR_ENTER();
 
-	type = SUBR_check_good_type(PARAM, NPARAM);
-	
-	if (type == T_NULL)
-		type = T_OBJECT;
-	
-	for (i = 0; i < NPARAM; i++)
-		VALUE_conv(&PARAM[i], type);
-	
-	GB_ArrayNew(&array, type, NPARAM);
-	OBJECT_REF(array);
+	next_reuse = code & CODE_CALL_VARIANT;
+
+	if (reuse)
+	{
+		array = (CARRAY *)(PARAM[-1]._object.object);
+		type = array->type;
+	}
+	else
+	{
+		type = SUBR_check_good_type(PARAM, NPARAM);
+
+		if (type == T_NULL)
+			type = T_OBJECT;
+	}
 
 	for (i = 0; i < NPARAM; i++)
+		VALUE_conv(&PARAM[i], type);
+
+	if (reuse)
 	{
-		GB_Store(type, (GB_VALUE *)&PARAM[i], GB_ArrayGet(array, i));
+		j = array->count;
+		CARRAY_resize(array, j + NPARAM);
+	}
+	else
+	{
+		j = 0;
+		GB_ArrayNew(POINTER(&array), type, NPARAM);
+		OBJECT_REF(array);
+	}
+
+	for (i = 0; i < NPARAM; i++, j++)
+	{
+		GB_Store(type, (GB_VALUE *)&PARAM[i], GB_ArrayGet(array, j));
 		RELEASE(&PARAM[i]);
 	}
-		
-	PARAM->_object.class = OBJECT_class(array); //CLASS_Array;
-	PARAM->_object.object = array;
-	SP = PARAM + 1;
+
+	if (reuse)
+	{
+		SP = PARAM;
+	}
+	else
+	{
+		PARAM->_object.class = OBJECT_class(array); //CLASS_Array;
+		PARAM->_object.object = array;
+		SP = PARAM + 1;
+	}
+
+	reuse = next_reuse;
 }
 
 void SUBR_collection(ushort code)
 {
+	static bool reuse = FALSE;
+
 	int i;
 	GB_COLLECTION col;
 	char *key;
 	int len;
 	VALUE *vkey, *vval;
+	bool next_reuse;
 
 	SUBR_ENTER();
 
-	GB_CollectionNew(&col, GB_COMP_BINARY);
+	next_reuse = code & CODE_CALL_VARIANT;
+
+	if (reuse)
+		col = (GB_COLLECTION)(PARAM[-1]._object.object);
+	else
+	{
+		GB_CollectionNew(&col, GB_COMP_BINARY);
+		OBJECT_REF(col);
+	}
 
 	for (i = 0; i < NPARAM; i += 2)
 	{
@@ -345,10 +417,18 @@ void SUBR_collection(ushort code)
 		RELEASE(&PARAM[i + 1]);
 	}
 
-	OBJECT_REF(col);
-	PARAM->_object.class = OBJECT_class(col); //CLASS_Array;
-	PARAM->_object.object = col;
-	SP = PARAM + 1;
+	if (reuse)
+	{
+		SP = PARAM;
+	}
+	else
+	{
+		PARAM->_object.class = OBJECT_class(col); //CLASS_Array;
+		PARAM->_object.object = col;
+		SP = PARAM + 1;
+	}
+
+	reuse = next_reuse;
 }
 
 
