@@ -34,6 +34,7 @@
 #include "gbx_regexp.h"
 #include "gbx_class.h"
 #include "gbx_string.h"
+#include "gbx_split.h"
 #include "gbx_c_array.h"
 #include "gbx_local.h"
 #include "gbx_compare.h"
@@ -677,8 +678,10 @@ void SUBR_split(ushort code)
 	CARRAY *array;
 	char *str;
 	int lstr;
-	char *sep = "";
-	char *esc = "";
+	char *sep = NULL;
+	int lsep = 0;
+	char *esc = NULL;
+	int lesc = 0;
 	bool no_void = FALSE;
 	bool keep_esc = FALSE;
 
@@ -687,14 +690,12 @@ void SUBR_split(ushort code)
 	VALUE_conv_string(PARAM);
 	VALUE_get_string(PARAM, &str, &lstr);
 
-	//SUBR_get_string_len(&PARAM[0], &str, &lstr);
-	
 	if (NPARAM >= 2)
 	{
-		sep = SUBR_get_string(&PARAM[1]);
+		SUBR_get_string_len(&PARAM[1], &sep, &lsep);
 		if (NPARAM >= 3)
 		{
-			esc = SUBR_get_string(&PARAM[2]);
+			SUBR_get_string_len(&PARAM[2], &esc, &lesc);
 			if (NPARAM >= 4)
 			{
 				no_void = SUBR_get_boolean(&PARAM[3]);
@@ -704,18 +705,7 @@ void SUBR_split(ushort code)
 		}
 	}
 
-	array = OBJECT_create(CLASS_StringArray, NULL, NULL, 0);
-
-	if (lstr)
-	{
-		if (*sep) STRING_ref(sep);
-		if (*esc) STRING_ref(esc);
-
-		CARRAY_split(array, str, lstr, sep, esc, no_void, keep_esc);
-
-		if (*sep) STRING_unref(&sep);
-		if (*esc) STRING_unref(&esc);
-	}
+	array = STRING_split(str, lstr, sep, lsep, esc, lesc, no_void, keep_esc);
 
 	RETURN->_object.class = CLASS_StringArray;
 	RETURN->_object.object = array;
@@ -865,11 +855,16 @@ static int _is_punct(int c)
 	return ((c > 32) && (c < 128) && !(_is_letter(c) || _is_digit(c)));
 }
 
+static int _is_alnum(int c)
+{
+	return _is_letter(c) || _is_digit(c);
+}
+
 void SUBR_is_chr(ushort code)
 {
 	static void *jump[] =
 	{
-		NULL, _is_ascii, _is_letter, _is_lower, _is_upper, _is_digit, _is_hexa, _is_space, _is_blank, _is_punct
+		NULL, _is_ascii, _is_letter, _is_lower, _is_upper, _is_digit, _is_hexa, _is_space, _is_blank, _is_punct, _is_alnum
 	};
 
 	char *addr;
@@ -924,9 +919,17 @@ void SUBR_tr(void)
 	SUBR_LEAVE();
 }
 
+static void make_hex_char(uchar c)
+{
+	static const char hex_digit[] = "0213456789ABCDEF";
+
+	STRING_make_char(hex_digit[c >> 4]);
+	STRING_make_char(hex_digit[c & 7]);
+}
+
 void SUBR_quote(ushort code)
 {
-	static void *jump[4] = { &&__QUOTE, &&__SHELL, &&__HTML, &&__BASE64 };
+	static void *jump[8] = { &&__QUOTE, &&__SHELL, &&__HTML, &&__BASE64, &&__URL , &&__ILLEGAL, &&__ILLEGAL, &&__ILLEGAL };
 	char *str;
 	int lstr;
 	int i;
@@ -942,7 +945,7 @@ void SUBR_quote(ushort code)
 	
 	STRING_start_len(lstr);
 	
-	goto *jump[code & 0x3];
+	goto *jump[code & 0x7];
 	
 __QUOTE:
 	
@@ -988,9 +991,17 @@ __SHELL:
 	
 	// TODO: The following works with bash, but not with dash!
 	
+	STRING_make_char('\'');
+
 	for (i = 0; i < lstr; i++)
 	{
 		c = str[i];
+		if (c == '\'')
+		{
+			STRING_make("'\\'", 3);
+		}
+		STRING_make_char(c);
+		/*
 		if (c == '\n')
 			STRING_make("$'\\n'", 5);
 		else if (c == '\r')
@@ -1009,8 +1020,11 @@ __SHELL:
 			STRING_make_char('\\');
 			STRING_make_char(c);
 		}
+		*/
 	}
 	
+	STRING_make_char('\'');
+
 	goto __END;
 
 __HTML:
@@ -1028,7 +1042,7 @@ __HTML:
 			STRING_make("&quot;", 6);
 		else if (c == '\'')
 			STRING_make("&#x27;", 6);
-		else if (c == 0xC2 && str[i + 1] == 0xA0)
+		else if (c == 0xC2 && (uchar)str[i + 1] == 0xA0)
 		{
 			STRING_make("&nbsp;", 6);
 			i++;
@@ -1066,9 +1080,69 @@ __BASE64:
 			STRING_make(out, 4);
 		}
 	}
+
+	goto __END;
+
+__URL:
+
+	// Warning! '/' is not encoded, so that the function is more pratical, by supposing that no file url will have '/' in its name.
+
+	for (i = 0; i < lstr; i++)
+	{
+		c = str[i];
+		if (c == ' ')
+			STRING_make_char('+');
+		else if ((c >= '0' && c <= '9') || (c >= 'A' && c <= 'Z') || (c >= 'a' && c <= 'z') || index("-._~,$!/", c))
+			STRING_make_char(c);
+		else
+		{
+			STRING_make_char('%');
+			make_hex_char(c);
+		}
+	}
+
+	goto __END;
+
+#if 0
+__JAVASCRIPT:
+	{
+		STRING_make_char('\'');
+
+		for (i = 0; i < lstr; i++)
+		{
+			c = str[i];
+			if (c >= ' ' && c <= 126 && c != '\\' && c != '\'')
+				STRING_make_char(c);
+			else
+			{
+				STRING_make_char('\\');
+				if (c == '\n')
+					c = 'n';
+				else if (c == '\r')
+					c = 'r';
+				else if (c == '\t')
+					c = 't';
+				else if (!(c == '\'' || c == '\\'))
+				{
+					snprintf(buf, sizeof(buf), "x%02X", c);
+					STRING_make(buf, 3);
+					continue;
+				}
+				STRING_make_char(c);
+			}
+		}
+
+		STRING_make_char('\'');
+	}
 	
 	goto __END;
 
+#endif
+
+__ILLEGAL:
+
+	THROW_ILLEGAL();
+
 __END:
 
 	RETURN->type = T_STRING;
@@ -1093,7 +1167,7 @@ static int read_hex_digit(unsigned char c)
 
 void SUBR_unquote(ushort code)
 {
-	static void *jump[2] = { &&__UNQUOTE, &&__UNBASE64 };
+	static void *jump[4] = { &&__UNQUOTE, &&__FROM_BASE64, &&__FROM_URL, &&__ILLEGAL };
 	
 	char *str;
 	int lstr;
@@ -1150,7 +1224,7 @@ __UNQUOTE:
 	
 	goto __END;
 	
-__UNBASE64:
+__FROM_BASE64:
 
 	{
 		char buf[4];
@@ -1189,6 +1263,31 @@ __UNBASE64:
 	}
 	
 	goto __END;
+
+__FROM_URL:
+
+	for (i = 0; i < lstr; i++)
+	{
+		c = str[i];
+		if (c == '+')
+			c = ' ';
+		else if (c == '%')
+		{
+			if (i >= (lstr - 2))
+				break;
+
+			c = (read_hex_digit(str[i + 1]) << 4) + read_hex_digit(str[i + 2]);
+			i += 2;
+		}
+
+		STRING_make_char(c);
+	}
+
+	goto __END;
+
+__ILLEGAL:
+
+	THROW_ILLEGAL();
 	
 __END:
 	
@@ -1202,6 +1301,9 @@ __END:
 
 void SUBR_swap(ushort code)
 {
+	char *src, *dst;
+	int len, i, j;
+
 	if (!(code & 0xFF))
 	{
 		SUBR_move(1);
@@ -1210,29 +1312,29 @@ void SUBR_swap(ushort code)
 	
 	SUBR_ENTER();
 	
-	if (NPARAM == 1 || (SUBR_get_integer(&PARAM[2]) == GB_BIG_ENDIAN) != EXEC_big_endian)
+	if (NPARAM == 2 && (SUBR_get_integer(&PARAM[1]) == GB_BIG_ENDIAN) == EXEC_big_endian)
 	{
-		char *src, *dst;
-		int len, i, j;
-		
-		if (SUBR_check_string(PARAM))
-			STRING_void_value(RETURN);
-		else
+		SP--;
+		return;
+	}
+
+	if (SUBR_check_string(PARAM))
+		STRING_void_value(RETURN);
+	else
+	{
+		len = PARAM->_string.len;
+		if (len > 0)
 		{
-			len = PARAM->_string.len;
-			if (len > 0)
-			{
-				src = PARAM->_string.addr + PARAM->_string.start;
-				dst = STRING_new_temp(NULL, PARAM->_string.len);
-			
-				for (i = 0, j = len - 1; i < len; i++,j--)
-					dst[i] = src[j];
-				
-				RETURN->type = T_STRING;
-				RETURN->_string.addr = dst;
-				RETURN->_string.start = 0;
-				RETURN->_string.len = len;
-			}
+			src = PARAM->_string.addr + PARAM->_string.start;
+			dst = STRING_new_temp(NULL, PARAM->_string.len);
+
+			for (i = 0, j = len - 1; i < len; i++,j--)
+				dst[i] = src[j];
+
+			RETURN->type = T_STRING;
+			RETURN->_string.addr = dst;
+			RETURN->_string.start = 0;
+			RETURN->_string.len = len;
 		}
 	}
 	
