@@ -2,7 +2,7 @@
 
   gbx_subr_test.c
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -309,6 +309,8 @@ void SUBR_if(ushort code)
 			if (TYPE_is_object(type))
 				type = T_OBJECT;
 			*PC |= (unsigned char)type;
+			
+			VALUE_conv(&PARAM[i], type);
 		}
 	}
 	else if (test != 0x1F)
@@ -331,7 +333,7 @@ void SUBR_choose(ushort code)
   VALUE_conv_integer(PARAM);
   val = PARAM->_integer.value;
 
-  if (val >= 1 && val <= NPARAM)
+  if (val >= 1 && val < NPARAM)
   {
     VALUE_conv_variant(&PARAM[val]);
     *RETURN = PARAM[val];
@@ -875,7 +877,7 @@ void SUBR_is(ushort code)
 	else
 		res = (OBJECT_class(object) == klass || CLASS_inherits(OBJECT_class(object), klass));
 
-	OBJECT_UNREF(object, "SUBR_is");
+	OBJECT_UNREF(object);
 
 	P1->type = T_BOOLEAN;
 	P1->_boolean.value = -(res ^ (code & 1));
