@@ -2,7 +2,7 @@
 
   gbx_value.c
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -54,7 +54,7 @@ static bool unknown_function(VALUE *value)
 		EXEC_special(SPEC_UNKNOWN, value->_function.class, value->_function.object, 0, FALSE);
 
 		//object = value->_function.object;
-		OBJECT_UNREF(value->_function.object, "VALUE_conv");
+		OBJECT_UNREF(value->_function.object);
 
 		SP--;
 		//*val = *SP;
@@ -249,7 +249,7 @@ void VALUE_convert(VALUE *value, TYPE type)
 {
 	static const void *jump[16][16] =
 	{
-	/*  ,------->  void       b          c          h          i          l          g          f          d          cs         s          p          v          func       class      n         */
+	/*   ,------>  void       b          c          h          i          l          g          f          d          cs         s          p          v          func       class      n         */
 	//  |
 	/* void   */ { &&__OK,    &&__NR,    &&__NR,    &&__NR,    &&__NR,    &&__NR,    &&__NR,    &&__NR,    &&__NR,    &&__NR,    &&__NR,    &&__NR,    &&__NR,    &&__NR,    &&__NR,    &&__NR,    },
 	/* b      */ { &&__N,     &&__OK,    &&__b2c,   &&__b2h,   &&__TYPE,  &&__b2l,   &&__b2g,   &&__b2f,   &&__N,     &&__b2s,   &&__b2s,   &&__N,     &&__2v,    &&__N,     &&__N,     &&__N,     },
@@ -262,11 +262,11 @@ void VALUE_convert(VALUE *value, TYPE type)
 	/* d      */ { &&__N,     &&__d2b,   &&__d2c,   &&__d2h,   &&__d2i,   &&__d2l,   &&__d2g,   &&__d2f,   &&__OK,    &&__d2s,   &&__d2s,   &&__N,     &&__2v,    &&__N,     &&__N,     &&__N,     },
 	/* cs     */ { &&__N,     &&__s2b,   &&__s2c,   &&__s2h,   &&__s2i,   &&__s2l,   &&__s2g,   &&__s2f,   &&__s2d,   &&__OK,    &&__OK,    &&__N,     &&__s2v,   &&__N,     &&__N,     &&__N,     },
 	/* s      */ { &&__N,     &&__s2b,   &&__s2c,   &&__s2h,   &&__s2i,   &&__s2l,   &&__s2g,   &&__s2f,   &&__s2d,   &&__OK,    &&__OK,    &&__N,     &&__s2v,   &&__N,     &&__N,     &&__N,     },
-	/* p      */ { &&__N,     &&__N,     &&__N,     &&__N,     &&__p2i,   &&__p2l,   &&__N,     &&__N,     &&__N,     &&__N,     &&__N,     &&__OK,    &&__2v,    &&__N,     &&__N,     &&__N,     },
+	/* p      */ { &&__N,     &&__N,     &&__N,     &&__N,     &&__p2i,   &&__p2l,   &&__N,     &&__N,     &&__N,     &&__p2s,   &&__p2s,   &&__OK,    &&__2v,    &&__N,     &&__N,     &&__N,     },
 	/* v      */ { &&__N,     &&__v2,    &&__v2,    &&__v2,    &&__v2,    &&__v2,    &&__v2,    &&__v2,    &&__v2,    &&__v2,    &&__v2,    &&__v2,    &&__OK,    &&__N,     &&__v2,    &&__v2,    },
 	/* func   */ { &&__N,     &&__func,  &&__func,  &&__func,  &&__func,  &&__func,  &&__func,  &&__func,  &&__func,  &&__func,  &&__func,  &&__F2p,   &&__func,  &&__OK,    &&__N,     &&__func,  },
 	/* class  */ { &&__N,     &&__N,     &&__N,     &&__N,     &&__N,     &&__N,     &&__N,     &&__N,     &&__N,     &&__N,     &&__N,     &&__N,     &&__2v,    &&__N,     &&__OK,    &&__N,     },
-	/* null   */ { &&__N,     &&__n2b,   &&__N,     &&__N,     &&__N,     &&__N,     &&__N,     &&__N,     &&__n2d,   &&__n2s,   &&__n2s,   &&__N,     &&__2v,    &&__N,     &&__N,     &&__OK,    },
+	/* null   */ { &&__N,     &&__n2b,   &&__N,     &&__N,     &&__N,     &&__N,     &&__N,     &&__N,     &&__n2d,   &&__n2s,   &&__n2s,   &&__n2p,     &&__2v,    &&__N,     &&__N,     &&__OK,    },
 	};
 
 	int len;
@@ -569,6 +569,15 @@ __f2s:
 	BORROW(value);
 	return;
 
+__p2s:
+	#if OS_64BITS
+		NUMBER_int_to_string((int64_t)(intptr_t)value->_pointer.value, 0, 16, value);
+	#else
+		NUMBER_int_to_string((int)(intptr_t)value->_pointer.value, 0, 16, value);
+	#endif
+	BORROW(value);
+	return;
+	
 __d2s:
 
 	len = DATE_to_string(COMMON_buffer, value);
@@ -657,6 +666,12 @@ __n2s:
 	STRING_void_value(value);
 	return;
 
+__n2p:
+
+	value->_pointer.value = 0;
+	value->type = T_POINTER;
+	return;
+	
 __v2:
 
 	VALUE_undo_variant(value);
@@ -719,7 +734,7 @@ __OBJECT:
 		if (type == T_BOOLEAN)
 		{
 			test = (value->_object.object != NULL);
-			OBJECT_UNREF(value->_object.object, "VALUE_convert");
+			OBJECT_UNREF(value->_object.object);
 			value->_boolean.value = -test;
 			value->type = T_BOOLEAN;
 			return;
@@ -728,6 +743,30 @@ __OBJECT:
 		if (type == T_VARIANT)
 			goto __2v;
 		
+		if (!value->_object.object)
+			goto __N;
+		
+		if (value->type == T_OBJECT)
+			class = OBJECT_class(value->_object.object);
+		else
+			class = value->_object.class;
+
+		if (class->has_convert)
+		{
+			void *unref = value->_object.object;
+			TYPE old_type = value->type;
+
+			if (!((*class->convert)(value->_object.object, type, value)))
+			{
+				OBJECT_UNREF(unref);
+				
+				if (value->type == old_type)
+					goto __TYPE;
+				else
+					goto __OK;
+			}
+		}
+		
 		goto __N;
 	}
 
@@ -735,10 +774,22 @@ __OBJECT:
 	{
 		if (value->type == T_NULL)
 		{
-			OBJECT_null(value, (CLASS *)type); /* marche aussi pour type = T_OBJECT */
+			OBJECT_null(value, (CLASS *)type); // Also works if type == T_OBJECT
 			goto __TYPE;
 		}
 
+		if (value->type == T_POINTER)
+		{
+			class = (CLASS *)type;
+			
+			if (CLASS_is_struct(class))
+			{
+				value->_object.object = CSTRUCT_create_static(STRUCT_CONST, class, value->_pointer.value);
+				OBJECT_REF(value->_object.object);
+				goto __TYPE;
+			}
+		}
+		
 		if (value->type == T_VARIANT)
 			goto __v2;
 
@@ -759,25 +810,35 @@ __OBJECT:
 			else
 				value->_object.object = class;
 
-			OBJECT_REF(value->_object.object, "VALUE_convert");
+			OBJECT_REF(value->_object.object);
 			value->type = T_OBJECT;
 			/* on continue... */
 		}
 		else
+		{
+			if (TYPE_is_pure_object(type))
+			{
+				class = (CLASS *)type;
+
+				if (class->has_convert)
+				{
+					if (!((*class->convert)(NULL, value->type, value)))
+					{
+						OBJECT_REF(value->_object.object);
+						goto __TYPE;
+					}
+				}
+			}
+			
 			goto __N;
+		}
 	}
 
 	if (value->_object.object == NULL)
 		goto __TYPE;
 
 	if (value->type == T_OBJECT)
-	{
-		/*if (value->_object.object == NULL)
-			goto __TYPE;*/
-
 		class = OBJECT_class(value->_object.object);
-		/* on continue */
-	}
 	else
 		class = value->_object.class;
 
@@ -799,14 +860,25 @@ __RETRY:
 		goto __RETRY;
 	}
 
-	if (class->special[SPEC_CONVERT] != NO_SYMBOL)
+	if (class->has_convert)
 	{
-		void *conv = ((void *(*)())(CLASS_get_desc(class, class->special[SPEC_CONVERT])->constant.value._pointer))(value->_object.object, type);
-		if (conv)
+		void *unref = value->_object.object;
+		if (!((*class->convert)(value->_object.object, type, value)))
 		{
-			OBJECT_REF(conv, "VALUE_conv");
-			OBJECT_UNREF(value->_object.object, "VALUE_conv");
-			value->_object.object = conv;
+			OBJECT_UNREF(unref);
+			OBJECT_REF(value->_object.object);
+			goto __TYPE;
+		}
+	}
+	
+	CLASS *class2 = (CLASS *)type;
+	if (class2->has_convert)
+	{
+		void *unref = value->_object.object;
+		if (!((*class2->convert)(NULL, OBJECT_class(unref), value)))
+		{
+			OBJECT_UNREF(unref);
+			OBJECT_REF(value->_object.object);
 			goto __TYPE;
 		}
 	}
@@ -914,8 +986,8 @@ __OBJECT:
 
 	VALUE_conv(value, type);
 
-	OBJECT_REF(value->_object.object, "VALUE_write");
-	OBJECT_UNREF(*((void **)addr), "VALUE_write");
+	OBJECT_REF(value->_object.object);
+	OBJECT_UNREF(*((void **)addr));
 	*((void **)addr) = value->_object.object;
 	return;
 
@@ -923,8 +995,8 @@ __CLASS:
 
 	VALUE_conv(value, type);
 
-	OBJECT_REF(value->_class.class, "VALUE_write");
-	OBJECT_UNREF(*((void **)addr), "VALUE_write");
+	OBJECT_REF(value->_class.class);
+	OBJECT_UNREF(*((void **)addr));
 	*((void **)addr) = value->_class.class;
 	return;
 
@@ -1088,7 +1160,7 @@ void VALUE_free(void *addr, TYPE type)
 	}
 	else if (TYPE_is_object(type))
 	{
-		OBJECT_UNREF(*((void **)addr), "VALUE_free");
+		OBJECT_UNREF(*((void **)addr));
 		*((void **)addr) = NULL;
 	}
 	else if (type == T_VARIANT)
@@ -1175,12 +1247,29 @@ __STRING:
 
 __OBJECT:
 
-	if (VALUE_is_null(value))
-		goto __NULL;
+	{
+		CLASS *class;
+		
+		if (VALUE_is_null(value))
+			goto __NULL;
 
-	*len = sprintf(COMMON_buffer, "(%s %p)", OBJECT_class(value->_object.object)->name, value->_object.object);
-	*addr = COMMON_buffer;
-	return;
+		class = OBJECT_class(value->_object.object);
+		if (class->has_convert)
+		{
+			VALUE temp;
+			if (!((*class->convert)(value->_object.object, T_CSTRING, &temp)))
+			{
+				*addr = temp._string.addr + temp._string.start;
+				*len = temp._string.len;
+				STRING_free_later(*addr);
+				return;
+			}
+		}
+		
+		*len = sprintf(COMMON_buffer, "(%s %p)", class->name, value->_object.object);
+		*addr = COMMON_buffer;
+		return;
+	}
 
 __POINTER:
 
@@ -1265,8 +1354,8 @@ void VALUE_class_write(CLASS *class, VALUE *value, char *addr, CTYPE ctype)
 		
 		VALUE_conv(value, type);
 
-		OBJECT_REF(value->_object.object, "VALUE_class_write");
-		OBJECT_UNREF(*((void **)addr), "VALUE_class_write");
+		OBJECT_REF(value->_object.object);
+		OBJECT_UNREF(*((void **)addr));
 		*((void **)addr) = value->_object.object;
 		//VALUE_write(value, addr, (ctype.value >= 0) ? (TYPE)class->load->class_ref[ctype.value] : T_OBJECT);
 	}
@@ -1756,7 +1845,7 @@ __NR:
 	THROW(E_NRETURN);
 }
 
-
+#if 0
 void VALUE_convert_object(VALUE *value, TYPE type)
 {
 	CLASS *class;
@@ -1769,7 +1858,7 @@ __CONV:
 		if (type == T_BOOLEAN)
 		{
 			test = (value->_object.object != NULL);
-			OBJECT_UNREF(value->_object.object, "VALUE_convert");
+			OBJECT_UNREF(value->_object.object);
 			value->_boolean.value = test ? -1 : 0;
 			goto __TYPE;
 		}
@@ -1809,7 +1898,7 @@ __CONV:
 			else
 				value->_object.object = class;
 
-			OBJECT_REF(value->_object.object, "VALUE_convert");
+			OBJECT_REF(value->_object.object);
 			value->type = T_OBJECT;
 			/* on continue... */
 		}
@@ -1855,8 +1944,8 @@ __RETRY:
 		void *conv = ((void *(*)())(CLASS_get_desc(class, class->special[SPEC_CONVERT])->constant.value._pointer))(value->_object.object, type);
 		if (conv)
 		{
-			OBJECT_REF(conv, "VALUE_conv");
-			OBJECT_UNREF(value->_object.object, "VALUE_conv");
+			OBJECT_REF(conv);
+			OBJECT_UNREF(value->_object.object);
 			value->_object.object = conv;
 			goto __TYPE;
 		}
@@ -1885,7 +1974,7 @@ __N:
 
 	THROW(E_TYPE, TYPE_get_name(type), TYPE_get_name(value->type));
 }
-
+#endif
 
 void VALUE_undo_variant(VALUE *value)
 {
