@@ -2,7 +2,7 @@
 
   gbx_variant.h
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -28,6 +28,8 @@
 #include "gbx_string.h"
 #include "gbx_object.h"
 
+// On ARM 32 bits, sizeof(VARIANT) = 12, so it must be packed like GB_VARIANT
+
 typedef
   struct 
 	{
@@ -48,7 +50,7 @@ typedef
 			}
 		value;
   }
-	PACKED
+  PACKED
   VARIANT;
 
 #define VARIANT_copy_value(_dst, _src) (_dst)->value.data = (_src)->value.data
@@ -67,7 +69,7 @@ typedef
   } \
   else if (TYPE_is_object((_var)->type)) \
   { \
-    OBJECT_UNREF((_var)->value._object, "VARIANT_free"); \
+    OBJECT_UNREF((_var)->value._object); \
   } \
 })
 
@@ -79,7 +81,7 @@ typedef
   } \
   else if (TYPE_is_object((_var)->type)) \
   { \
-    OBJECT_REF((_var)->value._object, "VARIANT_keep"); \
+    OBJECT_REF((_var)->value._object); \
   } \
 })
 
