@@ -53,6 +53,8 @@ static WATCH_TIMER *_timers = NULL;
 static int _do_not_really_delete_callback = 0;
 static bool _must_delete_callback = FALSE;
 
+static struct timeval _last_time = { 0 };
+
 #ifdef DEBUG_TIMER
 double _debug_time;
 static double time_to_double(const struct timeval *t)
@@ -98,7 +100,7 @@ static struct timeval *time_now(void)
 
 	gettimeofday(&current, NULL);
 	if (current.tv_usec < 0 || current.tv_usec >= 1000000)
-		fprintf(stderr, "gettimeofday: tv_usec = %ld!\n", current.tv_usec);
+		fprintf(stderr, "gbx3: warning: gettimeofday: tv_usec = %ld!\n", current.tv_usec);
 	return &current;
 }
 
@@ -286,17 +288,17 @@ void WATCH_exit(void)
 static void watch_fd(int fd, int flag, bool watch)
 {
 	#if DEBUG_WATCH
-	fprintf(stderr, "watch_fd: %d for %s: %s\n", fd, flag == WATCH_READ ? "read" : "write", watch ? "set" : "clear");
+	fprintf(stderr, "watch_fd: %d for %s: %s\n", fd, flag == GB_WATCH_READ ? "read" : "write", watch ? "set" : "clear");
 	#endif
 
-	if (flag == WATCH_READ)
+	if (flag == GB_WATCH_READ)
 	{
 		if (watch)
 			FD_SET(fd, &read_fd);
 		else
 			FD_CLR(fd, &read_fd);
 	}
-	else if (flag == WATCH_WRITE)
+	else if (flag == GB_WATCH_WRITE)
 	{
 		if (watch)
 			FD_SET(fd, &write_fd);
@@ -394,22 +396,29 @@ static void watch_delete_callback(int fd)
 	pos = watch_find_callback(fd);
 	if (pos < 0)
 		return;
-	
+
 	//watch_index[fd] = -1;
-	
+
 	wcb = &watch_callback[pos];
 	wcb->fd = -1;
-	watch_fd(fd, WATCH_READ, FALSE);
-	watch_fd(fd, WATCH_WRITE, FALSE);
+	watch_fd(fd, GB_WATCH_READ, FALSE);
+	watch_fd(fd, GB_WATCH_WRITE, FALSE);
 	max_fd = find_max_fd();
 	
 	if (_do_not_really_delete_callback)
 	{
+		#if DEBUG_WATCH
+		fprintf(stderr, "--> do not really delete\n");
+		#endif
 		_must_delete_callback = TRUE;
 		return;
 	}
 		
 	ARRAY_remove(&watch_callback, pos);
+
+	#if DEBUG_WATCH
+	fprintf(stderr, "--> deleted\n");
+	#endif
 }
 
 
@@ -419,17 +428,17 @@ void WATCH_watch(int fd, int type, void *callback, intptr_t param)
 
 	if (fd < 0 || fd > FD_SETSIZE)
 	{
-		if (type != WATCH_NONE)
+		if (type != GB_WATCH_NONE)
 			ERROR_warning("trying to watch fd #%d", fd);
 		return;
 	}
 
-	if (type == WATCH_NONE)
+	if (type == GB_WATCH_NONE)
 		watch_delete_callback(fd);
 	else
 	{
 		wcb = watch_create_callback(fd);
-		if (type == WATCH_READ)
+		if (type == GB_WATCH_READ)
 		{
 			wcb->callback_read = callback;
 			wcb->param_read = param;
@@ -448,8 +457,8 @@ void WATCH_watch(int fd, int type, void *callback, intptr_t param)
 			watch_delete_callback(fd);
 		else
 		{
-			watch_fd(fd, WATCH_READ, wcb->callback_read != NULL);
-			watch_fd(fd, WATCH_WRITE, wcb->callback_write != NULL);
+			watch_fd(fd, GB_WATCH_READ, wcb->callback_read != NULL);
+			watch_fd(fd, GB_WATCH_WRITE, wcb->callback_write != NULL);
 		}
 	}
 }
@@ -487,7 +496,7 @@ static void raise_callback(fd_set *rfd, fd_set *wfd)
 				#ifdef DEBUG_WATCH
 				fprintf(stderr, "call read callback on fd %d\n", wcb.fd);
 				#endif
-				(*(wcb.callback_read))(wcb.fd, WATCH_READ, wcb.param_read);
+				(*(wcb.callback_read))(wcb.fd, GB_WATCH_READ, wcb.param_read);
 			}
 		}
 
@@ -499,7 +508,7 @@ static void raise_callback(fd_set *rfd, fd_set *wfd)
 				#ifdef DEBUG_WATCH
 				fprintf(stderr, "call write callback on fd %d\n", wcb.fd);
 				#endif
-				(*(wcb.callback_write))(wcb.fd, WATCH_WRITE, wcb.param_write);
+				(*(wcb.callback_write))(wcb.fd, GB_WATCH_WRITE, wcb.param_write);
 			}
 		}
 	}
@@ -508,6 +517,9 @@ static void raise_callback(fd_set *rfd, fd_set *wfd)
 	
 	if (!_do_not_really_delete_callback && _must_delete_callback)
 	{
+		#if DEBUG_WATCH
+		fprintf(stderr, "do must delete callback\n");
+		#endif
 		i = 0;
 		while (i < ARRAY_count(watch_callback))
 		{
@@ -563,7 +575,7 @@ static bool do_loop(struct timeval *wait)
 	else
 	{
 		ret = do_select(&rfd, &wfd, &tv);
-		something_done = TRUE;
+		something_done = ARRAY_count(_timers) > 0;
 	}
 
 	if (ret < 0 && errno != EINTR)
@@ -703,3 +715,16 @@ void WATCH_timer(void *t, int on)
 	}
 }
 
+void WATCH_little_sleep(void)
+{
+	struct timeval tv;
+	struct timeval *now;
+
+	now = time_now();
+	tv = *now;
+	time_sub(&tv, &_last_time);
+	if (tv.tv_sec == 0 && tv.tv_usec < 10000)
+		usleep(10000 - tv.tv_usec);
+	_last_time = *now;
+}
+
