@@ -32,12 +32,6 @@
 #include <signal.h>
 #endif
 
-enum {
-  WATCH_NONE = 0,
-  WATCH_READ = 1,
-  WATCH_WRITE = 2,
-  };
-
 enum {
 	WP_NOTHING = 0,
 	WP_END = 1,
@@ -62,7 +56,6 @@ typedef
 	}
 	WATCH_TIMER;
 
-
 void WATCH_init(void);
 void WATCH_exit(void);
 
@@ -74,5 +67,6 @@ int WATCH_loop_signal(const sigset_t *sig);
 int WATCH_process(int fd_end, int fd_output, int timeout);
 void WATCH_timer(void *t, int on);
 double WATCH_get_timeout(GB_TIMER *timer);
+void WATCH_little_sleep(void);
 
 #endif /* */
