@@ -436,7 +436,7 @@ GB_DESC PolygonDesc[] =
 	GB_METHOD("_free", NULL, Polygon_free, NULL),
 
 	GB_METHOD("_get", "PointF", Polygon_get, "(Index)i"),
-	GB_METHOD("_put", NULL, Polygon_put, "(Index)i(Point)PointF;"),
+	GB_METHOD("_put", NULL, Polygon_put, "(Point)PointF;(Index)i"),
 
 	GB_PROPERTY_READ("Count", "i", Polygon_Count),
 	GB_PROPERTY_READ("Max", "i", Polygon_Max),
@@ -448,7 +448,7 @@ GB_DESC PolygonDesc[] =
 	GB_METHOD("Clean", "Polygon", Polygon_Clean, "[(Distance)f]"),
 
 	GB_METHOD("Add", NULL, Polygon_Add, "(X)f(Y)f"),
-	GB_METHOD("AddPoint", NULL, Polygon_AddPoint, "(Point)Point;"),
+	GB_METHOD("AddPoint", NULL, Polygon_AddPoint, "(Point)PointF;"),
 	GB_METHOD("Remove", NULL, Polygon_Remove, "(Index)i[(Count)i]"),
 
 	GB_INTERFACE("_convert", &_convert_polygon),
