@@ -60,6 +60,9 @@ enum
 	GB_DEBUG_SET_READ_ONLY = 2
 };
 
+typedef
+	void (*GB_DEBUG_ENUM_CB)(char *key, int len);
+
 typedef
 	struct {
 		void *(*GetExec)(void);
@@ -73,11 +76,14 @@ typedef
 		bool (*GetValue)(const char *sym, int len, GB_VARIANT *ret);
 		int (*SetValue)(const char *sym, int len, VALUE *value);
 		void (*GetArrayValue)(GB_ARRAY array, int index, GB_VALUE *value);
-		void (*EnumKeys)(void *collection, char **key);
+		void (*EnumKeys)(void *collection, GB_DEBUG_ENUM_CB cb);
 		void *(*GetNextSortedSymbol)(void *klass, int *index);
 		int (*GetObjectAccessType)(void *object, CLASS *klass, int *count);
 		GB_CLASS (*FindClass)(const char *name);
 		int *(*GetArrayBounds)(void *array);
+		void (*BreakOnError)(bool);
+		void (*EnterEval)(void);
+		void (*LeaveEval)(void);
 		}
 	GB_DEBUG_INTERFACE;
 
@@ -101,6 +107,7 @@ typedef
 			void (*Exit)(void);
 			void (*Begin)(void *cp, void *fp);
 			void (*End)(void *cp, void *fp);
+			void (*Cancel)(void);
 			}
 			Profile;
 		void *_null;
