@@ -2,7 +2,7 @@
 
   main.h
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -26,12 +26,14 @@
 
 #include "gambas.h"
 #include "gb.image.h"
+#include "gb.geom.h"
 #include "gb.draw.h"
 #include "gb_common.h"
 
 #ifndef __MAIN_C
 extern GB_INTERFACE GB;
 extern IMAGE_INTERFACE IMAGE;
+extern GEOM_INTERFACE GEOM;
 #endif
 
 #endif /* __MAIN_H */
