@@ -2,7 +2,7 @@
 
   CHighlight.c
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -144,7 +144,6 @@ static int convState(int state)
 		case EVAL_TYPE_OPERATOR: return HIGHLIGHT_OPERATOR;
 		case EVAL_TYPE_DATATYPE: return HIGHLIGHT_DATATYPE;
 		case EVAL_TYPE_ERROR: return HIGHLIGHT_ERROR;
-		case EVAL_TYPE_ALTERNATE: return HIGHLIGHT_ALTERNATE;
 		case EVAL_TYPE_HELP: return HIGHLIGHT_HELP;
 		case EVAL_TYPE_PREPROCESSOR: return HIGHLIGHT_PREPROCESSOR;
 		default: return HIGHLIGHT_NORMAL;
@@ -207,13 +206,8 @@ static void analyze(const char *src, int len_src, bool rewrite, int state)
 	_analyze_pos = parray;
 	GB.Ref(parray);
 	
-	if (rewrite)
-	{
-		GB.FreeString(&_analyze_text);
-		_analyze_text = result.str;
-	}
-	else
-		GB.FreeString(&result.str);  
+	GB.FreeString(&_analyze_text);
+	_analyze_text = result.str;
 }
 
 BEGIN_METHOD_VOID(CHIGHLIGHT_exit)
@@ -271,6 +265,12 @@ BEGIN_PROPERTY(CHIGHLIGHT_analyze_text)
 
 END_PROPERTY
 
+BEGIN_PROPERTY(Highlight_Alternate)
+
+	GB.Deprecated("gb.eval", "Highlight.Alternate", NULL);
+	GB.ReturnInteger(-1);
+
+END_PROPERTY
 
 GB_DESC CHighlightDesc[] =
 {
@@ -292,9 +292,9 @@ GB_DESC CHighlightDesc[] =
 	GB_CONSTANT("Highlight", "i", HIGHLIGHT_HIGHLIGHT),
 	GB_CONSTANT("CurrentLine", "i", HIGHLIGHT_LINE),
 	GB_CONSTANT("Error", "i", HIGHLIGHT_ERROR),
-	GB_CONSTANT("Alternate", "i", HIGHLIGHT_ALTERNATE),
 	GB_CONSTANT("Help", "i", HIGHLIGHT_HELP),
 	GB_CONSTANT("Preprocessor", "i", HIGHLIGHT_PREPROCESSOR),
+	GB_STATIC_PROPERTY_READ("Alternate", "i", Highlight_Alternate),
 	
 	GB_STATIC_METHOD("_exit", NULL, CHIGHLIGHT_exit, NULL),
 	GB_STATIC_METHOD("Analyze", "String[]", CHIGHLIGHT_analyze, "(Code)s[(Rewrite)b(State)i]"),
