@@ -2,7 +2,7 @@
 
   gb_alloc_override.h
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -26,9 +26,9 @@
 
 #include "main.h"
 
-#define ALLOC(_ptr, _len, _where) GB.Alloc((void **)(void *)_ptr, _len)
-#define FREE(_ptr, _where) GB.Free((void **)(void *)_ptr)
-#define REALLOC(_ptr, _len, _where) GB.Realloc((void **)(void *)_ptr, _len)
+#define ALLOC(_ptr, _len) GB.Alloc((void **)(void *)_ptr, _len)
+#define FREE(_ptr) GB.Free((void **)(void *)_ptr)
+#define REALLOC(_ptr, _len) GB.Realloc((void **)(void *)_ptr, _len)
 
 #endif
 
