@@ -55,6 +55,7 @@ bool CODE_check_pop_local_last(short *local);
 bool CODE_check_jump_not(void);
 
 bool CODE_check_varptr(void);
+bool CODE_check_ismissing(void);
 
 void CODE_allow_break(void);
 //void CODE_break(void);
@@ -126,15 +127,18 @@ void CODE_push_boolean(bool value);
 void CODE_push_inf(bool neg);
 void CODE_push_complex();
 
+void CODE_push_vargs();
+void CODE_drop_vargs();
+
 void CODE_dup(void);
 
 void CODE_return(int return_value);
 
-void CODE_push_char(char car);
+void CODE_push_char(uchar car);
 void CODE_push_void(void);
 
-void CODE_subr(short subr, short nparam, short optype, bool output, bool fixed);
-void CODE_subr_output(short subr, short nparam, int output);
+void CODE_subr(short subr, short nparam, short optype, bool fixed);
+//void CODE_subr_output(short subr, short nparam, int output);
 
 void CODE_call(short nparam);
 void CODE_call_byref(short nparam, uint64_t byref);
