@@ -205,7 +205,7 @@ void CODE_end_function(FUNCTION *func)
 
 static ushort *get_last_code()
 {
-	if (cur_func->last_code == CODE_NO_POS)
+	if ((ushort)cur_func->last_code == CODE_NO_POS)
 		return NULL;
 
 	return &cur_func->code[cur_func->last_code];
@@ -213,7 +213,7 @@ static ushort *get_last_code()
 
 static ushort *get_last_code2()
 {
-	if (cur_func->last_code2 == CODE_NO_POS)
+	if ((ushort)cur_func->last_code2 == CODE_NO_POS)
 		return NULL;
 
 	return &cur_func->code[cur_func->last_code2];
@@ -349,12 +349,29 @@ bool CODE_check_varptr(void)
 	op = *last_code;
 	if (!((op & 0xFF00) == C_PUSH_LOCAL || (op & 0xFF00) == C_PUSH_PARAM || (op & 0xF800) == C_PUSH_STATIC || (op & 0xF800) == C_PUSH_DYNAMIC))
 		return TRUE;
-	
+
 	*last_code = C_PUSH_INTEGER;
 	write_short((short)op);
 	return FALSE;
 }
 
+bool CODE_check_ismissing(void)
+{
+	unsigned short op;
+	PCODE *last_code;
+
+	last_code = get_last_code();
+	if (!last_code)
+		return TRUE;
+
+	op = *last_code;
+	if ((op & 0xFF00) != C_PUSH_PARAM)
+		return TRUE;
+
+	*last_code = C_PUSH_QUICK | (op & 0xFF);
+	return FALSE;
+}
+
 void CODE_nop(void)
 {
 	LAST_CODE;
@@ -1014,7 +1031,7 @@ void CODE_stop(void)
 #endif /* PROJECT_COMP */
 
 
-void CODE_push_char(char car)
+void CODE_push_char(uchar car)
 {
 	LAST_CODE;
 
@@ -1056,26 +1073,20 @@ void CODE_stop_event(void)
 #endif
 
 
-void CODE_subr(short subr, short nparam, short optype, bool output, bool fixed)
+void CODE_subr(short subr, short nparam, short optype, bool fixed)
 {
 	LAST_CODE;
 
-	if (output)
-		use_stack(0);
-	else
-		use_stack(1 - nparam);
+	use_stack(1 - nparam);
 
 	#ifdef DEBUG
-	printf("SUBR %s %d (%d)\n", output ? "OUTPUT" : "", subr, nparam);
+	printf("SUBR %d (%d)\n", subr, nparam);
 	#endif
 
 	if (optype == 0)
 	{
 		if (fixed)
 			nparam = 0;
-
-		/*if (output)
-			nparam |= CODE_CALL_OUTPUT;*/
 	}
 	else
 	{
@@ -1087,7 +1098,7 @@ void CODE_subr(short subr, short nparam, short optype, bool output, bool fixed)
 }
 
 
-void CODE_subr_output(short subr, short nparam, int output)
+/*void CODE_subr_output(short subr, short nparam, int output)
 {
 	LAST_CODE;
 
@@ -1099,7 +1110,7 @@ void CODE_subr_output(short subr, short nparam, int output)
 
 	subr += CODE_FIRST_SUBR;
 	write_short(((subr & 0xFF) << 8) | (nparam & 0xFF));
-}
+}*/
 
 
 void CODE_call(short nparam)
@@ -1360,6 +1371,28 @@ void CODE_push_complex(void)
 	write_ZZxx(C_PUSH_MISC, CPM_COMPLEX);
 }
 
+void CODE_push_vargs(void)
+{
+	LAST_CODE;
+
+	#ifdef DEBUG
+	printf("PUSH VARGS\n");
+	#endif
+
+	write_ZZxx(C_PUSH_MISC, CPM_VARGS);
+}
+
+void CODE_drop_vargs(void)
+{
+	LAST_CODE;
+
+	#ifdef DEBUG
+	printf("DROP VARGS\n");
+	#endif
+
+	write_ZZxx(C_PUSH_MISC, CPM_DROP_VARGS);
+}
+
 #ifdef CODE_DUMP
 
 void CODE_dump(PCODE *code, int count)
