@@ -77,6 +77,8 @@
 #define CPM_PINF         6
 #define CPM_MINF         7
 #define CPM_COMPLEX      8
+#define CPM_VARGS        9
+#define CPM_DROP_VARGS   10
 
 #define C_TRY                   0x1600
 #define C_END_TRY               0x1700
