@@ -110,6 +110,7 @@ COMP_INFO COMP_res_info[] =
 	{ "Let"                                                       },
 	{ "+Inf"                                                      },
 	{ "-Inf"                                                      },
+	{ "Use"                                                       },
 
 	{ "Print"                                                     },
 	{ "Input"                                                     },
@@ -448,6 +449,7 @@ SUBR_INFO COMP_subr_info[] =
 
 	{ ".Lock",             64,  0,  1     },
 	{ ".Unlock",           64,  1,  1     },
+	{ ".LockWait",         64,  2,  2     },
 
 	{ ".InputFrom",        65,  0,  1     },
 	{ ".OutputTo",         65,  1,  1     },
@@ -464,6 +466,8 @@ SUBR_INFO COMP_subr_info[] =
 	//{ ".Mkdir",            70,  0,  1     }, // The old Mkdir from 3.0
 	{ "Even",              70,  1,  1     },
 	{ "Odd",               70,  2,  1     },
+	//{ ".Rmdir",            71,  0,  1     }, // The old Rmdir from 3.0
+	{ "Rand",              71,  0,  1,  2 },
 #ifndef __EVAL_RESERVED_C
 	{ ".Move",             72,  0,  2     },
 	{ ".Copy",             72,  1,  2     },
@@ -472,8 +476,8 @@ SUBR_INFO COMP_subr_info[] =
 	{ ".Chown",            72,  4,  2     },
 	{ ".Chgrp",            72,  5,  2     },
 #endif
-	{ "Swap",              73,  1,  1,  2 }, // at least one argument, because 73 is a deprecated Copy() too.
-	{ "Swap$",             73,  1,  1,  2 },
+	{ "Swap",              73,  0,  1,  2 }, // at least one argument, because 73 is a deprecated Copy() too.
+	{ "Swap$",             73,  0,  1,  2 },
 	
 	{ "IsNan",             74,  1,  1     },
 	{ "IsInf",             74,  2,  1     },
@@ -504,9 +508,12 @@ SUBR_INFO COMP_subr_info[] =
 
 #ifndef __EVAL_RESERVED_C
 	{ ".Sleep",            88,  0,  1     },
+	{ ".Use",              88,  1,  1     },
+	{ ".CheckExec",        88,  2,  1     },
 #endif
 
 	{ "VarPtr",            89,  0,  1     },
+	{ "IsMissing",         89,  1,  1     },
 
 	{ ".Collection",       90,  0,  1, 63 },
 	
@@ -521,12 +528,18 @@ SUBR_INFO COMP_subr_info[] =
 	{ "Html$",             92,  2,  1     },
 	{ "Base64",            92,  3,  1     },
 	{ "Base64$",           92,  3,  1     },
-	
+	{ "Url",               92,  4,  1     },
+	{ "Url$",              92,  4,  1     },
+
 	{ "UnQuote",           93,  0,  1     },
 	{ "UnQuote$",          93,  0,  1     },
 	{ "UnBase64",          93,  1,  1     },
 	{ "UnBase64$",         93,  1,  1     },
-	
+	{ "FromBase64",        93,  1,  1     },
+	{ "FromBase64$",       93,  1,  1     },
+	{ "FromUrl",           93,  2,  1     },
+	{ "FromUrl$",          93,  2,  1     },
+
 	{ "MkBool",            94,  1,  1     },
 	{ "MkBool$",           94,  1,  1     },
 	{ "MkBoolean",         94,  1,  1     },
